/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.JWindow;
import javax.swing.border.EmptyBorder;

/**
 * @author Vincent Tscherter
 */
public final class Browser extends JComponent {

	static final String TITLE = Messages.getString("Browser.title") + " [" + Locale.getDefault().getLanguage() + "]"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	public static void main(String[] args) {
		Browser.showBrowser(Shared.HOME, new HTML(Messages
				.getString("Browser.file.home"))); //$NON-NLS-1$ //$NON-NLS-2$
	}

	

	public static Browser getBrowser(JComponent c) {
		JComponent p = c;
		while (p != null && p.getParent() != null) {
			if (p instanceof Browser) {
				lastused = (Browser) p;
				return lastused;
			}
			if (p.getParent() != null && p.getParent() instanceof JComponent) {
				p = (JComponent) p.getParent();
			} else {
				p = null;
			}
		}
		return lastused;
	}

	private static Browser lastused;

	JLabel statuslabel = new JLabel();

	public static void showBrowser(String label, Page home) {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		String version = System.getProperties().getProperty("java.version"); //$NON-NLS-1$
		StringTokenizer st = new StringTokenizer(version, "."); //$NON-NLS-1$
		int v = Integer.parseInt(st.nextToken());
		if (v < 1) {
			showWrongVersionDialog(version);
		} else {
			v = Integer.parseInt(st.nextToken());
			if (v < 4) {
				showWrongVersionDialog(version);
			}
		}
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		JWindow splash;
		splash = new JWindow();
		splash.getContentPane().setLayout(new BorderLayout());
		splash.getContentPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		splash.getContentPane().add(
				new JLabel(Shared.loadIcon(Messages
						.getString("Browser.image.splash"))), //$NON-NLS-1$
				BorderLayout.CENTER);
		splash.pack();
		splash.setLocation((screen.width - splash.getWidth()) / 2,
				(screen.height - splash.getHeight()) / 2);
		splash.setVisible(true);
		JFrame f = new JFrame();

		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		f.getContentPane().setBackground(Color.white);
		f.getContentPane().setLayout(new BorderLayout());
		lastused = new Browser(home, label);
		Browser.statusOut(lastused, "java.version: " //$NON-NLS-1$
				+ System.getProperties().getProperty("java.version")); //$NON-NLS-1$
		f.getContentPane().add(lastused, BorderLayout.CENTER);
		f.setBounds(screen.width / 12, screen.height / 12, screen.width - 2
				* screen.width / 12, screen.height - screen.height / 6);
		f.validate();
		f.setVisible(true);
		f.setTitle(TITLE);
		splash.dispose();
	}

	private static void showWrongVersionDialog(String version) {
		JOptionPane
				.showMessageDialog(
						null,
						"Your are using Java $VERSION.\nTo ensure Exorciser to work properly, you should use Java 1.4 or higher!" //$NON-NLS-1$
						.replaceFirst("\\$VERSION", //$NON-NLS-1$ //$NON-NLS-2$
								version));
	}

	private Action homeAction = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
			.getString("Browser.icon.home"))) { //$NON-NLS-1$

		public void actionPerformed(ActionEvent e) {
			setPage((Page)pages.get(0));
			statuslabel.setText(" "); //$NON-NLS-1$
		}
	};

	private Action forwardAction = new AbstractAction("", Shared//$NON-NLS-1$
			.loadIcon(Messages.getString("Browser.icon.forward"))) { //$NON-NLS-1$

		public void actionPerformed(ActionEvent e) {
			remove(((Page)pages.get(current)).getPageUI());
			current++;
			showPage((Page)pages.get(current));
			statuslabel.setText(" "); //$NON-NLS-1$
		}
	};

	private Action backAction = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
			.getString("Browser.icon.back"))) { //$NON-NLS-1$

		public void actionPerformed(ActionEvent e) {
			remove(((Page)pages.get(current)).getPageUI());
			current--;
			showPage((Page)pages.get(current));
			statuslabel.setText(" "); //$NON-NLS-1$
		}
	};

	List pages = new ArrayList();

	int current = -1;

	public Browser(Page home, String title) {
		setLayout(new BorderLayout());
		setPage(home);
		JToolBar left = new JToolBar();
		left.setFloatable(false);
		JButton homeb = new JButton();
		homeb.setAction(homeAction);
		homeb.setText(title);
		homeb.setToolTipText(Shared.HOME);
		homeb.setBorder(new EmptyBorder(5, 5, 5, 5));
		left.add(homeb);
		//
		JButton backButton = new JButton(backAction);
		backButton.setToolTipText(Shared.BACK);
		backButton.setBorder(new EmptyBorder(5, 5, 5, 5));
		left.add(backButton);
		//
		JButton forwardButton = new JButton(forwardAction);
		forwardButton.setToolTipText(Shared.FORWARD);
		forwardButton.setBorder(new EmptyBorder(5, 5, 5, 5));
		left.add(forwardButton);
		//
		left.setBorder(BorderFactory.createEtchedBorder());
		add(left, BorderLayout.NORTH);
		statuslabel.setOpaque(true);
		statuslabel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createEtchedBorder(), BorderFactory.createEmptyBorder(2, 5, 2,
				5)));
		statuslabel.setFont(statuslabel.getFont().deriveFont(Font.PLAIN));
		add(statuslabel, BorderLayout.SOUTH);
	}

	public static void statusOut(Object source, String text) {
		if (source instanceof JComponent) {
			Browser b = Browser.getBrowser((JComponent) source);
			if (b != null) {
				b.statuslabel.setText(text);
				return;
			}
		}
		Debug.showException(new Exception("no browser for message: " + text)); //$NON-NLS-1$
	}

	public void setPage(Page page) {
		if (0 <= current && current < pages.size()) {
			remove(((Page)pages.get(current)).getPageUI());
		}
		while (pages.size() > current + 1) {
			pages.remove(pages.size() - 1);
		}
		pages.add(page);
		current++;
		showPage(page);
	}

	public Page getPage() {
		return (Page)pages.get(current);
	}

	void showPage(Page page) {
		backAction.setEnabled((current > 0));
		forwardAction.setEnabled(current < pages.size() - 1);
		add(page.getPageUI(), BorderLayout.CENTER);
		if (getRootPane() != null
				&& getRootPane().getParent() instanceof JFrame) {
			String title = page.getTitle();
			if (title != null) {
				title = title.trim();
			}
			if (title == null || title.equals("")) { //$NON-NLS-1$
				title = TITLE;
			} else {
				title += " - " + TITLE; //$NON-NLS-1$
			}
			((JFrame) getRootPane().getParent()).setTitle(title);
		}
		revalidate();
		repaint();
	}
}