/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;

/**
 * @author Vincent Tscherter
 */
public class Notifications extends JPanel implements Scrollable {

   public Notifications () {
      super();
      setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
   }

   public static String createFeedback (String title, String text) {
      StringBuffer s = new StringBuffer("<html><head><style>body {color : #000000; background : #FFFFFF; font-family : Dialog, sans-serif; } .title { font-size: 110%; font-weight: bold; color: #444444;} a {text-decoration: none}</style></head><body>"); //$NON-NLS-1$
      if (title != null && !title.equals("")) { //$NON-NLS-1$
         s.append("<div class=\"title\">" + title + "</div>"); //$NON-NLS-1$ //$NON-NLS-2$
      }
      if (text != null && !text.equals("")) { //$NON-NLS-1$
      	s.append("<div>" + text + "</div>"); //$NON-NLS-1$ //$NON-NLS-2$
      }
      s.append("</body></html>"); //$NON-NLS-1$
      return s.toString();
   }

   public JEditorPane addNotification (String title, String text) {
      return addNotification(title, text, Shared.loadIcon(Messages
            .getString("Notifications.icon.blank"))); //$NON-NLS-1$
   }

   public JEditorPane addWarningNotification (String title, String text) {
      return addNotification(title, text, Shared.loadIcon(Messages
            .getString("Notifications.icon.warning"))); //$NON-NLS-1$
   }

   public JEditorPane addHintNotification (String title, String text) {
      return addNotification(title, text, Shared.loadIcon(Messages
            .getString("Notifications.icon.hint"))); //$NON-NLS-1$
   }

   public JEditorPane addCorrectNotification (String title, String text) {
      return addNotification(title, text, Shared.loadIcon(Messages
            .getString("Notifications.icon.correct"))); //$NON-NLS-1$
   }

   public JEditorPane addErrorNotification (String title, String text) {
      return addNotification(title, text, Shared.loadIcon(Messages
            .getString("Notifications.icon.error"))); //$NON-NLS-1$
   }

   public JEditorPane addNotification (String title, String text, ImageIcon icon) {
      JEditorPane p = new JEditorPane("text/html", //$NON-NLS-1$
            createFeedback(title, text));
      p.setMargin(new Insets(10, 10, 10, 10));
      p.setEditable(false);
      p.addHyperlinkListener(new ExtendedHyperlinkListener());
      JPanel q = new JPanel(new BorderLayout());
      JLabel l = new JLabel(icon);
      l.setVerticalAlignment(SwingConstants.VERTICAL);
      l.setBorder(BorderFactory.createEmptyBorder(10, 1, 10, 1));
      q.add(l, BorderLayout.WEST);
      q.add(p, BorderLayout.CENTER);
      q.setBorder(BorderFactory.createEtchedBorder());
      add(q);
      revalidate();
      repaint();
      return p;
   }

   Dimension d = new Dimension(200, 150);

   public Dimension getPreferredScrollableViewportSize () {
      return d;
   }

   public int getScrollableBlockIncrement (Rectangle arg0, int arg1, int arg2) {
      return 5;
   }

   public boolean getScrollableTracksViewportHeight () {
      return false;
   }

   public boolean getScrollableTracksViewportWidth () {
      return true;
   }

   public int getScrollableUnitIncrement (Rectangle arg0, int arg1, int arg2) {
      return 5;
   }

   public void removeAll () {
      super.removeAll();
      revalidate();
      repaint();
   }
}