/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.cyk;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.cyk.Tools.CouldNotGenerateWordException;
import ch.ethz.exorciser.cyk.Tools.ParseException;

/**
 * @author Vincent Tscherter
 */
public class CYKExercise extends JPanel implements Exercise {

    Notifications notifications = new Notifications();

    public void random(String query) {
        String level = Query.getValue(query, "level"); //$NON-NLS-1$
        notifications.removeAll();
        try {
            if (level.equals("beginner")) { //$NON-NLS-1$
                word = Tools.createWord(grammar, 4, 50);
            } else if (level.equals("advanced")) { //$NON-NLS-1$
                word = Tools.createWord(grammar, 7, 100);
            } else if (level.equals("expert")) { //$NON-NLS-1$
                word = Tools.createWord(grammar, 11, 200);
            }
            cykpanel.setWord(word);
        } catch (CouldNotGenerateWordException e) {
            notifications.addWarningNotification(Messages
                    .getString("CYK.sorry.title"), //$NON-NLS-1$
                    Messages.getString("CYK.sorry")); //$NON-NLS-1$
        }
    }

    public void reset() {
        notifications.removeAll();
        cykpanel.reset();
    }

    public void solve() {
        notifications.removeAll();
        if (cykpanel.solve()) {
            notifications.addNotification(Messages.getString("CYK.fill_table"), //$NON-NLS-1$
                    Shared.DONE);
        } else {
            notifications.addNotification(Messages.getString("CYK.fill_table"), //$NON-NLS-1$
                    Messages.getString("CYK.nothing_to_do")); //$NON-NLS-1$
        }
    }

    public void next() {
        notifications.removeAll();
        cykpanel.next(notifications);
    }

    public void check() {
        notifications.removeAll();
        cykpanel.check(notifications);
    }

    static final Object[] w1 = {};

    CYKPanel cykpanel;

    Grammar grammar;

    Object[] word = { "s", "o", "r", "r", "y" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

    public CYKExercise(String query) {
        String id = Query.getValue(query, "id"); //$NON-NLS-1$
        BufferedReader br = null;
        try {
            br = new BufferedReader(new InputStreamReader(ClassLoader
                    .getSystemResourceAsStream("resources/cyk/" + id //$NON-NLS-1$
                            + ".grammar"), "UTF-8")); //$NON-NLS-1$ //$NON-NLS-2$
            String g = ""; //$NON-NLS-1$
            String l = br.readLine();
            while (l != null) {
                g += l + "\n"; //$NON-NLS-1$
                l = br.readLine();
            }
            grammar = Tools.createGrammar(g);
            br.close();
        } catch (Exception e2) {
            e2.printStackTrace();
        } finally {
            try {
                if (br != null) {
                    br.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            word = Tools.createWord(grammar, 4, 50);
        } catch (CouldNotGenerateWordException e1) {
            e1.printStackTrace();
        }
        cykpanel = new CYKPanel(grammar, word);
        JEditorPane task = ExerciseUIFactory.createTaskPane(this);
        //
        JPanel main = new JPanel(new BorderLayout());
        main.setBackground(Color.white);
        main.add(task, BorderLayout.NORTH);
        main.add(cykpanel, BorderLayout.CENTER);
        JEditorPane p = new JEditorPane();
        p.setEditable(false);
        p.addHyperlinkListener(new ExtendedHyperlinkListener());
        try {
            p.setPage(ClassLoader.getSystemResource(Messages
                    .getString("CYK.file.control"))); //$NON-NLS-1$
        } catch (IOException e) {
            Debug.showException(e);
        }
        ExerciseUIFactory.createExerciseUI(main, p, notifications, this);
    }

    public JComponent getPageUI() {
        return this;
    }

    public String getTitle() {
        return Messages.getString("CYK.exercise.title"); //$NON-NLS-1$
    }

    JDialog jd;

    JTextField tf;

    JTextArea ta;

    public void custom() {
        jd = new JDialog((Frame) null,
                Messages.getString("CYK.edit_word"), true); //$NON-NLS-1$
        jd.getContentPane().setLayout(new BorderLayout());
        tf = new JTextField(20);
        tf.setText(Tools.WordToString(word));
        JButton ok = new JButton(Shared.OKAY); //$NON-NLS-1$
        ok.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                Object[] w = Tools.extractSymbols(tf.getText());
                if (w.length < 1) {
                    JOptionPane.showMessageDialog(null, Messages
                            .getString("CYK.word_is_to_short"), //$NON-NLS-1$
                            Shared.ERROR, //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                } else {
                    word = w;
                    cykpanel.setWord(word);
                    notifications.removeAll();
                    jd.dispose();
                }
            }
        });
        JButton cancel = new JButton(Shared.CANCEL); //$NON-NLS-1$
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                jd.dispose();
            }
        });
        JPanel buttons = new JPanel(new GridLayout(1, 2));
        buttons.add(ok);
        buttons.add(cancel);
        jd.getContentPane().add(buttons, BorderLayout.SOUTH);
        jd.getContentPane().add(tf, BorderLayout.CENTER);
        jd.pack();
        Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        jd.setLocation((screen.width - jd.getWidth()) / 2, (screen.height - jd
                .getHeight()) / 2);
        jd.setVisible(true);
    }

    public void edit() {
        jd = new JDialog((Frame) null, Messages.getString("CYK.edit_grammar"), //$NON-NLS-1$
                true);
        jd.getContentPane().setLayout(new BorderLayout());
        ta = new JTextArea(20, 30);
        ta.setText(grammar.toString());
        JButton ok = new JButton(Shared.OKAY);
        ok.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                try {
                    Grammar g = Tools.createGrammar(ta.getText());
                    if (!CYKParser.isCNFGrammer(g)) {
                        throw new Tools.ParseException();
                    }
					grammar = g;
					cykpanel.setGrammar(g);
					random("level=beginner"); //$NON-NLS-1$
					jd.dispose();
                } catch (ParseException e) {
                    JOptionPane.showMessageDialog(null, Messages
                            .getString("CYK.not_a_cnf_grammar"), //$NON-NLS-1$
                            Shared.ERROR, //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                }
            }
        });
        JButton cancel = new JButton(Shared.CANCEL); //$NON-NLS-1$
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                jd.dispose();
            }
        });
        JPanel buttons = new JPanel(new GridLayout(1, 2));
        buttons.add(ok);
        buttons.add(cancel);
        jd.getContentPane().add(buttons, BorderLayout.SOUTH);
        jd.getContentPane().add(new JScrollPane(ta), BorderLayout.CENTER);
        jd.pack();
        Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        jd.setLocation((screen.width - jd.getWidth()) / 2, (screen.height - jd
                .getHeight()) / 2);
        jd.setVisible(true);
    }

    public String getTask() {
        return Messages.getString("CYK.exercise"); //$NON-NLS-1$
    }
}