/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.cyk;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;

import javax.swing.JComponent;
import javax.swing.event.MouseInputAdapter;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Shared;

/**
 * @author Vincent Tscherter
 */
public class CYKPanel extends JComponent {

   private static final int OFF = 30;

   private static final int SEP = 60;

   private static final int PAD = 4;

   Grammar grammar;

   private Object[] word;

   CYKParseTable working;

   CYKParseTable solution;

   int mousex, mousey, dragx, dragy, rule, targetx, targety, kx, ky, kz;

   boolean dragging, notarget;

   private MouseInputAdapter ma = new MouseInputAdapter() {

      public void mouseMoved (MouseEvent e) {
         rule = -1;
         kz = -1;
         dragging = false;
         mousex = e.getX();
         mousey = e.getY();
         repaint();
      }

      public void mouseDragged (MouseEvent e) {
         if (rule != -1) {
            dragging = true;
            dragx = e.getX();
            dragy = e.getY();
            repaint();
         }
      }

      public void mouseClicked (MouseEvent e) {
         if (kz != -1) {
            working.remove(kx, ky, kz);
            working.validate();
            mouseMoved(e);
         }
      }

      public void mouseReleased (MouseEvent e) {
         if (dragging) {
            if (!notarget) {
               working.add(grammar.getRule(rule), targetx, targety);
               working.validate();
            }
            mouseMoved(e);
         }
      }
   };

   public CYKPanel (Grammar grammar, Object[] word) {
      solution = CYKParser.createCYKParseTable(grammar, word);
      this.setBackground(Color.white);
      //this.setOpaque(true);
      this.grammar = grammar;
      this.word = word;
      this.working = new CYKParseTable(word);
      //        this.working = CYKParser.createCYKParseTable(grammar, word);
      this.working.validate();
      this.setMinimumSize(new Dimension(100, 100));
      this.addMouseListener(ma);
      this.addMouseMotionListener(ma);
   }

   int h = 0, w = 0, fontoffset;

   int[] deltas;

   public void paint (Graphics g) {
      ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      int W = this.getWidth();
      int H = this.getHeight();
      g.clearRect(0, 0, W, H);
      // init
      g.setFont(g.getFont().deriveFont(Font.BOLD, 16f));
      //g.setFont(g.getFont().deriveFont( 16f));
      // draw grammar
      deltas = new int[word.length + 1];
      if (h == 0 || w == 0) {
         h = g.getFontMetrics().getHeight() + 2 * PAD;
         w = 0;
         for (int r = 0; r < grammar.nOfRules(); r++) {
            w = Math.max(w, g.getFontMetrics().stringWidth(
                  grammar.getRule(r).toString()));
         }
         w += 4 * PAD;
         fontoffset = g.getFontMetrics().getHeight()
               - g.getFontMetrics().getDescent();
      }
      //
      g.setColor(Color.gray);
      g.drawString("w = ", OFF + w + SEP //$NON-NLS-1$
            - g.getFontMetrics().stringWidth("w = "), OFF + PAD //$NON-NLS-1$
            + fontoffset);
      g.drawRoundRect(OFF + w + SEP, OFF + PAD, w * working.size(),
            h - 2 * PAD, 2 * PAD, 2 * PAD);
      g.drawString("G:", OFF + 2 * PAD, OFF + PAD + fontoffset); //$NON-NLS-1$
      g
            .drawRoundRect(OFF, OFF + h, w, h * grammar.nOfRules(), 2 * PAD,
                  2 * PAD);
      // draw grammar
      for (int r = 0; r < grammar.nOfRules(); r++) {
         if (OFF < mousex && mousex < OFF + w && OFF + (r + 1) * h < mousey
               && mousey < OFF + (r + 2) * h) {
            g.setColor(new Color(0f, 0f, 1f, 0.2f));
            g.fillRoundRect(OFF, OFF + (r + 1) * h, w, h, 2 * PAD, 2 * PAD);
            rule = r;
         }
         //            g.setColor(Color.black);
         drawRule(g, grammar.getRule(r), OFF + 2 * PAD, OFF + h + PAD + r * h
               + fontoffset);
         //            g.drawString(grammar.getRule(r).toString(), OFF + 2 * PAD, OFF +
         // h
         //                    + PAD + r * h + fontoffset);
      }
      // draw word
      for (int i = 0; i < word.length; i++) {
         g.setColor(Color.black);
         g.drawString(word[i].toString(), OFF + w + SEP + i * w + 2 * PAD, OFF
               + PAD + fontoffset);
         g.setColor(Color.gray);
         g.drawString(i + " ", OFF + w + SEP + i * w + 2 * PAD, OFF - h + 3 //$NON-NLS-1$
               * PAD + fontoffset);
      }
      // draw table
      deltas[0] = OFF + h;
      boolean targetfound = false;
      for (int y = 0; y < working.size(); y++) {
         int maxz = 1;
         for (int x = 0; x < working.size() - y; x++) {
            maxz = Math.max(maxz, working.height(x, y));
            for (int z = 0; z < working.height(x, y); z++) {
               if (OFF + SEP + w * (x + 1) < mousex
                     && mousex < OFF + SEP + w * (x + 2)
                     && deltas[y] + z * h < mousey
                     && mousey < deltas[y] + (z + 1) * h) {
                  kx = x;
                  ky = y;
                  kz = z;
               }
               //                    g.setColor(Color.black);
               //                    g.drawString(working.getRule(x, y, z).toString(), OFF +
               // SEP
               //                            + (x + 1) * w + 2 * PAD, deltas[y] + z * h + PAD
               //                            + fontoffset);
               drawRule(g, working.getRule(x, y, z), OFF + SEP + (x + 1) * w
                     + 2 * PAD, deltas[y] + z * h + PAD + fontoffset);
            }
         }
         deltas[y + 1] = deltas[y] + maxz * h;
         // draw cell borders & drag targets
         g.setColor(Color.gray);
         g.drawString(y + " ", OFF + w + SEP //$NON-NLS-1$
               - g.getFontMetrics().stringWidth(y + " "), deltas[y] + PAD //$NON-NLS-1$
               + fontoffset);
         for (int x = 0; x < working.size() - y; x++) {
            // cell borders;
            g.setColor(Color.gray);
            g.drawRoundRect(OFF + SEP + w + x * w, deltas[y], w, maxz * h,
                  2 * PAD, 2 * PAD);
            if (dragging && OFF + SEP + w * (x + 1) < dragx
                  && dragx < OFF + SEP + w * (x + 2) && deltas[y] < dragy
                  && dragy < deltas[y] + maxz * h) {
               targetx = x;
               targety = y;
               targetfound = true;
               if (!working.contains(grammar.getRule(rule), x, y)) {
                  drawGradient(g, y, x);
               } else {
                  g.setColor(new Color(1f, 0f, 0f, 0.25f));
                  g.fillRoundRect(OFF + SEP + w + x * w, deltas[y], w,
                        maxz * h, 2 * PAD, 2 * PAD);
               }
            }
         }
      }
      // draw tree ...
      for (int y = 0; y < working.size(); y++) {
         for (int x = 0; x < working.size() - y; x++) {
            if (OFF + SEP + w * (x + 1) < mousex
                  && mousex < OFF + SEP + w * (x + 2) && deltas[y] < mousey
                  && mousey < deltas[y + 1]) {
               boolean done = false;
               for (int z = 0; z < working.height(x, y); z++) {
                  if (deltas[y] + z * h < mousey
                        && mousey < deltas[y] + (z + 1) * h) {
                     ((Graphics2D) g).setStroke(new BasicStroke(2f,
                           BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                     drawTree(x, y, z, g);
                     done = true;
                  }
               }//} else {
               if (!done) {
                  drawGradient(g, y, x);
               }
            }
         }
      }
      // dragging
      if (dragging) {
         g.setColor(Color.black);
         //g.drawString(grammar.getRule(rule).toString(), dragx, dragy);
         drawRule(g, grammar.getRule(rule), dragx, dragy);
      }
      //
      notarget = !targetfound;
   }

   private void drawRule (Graphics g, Rule rule_, int x, int y) {
      Object[] p = rule_.getPattern();
      Object[] r = rule_.getReplacement();
      for (int i = 0; i < p.length; i++) {
         if (grammar.getStartSymbol().equals(p[i])) {
            g.setColor(Color.blue);
         } else {
            g.setColor(Color.black);
         }
         g.drawString(p[i].toString(), x, y);
         x += g.getFontMetrics().stringWidth(p[i] + " "); //$NON-NLS-1$
      }
      g.setColor(Color.black);
      g.drawString(Rule.ARROW, x, y);
      x += g.getFontMetrics().stringWidth(Rule.ARROW + " "); //$NON-NLS-1$
      for (int i = 0; i < r.length; i++) {
         if (grammar.getStartSymbol().equals(r[i])) {
            g.setColor(Color.blue);
         } else {
            g.setColor(Color.black);
         }
         g.drawString(r[i].toString(), x, y);
         x += g.getFontMetrics().stringWidth(r[i] + " "); //$NON-NLS-1$
      }
   }

   private void drawGradient (Graphics g, int y, int x) {
      for (int k = 0; k < y; k++) {
         g.setColor(new Color(k / (float) y, k / (float) y, 1f, 0.375f));
         g.fillRoundRect(OFF + SEP + w + x * w, deltas[k], w, deltas[k + 1]
               - deltas[k], 2 * PAD, 2 * PAD);
         g.fillRoundRect(OFF + SEP + w + (x + k + 1) * w, deltas[y - k - 1], w,
               deltas[y - k] - deltas[y - k - 1], 2 * PAD, 2 * PAD);
      }
      g.setColor(new Color(1f, 1f, 0f, 0.25f));
      g.fillRoundRect(OFF + SEP + (x + 1) * w, OFF + PAD, w * (y + 1), h - 2
            * PAD, 2 * PAD, 2 * PAD);
      g.fillRoundRect(OFF + SEP + w + x * w, deltas[y], w, deltas[y + 1]
            - deltas[y], 2 * PAD, 2 * PAD);
   }

   private void drawTree (int x, int y, int z, Graphics g) {
      g.setColor(new Color(0f, 0f, 1f, .1f));
      g.fillRoundRect(OFF + SEP + (x + 1) * w + PAD, deltas[y] + z * h + PAD, w
            - 2 * PAD, h - 2 * PAD, 2 * PAD, 2 * PAD);
      g.setColor(new Color(0f, 0f, 1f, 1f));
      g.drawRoundRect(OFF + SEP + (x + 1) * w + PAD, deltas[y] + z * h + PAD, w
            - 2 * PAD, h - 2 * PAD, 2 * PAD, 2 * PAD);
      if (working.isValid(x, y, z)) {
         if (y > 0) {
            int dy = working.getDY(x, y, z);
            int z0 = working.getZ0(x, y, z);
            int z1 = working.getZ1(x, y, z);
            g.drawLine(OFF + SEP + (x + 1) * w + w / 2, deltas[dy] + (z0 + 1)
                  * h + -PAD, OFF + SEP + (x + 1) * w + w / 2, deltas[y] + z
                  * h + PAD);
            GeneralPath gp = new GeneralPath();
            int x1 = OFF + SEP + (x + 2 + dy) * w + w / 2;
            int y1 = deltas[y - dy - 1] + (z1 + 1) * h - PAD;
            int x2 = OFF + SEP + (x + 2) * w - PAD;
            int y2 = deltas[y] + z * h + h / 2;
            gp.moveTo(x2, y2);
            gp.curveTo(x1, y2, x1, y2, x1, y1);
            ((Graphics2D) g).draw(gp);
            drawTree(x, dy, z0, g);
            drawTree(x + dy + 1, y - dy - 1, z1, g);
         } else {
            g.drawLine(OFF + SEP + (x + 1) * w + w / 2, OFF + h - PAD, OFF
                  + SEP + (x + 1) * w + w / 2, deltas[y] + z * h + PAD);
            g.setColor(new Color(0f, 0f, 1f, .1f));
            g.fillRoundRect(OFF + SEP + (x + 1) * w + PAD, OFF + PAD, w - 2
                  * PAD, h - 2 * PAD, 2 * PAD, 2 * PAD);
            g.setColor(new Color(0f, 0f, 1f, 1f));
            g.drawRoundRect(OFF + SEP + (x + 1) * w + PAD, OFF + PAD, w - 2
                  * PAD, h - 2 * PAD, 2 * PAD, 2 * PAD);
         }
      } 
   }

   /**
    * 
    */
   public void reset () {
      working.removeAll();
      repaint();
   }

   /**
    * @param n
    */
   public void check (Notifications n) {
      for (int y = 0; y < word.length; y++) {
         for (int x = 0; x < word.length - y; x++) {
            for (int z = 0; z < working.height(x, y); z++) {
               if (!solution.contains(working.getRule(x, y, z), x, y)) {
                  n.addErrorNotification("", //$NON-NLS-1$
                        Messages.getString("CYK.no_reason_for_rule") //$NON-NLS-1$
                              .replaceAll("\\$RULE", //$NON-NLS-1$
                                    working.getRule(x, y, z).toString())
                              .replaceAll("\\$X", x + "").replaceAll( //$NON-NLS-1$ //$NON-NLS-2$
                                    "\\$Y", y + "")); //$NON-NLS-1$ //$NON-NLS-2$
                  return;
               }
            }
         }
      }
      for (int y = 0; y < word.length; y++) {
         for (int x = 0; x < word.length - y; x++) {
            for (int z = 0; z < solution.height(x, y); z++) {
               if (!working.contains(solution.getRule(x, y, z), x, y)) {
                  n.addWarningNotification("", //$NON-NLS-1$
                        Messages.getString("CYK.missnig_entry").replaceAll( //$NON-NLS-1$
                              "\\$X", x + "").replaceAll("\\$Y", y + "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
                  return;
               }
            }
         }
      }
      n.addCorrectNotification(Shared.CORRECT, ""); //$NON-NLS-1$ //$NON-NLS-2$
   }

   /**
    * @return
    */
   public boolean solve () {
      boolean done = false;
      for (int x = 0; x < word.length; x++) {
         for (int y = 0; y < word.length - x; y++) {
            for (int z = 0; z < solution.height(x, y); z++) {
               if (!working.contains(solution.getRule(x, y, z), x, y)) {
                  working.add(solution.getRule(x, y, z), x, y);
                  done = true;
               }
            }
         }
      }
      working.validate();
      repaint();
      return done;
   }

   /**
    * @param n
    */
   public void next (Notifications n) {
      for (int y = 0; y < word.length; y++) {
         for (int x = 0; x < word.length - y; x++) {
            for (int z = 0; z < working.height(x, y); z++) {
               if (!solution.contains(working.getRule(x, y, z), x, y)) {
                  Rule r = working.getRule(x, y, z);
                  working.remove(x, y, z);
                  working.validate();
                  repaint();
                  n.addNotification(Messages.getString("CYK.next_step"), //$NON-NLS-1$
                        Messages.getString("CYK.unfounded_rule") //$NON-NLS-1$
                              .replaceAll("\\$RULE", r.toString()) //$NON-NLS-1$
                              .replaceAll("\\$X", x + "").replaceAll( //$NON-NLS-1$ //$NON-NLS-2$
                                    "\\$Y", y + "")); //$NON-NLS-1$ //$NON-NLS-2$
                  return;
               }
            }
            for (int z = 0; z < solution.height(x, y); z++) {
               if (!working.contains(solution.getRule(x, y, z), x, y)) {
                  working.add(solution.getRule(x, y, z), x, y);
                  working.validate();
                  repaint();
                  n.addNotification(Messages.getString("CYK.next_step"), //$NON-NLS-1$
                        Messages.getString("CYK.rule_added").replaceAll( //$NON-NLS-1$
                              "\\$RULE", solution.getRule(x, y, z).toString()) //$NON-NLS-1$
                              .replaceAll("\\$X", x + "").replaceAll("\\$Y", //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
                                    y + "")); //$NON-NLS-1$
                  return;
               }
            }
         }
      }
      n.addNotification(Messages.getString("CYK.next_step"), Messages //$NON-NLS-1$
            .getString("CYK.nothing_to_do")); //$NON-NLS-1$
   }

   /**
    * @param word
    */
   public void setWord (Object[] word) {
      this.word = word;
      working = new CYKParseTable(word);
      solution = CYKParser.createCYKParseTable(grammar, this.word);
      repaint();
   }

   /**
    * @param g
    */
   public void setGrammar (Grammar g) {
      this.grammar = g;
   }
}