/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.cyk;

/**
 * @author Vincent Tscherter
 */
public class CYKParser {

   public static CYKParseTable createCYKParseTable (Grammar g, Object[] word) {
      CYKParseTable table = new CYKParseTable(word);
      if (!isCNFGrammer(g)) { return null; }
      // unterste zeile fllen
      for (int x = 0; x < word.length; x++) {
         for (int r = 0; r < g.nOfRules(); r++) {
            Object[] replacement = g.getRule(r).getReplacement();
            if (replacement.length == 1) {
               if (word[x].equals(replacement[0])) {
                  table.add(g.getRule(r), x, 0);
               }
            }
         }
      }
      // rest auffllen
      for (int y = 1; y < word.length; y++) {
         for (int x = 0; x < word.length - y; x++) {
            for (int r = 0; r < g.nOfRules(); r++) {
               //                    Object[] pattern = g.getRule(r).getPattern();
               Object[] replacement = g.getRule(r).getReplacement();
               if (replacement.length == 2) {
                  for (int z = 0; z < y; z++) {
                     if (table.containsPattern(replacement[0], x, z)
                           && table.containsPattern(replacement[1], x + z + 1,
                                 y - z - 1)) {
                        table.add(g.getRule(r), x, y);
                     }
                  }
               }
            }
         }
      }
      return table;
   }

   public static boolean isCNFGrammer (Grammar g) {
      for (int i = 0; i < g.nOfRules(); i++) {
         Object[] pattern = g.getRule(i).getPattern();
         Object[] replacement = g.getRule(i).getReplacement();
         if (pattern.length != 1) { return false; }
         if (g.isTerminal(pattern[0])) { return false; }
         if (replacement.length == 1) {
            if (!g.isTerminal(replacement[0])) { return false; }
         } else if (replacement.length == 2) {
            if (g.isTerminal(replacement[0]) || g.isTerminal(replacement[0])) { return false; }
         } else {
            return false;
         }
      }
      return true;
   }
}