/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.cyk;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Vincent Tscherter
 */
public class Grammar {

    private Object startSymbol;

    private Rule[] rules;

    private List terminals = new ArrayList();

    public Grammar(Object startSymbol, Object[] terminals, Rule[] rules) {
        this.startSymbol = startSymbol;
        for (int i = 0; i < terminals.length; i++) {
            this.terminals.add(terminals[i]);
        }
        this.rules = rules;
    }

    public Grammar(Object startSymbol, Object[] terminals, List rules) {
        this.startSymbol = startSymbol;
        for (int i = 0; i < terminals.length; i++) {
            this.terminals.add(terminals[i]);
        }
        this.rules = new Rule[rules.size()];
        for (int i = 0; i < rules.size(); i++) {
            this.rules[i] = (Rule)rules.get(i);
        }
    }

    public Object getStartSymbol() {
        return startSymbol;
    }

    public Rule getRule(int index) {
        return rules[index];
    }

    public int nOfRules() {
        return rules.length;
    }

    public int nOfTerminals() {
        return terminals.size();
    }

    public boolean isTerminal(Object symbol) {
        return terminals.contains(symbol);
    }

    public String toString() {
        String d = getStartSymbol() + "\n\n"; //$NON-NLS-1$
        for (int i = 0; i<terminals.size();i++) {
            d += terminals.get(i);
        }
        d += "\n\n"; //$NON-NLS-1$
        for (int i = 0; i < rules.length; i++) {
            d += rules[i] + "\n"; //$NON-NLS-1$
        }
        return d;
    }
}