/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.cyk;

/**
 * @author Vincent Tscherter
 */
public class Rule {

   public static final String ARROW = "\u2192"; //    //$NON-NLS-1$

   public static final String DELIMITER = " "; //  //$NON-NLS-1$

   private Object[] pattern, replacement;

   private Rule (Object[] pattern, Object[] replacement) {
      this.pattern = pattern;
      this.replacement = replacement;
   }

   public String toString () {
      String desc = ""; //$NON-NLS-1$
      for (int i = 0; i < pattern.length; i++) {
         desc += pattern[i] + DELIMITER;
      }
      desc += ARROW;
      for (int i = 0; i < replacement.length; i++) {
         desc += DELIMITER + replacement[i];
      }
      return desc;
   }

   public static Rule createCNFRule (String desc) {
      return new Rule(Tools.extractSymbols(extractPattern(desc)), Tools
            .extractSymbols(extractReplacement(desc)));
   }

   private static String extractPattern (String rule) {
      if (rule.indexOf(ARROW) != -1) { return rule.substring(0,
            rule.indexOf(ARROW)).trim(); }
      return null;
   }

   private static String extractReplacement (String rule) {
      if (rule.indexOf(ARROW) != -1) { return rule.substring(
            rule.indexOf(ARROW) + ARROW.length()).trim(); }
      return null;
   }

   public Object[] getPattern () {
      return pattern;
   }

   public Object[] getReplacement () {
      return replacement;
   }

   public boolean equals (Object obj) {
      if (obj instanceof Rule) {
         Rule rule = (Rule) obj;
         if (this.pattern.length != rule.pattern.length
               || this.replacement.length != rule.replacement.length) { return false; }
         for (int i = 0; i < rule.pattern.length; i++) {
            if (!rule.pattern[i].equals(this.pattern[i])) { return false; }
         }
         for (int i = 0; i < rule.replacement.length; i++) {
            if (!rule.replacement[i].equals(this.replacement[i])) { return false; }
         }
         return true;
      }
      return false;
   }
}