/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.cyk;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * @author Vincent Tscherter
 */
public class Tools {

	public static Object[] createWord(Grammar g)
			throws CouldNotGenerateWordException {
		List w = new ArrayList();
		// int
		int stop = g.nOfRules() * g.nOfRules() * g.nOfRules();
		if (CYKParser.isCNFGrammer(g)) {
			w.add(g.getStartSymbol());
			int vars = 1;
			while (vars > 0) {
				List w_ = new ArrayList();
				for (int i = 0; i < w.size(); i++) {
					if (!g.isTerminal(w.get(i))) {
						boolean done = false;
						int counter = 0;
						while (!done) {
							Rule r = g.getRule((int) (Math.random() * g
									.nOfRules()));
							if (r.getPattern()[0].equals(w.get(i))) {
								if (r.getReplacement().length == 2
										&& Math.random() < .25) {
									w_.add(r.getReplacement()[0]);
									w_.add(r.getReplacement()[1]);
									vars++;
									done = true;
								} else if (r.getReplacement().length == 1) {
									w_.add(r.getReplacement()[0]);
									vars--;
									done = true;
								}
							}
							counter++;
							if (counter > stop || w_.size() > 200) {
								throw new CouldNotGenerateWordException();
							}
						}
					} else {
						w_.add(w.get(i));
					}
				}
				w = w_;
			}
		}
		return w.toArray();
	}

	public static Object[] createWord(Grammar g, int length, int maxiter)
			throws CouldNotGenerateWordException {
		Object[] cand = createWord(g);
		while (cand.length != length && maxiter > 0) {
			Object[] cand_ = createWord(g);
			if (Math.abs(length - cand_.length) < Math
					.abs(length - cand.length)) {
				cand = cand_;
			}
			maxiter--;
		}
		return cand;
	}

	public static Object[] extractSymbols(String word) {
		StringTokenizer st = new StringTokenizer(word, Rule.DELIMITER);
		Object[] symbols = new Object[st.countTokens()];
		int i = 0;
		while (st.hasMoreTokens()) {
			symbols[i++] = st.nextToken();
		}
		return symbols;
	}

	public static String WordToString(Object[] word) {
		String d = ""; //$NON-NLS-1$
		for (int i = 0; i < word.length; i++) {
			d += word[i] + " "; //$NON-NLS-1$
		}
		return d + ""; //$NON-NLS-1$
	}

	public static Grammar createGrammar(String desc) throws ParseException {
		StringTokenizer st = new StringTokenizer(desc, "\n"); //$NON-NLS-1$
		Object start;
		if (st.hasMoreTokens()) {
			Object[] s = extractSymbols(st.nextToken().trim());
			if (s.length != 1) {
				throw new ParseException();
			}
			start = s[0];
		} else {
			throw new ParseException();
		}
		Object[] terminals;
		if (st.hasMoreTokens()) {
			terminals = extractSymbols(st.nextToken());
			if (terminals.length < 1) {
				throw new ParseException();
			}
		} else {
			throw new ParseException();
		}
		List rules = new ArrayList();
		while (st.hasMoreTokens()) {
			rules.add(Rule.createCNFRule(st.nextToken()));
		}
		Grammar g = new Grammar(start, terminals, rules);
		return g;
	}

	static class CouldNotGenerateWordException extends Exception {
		// empty block
	}

	static class ParseException extends Exception {
		// empty block
	}

}