/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Shared;

/**
 * Class to build the context menu for selecting transition labels and to handle
 * the selections.
 * 
 * @author Reto Lamprecht
 */
public class BinaryCharTransitionPopup implements ActionListener {

	/** The transition that this menu is appearing on. */
	protected Transition concernedTrans;

	protected FSMEditorOperations editorOps;

	protected FSMView fsmView;

	protected FSMModelInterface model;

	protected Alphabet alphabet;

	JPopupMenu menu;

	Character zero, one;

	public BinaryCharTransitionPopup(FSMControllerInterface controller) {
		this.editorOps = controller.getFSMEditorOperations();
		this.fsmView = controller.getView();
		this.model = controller.getModel();
		if (!model.getAlphabet().isDualAlphabeh()) {
			Debug.showException(new Exception(
					"BinaryCharTransitionPopup not suited for Alphabet of current FSM-model " //$NON-NLS-1$
							+ model.getAlphabet().toString()));
		}
		menu = new JPopupMenu();

	}

	public JPopupMenu getPopup(Transition fat) {
		this.concernedTrans = fat;
		this.alphabet = model.getAlphabet();
		List existingInputs = new ArrayList();
		Transition already = fsmView.getTransition(fat.from, fat.to);
		if (already != null) {
			existingInputs = already.label.getInputs();
		}
		boolean epsilonEnabled = (alphabet.containsEpsilon() && fat.to != fat.from);
		boolean emptyTrans = concernedTrans.label.number() < 1;
		menu.removeAll();
		ButtonGroup group = new ButtonGroup();
		JMenuItem mi;
		JRadioButtonMenuItem rmi;
		Iterator i = alphabet.iterator();
		Character c = (Character) i.next();
		while (c.equals(Alphabet.EPSILON)) {
			c = (Character) i.next();
		}
		zero = c;
		c = (Character) i.next();
		while (c.equals(Alphabet.EPSILON)) {
			c = (Character) i.next();
		}
		one = c;
		rmi = new JRadioButtonMenuItem(zero.toString());
		menu.add(rmi);
		group.add(rmi);
		rmi.addActionListener(this);
		rmi.setSelected(existingInputs.size() == 1
				&& existingInputs.contains(zero));
		rmi = new JRadioButtonMenuItem(one.toString());
		menu.add(rmi);
		group.add(rmi);
		rmi.addActionListener(this);
		rmi.setSelected(existingInputs.size() == 1
				&& existingInputs.contains(one));
		rmi = new JRadioButtonMenuItem(zero.toString() + "|" + one.toString()); //$NON-NLS-1$
		menu.add(rmi);
		group.add(rmi);
		rmi.addActionListener(this);
		rmi.setSelected(existingInputs.size() == 2
				&& existingInputs.contains(zero)
				&& existingInputs.contains(one));
		if (epsilonEnabled) {
			menu.addSeparator();
			rmi = new JRadioButtonMenuItem(Alphabet.EPSILON.toString());
			menu.add(rmi);
			group.add(rmi);
			rmi.addActionListener(this);
			rmi.setSelected(existingInputs.contains(Alphabet.EPSILON));
			rmi = new JRadioButtonMenuItem(zero.toString() + "|" //$NON-NLS-1$
					+ Alphabet.EPSILON.toString());
			menu.add(rmi);
			group.add(rmi);
			rmi.addActionListener(this);
			rmi.setSelected(existingInputs.size() == 2
					&& existingInputs.contains(zero)
					&& existingInputs.contains(Alphabet.EPSILON));
			rmi = new JRadioButtonMenuItem(one.toString() + "|" //$NON-NLS-1$
					+ Alphabet.EPSILON.toString());
			menu.add(rmi);
			group.add(rmi);
			rmi.addActionListener(this);
			rmi.setSelected(existingInputs.size() == 2
					&& existingInputs.contains(one)
					&& existingInputs.contains(Alphabet.EPSILON));
			rmi = new JRadioButtonMenuItem(zero.toString() + "|" //$NON-NLS-1$
					+ one.toString() + "|" + Alphabet.EPSILON.toString()); //$NON-NLS-1$
			menu.add(rmi);
			group.add(rmi);
			rmi.addActionListener(this);
			rmi.setSelected(existingInputs.size() == 3
					&& existingInputs.contains(zero)
					&& existingInputs.contains(one)
					&& existingInputs.contains(Alphabet.EPSILON));
		}

		if (!emptyTrans) {
			menu.addSeparator();
			mi = new JMenuItem(Shared.DELETE); //$NON-NLS-1$
			menu.add(mi);
			mi.addActionListener(this);
		}

		return menu;
	}

	public void actionPerformed(ActionEvent e) {
		if (concernedTrans == null) {
			return;
		}
		String item = e.getActionCommand();
		JMenuItem source = (JMenuItem) (e.getSource());
		if (item == null) {
			// empty block
		} else if (Shared.DELETE == item) { //$NON-NLS-1$
			editorOps.setTransitionInput(concernedTrans, new Object[0]);
		} else if (source instanceof JRadioButtonMenuItem) {
			if (Alphabet.EPSILON.toString().equals(item)) {
				Character[] c = { Alphabet.EPSILON };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if (zero.toString().equals(item)) {
				Character[] c = { zero };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if (one.toString().equals(item)) {
				Character[] c = { one };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if ((zero.toString() + "|" + one.toString()).equals(item)) { //$NON-NLS-1$
				Character[] c = { zero, one };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if ((zero.toString() + "|" + Alphabet.EPSILON.toString()) //$NON-NLS-1$
					.equals(item)) {
				Character[] c = { zero, Alphabet.EPSILON };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if ((one.toString() + "|" + Alphabet.EPSILON.toString()) //$NON-NLS-1$
					.equals(item)) {
				Character[] c = { one, Alphabet.EPSILON };
				editorOps.setTransitionInput(concernedTrans, c);
			} else if ((zero.toString() + "|" + one.toString() + "|" + Alphabet.EPSILON //$NON-NLS-1$ //$NON-NLS-2$
					.toString()).equals(item)) {
				Character[] c = { zero, one, Alphabet.EPSILON };
				editorOps.setTransitionInput(concernedTrans, c);
			}
		}

	}
}