/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import ch.ethz.exorciser.Messages;

/**
 * @author Reto Lamprecht
 */
public class Component3DRenderer extends ComponentRendererAutoCalcTrans {

	public static String key = Messages.getString("FSM.renderer.3d.key"); //$NON-NLS-1$

	// //////////
	// Colors //

	public static int LIGHT = 75;

	public static Color BGR = new Color(0, 0, 10);

	public static Color TEXT = new Color(125, 205, 225);

	public static Color STATE = new Color(105, 125, 130);

	public static Color STATE_HIGHLIGHTED = new Color(125, 205, 225);

	public static Color STATE_SELECTED = new Color(78, 129, 151);

	public static Color STATE_TEXT = new Color(40, 0, 45);

	public static Color STATE_TEXT_HIGHLIGHTED = new Color(90, 60, 95);

	public static Color STATE_TEXT_SELECTED = STATE_TEXT;

	public static Color STATE_MARKED_RED = new Color(170, 65, 80);

	public static Color STATE_MARKED_RED_HIGHLIGHTED = new Color(230, 150, 160);

	public static Color STATE_MARKED_RED_SELECTED = STATE_MARKED_RED_HIGHLIGHTED;

	public static Color STATE_MARKED_RED_TEXT = new Color(0, 25, 20);

	public static Color STATE_MARKED_RED_TEXT_HIGHLIGHTED = new Color(40, 65,
			60);

	public static Color STATE_MARKED_RED_TEXT_SELECTED = STATE_MARKED_RED_TEXT_HIGHLIGHTED;

	public static Color STATE_MARKED_GREEN = new Color(75, 165, 75);

	public static Color STATE_MARKED_GREEN_HIGHLIGHTED = new Color(175, 240,
			140);

	public static Color STATE_MARKED_GREEN_SELECTED = STATE_MARKED_GREEN_HIGHLIGHTED;

	public static Color STATE_MARKED_GREEN_TEXT = STATE_TEXT;

	public static Color STATE_MARKED_GREEN_TEXT_HIGHLIGHTED = STATE_TEXT_HIGHLIGHTED;

	public static Color STATE_MARKED_GREEN_TEXT_SELECTED = STATE_TEXT_HIGHLIGHTED;

	public static Color STATE_MARKED_BLUE = new Color(70, 130, 180);

	public static Color STATE_MARKED_BLUE_HIGHLIGHTED = new Color(75, 190, 255);

	public static Color STATE_MARKED_BLUE_SELECTED = STATE_MARKED_BLUE_HIGHLIGHTED;

	public static Color STATE_MARKED_BLUE_TEXT = new Color(25, 20, 0);

	public static Color STATE_MARKED_BLUE_TEXT_HIGHLIGHTED = new Color(65, 60,
			40);

	public static Color STATE_MARKED_BLUE_TEXT_SELECTED = STATE_MARKED_BLUE_TEXT_HIGHLIGHTED;

	public static Color TRANSITION = new Color(110, 120, 105);

	public static Color TRANSITION_HIGHLIGHTED = new Color(137, 165, 132);

	public static Color TRANSITION_SELECTED = TRANSITION_HIGHLIGHTED;

	public static Color TRANSITION_TEXT = new Color(30, 20, 20);

	public static Color TRANSITION_TEXT_HIGHLIGHTED = TRANSITION_TEXT;

	public static Color TRANSITION_TEXT_SELECTED = TRANSITION_TEXT;

	public static Color TRANSITION_MARKED_RED = STATE_MARKED_RED;

	public static Color TRANSITION_MARKED_RED_HIGHLIGHTED = brighter(
			STATE_MARKED_RED, 40);

	public static Color TRANSITION_MARKED_RED_SELECTED = TRANSITION_MARKED_RED_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_RED_TEXT = STATE_MARKED_RED_TEXT;

	public static Color TRANSITION_MARKED_RED_TEXT_HIGHLIGHTED = STATE_MARKED_RED_TEXT_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_RED_TEXT_SELECTED = STATE_MARKED_RED_TEXT_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_GREEN = STATE_MARKED_GREEN;

	public static Color TRANSITION_MARKED_GREEN_HIGHLIGHTED = brighter(
			STATE_MARKED_GREEN, 40);

	public static Color TRANSITION_MARKED_GREEN_SELECTED = TRANSITION_MARKED_GREEN_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_GREEN_TEXT = STATE_MARKED_GREEN_TEXT;

	public static Color TRANSITION_MARKED_GREEN_TEXT_HIGHLIGHTED = STATE_MARKED_GREEN_TEXT_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_GREEN_TEXT_SELECTED = STATE_MARKED_GREEN_TEXT_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_BLUE = STATE_MARKED_BLUE;

	public static Color TRANSITION_MARKED_BLUE_HIGHLIGHTED = brighter(
			STATE_MARKED_BLUE, 40);

	public static Color TRANSITION_MARKED_BLUE_SELECTED = TRANSITION_MARKED_BLUE_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_BLUE_TEXT = STATE_MARKED_BLUE_TEXT;

	public static Color TRANSITION_MARKED_BLUE_TEXT_HIGHLIGHTED = STATE_MARKED_BLUE_TEXT_HIGHLIGHTED;

	public static Color TRANSITION_MARKED_BLUE_TEXT_SELECTED = STATE_MARKED_BLUE_TEXT_HIGHLIGHTED;

	/** Returns the Color for the specified marks and selection status. */
	protected Color getStateColor(Object mark, boolean selected,
			boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return STATE_MARKED_RED_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_RED_SELECTED;
			} else {
				return STATE_MARKED_RED;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return STATE_MARKED_GREEN_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_GREEN_SELECTED;
			} else {
				return STATE_MARKED_GREEN;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return STATE_MARKED_BLUE_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_BLUE_SELECTED;
			} else {
				return STATE_MARKED_BLUE;
			}
		} else {
			if (highlighted) {
				return STATE_HIGHLIGHTED;
			} else if (selected) {
				return STATE_SELECTED;
			} else {
				return STATE;
			}
		}
	}

	/** Returns the Color for the specified marks and selection status. */
	protected Color getStateTextColor(Object mark, boolean selected,
			boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return STATE_MARKED_RED_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_RED_TEXT_SELECTED;
			} else {
				return STATE_MARKED_RED_TEXT;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return STATE_MARKED_GREEN_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_GREEN_TEXT_SELECTED;
			} else {
				return STATE_MARKED_GREEN_TEXT;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return STATE_MARKED_BLUE_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_BLUE_TEXT_SELECTED;
			} else {
				return STATE_MARKED_BLUE_TEXT;
			}
		} else {
			if (highlighted) {
				return STATE_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return STATE_TEXT_SELECTED;
			} else {
				return STATE_TEXT;
			}
		}
	}

	/** Returns the Color for the specified marks and selection status. */
	protected Color getTransitionColor(Object mark, boolean selected,
			boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return TRANSITION_MARKED_RED_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_RED_SELECTED;
			} else {
				return TRANSITION_MARKED_RED;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return TRANSITION_MARKED_GREEN_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_GREEN_SELECTED;
			} else {
				return TRANSITION_MARKED_GREEN;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return TRANSITION_MARKED_BLUE_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_BLUE_SELECTED;
			} else {
				return TRANSITION_MARKED_BLUE;
			}
		} else {
			if (highlighted) {
				return TRANSITION_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_SELECTED;
			} else {
				return TRANSITION;
			}
		}
	}

	/** Returns the Color for the specified marks and selection status. */
	protected Color getTransitionLabelColor(Object mark, boolean selected,
			boolean highlighted) {
		return brighter(getTransitionColor(mark, selected, highlighted),
				labelBright);
	}

	/** Returns the Color for the specified marks and selection status. */
	protected Color getTransitionTextColor(Object mark, boolean selected,
			boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return TRANSITION_MARKED_RED_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_RED_TEXT_SELECTED;
			} else {
				return TRANSITION_MARKED_RED_TEXT;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return TRANSITION_MARKED_GREEN_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_GREEN_TEXT_SELECTED;
			} else {
				return TRANSITION_MARKED_GREEN_TEXT;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return TRANSITION_MARKED_BLUE_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_BLUE_TEXT_SELECTED;
			} else {
				return TRANSITION_MARKED_BLUE_TEXT;
			}
		} else {
			if (highlighted) {
				return TRANSITION_TEXT_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_TEXT_SELECTED;
			} else {
				return TRANSITION_TEXT;
			}
		}
	}

	public static Color darker(Color color, int light) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		return new Color(Math.max(r - light, 0), Math.max(g - light, 0), Math
				.max(b - light, 0));
	}

	public static Color brighter(Color color, int light) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		return new Color(Math.min(r + light, 255), Math.min(g + light, 255),
				Math.min(b + light, 255));
	}

	/** The Font used for state- and transitionlabels. */
	Font font = new Font("Dialog", Font.PLAIN, 12); //$NON-NLS-1$

	// ////////////////////
	// State parameters //

	/**
	 * Radius of states. Should be small enough to avoid overlapping. States
	 * have minimal distance Fascape.minStateDist from each other.
	 */
	protected double stateRadius = 24;

	/** Width of 3D border. */
	double border = stateRadius * (3.0 / 30.0);

	/** Width of accepting furrow */
	double acptBord = stateRadius * (6.0 / 30.0);

	/** Width of selectable border. */
	double selBorder = border + acptBord + stateRadius * (2.0 / 30.0);

	/** Maximum length of state labels. */
	double MAX_LABEL_LENGTH = 2 * stateRadius - 2 * selBorder;

	/** The angle of the startstate's nick. */
	double startNickAngle = Math.toRadians(0);

	/** Width of the startstate text mark. Used in getStateBounds. */
	double startMarkWidth = 74;

	/** Distance of startstate text mark from stateborder. */
	double smDist = 17.0;

	/** the nick's depth on the 3D states surface. */
	double nDepth = selBorder * 1.7;

	/**
	 * The angle around the state's center that the nick covers at the state's
	 * edge.
	 */
	double nAngle = Math.toRadians(9);

	/**
	 * The angle around the state's center that the nick covers at the inner
	 * border limit.
	 */
	double nAngleB = Math.toRadians(13);

	/**
	 * The nick's depth on the 3D states ground. Set to the value that makes the
	 * nick's borders parallel.
	 */
	double nDepth2 = stateRadius
			- (stateRadius)
			* Math.cos(nAngle)
			+ ((stateRadius) * Math.sin(nAngle))
			* (((stateRadius - border) * Math.cos(nAngleB) + nDepth - stateRadius))
			/ ((stateRadius - border) * Math.sin(nAngleB));

	/** Polygons to draw the startstate's 3D nick. */
	double[] startSymbolCoords = { nDepth2, 0.0,
			stateRadius - (stateRadius + 1.0) * Math.cos(nAngle),
			(stateRadius + 3.0) * Math.sin(nAngle), -1.0, 0.0,
			stateRadius - (stateRadius + 1.0) * Math.cos(nAngle),
			-(stateRadius + 3.0) * Math.sin(nAngle) };

	double[] startSymbolCoordsUp = { nDepth, 0.0,
			stateRadius - (stateRadius - border) * Math.cos(nAngleB),
			-(stateRadius - border) * Math.sin(nAngleB),
			stateRadius - (stateRadius) * Math.cos(nAngle),
			-(stateRadius) * Math.sin(nAngle), 0.0, 0.0 };

	double[] startSymbolCoordsDown = { nDepth, 0.0,
			stateRadius - (stateRadius - border) * Math.cos(nAngleB),
			(stateRadius - border) * Math.sin(nAngleB),
			stateRadius - (stateRadius) * Math.cos(nAngle),
			(stateRadius) * Math.sin(nAngle), 0.0, 0.0 };

	// /////////////////////////
	// Transition parameters //

	/** The line width for transitions */
	float transWidth = 2.5f;

	BasicStroke transStroke = new BasicStroke(transWidth, BasicStroke.CAP_BUTT,
			BasicStroke.JOIN_MITER);

	float toolTipWidth = 1.5f;

	BasicStroke toolTipStroke = new BasicStroke(toolTipWidth,
			BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);

	/** the head's width. */
	double HEADWIDTH = 10.0;

	/** the head's sensitive area width . */
	double SENS_HEADWIDTH = 18.0;

	/** the head's sensitive area length . */
	double SENS_HEADLENGTH = 18.0;

	/** The coordinates of the head's shape. */
	double[] head = { 0.0, -HEADLENGTH / 2, -HEADWIDTH / 2, HEADLENGTH / 2,
			HEADWIDTH / 2, HEADLENGTH / 2 };

	double[] sensHead = { 0.0, -SENS_HEADLENGTH / 2, -SENS_HEADWIDTH / 2,
			SENS_HEADLENGTH / 2, SENS_HEADWIDTH / 2, SENS_HEADLENGTH / 2 };

	/** FontMetrics of the current font. */
	FontMetrics fm = null;

	// //////////////////////////////
	// Transitionlabel parameters //

	/** margin between text and labelbox. */
	double labelMargin = 3;

	/** Roundness of label box corners */
	double labelCorner = 14;

	/** Says how much brighter the labels are than the transitions. */
	int labelBright = 55;

	public Component3DRenderer() {
		this(null);
	}

	public Component3DRenderer(FSMView fsmView) {
		this.fsmView = fsmView;
	}

	public String getKey() {
		return key;
	}

	public void setStateRadius(double r) {
		stateRadius = r;
		acptBord = stateRadius * (6.0 / 30.0);
		selBorder = border + acptBord + stateRadius * (2.0 / 30.0);
		MAX_LABEL_LENGTH = 2 * stateRadius - 2 * selBorder;
	}

	public void setBackground(Color bgr) {
		BGR = bgr;
	}

	/** Draws the background. */
	public void drawBackground(int width, int height, Graphics2D g) {
		g.setColor(BGR);
		g.fillRect(0, 0, width, height);
		stardust(width, height, g);
	}

	int[] stars;

	double density = 0.001;

	protected void stardust(int w, int h, Graphics2D g) {

		if (stars == null || stars.length != 3 * (int) (w * h * density)) {
			stars = new int[3 * (int) (w * h * density)];
			for (int i = 0; i < stars.length - 2; i += 3) {
				stars[i] = 70 + (int) (Math.random() * (220 - 70));
				stars[i + 1] = (int) (Math.random() * w);
				stars[i + 2] = (int) (Math.random() * h);
			}
		}
		for (int i = 0; i < stars.length - 2; i += 3) {
			g.setColor(new Color(stars[i], stars[i], stars[i]));
			g.drawLine(stars[i + 1], stars[i + 2], stars[i + 1], stars[i + 2]);
		}
	}

	/** Returns the text color of this renderer. */
	public Color getTextColor() {
		return TEXT;
	}

	/**
	 * Returns the distance of the state's edge from it's center for a given
	 * angle.
	 */
	public double getStateDiameter(double angle) {
		return stateRadius * 2;
	}

	/**
	 * Returns the bounding box of the specified state including his
	 * selftransition.
	 */
	public Rectangle2D getStateBounds(State s) {
		double x = s.centerX - stateRadius;
		double y = s.centerY - stateRadius;
		double width = 2 * stateRadius;
		double height = 2 * stateRadius;
		Transition self = s.getSelfTransition();
		if (null != self) {
			// calc bounds of selftransition
			calcTransitionCoords(self);
			Rectangle selfbounds = new CubicCurve2D.Double(self.fromX,
					self.fromY, self.ctrlFromX, self.ctrlFromY, self.ctrlToX,
					self.ctrlToY, self.toX, self.toY).getBounds();

			// calc bounds of state including selftransition
			double maxX = Math.max(selfbounds.x + selfbounds.width, x + width);
			double maxY = Math
					.max(selfbounds.y + selfbounds.height, y + height);
			x = Math.min(selfbounds.x, x);
			y = Math.min(selfbounds.y, y);
			width = maxX - x;
			height = maxY - y;
		}
		if (s.isStart()) {
			// precalculated in markState. depending on FontMetrics...
			double dx = x - Math.min(x, s.centerX - startMarkWidth);
			x -= dx;
			width += dx;
		}
		return new Rectangle2D.Double((int) x, (int) y, (int) width,
				(int) height);
	}

	/**
	 * Draws the specified state in the specified Graphics window.
	 */
	public void drawState(State s, Graphics2D g) {

		Color borderCol = getStateColor(s.mark, s.SELECTED, s.HIGHLIGHT_BORDER);
		Color centerCol = getStateColor(s.mark, s.SELECTED, s.HIGHLIGHT_CENTER);
		Color txtCol = getStateTextColor(s.mark, s.SELECTED, s.HIGHLIGHT_CENTER);

		// update state bounds
		s.x = s.centerX - stateRadius;
		s.y = s.centerY - stateRadius;
		s.width = 2 * stateRadius;
		s.height = 2 * stateRadius;

		GradientPaint bordGrad1 = new GradientPaint((float) s.x, (float) s.y,
				brighter(borderCol, LIGHT), (float) (s.x + s.width),
				(float) (s.y + s.height), darker(borderCol, LIGHT));
		GradientPaint bordGrad2 = new GradientPaint((float) s.x, (float) s.y,
				darker(borderCol, LIGHT), (float) (s.x + s.width),
				(float) (s.y + s.height), brighter(borderCol, LIGHT));
		GradientPaint centGrad = new GradientPaint((float) s.x, (float) s.y,
				darker(centerCol, LIGHT), (float) (s.x + s.width),
				(float) (s.y + s.height), brighter(centerCol, LIGHT));

		// draw state symbol:
		g.setPaint(bordGrad1);
		g.fill(new Ellipse2D.Double(s.x, s.y, s.width, s.height));
		if (s.isAccepting()) {
			g.setPaint(bordGrad1);
			g.fill(new Ellipse2D.Double(s.x + border, s.y + border, s.width - 2
					* border, s.height - 2 * border));
			g.setPaint(bordGrad2);
			g.fill(new Ellipse2D.Double(s.x + border + acptBord * 1 / 3, s.y
					+ border + acptBord * 1 / 3, s.width - 2
					* (border + acptBord * 1 / 3), s.height - 2
					* (border + acptBord * 1 / 3)));
			g.setPaint(bordGrad1);
			g.fill(new Ellipse2D.Double(s.x + border + acptBord * 2 / 3, s.y
					+ border + acptBord * 2 / 3, s.width - 2
					* (border + acptBord * 2 / 3), s.height - 2
					* (border + acptBord * 2 / 3)));
			g.setPaint(bordGrad2);
			g.fill(new Ellipse2D.Double(s.x + border + acptBord, s.y + border
					+ acptBord, s.width - 2 * (border + acptBord), s.height - 2
					* (border + acptBord)));
		} else {
			g.setPaint(bordGrad2);
			g.fill(new Ellipse2D.Double(s.x + border, s.y + border, s.width - 2
					* border, s.height - 2 * border));
		}

		g.setPaint(centGrad);
		g.fill(new Ellipse2D.Double(s.x + selBorder, s.y + selBorder, s.width
				- 2 * selBorder, s.height - 2 * selBorder));

		// draw startstate nick:
		if (s.isStart()) {
			AffineTransform at = new AffineTransform();
			at.translate(s.centerX - stateRadius * Math.cos(startNickAngle),
					s.centerY - stateRadius * Math.sin(startNickAngle));
			at.rotate(startNickAngle);
			// GeneralPath needs float coordinates
			float[] res = new float[startSymbolCoords.length];
			float[] resUp = new float[startSymbolCoordsUp.length];
			float[] resDown = new float[startSymbolCoordsDown.length];
			at.transform(startSymbolCoords, 0, res, 0,
					startSymbolCoords.length >> 1);
			at.transform(startSymbolCoordsUp, 0, resUp, 0,
					startSymbolCoordsUp.length >> 1);
			at.transform(startSymbolCoordsDown, 0, resDown, 0,
					startSymbolCoordsDown.length >> 1);
			g.setPaint(bordGrad2);
			g.fill(makePath(resUp));
			g.setPaint(bordGrad1);
			g.fill(makePath(resDown));
			g.setPaint(BGR);
			g.fill(makePath(res));
			at.translate(-2.0, 0.0);
			at.transform(startSymbolCoords, 0, res, 0,
					startSymbolCoords.length >> 1);
			g.setColor(TRANSITION);
			g.fill(makePath(res));
		}

		// draw state label
		Font backup = g.getFont();
		FontMetrics fm_ = g.getFontMetrics();

		if (s.getFontSize(this) == null || s.getFont() == null) {
			String visibleLabel = s.label;

			float fsize = g.getFont().getSize();

			while (fm_.stringWidth(visibleLabel) > MAX_LABEL_LENGTH
					&& fsize > 6.0) {
				fsize *= 0.9;
				g.setFont(g.getFont().deriveFont(fsize));
				fm_ = g.getFontMetrics();
			}

			while (fm_.stringWidth(visibleLabel) > MAX_LABEL_LENGTH) {
				visibleLabel = visibleLabel.substring(0,
						visibleLabel.length() - 1);
			}

			s.labelOffsX = (fm_.stringWidth(visibleLabel) / 2);
			s.labelOffsY = (fm_.getAscent() / 3);

			s.setFontSize(this, fsize);
			s.setVisibleLabel(visibleLabel);
			s.setFont(g.getFont().deriveFont(s.getFontSize(this).floatValue()));
		}

		g.setFont(s.getFont());

		fm_ = g.getFontMetrics();
		g.setPaint(txtCol);

		g.drawString(s.getVisibleLabel(), (float) (s.centerX - s.labelOffsX),
				(float) (s.centerY + s.labelOffsY));

		g.setFont(backup);

	}

	/**
	 * Draws special marks separately.
	 */
	public void markState(State s, Graphics2D g) {
		if (null != s && s.isStart()) {
			g.setFont(font);
			FontMetrics fm_ = g.getFontMetrics();
			startMarkWidth = stateRadius * Math.cos(startNickAngle) + smDist
					+ fm_.stringWidth(ComponentRenderer.START);

			g.setStroke(transStroke);
			g.setColor(TRANSITION);
			g.draw(new Line2D.Double(s.centerX - stateRadius
					* Math.cos(startNickAngle) - smDist - 2.0, s.centerY,
					s.centerX - stateRadius * Math.cos(startNickAngle),
					s.centerY));
			g.setColor(brighter(TRANSITION, labelBright));
			g.fill(new RoundRectangle2D.Double(s.centerX - stateRadius
					* Math.cos(startNickAngle) - smDist
					- fm_.stringWidth(ComponentRenderer.START), s.centerY
					- stateRadius * Math.sin(startNickAngle) - fm_.getAscent()
					* 0.7, fm_.stringWidth(ComponentRenderer.START), fm_
					.getAscent()
					+ fm_.getDescent(), 7, 7));
			g.setColor(TRANSITION_TEXT);
			g
					.drawString(ComponentRenderer.START,
							(int) (s.centerX - stateRadius
									* Math.cos(startNickAngle) - smDist - fm_
									.stringWidth(ComponentRenderer.START)),
							(int) (s.centerY - stateRadius
									* Math.sin(startNickAngle) + fm_
									.getAscent() / 3));
		}
		// draw tooltip
		if (s.SHOW_WHOLE_LABEL == true
				&& (s.getVisibleLabel().length() != s.getLabel().length() || s
						.getFontSize(this).floatValue() < 8.0)) {

			fm = g.getFontMetrics();
			g.setColor(getStateColor(s.mark, s.SELECTED, false));
			g.fill(s.getWholeLabelBounds(fm));
			g.setColor(getStateColor(s.mark, s.SELECTED, true));
			g.setStroke(toolTipStroke);
			g.draw(s.getWholeLabelBounds(fm));
			g.setColor(STATE_TEXT);
			g
					.drawString(s.getLabel(), (float) (s.centerX - fm
							.stringWidth(s.getLabel()) / 2),
							(float) (s.centerY + fm.getAscent() / 3));
		}
	}

	/**
	 * Draws the specified transition in the specified Graphics window.
	 */
	public void drawTransition(Transition t, Graphics2D g) {

		calcTransitionCoords(t);

		g.setStroke(transStroke);
		g.setPaint(getTransitionColor(t.mark, t.SELECTED, t.HIGHLIGHTED));

		AffineTransform at = new AffineTransform();
		at.translate(t.toX, t.toY);
		at.rotate(Math.PI - t.angleHead);
		// GeneralPath needs float coordinates
		float[] headtf = new float[head.length];
		at.transform(head, 0, headtf, 0, head.length >> 1);
		t.headGP = makePath(headtf);
		float[] sensHeadtf = new float[sensHead.length];
		at.transform(sensHead, 0, sensHeadtf, 0, sensHead.length >> 1);
		t.sensHeadGP = makePath(sensHeadtf);

		if (t.type == Transition.CUBIC) {
			g.draw(new CubicCurve2D.Double(t.fromX, t.fromY, t.ctrlFromX,
					t.ctrlFromY, t.ctrlToX, t.ctrlToY, t.toX, t.toY));
			g.fill(t.headGP);
		} else if (t.type == Transition.QUAD) {
			g.draw(new QuadCurve2D.Double(t.fromX, t.fromY, t.ctrlFromX,
					t.ctrlFromY, t.toX, t.toY));
			g.fill(t.headGP);
		} else if (t.type == Transition.LINE) {
			g.draw(new Line2D.Double(t.fromX, t.fromY, t.toX, t.toY));
			g.fill(t.headGP);
		}
	}

	public void drawTransitionLabelQuick(TransitionLabel tl, Graphics2D g) {
		// empty block
	}

	/**
	 * Draws the specified transitionlabel in the specified Graphics window.
	 */
	public void drawTransitionLabel(TransitionLabel tl, Graphics2D g) {
		g.setFont(font);
		if (this.fm == null) {
			this.fm = g.getFontMetrics();
		}
		tl.updateLabelString();
		if (null != tl.labelString) {
			calcTransitionLabelCoords(tl);
			updateTransitionLabelBounds(tl);

			g.setPaint(getTransitionLabelColor(tl.fat.mark, tl.fat.SELECTED,
					tl.fat.HIGHLIGHTED));
			g
					.fill(new RoundRectangle2D.Double(tl.bounds.x, tl.bounds.y,
							tl.bounds.width, tl.bounds.height, labelCorner,
							labelCorner));

			g.setColor(getTransitionTextColor(tl.fat.mark, tl.fat.SELECTED,
					tl.fat.HIGHLIGHTED));
			g.drawString(tl.labelString, (float) (tl.x
					- this.fm.stringWidth(tl.labelString) * 0.5 + 0.47),
					(float) (tl.y + this.fm.getAscent() * 0.4));

		}
	}

	/**
	 * Alters the specified state's state depending on mouse position. Returns
	 * true if any changes have to be repainted.
	 */
	public boolean onMouseOver(State s, double x, double y) {
		double dx = x - s.centerX;
		double dy = y - s.centerY;
		double dist = Math.sqrt(dx * dx + dy * dy);
		boolean changed = false;
		if (dist < (stateRadius - selBorder)) {
			if (!s.HIGHLIGHT_CENTER)
				changed = true;
			s.HIGHLIGHT_CENTER = true;
			s.HIGHLIGHT_BORDER = false;
			s.SHOW_WHOLE_LABEL = true;
		} else if (dist < stateRadius) {
			if (!s.HIGHLIGHT_BORDER)
				changed = true;
			s.HIGHLIGHT_BORDER = true;
			s.HIGHLIGHT_CENTER = false;
			s.SHOW_WHOLE_LABEL = false;
		} else
			s.SHOW_WHOLE_LABEL = false;
		return changed;
	}

	/**
	 * Returns true if (x, y) lies inside the specified state.
	 */
	public boolean contains(State s, double x, double y) {
		double dx = x - s.centerX;
		double dy = y - s.centerY;
		return (Math.sqrt(dx * dx + dy * dy) < stateRadius);
	}

	/**
	 * Alters the specified transitions's state depending on mouse position.
	 */
	public void onMouseOver(Transition t, double x, double y) {
		// empty block
	}

	/**
	 * Returns true if (x, y) lies inside the specified transition.
	 */
	public boolean contains(Transition t, double x, double y) {
		return t.sensHeadGP.contains(x, y);
	}

	/**
	 * Alters the specified transitionlabel's state depending on mouse position.
	 */
	public void onMouseOver(TransitionLabel tl, double x, double y) {
		// empty block
	}

	/**
	 * Returns true if (x, y) lies inside the specified transitionlabel.
	 */
	public boolean contains(TransitionLabel tl, double x, double y) {
		return tl.bounds.contains((int) x, (int) y);
	}

	/**
	 * Sets the bounding rectangle of the specified transitionlabel.
	 */
	protected void updateTransitionLabelBounds(TransitionLabel tl) {
		if (null != tl.labelString && null != this.fm) {
			tl.bounds.x = tl.x - this.fm.stringWidth(tl.labelString) / 2
					- labelMargin;
			tl.bounds.y = tl.y - this.fm.getAscent() / 2 - labelMargin;
			tl.bounds.width = this.fm.stringWidth(tl.labelString) + 2
					* labelMargin;
			tl.bounds.height = this.fm.getAscent() + 2 * labelMargin;
		}
	}

	protected double getAdjustX(TransitionLabel tl, double adjustAngle) {
		return 0;
	}

	protected double getAdjustY(TransitionLabel tl, double adjustAngle) {
		return 0;
	}
}