/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import ch.ethz.exorciser.Messages;

/**
 * @author		Reto Lamprecht
 */
public class ComponentContrastRenderer extends ComponentRendererAutoCalcTrans {

	public static String key = Messages.getString("FSM.renderer.contrast.key"); //$NON-NLS-1$

	////////////
	// Colors //

	public static Color BGR = new Color(255, 255, 255);
	public static Color TEXT = Color.black;

	public static Color STATE_LINE = Color.black;
	public static Color STATE_HIGHLIGHTED = new Color(110, 175, 245);
	// new Color(245, 175, 110);
	public static Color STATE_SELECTED = new Color(215, 195, 160);
	public static Color STATE_TEXT = Color.black;

	public static Color STATE_MARKED_RED_LINE = Color.black;
	public static Color STATE_MARKED_RED = new Color(235, 135, 135);
	//new Color(235, 130, 150);
	public static Color STATE_MARKED_RED_HIGHLIGHTED = new Color(255, 50, 50);
	public static Color STATE_MARKED_RED_SELECTED = STATE_MARKED_RED_HIGHLIGHTED;

	public static Color STATE_MARKED_GREEN_LINE = Color.black;
	public static Color STATE_MARKED_GREEN = new Color(135, 235, 135);
	//new Color(120, 220, 120);
	public static Color STATE_MARKED_GREEN_HIGHLIGHTED = new Color(50, 255, 50);
	public static Color STATE_MARKED_GREEN_SELECTED =
		STATE_MARKED_GREEN_HIGHLIGHTED;

	public static Color STATE_MARKED_BLUE_LINE = Color.black;
	public static Color STATE_MARKED_BLUE = new Color(135, 135, 235);
	//Color(125, 165, 250);
	public static Color STATE_MARKED_BLUE_HIGHLIGHTED = new Color(50, 50, 255);
	public static Color STATE_MARKED_BLUE_SELECTED = STATE_MARKED_BLUE_HIGHLIGHTED;

	public static Color TRANSITION = Color.black;
	public static Color TRANSITION_HIGHLIGHTED = Color.black;
	public static Color TRANSITION_SELECTED = Color.black;
	public static Color TRANSITION_TEXT = Color.black;

	public static Color TRANSITION_MARKED_RED = new Color(190, 80, 80);
	public static Color TRANSITION_MARKED_RED_HIGHLIGHTED = TRANSITION_MARKED_RED;
	public static Color TRANSITION_MARKED_RED_SELECTED = TRANSITION_MARKED_RED;

	public static Color TRANSITION_MARKED_GREEN = new Color(80, 190, 80);
	public static Color TRANSITION_MARKED_GREEN_HIGHLIGHTED =
		TRANSITION_MARKED_GREEN;
	public static Color TRANSITION_MARKED_GREEN_SELECTED = TRANSITION_MARKED_GREEN;

	public static Color TRANSITION_MARKED_BLUE = new Color(80, 80, 190);
	public static Color TRANSITION_MARKED_BLUE_HIGHLIGHTED = TRANSITION_MARKED_BLUE;
	public static Color TRANSITION_MARKED_BLUE_SELECTED = TRANSITION_MARKED_BLUE;

	/** Returns the Color for the specified marks and selection status. */
	protected static Color getStateColor(
		Object mark,
		boolean selected,
		boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return STATE_MARKED_RED_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_RED_SELECTED;
			} else {
				return STATE_MARKED_RED;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return STATE_MARKED_GREEN_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_GREEN_SELECTED;
			} else {
				return STATE_MARKED_GREEN;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return STATE_MARKED_BLUE_HIGHLIGHTED;
			} else if (selected) {
				return STATE_MARKED_BLUE_SELECTED;
			} else {
				return STATE_MARKED_BLUE;
			}
		} else {
			if (highlighted) {
				return STATE_HIGHLIGHTED;
			} else if (selected) {
				return STATE_SELECTED;
			} else {
				return BGR;
			}
		}
	}

	/** Returns the Color for the specified marks and selection status. */
	protected static Color getTransitionColor(
		Object mark,
		boolean selected,
		boolean highlighted) {
		if (mark == MARK_MARKED || mark == MARK_RED) {
			if (highlighted) {
				return TRANSITION_MARKED_RED_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_RED_SELECTED;
			} else {
				return TRANSITION_MARKED_RED;
			}
		} else if (mark == MARK_GREEN) {
			if (highlighted) {
				return TRANSITION_MARKED_GREEN_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_GREEN_SELECTED;
			} else {
				return TRANSITION_MARKED_GREEN;
			}
		} else if (mark == MARK_BLUE) {
			if (highlighted) {
				return TRANSITION_MARKED_BLUE_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_MARKED_BLUE_SELECTED;
			} else {
				return TRANSITION_MARKED_BLUE;
			}
		} else {
			if (highlighted) {
				return TRANSITION_HIGHLIGHTED;
			} else if (selected) {
				return TRANSITION_SELECTED;
			} else {
				return TRANSITION;
			}
		}
	}

	/** Returns the Color for the specified marks and selection status. */
	protected static Color getTransitionTextColor(
		Object mark,
		boolean selected,
		boolean highlighted) {
		return getTransitionColor(mark, selected, highlighted);
	}

	public static Color darker(Color color, int light) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		return new Color(
			Math.max(r - light, 0),
			Math.max(g - light, 0),
			Math.max(b - light, 0));
	}

	public static Color brighter(Color color, int light) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		return new Color(
			Math.min(r + light, 255),
			Math.min(g + light, 255),
			Math.min(b + light, 255));
	}

	//////////////////////
	// State parameters //

	/** Radius of states. Should be small enough to avoid overlapping.
	 *  States have minimal distance FSMView.minStateDist from each other. */
	protected double stateRadius = 23;

	double sensitive_Border_Overlap = 8;

	/** Width of strokes. */
	double strokeWidth = 23 * (2.0 / 30.0);
	/** Accessor to let superclass use this value. */
	protected double getStateStrokeWidth() {
		return strokeWidth;
	}
	BasicStroke stateStroke =
		new BasicStroke(
			(float) strokeWidth,
			BasicStroke.CAP_BUTT,
			BasicStroke.JOIN_MITER);
	/** Width of accepting furrow */
	double acptBord = strokeWidth + stateRadius * (8.0 / 30.0);
	/** Width of selectable border. */
	double selBorder = acptBord;
	/** Maximum length of state labels. */
	double MAX_LABEL_LENGTH = 2 * stateRadius - 2 * selBorder;

	/** The angle of the startstate's nick. */
	double startNickAngle = Math.toRadians(0.0);
	/** Width of the startstate text mark. Used in getStateBounds. */
	double startMarkWidth = 80;

	/** The start arrows head length */
	double saDepth = 12.0;
	/** The start arrows head width */
	double saWidth = 7.2;
	/** The start arrows tail length */
	double saTail = 12.0;
	/** Polygons to draw the startstate's 3D nick. */
	double[] startSymbolCoords =
		{
			0.0,
			0.0,
			-saDepth - 1.0,
			-saWidth / 2,
			-saDepth,
			-strokeWidth / 2,
			-saDepth - saTail,
			-strokeWidth / 2,
			-saDepth - saTail,
			strokeWidth / 2,
			-saDepth,
			strokeWidth / 2,
			-saDepth - 1.0,
			saWidth / 2 };

	///////////////////////////
	// Transition parameters //

	/** The line width for transitions */
	float transWidth = (float) strokeWidth;
	float markedTransWidth = (float) (strokeWidth + 1.0);
	BasicStroke transStroke =
		new BasicStroke(transWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);
	BasicStroke markedTransStroke =
		new BasicStroke(markedTransWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);

	/** the head's width. */
	double HEADWIDTH = 7.2;
	/** the head's sensitive area width . */
	double SENS_HEADWIDTH = 25.0;
	/** the head's sensitive area length . */
	double SENS_HEADLENGTH = 25.0;
	/** determines the angle of the head's back line. */
	double EDGEMIDDLEDIFF = 2.0;

	/** The coordinates of the head's shape. (0,0) is in the middle of the head.*/
	double[] head =
		{
			0.0,
			-HEADLENGTH / 2,
			-HEADWIDTH / 2,
			HEADLENGTH / 2 + EDGEMIDDLEDIFF,
			0.0,
			HEADLENGTH / 2,
			HEADWIDTH / 2,
			HEADLENGTH / 2 + EDGEMIDDLEDIFF };

	/** The coordinates of the head's shape. */
	double[] sensHead =
		{
			0.0,
			-SENS_HEADLENGTH / 2,
			-SENS_HEADWIDTH / 2,
			SENS_HEADLENGTH / 2,
			SENS_HEADWIDTH / 2,
			SENS_HEADLENGTH / 2 };

	/** FontMetrics of the current font. */
	FontMetrics fm = null;

	////////////////////////////////
	// Transitionlabel parameters //

	/** margin between text and labelbox. */
	double labelMargin = 5;
	/** width from label border within which the mouse is considered as inside the label. */
	double sensitive_TransLabel_Border_Overlap = 6;
	/** Roundness of label box corners */
	double labelCorner = 8.0;

	public ComponentContrastRenderer() {
		this(null);
	}

	public ComponentContrastRenderer(FSMView fsmView) {
		this.fsmView = fsmView;
	}

	public String getKey() {
		return key;
	}

	public void setStateRadius(double r) {
		stateRadius = r;
		acptBord = strokeWidth + stateRadius * (8.0 / 30.0);
		selBorder = acptBord;
		MAX_LABEL_LENGTH = 2 * stateRadius - 2 * selBorder;
	}

	public void setBackground(Color bgr) {
		BGR = bgr;
	}

	/** Draws the background. */
	public void drawBackground(int width, int height, Graphics2D g) {
		g.setBackground(fsmView.getMyBackground());
		g.clearRect(0, 0, width, height);
	}

	/** Returns the text color of this renderer. */
	public Color getTextColor() {
		return TEXT;
	}

	/** Returns the distance of the state's edge from it's center for a given angle.
	 */
	public double getStateDiameter(double angle) {
		return stateRadius * 2;
	}

	/** Returns the bounding box of the specified state including his selftransition.
	 */
	public Rectangle2D getStateBounds(State s) {
		if (null == s)
			return null;
		double x = s.centerX - stateRadius;
		double y = s.centerY - stateRadius;
		double width = 2 * stateRadius;
		double height = 2 * stateRadius;
		Transition self = s.getSelfTransition();
		if (null != self) {
			// calc bounds of selftransition
			calcTransitionCoords(self);
			Rectangle selfbounds =
				new CubicCurve2D
					.Double(
						self.fromX,
						self.fromY,
						self.ctrlFromX,
						self.ctrlFromY,
						self.ctrlToX,
						self.ctrlToY,
						self.toX,
						self.toY)
					.getBounds();

			// calc bounds of state including selftransition
			double maxX = Math.max(selfbounds.x + selfbounds.width, x + width);
			double maxY = Math.max(selfbounds.y + selfbounds.height, y + height);
			x = Math.min(selfbounds.x, x);
			y = Math.min(selfbounds.y, y);
			width = maxX - x;
			height = maxY - y;
		}
		if (s.isStart()) {
			// precalculated in markState. depending on FontMetrics...
			double dx = x - Math.min(x, s.centerX - startMarkWidth);
			x -= dx;
			width += dx;
		}
		return new Rectangle2D.Double(x, y, width, height);
	}

	/** Draws the specified state in the specified Graphics window.
	 */
	public void drawState(State s, Graphics2D g) {

		Color borderCol = getStateColor(s.mark, s.SELECTED, s.HIGHLIGHT_BORDER);
		Color centerCol = getStateColor(s.mark, s.SELECTED, s.HIGHLIGHT_CENTER);

		// update state bounds
		s.x = s.centerX - stateRadius;
		s.y = s.centerY - stateRadius;
		s.width = 2 * stateRadius;
		s.height = 2 * stateRadius;

		// paint highlighting
		g.setPaint(borderCol);
		//try {
			g.fill(new Ellipse2D.Double(s.x, s.y, s.width, s.height));
		//} catch (Exception e) {
		//	Debug.showException(e);
		//}
		g.setPaint(centerCol);
		g.fill(
			new Ellipse2D.Double(
				s.x + selBorder,
				s.y + selBorder,
				s.width - 2 * selBorder,
				s.height - 2 * selBorder));

		// draw state symbol
		g.setStroke(stateStroke);
		g.setPaint(STATE_LINE);
		drawStateOutline(s, g);
		if (s.isAccepting()) {
			drawAcceptingBlip(s, g);
		}

		// draw state label
		Font backup = g.getFont();
		FontMetrics fm_ = g.getFontMetrics();

		if (s.getFontSize(this) == null || s.getFont() == null) {
			String visibleLabel = s.label;

			float fsize = g.getFont().getSize();

			while (fm_.stringWidth(visibleLabel) > MAX_LABEL_LENGTH && fsize > 6.0) {
				fsize *= 0.9;
				g.setFont(g.getFont().deriveFont(fsize));
				fm_ = g.getFontMetrics();
			}

			while (fm_.stringWidth(visibleLabel) > MAX_LABEL_LENGTH) {
				visibleLabel = visibleLabel.substring(0, visibleLabel.length() - 1);
			}

			s.labelOffsX = (fm_.stringWidth(visibleLabel) / 2);
			s.labelOffsY = (fm_.getAscent() / 3);

			s.setFontSize(this, fsize);
			s.setVisibleLabel(visibleLabel);

			s.setFont(g.getFont().deriveFont(s.getFontSize(this).floatValue()));
		}

		g.setFont(s.getFont());
		fm_ = g.getFontMetrics();
		g.setPaint(STATE_TEXT);

		g.drawString(
			s.getVisibleLabel(),
			(float) (s.centerX - s.labelOffsX),
			(float) (s.centerY + s.labelOffsY));

		g.setFont(backup);

	}

	public void drawAcceptingBlip(State s, Graphics2D g) {
		g.draw(
			new Ellipse2D.Double(
				s.x + acptBord,
				s.y + acptBord,
				s.width - 2 * acptBord,
				s.height - 2 * acptBord));
	}

	public void drawStateOutline(State s, Graphics2D g) {
		g.draw(new Ellipse2D.Double(s.x, s.y, s.width, s.height));
	}

	/** Draws special marks separately.
	 */
	public void markState(State s, Graphics2D g) {
		if (null != s && s.isStart()) {
			FontMetrics fm_ = g.getFontMetrics();
			startMarkWidth =
				stateRadius * Math.cos(startNickAngle)
					+ saDepth
					+ saTail
					+ fm_.stringWidth(ComponentRenderer.START);

			AffineTransform at = new AffineTransform();
			at.translate(
				s.centerX - stateRadius * Math.cos(startNickAngle),
				s.centerY - stateRadius * Math.sin(startNickAngle));
			at.rotate(startNickAngle);
			// GeneralPath needs float coordinates
			float[] res = new float[startSymbolCoords.length];
			at.transform(startSymbolCoords, 0, res, 0, startSymbolCoords.length >> 1);
			g.setPaint(Color.black);
			g.fill(makePath(res));

			//			//white background of start notation
			//			g.setColor(BGR);
			//			g.fill(
			//				new RoundRectangle2D.Double(
			//					s.centerX - stateRadius * Math.cos(startNickAngle) - saDepth - saTail - fm.stringWidth(" start "),
			//					s.centerY - stateRadius * Math.sin(startNickAngle) - fm.getAscent() * 0.6,
			//					fm.stringWidth(" start "),
			//					fm.getAscent() + fm.getDescent(),
			//					7,
			//					7));

			g.setColor(TRANSITION_TEXT);
			g.drawString(
				ComponentRenderer.START,
				(float) (s.centerX
					- stateRadius * Math.cos(startNickAngle)
					- saDepth
					- saTail
					- fm_.stringWidth(ComponentRenderer.START)),
				(float) (s.centerY
					- stateRadius * Math.sin(startNickAngle)
					+ fm_.getAscent() / 3));
		}
		// draw tooltip
		if (s.SHOW_WHOLE_LABEL == true
			&& (s.getVisibleLabel().length() != s.getLabel().length()
				|| s.getFontSize(this).floatValue() < 8.0)) {
			fm = g.getFontMetrics();
			g.setColor(BGR);
			g.fill(s.getWholeLabelBounds(fm));
			g.setColor(Color.gray);
			g.draw(s.getWholeLabelBounds(fm));
			g.setColor(STATE_TEXT);
			g.drawString(
				s.getLabel(),
				(float) (s.centerX - fm.stringWidth(s.getLabel()) / 2),
				(float) (s.centerY + fm.getAscent() / 3));
		}
	}

	/** Draws the specified transition in the specified Graphics window.
	 */
	public void drawTransition(Transition t, Graphics2D g) {

		calcTransitionCoords(t);

		AffineTransform at = new AffineTransform();
		at.translate(t.toX, t.toY);
		at.rotate(Math.PI - t.angleHead);
		// GeneralPath needs float coordinates
		float[] headtf = new float[head.length];
		at.transform(head, 0, headtf, 0, head.length >> 1);
		t.headGP = makePath(headtf);
		float[] sensHeadtf = new float[sensHead.length];
		at.transform(sensHead, 0, sensHeadtf, 0, sensHead.length >> 1);
		t.sensHeadGP = makePath(sensHeadtf);

		if (t.mark != MARK_NOT_MARKED)
			g.setStroke(markedTransStroke);
		else
			g.setStroke(transStroke);

		g.setPaint(getTransitionColor(t.mark, t.SELECTED, t.HIGHLIGHTED));
		if (t.type == Transition.CUBIC) {
			g.draw(
				new CubicCurve2D.Double(
					t.fromX,
					t.fromY,
					t.ctrlFromX,
					t.ctrlFromY,
					t.ctrlToX,
					t.ctrlToY,
					t.toX,
					t.toY));
			g.fill(t.headGP);
		} else if (t.type == Transition.QUAD) {
			g.draw(
				new QuadCurve2D.Double(
					t.fromX,
					t.fromY,
					t.ctrlFromX,
					t.ctrlFromY,
					t.toX,
					t.toY));
			g.fill(t.headGP);
		} else if (t.type == Transition.LINE) {
			g.draw(new Line2D.Double(t.fromX, t.fromY, t.toX, t.toY));
			g.fill(t.headGP);
		}
	}

	public void drawTransitionLabelQuick(TransitionLabel tl, Graphics2D g) {}

	/** Draws the specified transitionlabel in the specified Graphics window.
	 */
	public void drawTransitionLabel(TransitionLabel tl, Graphics2D g) {
		if (this.fm == null) {
			this.fm = g.getFontMetrics();
		}
		tl.updateLabelString();
		if (null != tl.labelString) {
			if (fsmView.TOPOLOGY_CHANGED) {
				calcTransitionLabelCoords(tl);
				updateTransitionLabelBounds(tl);
			}

			if (tl.hasRenderingHint(RenderingHint_WithBGR)) {
				// draw backgroundcolor box
				g.setColor(fsmView.getMyBackground());
				g.fill(
					new RoundRectangle2D.Double(
						tl.x - this.fm.stringWidth(tl.labelString) / 2,
						tl.y - this.fm.getAscent() * 0.5,
						this.fm.stringWidth(tl.labelString) + 0.3,
						this.fm.getAscent() * 1.0,
						labelCorner,
						labelCorner));
			}

			// draw input charachters
			g.setColor(
				getTransitionTextColor(tl.fat.mark, tl.fat.SELECTED, tl.fat.HIGHLIGHTED));
			g.drawString(
				tl.labelString,
				(float) (tl.x - this.fm.stringWidth(tl.labelString) * 0.5 + 0.9),
				(float) (tl.y + this.fm.getAscent() * 0.35));
		}
	}

	/** Calculates the specified transitionlabel's coordinates depending on it's transition's type and from- and to-states.
	 */

	private double fromLineOffset = 1;

	protected double getAdjustX(TransitionLabel tl, double adjustAngle) {
		return (tl.bounds.width * 0.5 + fromLineOffset) * Math.sin(adjustAngle);
	}

	protected double getAdjustY(TransitionLabel tl, double adjustAngle) {
		return (tl.bounds.height * 0.5 + fromLineOffset) * Math.cos(adjustAngle);
	}

	/** Alters the specified state's state depending on mouse position.
	 */
	public boolean onMouseOver(State s, double x, double y) {
		double dx = x - s.centerX;
		double dy = y - s.centerY;
		double dist = Math.sqrt(dx * dx + dy * dy);
		boolean changed = false;
		if (dist < (stateRadius - selBorder)) {
			if (!s.HIGHLIGHT_CENTER)
				changed = true;
			s.HIGHLIGHT_CENTER = true;
			s.HIGHLIGHT_BORDER = false;
			s.SHOW_WHOLE_LABEL = true;
		} else if (dist < stateRadius + sensitive_Border_Overlap) {
			if (!s.HIGHLIGHT_BORDER)
				changed = true;
			s.HIGHLIGHT_BORDER = true;
			s.HIGHLIGHT_CENTER = false;
			s.SHOW_WHOLE_LABEL = false;
		} else
			s.SHOW_WHOLE_LABEL = false;
		return changed;
	}

	/** Returns true if (x, y) lies inside the specified state.
	 */
	public boolean contains(State s, double x, double y) {
		double dx = x - s.centerX;
		double dy = y - s.centerY;
		return (Math.sqrt(dx * dx + dy * dy) < stateRadius + sensitive_Border_Overlap);
	}

	/** Alters the specified transitions's state depending on mouse position.
	 */
	public void onMouseOver(Transition t, double x, double y) {}

	/** Returns true if (x, y) lies inside the specified transition.
	 */
	public boolean contains(Transition t, double x, double y) {
		return t.sensHeadGP.contains(x, y);
	}

	/** Alters the specified transitionlabel's state depending on mouse position.
	 */
	public void onMouseOver(TransitionLabel tl, double x, double y) {}

	/** Returns true if (x, y) lies inside the specified transitionlabel.
	 */
	public boolean contains(TransitionLabel tl, double x, double y) {
		return tl.sensBounds.contains((int) x, (int) y);
	}

	/** Sets the bounding rectangle of the specified transitionlabel.
	 */
	protected void updateTransitionLabelBounds(TransitionLabel tl) {
		if (null != tl.labelString && null != this.fm) {
			tl.bounds.x = tl.x - this.fm.stringWidth(tl.labelString) / 2 - labelMargin;
			tl.bounds.y = tl.y - this.fm.getAscent() / 2 - labelMargin;
			tl.bounds.width = this.fm.stringWidth(tl.labelString) + 2 * labelMargin;
			tl.bounds.height = this.fm.getAscent() + 2 * labelMargin;

			tl.sensBounds.x =
				tl.x
					- this.fm.stringWidth(tl.labelString) / 2
					- labelMargin
					- sensitive_TransLabel_Border_Overlap;
			tl.sensBounds.y =
				tl.y
					- this.fm.getAscent() / 2
					- labelMargin
					- sensitive_TransLabel_Border_Overlap;
			tl.sensBounds.width =
				this.fm.stringWidth(tl.labelString)
					+ 2 * (labelMargin + sensitive_TransLabel_Border_Overlap);
			tl.sensBounds.height =
				this.fm.getAscent() + 2 * (labelMargin + sensitive_TransLabel_Border_Overlap);
		}
	}

}