/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import ch.ethz.exorciser.Messages;

/**
 * @author		Reto Lamprecht
 */
public interface ComponentRenderer {

	///////////
	// Marks //
	///////////

    public static final String START = Messages.getString("FSM.label.start"); //$NON-NLS-1$
    
	public static final Object MARK_MARKED = new Object();
	// the editor determines the color of the mark
	public static final Object MARK_NOT_MARKED = new Object();
	// unmarks the object --> neutral color
	public static final Object MARK_RED = new Object();
	// should mark the object in a red-color-style
	public static final Object MARK_BLUE = new Object();
	// should mark the object in a blue-color-style
	public static final Object MARK_GREEN = new Object();
	// should mark the object in a greeb-color-style

	public String getKey();

	/** Sets this Renderer's FSMView Object. */
	public void setFSMView(FSMView fsmView);

	/** Draws the background. */
	public void drawBackground(int width, int height, Graphics2D g);

	/** Returns the text color of this renderer. */
	public Color getTextColor();

	/** Returns the distance of the state's edge from it's center for a given angle.
	 */
	public double getStateDiameter(double angle);

	/** Returns the bounding box of the specified state including his selftransition.
	 */
	public Rectangle2D getStateBounds(State s);

	/** Draws the specified state in the specified Graphics window.
	 */
	public void drawState(State s, Graphics2D g);

	/** Draws special marks separately.
	 */
	public void markState(State s, Graphics2D g);

	/** Draws the specified transition in the specified Graphics window.
	 */
	public void drawTransition(Transition t, Graphics2D g);

	/** Calculates the specified transition's coordinates depending on it's type and from- and to-states.
	 */
	public void calcTransitionCoords(Transition t);

	/** Returns the bounding box of the specified transitionlabel.
	 */
	public Rectangle2D getTransitionLabelBounds(TransitionLabel tl);

	/** Draws the specified transitionlabel in the specified Graphics window without sophisticated calculations.
	 */
	public void drawTransitionLabelQuick(TransitionLabel tl, Graphics2D g);

	/** Draws the specified transitionlabel in the specified Graphics window.
	 */
	public void drawTransitionLabel(TransitionLabel tl, Graphics2D g);

	/** Calculates the specified transitionlabel's coordinates depending on it's transition's type and from- and to-states.
	 */
	public void calcTransitionLabelCoords(TransitionLabel tl);

	/** Alters the specified state's state depending on mouse position.
	 *  Returns true if any changes have to be repainted.
	 */
	public boolean onMouseOver(State s, double x, double y);

	/** Alters the specified transitions's state depending on mouse position.
	 */
	public void onMouseOver(Transition t, double x, double y);

	/** Alters the specified transitionlabel's state depending on mouse position.
	 */
	public void onMouseOver(TransitionLabel tl, double x, double y);

	/** Returns true if (x, y) lies inside the specified state.
	 */
	public boolean contains(State s, double x, double y);

	/** Returns true if (x, y) lies inside the specified transition.
	 */
	public boolean contains(Transition t, double x, double y);

	/** Returns true if (x, y) lies inside the specified transitionlabel.
	 */
	public boolean contains(TransitionLabel tl, double x, double y);

}