/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Point;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Shared;

/**
 * Class to build the default context menu and to handle the selections.
 * 
 * @author Reto Lamprecht
 */
public class DefaultPopup implements ActionListener {

	/** Receiver of "change Name" and "clear" actions. */
	protected FSMEditorOperations editorOps;

	/** Receiver of "Add state" action. */
	protected FSMControllerInterface controller;

	/** Receiver of layout and style actions. */
	protected FSMView fsmView;

	/** The coordinates of the mouseClick that made this menu show up. */
	protected int lastClickX, lastClickY;

	private Point2D modCoords = new Point2D.Double();

	/** True if epsilons are allowed as transitionlabels. */
	protected boolean EPSILONS;

	protected JPopupMenu menu;

	protected JMenuItem undoItem;

	protected JMenuItem redoItem;

	protected JMenuItem copyItem;

	protected JMenuItem pasteItem;

	protected JMenuItem addStateItem;

	protected JMenuItem clearItem;

	protected JMenuItem changeNameItem;

	protected JMenu layoutMenu;

	protected JMenuItem fitItem;

	protected JMenuItem organicItem;

	protected JMenuItem circleItem;

	protected JCheckBoxMenuItem alignItem;

	protected JMenu styleMenu;

	protected ButtonGroup styleGroup;

	protected JRadioButtonMenuItem style3D;

	protected JRadioButtonMenuItem styleContrast;

	protected JRadioButtonMenuItem styleDiode;

	protected Frame parent;

	protected FANameDialog faNameDialog;

	public DefaultPopup(FSMControllerInterface controller, Frame parent) {
		this.controller = controller;
		this.editorOps = controller.getFSMEditorOperations();
		this.fsmView = controller.getView();
		this.parent = parent;

		menu = new JPopupMenu();

		undoItem = new JMenuItem(Shared.UNDO); //$NON-NLS-1$
		undoItem.addActionListener(this);
		redoItem = new JMenuItem(Shared.REDO); //$NON-NLS-1$
		redoItem.addActionListener(this);

		copyItem = new JMenuItem(Shared.COPY); //$NON-NLS-1$
		copyItem.addActionListener(this);
		pasteItem = new JMenuItem(Shared.PASTE); //$NON-NLS-1$
		pasteItem.addActionListener(this);

		addStateItem = new JMenuItem(Messages.getString("FSM.popup.add_state")); //$NON-NLS-1$
		addStateItem.addActionListener(this);
		clearItem = new JMenuItem(Shared.DELETE); //$NON-NLS-1$
		clearItem.addActionListener(this);
		changeNameItem = new JMenuItem(Messages
				.getString("FSM.popup.change_name")); //$NON-NLS-1$
		changeNameItem.addActionListener(this);

		layoutMenu = new JMenu(Messages.getString("FSM.popup.layout")); //$NON-NLS-1$
		fitItem = new JMenuItem(Messages.getString("FSM.popup.fit")); //$NON-NLS-1$
		fitItem.addActionListener(this);
		organicItem = new JMenuItem(Messages.getString("FSM.popup.organic")); //$NON-NLS-1$
		organicItem.addActionListener(this);
		circleItem = new JMenuItem(Messages.getString("FSM.popup.circle")); //$NON-NLS-1$
		circleItem.addActionListener(this);
		alignItem = new JCheckBoxMenuItem(Messages
				.getString("FSM.popup.align_with_grid")); //$NON-NLS-1$
		alignItem.addActionListener(this);

		styleMenu = new JMenu(Messages.getString("FSM.popup.style")); //$NON-NLS-1$
		styleGroup = new ButtonGroup();
		style3D = new JRadioButtonMenuItem(Component3DRenderer.key); //$NON-NLS-1$
		styleGroup.add(style3D);
		style3D.addActionListener(this);
		styleContrast = new JRadioButtonMenuItem(ComponentContrastRenderer.key); //$NON-NLS-1$
		styleGroup.add(styleContrast);
		styleContrast.addActionListener(this);
		styleDiode = new JRadioButtonMenuItem(ComponentDiodeRenderer.key); //$NON-NLS-1$
		styleGroup.add(styleDiode);
		styleDiode.addActionListener(this);

	}

	public void setEditTarget(FSMControllerInterface controller) {
		// [R.L. 15.1.2002]
		this.controller = controller;
		this.editorOps = controller.getFSMEditorOperations();
		this.fsmView = controller.getView();
		// [R.L. 15.1.2002]
	}

	public JPopupMenu getPopup(int x, int y) {
		this.lastClickX = x;
		this.lastClickY = y;

		menu.removeAll();

		menu.add(undoItem);
		undoItem.setEnabled(!editorOps.undoListEmpty());

		menu.add(redoItem);
		redoItem.setEnabled(!editorOps.redoListEmpty());

		menu.addSeparator();
		menu.add(copyItem);
		menu.add(pasteItem);
		pasteItem.setEnabled(controller.getModel().isPasteable());

		menu.addSeparator();

		// "Add state"
		menu.add(addStateItem);
		// "Clear"
		menu.add(clearItem);
		menu.add(getLayoutMenu());
		menu.add(getStyleMenu());

		// "Change name"
		menu.add(changeNameItem);

		return menu;
	}

	protected JMenu getLayoutMenu() {

		layoutMenu.removeAll();

		// "Fit"
		layoutMenu.add(fitItem);

		layoutMenu.addSeparator();
		// "Organic"
		layoutMenu.add(organicItem);
		// "Circle"
		layoutMenu.add(circleItem);

		layoutMenu.addSeparator();
		// "Align with grid"
		layoutMenu.add(alignItem);
		if (fsmView.isGridAligned()) {
			alignItem.setState(true);
		}

		return layoutMenu;
	}

	protected JMenu getStyleMenu() {
		styleMenu.removeAll();

		styleMenu.add(style3D);
		if (fsmView.getChosenRendererKey().equals(Component3DRenderer.key)) {
			style3D.setSelected(true);
		}

		styleMenu.add(styleContrast);
		if (fsmView.getChosenRendererKey()
				.equals(ComponentContrastRenderer.key)) {
			styleContrast.setSelected(true);
		}

		styleMenu.add(styleDiode);
		if (fsmView.getChosenRendererKey().equals(ComponentDiodeRenderer.key)) {
			styleDiode.setSelected(true);
		}
		return styleMenu;
	}

	public void actionPerformed(ActionEvent e) {
		String item = e.getActionCommand();

		if (Shared.UNDO.equals(item)) {
			editorOps.undo();
		} else if (Shared.REDO.equals(item)) {
			editorOps.redo();
		} else if (Shared.COPY.equals(item)) {
			controller.getModel().copy();
		} else if (Shared.PASTE.equals(item)) {
			if (controller.getModel().isPasteable()) {
				controller.getModel().paste();
				controller.getView().fit();
			}
		}
		if (Messages.getString("FSM.popup.add_state").equals(item)) { //$NON-NLS-1$
			fsmView.getModelCoords(modCoords, lastClickX, lastClickY);
			double modX = modCoords.getX();
			double modY = modCoords.getY();
			/*
			 * int modX = (int)((double)(lastClickX-fsmView.dx)/fsmView.scale);
			 * int modY = (int)((double)(lastClickY-fsmView.dy)/fsmView.scale);
			 */
			Point p = fsmView.getNearestFreePoint(null, (int) modX, (int) modY);
			if (null != p) {
				controller.addState(p.x, p.y);
				// faui.addState(s);
				/*
				 * fsmView.getModel().addState(s); if (fsmView.states.size() ==
				 * 1) { try { fsmView.getModel().setStartState(s); } catch
				 * (FAException fae) {} }
				 */
			}
		}
		/*
		 * else if ("complete".equals(item)) {
		 * editorOps.addTrapState((double)lastClickX, (double)lastClickY); }
		 */
		else if (Shared.DELETE.equals(item)) { //$NON-NLS-1$
			editorOps.clear();
		} else if (Messages.getString("FSM.popup.fit").equals(item)) { //$NON-NLS-1$
			fsmView.fit();
			fsmView.repaint();
		} else if (Messages.getString("FSM.popup.align_with_grid").equals(item)) { //$NON-NLS-1$
			if (((JCheckBoxMenuItem) e.getSource()).getState()) {
				fsmView.alignWithGrid(true);
				fsmView.fit();
				fsmView.repaint();
			} else {
				fsmView.alignWithGrid(false);
			}
		} else if (Messages.getString("FSM.popup.organic").equals(item)) { //$NON-NLS-1$
			OrganicLayouter.layout(fsmView);
			fsmView.fit();
			fsmView.repaint();
		} else if (Messages.getString("FSM.popup.circle").equals(item)) { //$NON-NLS-1$
			CircleLayouter.layout(fsmView);
			fsmView.fit();
			fsmView.repaint();
		} else if (Messages.getString("FSM.popup.align_with_grid").equals(item)) { //$NON-NLS-1$
			fsmView.alignWithGrid(true);
			fsmView.fit();
			fsmView.repaint();
		} else if (Component3DRenderer.key.equals(item)) {
			fsmView.chooseRenderer(Component3DRenderer.key);
			fsmView.repaint();
		} else if (ComponentContrastRenderer.key.equals(item)) {
			fsmView.chooseRenderer(ComponentContrastRenderer.key);
			fsmView.repaint();
		} else if (ComponentDiodeRenderer.key.equals(item)) {
			fsmView.chooseRenderer(ComponentDiodeRenderer.key);
			fsmView.repaint();
		} else if (Messages.getString("FSM.popup.change_name").equals(item)) { //$NON-NLS-1$
			if (null == faNameDialog) {
				faNameDialog = new FANameDialog(parent);
			}
			faNameDialog.setVisible(true);
		}
	}

	/**
	 * A Dialog containing a TextField and an ok-button to set the FA's name.
	 * Instantiated on "change name".
	 */
	public class FANameDialog extends Dialog implements ActionListener {

		TextField t;

		FANameDialog(Frame parent) {
			super(parent, Messages.getString("FSM.popup.change_name"), true); //$NON-NLS-1$
			addWindowListener(new WindowAdapter() {

				public void windowClosing(WindowEvent e) {
					dispose();
				}
			});
			setLayout(new FlowLayout());
			t = new TextField(15);
			t.setText(fsmView.getModel().getName());
			add(t);
			t.addActionListener(this);
			JButton ok = new JButton(Shared.OKAY); //$NON-NLS-1$
			add(ok);
			ok.addActionListener(this);
			pack();
			Point p = fsmView.getLocationOnScreen();
			setLocation(p.x + fsmView.getWidth() / 2 - getWidth() / 2, p.y
					+ fsmView.getHeight() / 2 - getHeight() / 2);
		}

		public void actionPerformed(ActionEvent e) {
			editorOps.setFAName(t.getText());
			dispose();
		}
	}

}