/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


public class EditorAction {

	// DropTarget
	FSMModelInterface model;

	// Change information
	List undo = new ArrayList();
	List redo = new ArrayList();

	public EditorAction(FSMModelInterface model) {
		this.model = model;
	}

	public EditorAction(FSMModelInterface model, FSMEvent undo, FSMEvent redo) {
		this.model = model;
		this.undo.add(undo);
		this.redo.add(redo);
	}

	public void addUndoEvent(FSMEvent event) {
		this.undo.add(0, event);
	}

	public void addRedoEvent(FSMEvent event) {
		this.redo.add(event);
	}

	public EditorAction getRedoEditorAction() {
		EditorAction mc = new EditorAction(model);
		mc.undo = this.redo;
		mc.redo = this.undo;
		return mc;
	}

	public void undo() {
		for (int i = 0; i < undo.size(); i++) {
			FSMEvent event = (FSMEvent) undo.get(i);
			switch (event.getId()) {
				case FSMEvent.ALPHABET_CHANGED :
					model.setAlphabet(event.getAlphabet());
					break;
				case FSMEvent.STATE_ADDED :
					model.addState(event.getState());
					// add inTrans
					// add outTrans
					break;
				case FSMEvent.STATE_DELETED :
					model.removeState(event.getState());
					break;
				case FSMEvent.STATE_LABEL_CHANGED :
					event.getState().setLabel(event.getStateLabel());
					break;
				case FSMEvent.STATE_MODE_CHANGED :
					event.getState().setAccepting(event.isAccepting());
					break;
				case FSMEvent.START_STATE_CHANGED :
					model.setStartState(event.getState());
					break;
				case FSMEvent.CLEAR_ALL :
					model.clear();
					break;
				case FSMEvent.TRANSITION_ADDED :
					event.getFromState().addTransition(event.getInput(), event.getToState());
					break;
				case FSMEvent.TRANSITION_DELETED :
					event.getFromState().removeTransition(event.getInput(), event.getToState());
					break;
				case FSMEvent.FSM_NAME_CHANGED :
					model.setName(event.getFSMName());
					break;
				case FSMEvent.MARK :
					model.notifyFSMModelListeners(event);
					break;
				case FSMEvent.UNMARK_ALL :
					model.notifyFSMModelListeners(event);
					break;
				case FSMEvent.RESET_MODEL :

					model.clear();
					Iterator states = event.getStateSet().iterator();
					FSMStateInterface s;
					Iterator inputs;
					Character input;
					Iterator nextStates;
					FSMStateInterface next;
					while (null != states && states.hasNext()) {
						s = (FSMStateInterface) states.next();
						model.addState(s);
					}
					// notify view of existing transitions
					states = event.getStateSet().iterator();
					while (null != states && states.hasNext()) {
						s = (FSMStateInterface) states.next();
						inputs = s.getInputs().iterator();
						while (inputs.hasNext()) {
							input = (Character) inputs.next();
							nextStates = s.getNextStateSet(input).iterator();
							while (nextStates.hasNext()) {
								next = (FSMStateInterface) nextStates.next();
								model.notifyFSMModelListeners(
									FSMEvent.createAddTransitionEvent(s, next, input));
							}
						}
					}
					break;
			}
		}
	}

}