/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.util.Hashtable;
import java.util.Set;


public class FSMEvent {

	public final static int STATE_ADDED = 1;
	public final static int STATE_DELETED = 2;
	public final static int STATE_MODE_CHANGED = 3;
	public final static int START_STATE_CHANGED = 4;
	public final static int STATE_LABEL_CHANGED = 5;
	public final static int STATE_POS_CHANGED = 6;

	public final static int TRANSITION_ADDED = 7;
	public final static int TRANSITION_DELETED = 8;

	public final static int CLEAR_ALL = 9;
	public final static int RESET_MODEL = 10;
	public final static int ALPHABET_CHANGED = 11;
	public final static int FSM_NAME_CHANGED = 12;

	public final static int MARK = 13;
	public final static int UNMARK_ALL = 14;

	private int id;
	private FSMStateInterface state1 = null;
	private FSMStateInterface state2 = null;
	private Object object;
	private boolean ACCEPTING;
	private Set stateSet;
	private Alphabet alphabet = null;
	private Hashtable marks = null;

	public static FSMEvent createAddStateEvent(FSMStateInterface state) {
		FSMEvent e = new FSMEvent(STATE_ADDED);
		e.state1 = state;
		return e;
	}

	public static FSMEvent createDeleteStateEvent(FSMStateInterface state) {
		FSMEvent e = new FSMEvent(STATE_DELETED);
		e.state1 = state;
		return e;
	}

	public static FSMEvent createSetAcceptingEvent(
		FSMStateInterface state,
		boolean accepting) {
		FSMEvent e = new FSMEvent(STATE_MODE_CHANGED);
		e.state1 = state;
		e.ACCEPTING = accepting;
		return e;
	}

	public static FSMEvent createSetStartStateEvent(FSMStateInterface state) {
		FSMEvent e = new FSMEvent(START_STATE_CHANGED);
		e.state1 = state;
		return e;
	}

	public static FSMEvent createSetStateLabelEvent(
		FSMStateInterface state,
		Object label) {
		FSMEvent e = new FSMEvent(STATE_LABEL_CHANGED);
		e.state1 = state;
		e.object = label;
		return e;
	}

	// only from model to view...
	public static FSMEvent createStatePosChangedEvent(FSMStateInterface state) {
		FSMEvent e = new FSMEvent(STATE_POS_CHANGED);
		e.state1 = state;
		return e;
	}

	public static FSMEvent createAddTransitionEvent(
		FSMStateInterface from,
		FSMStateInterface to,
		Object input) {
		FSMEvent e = new FSMEvent(TRANSITION_ADDED);
		e.state1 = from;
		e.state2 = to;
		e.object = input;
		return e;
	}

	public static FSMEvent createDeleteTransitionEvent(
		FSMStateInterface from,
		FSMStateInterface to,
		Object input) {
		FSMEvent e = new FSMEvent(TRANSITION_DELETED);
		e.state1 = from;
		e.state2 = to;
		e.object = input;
		return e;
	}

	public static FSMEvent createClearAllEvent() {
		return new FSMEvent(CLEAR_ALL);
	}

	public static FSMEvent createResetModelEvent(Set stateSet) {
		FSMEvent e = new FSMEvent(RESET_MODEL);
		e.stateSet = stateSet;
		return e;
	}

	public static FSMEvent createSetAlphabetEvent(Alphabet alphabet) {
		FSMEvent e = new FSMEvent(ALPHABET_CHANGED);
		e.alphabet = alphabet;
		return e;
	}

	public static FSMEvent createSetFSMNameEvent(String name) {
		FSMEvent e = new FSMEvent(FSM_NAME_CHANGED);
		e.object = name;
		return e;
	}

	public static FSMEvent createMarkEvent(Hashtable marks) {
		FSMEvent e = new FSMEvent(MARK);
		e.marks = marks;
		return e;
	}

	public static FSMEvent createUnMarkAllEvent() {
		FSMEvent e = new FSMEvent(UNMARK_ALL);
		return e;
	}

	private FSMEvent(int id) {
		this.id = id;
	}

	public int getId() {
		return id;
	}
	public FSMStateInterface getFromState() {
		return state1;
	}
	public FSMStateInterface getToState() {
		return state2;
	}
	public FSMStateInterface getState() {
		return state1;
	}
	public Object getStateLabel() {
		return object;
	}
	public boolean isAccepting() {
		return ACCEPTING;
	}
	public Set getStateSet() {
		return stateSet;
	}
	public Alphabet getAlphabet() {
		return alphabet;
	}
	public String getFSMName() {
		return (String) object;
	}
	public Object getInput() {
		return object;
	}
	public Hashtable getMarks() {
		return marks;
	}

}