/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import javax.swing.event.MouseInputListener;


public class FSMMouseInputListener implements MouseInputListener {

	int sampleCount = 0;

	/** Indicates that a state was clicked at it's center and was not yet released. */
	protected static int ABOUT_TO_MOVE_STATE = 1;
	/** Indicates that a state was clicked at it's border and was neither released
	 *  nor has the mouse left the state. */
	protected static int ABOUT_TO_ADD_TRANS = 2;
	/** Indicates that the user has added a new transition and is dragging it around. */
	protected static int READY_TO_ATTACH_NEW = 4;
	/** Indicates that a transition was clicked at it's head. */
	protected static int GRABBED_TRANSITION = 5;
	/** Indicates that the user has grabbed a transition and is dragging it around. */
	protected static int READY_TO_ATTACH_EXISTING = 3;

	/** Indicates that the FSMView plane is moved */
	protected static int TRANSLATE_PLANE = 6;
	/** Indicates that the FSMView plane is moved */
	protected static int ZOOM_PLANE = 7;

	/** Indicates that the mouse is just rambling around. */
	protected static int JUST_LOOKING = 8;

	/** the status indicating the users action. */
	protected int status = JUST_LOOKING;

	/** The state that was clicked. */
	protected State concernedFAState;
	/** The concernedFAState's corresponding ch.ethz.ifa.model.State. */
	protected FSMStateInterface concernedState;
	/** The transition that was clicked at it's head. */
	protected Transition concernedTransHead;
	/** The transition that was clicked on it's label. */
	protected Transition concernedTransLabel;
	/** The state that is currently highlighted. */
	protected State highlightedState = null;
	/** The state that a transition was pointing to before it was grabbed and dragged away. */
	protected State formerToState = null;
	/** The TransitionLabel containing the labels that are added to a temporary attached transition. */
	protected TransitionLabel draggedFATLabel;

	/** Values to store the position of a mouseclick. */
	protected int lastClickX, lastClickY;

	/** Variables to store the current translation of fascape when changing the translation. */
	protected double currdx, currdy;
	/** Variable to store the current scale of fascape when changing the scale. */
	protected double currScale;

	private Point2D modCoords = new Point2D.Double();

	/** The FSMView this editor is working with.
	 */
	private FSMView fascape;
	private FSMControllerInterface controller;
	//    private FSMEditorOperations editorOps;

	public FSMMouseInputListener(FSMControllerInterface ctrl) {
		controller = ctrl;
		fascape = controller.getView();
		//this.editorOps = controller.getFSMEditorOperations();
	}

	public void setEditTarget(FSMControllerInterface ctrl) {
		controller = ctrl;
		//this.editorOps = controller.getFSMEditorOperations();
	}

	public void mouseMoved(MouseEvent e) {
		fascape.getModelCoords(modCoords, e.getX(), e.getY());
		double modX = modCoords.getX();
		double modY = modCoords.getY();
		highlight(modX, modY);
	}

	public void mousePressed(MouseEvent e) {
		// the menu to choose an input for a new created transition was open but nothing was selected
		if (status == READY_TO_ATTACH_NEW) {
			fascape.removeTransition(concernedTransHead);
			status = JUST_LOOKING;
		}

		lastClickX = e.getX();
		lastClickY = e.getY();
		fascape.getModelCoords(modCoords, e.getX(), e.getY());
		double modX = modCoords.getX();
		double modY = modCoords.getY();
		highlight(modX, modY);
		concernedFAState = fascape.getStateAt(modX, modY);
		concernedState = null;
		if (null != concernedFAState) {
			concernedState = concernedFAState.model;
		}
		concernedTransLabel = fascape.getTransitionLabelAt(modX, modY);
		fascape.toTop(concernedTransLabel);
		concernedTransHead = null;
		if (null == concernedTransLabel) {
			// labels are drawn over heads --> select visible
			concernedTransHead = fascape.getTransitionHeadAt(modX, modY);
			fascape.toTop(concernedTransHead);
		}

		int modif = e.getModifiers();

		// handle right mouseclicks resp. Ctrl-clicks
		if ((modif & InputEvent.BUTTON3_MASK) > 0
			|| (((modif & InputEvent.BUTTON1_MASK) > 0) & ((modif & InputEvent.CTRL_MASK) > 0))) {
			// transitions are drawn over states -> handle transitions first
			if (null != concernedTransLabel) {
				fascape.selectTransition(concernedTransLabel);
				controller.changeTransition(concernedTransLabel, e.getX(), e.getY());
			} else if (null != concernedTransHead) {
				fascape.selectTransition(concernedTransHead);
			} else if (null != concernedState) {
				controller.changeState(concernedState, e.getX(), e.getY());
			} else if (null == concernedTransHead) { // condition really necessary??
				controller.changeFSM(e.getX(), e.getY());
			}
		}

		// handle middle mouseclicks resp. Shift-clicks
		else if (
			(modif & InputEvent.BUTTON2_MASK) > 0
				|| (((modif & InputEvent.BUTTON1_MASK) > 0) & ((modif & InputEvent.SHIFT_MASK) > 0))) {
			if (e.getClickCount() > 1) {
				fascape.fit();
				fascape.repaint();
			} else {
				fascape.setScaleRef(e.getX(), e.getY());
				status = ZOOM_PLANE;
			}
		}

		// handle left mouseclicks
		else {
			// transitions are drawn over states -> handle transitions first
			if (null == concernedTransHead && null == concernedTransLabel) {
				if (null != concernedState) {
					if (concernedFAState.isBorderHighlighted() && controller.addTransitionAllowed(concernedState)) {
						status = ABOUT_TO_ADD_TRANS;
					} else if (concernedFAState.isCenterHighlighted()) {
						controller.selectState(concernedFAState);
						//fascape.selectState(concernedFAState, true);
						fascape.grabState(concernedFAState, modX, modY);
						status = ABOUT_TO_MOVE_STATE;
					}
				} else if (e.getClickCount() > 1) {
					// only add state if mousepointer is not on a state or a transition
					Point p = fascape.getNearestFreePoint(null, (int) modX, (int) modY);
					if (null != p) {
						concernedState = controller.addState(p.x, p.y);
						concernedFAState = fascape.getState(concernedState);
						if (concernedFAState != null) {
							highlight(e.getX(), e.getY());
							fascape.repaint();
							status = ABOUT_TO_MOVE_STATE;
						}
					}
				} else {
					State state = fascape.getSelectedState();
					if (state != null) {
						fascape.selectState(state, false);
					}
					fascape.selectState(null, true);
					fascape.setTranslationRef(e.getX(), e.getY());
					status = TRANSLATE_PLANE;
				}
			} else {
				// if a transition label is hit, select the transition.
				fascape.selectTransition(concernedTransLabel);
				if (null != concernedTransHead) {
					fascape.selectTransition(concernedTransHead);
					status = GRABBED_TRANSITION;
				}
			}
		}

	}

	public void mouseDragged(MouseEvent e) {
		int modif = e.getModifiers();

		fascape.getModelCoords(modCoords, e.getX(), e.getY());
		double modX = modCoords.getX();
		double modY = modCoords.getY();

		if ((modif & InputEvent.BUTTON2_MASK) > 0) {
			if (status == ZOOM_PLANE) {
				fascape.scale(e.getY());
				fascape.repaint();
			}
		} else if ((modif & InputEvent.BUTTON1_MASK) > 0) {
			limitMousePos(e);
			fascape.getModelCoords(modCoords, e.getX(), e.getY());
			modX = modCoords.getX();
			modY = modCoords.getY();
			// last mouseclick was a single left click into the empty plane:
			if (status == TRANSLATE_PLANE) {
				//limitMousePos(e);
				fascape.translate(e.getX(), e.getY());
				fascape.repaint();
			}
			// last mousePressed occured in center of a state:
			else if (status == ABOUT_TO_MOVE_STATE) {
				fascape.moveState(concernedFAState, modX, modY);
			}
			// last mousePressed occured on border of a state:
			else if (status == ABOUT_TO_ADD_TRANS) {
				// mouse must leave state before transition is added!
				if (concernedFAState != fascape.getStateAt(modX, modY)) {
					concernedTransHead = fascape.addTransition(concernedFAState);
					fascape.moveTransition(modX, modY, concernedTransHead);
					draggedFATLabel = concernedTransHead.label.copy();
					status = READY_TO_ATTACH_NEW;
				}
			}
			// last mousePressed occured on head of a transition.
			else if (status == GRABBED_TRANSITION) {
				formerToState = concernedTransHead.to;
				fascape.moveTransition(modX, modY, concernedTransHead);
				// 10.10 call editorOps.removeTrans??
				draggedFATLabel = concernedTransHead.label.copy();
				status = READY_TO_ATTACH_EXISTING;
			}
			// new transition looking for destination state:
			else if (status == READY_TO_ATTACH_EXISTING || status == READY_TO_ATTACH_NEW) {
				State selState = fascape.getStateAt(modX, modY);
				// temporarily attach transition if mouse crosses states:
				if (null != selState) {
					if (concernedTransHead.to != selState) {
						// remove transition from old temporary to-state first
						if (null != concernedTransHead.to) {
							concernedTransHead.to.removeInTransition(concernedTransHead);
							concernedTransHead.label.setFATLabel(draggedFATLabel);
						}
						// attach to new state, add labels from existing transition
						Transition alreadyThere = fascape.getTransition(concernedTransHead.from, selState);
						if (null != alreadyThere) {
							concernedTransHead.label.addFATLabel(alreadyThere.label);
						}
						fascape.TOPOLOGY_CHANGED = true;
						concernedTransHead.setToState(selState);
						selState.addInTransition(concernedTransHead);
					}
				} else {
					// remove transition from old temporary to-state
					concernedTransHead.label.setFATLabel(draggedFATLabel);
					// transition is removed from to-state's intransitions in moveState().
					fascape.moveTransition(modX, modY, concernedTransHead);
				}
			}
			fascape.repaint();
		}
	}

	public void mouseReleased(MouseEvent e) {
		limitMousePos(e);
		fascape.getModelCoords(modCoords, e.getX(), e.getY());
		double modX = modCoords.getX();
		double modY = modCoords.getY();
		highlight(modX, modY);
		if (status == ABOUT_TO_MOVE_STATE && null != concernedFAState) {
			Point p =
				fascape.getNearestFreePoint(
					concernedFAState,
					(int) (modX - concernedFAState.getMouseRefX()),
					(int) (modY - concernedFAState.getMouseRefY()));
			if (null != p) {
				concernedState.setPosition(p.x, p.y);
			}
			concernedFAState.release();
			status = JUST_LOOKING;
		}

		if (status == READY_TO_ATTACH_EXISTING || status == READY_TO_ATTACH_NEW) {
			State s = fascape.getStateAt(modX, modY);
			concernedFAState = s;
			if (null == s) {
				fascape.removeTransition(concernedTransHead);
				//editorOps.changeTransition(concernedTransHead, formerToState, null);
				controller.changeTransition(concernedTransHead, formerToState, null);
				status = JUST_LOOKING;
			} else {
				if (status == READY_TO_ATTACH_EXISTING) {
					fascape.removeTransition(concernedTransHead);
					concernedTransHead.label.setFATLabel(draggedFATLabel);
					controller.changeTransition(concernedTransHead, formerToState, s);
					//editorOps.changeTransition(concernedTransHead, formerToState, s);
					status = JUST_LOOKING;
				} else if (null != s && status == READY_TO_ATTACH_NEW) {
					if (concernedTransHead.label.number() > 0) {
						// situation: a new (empty) transition was drawn over an existing one.
						fascape.removeTransition(concernedTransHead);
						controller.addTransition(
							fascape.getTransition(concernedTransHead.from, concernedTransHead.to),
							e.getX(),
							e.getY());
					} else {
						concernedTransHead.to = s; // just to be sure
						controller.addTransition(concernedTransHead, e.getX(), e.getY());
					}
					// dont reset the state to delete the temporary transition in the next mousePressed!
				}
			}
		}
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	/** Limits mouse position to values inside fascape.
	 */
	protected void limitMousePos(MouseEvent e) {
		if (e.getX() < 0)
			e.translatePoint(-e.getX(), 0);
		if (e.getY() < 0)
			e.translatePoint(0, -e.getY());
		if (e.getX() > fascape.getWidth())
			e.translatePoint(fascape.getWidth() - e.getX(), 0);
		if (e.getY() > fascape.getHeight())
			e.translatePoint(0, fascape.getHeight() - e.getY());
	}

	/** Needs model coordinates. */
	protected void highlight(double x, double y) {
		State selState = fascape.getStateAt((int) x, (int) y);
		if (null != selState) {
			if (highlightedState != selState) {
				// dehighlight old
				fascape.dehighlightStates();
			}
			// highlight state under mousepointer and repaint if something changed
			if (fascape.onMouseOver(selState, (int) x, (int) y)) {
				fascape.repaint();
			}
			highlightedState = selState;
		} else if (highlightedState != null) {
			// no state under mouse pointer. dehighlight old
			fascape.dehighlightStates();
			highlightedState = null;
		}
		Transition selTrans = fascape.getTransitionAt((int) x, (int) y);
		if (null != selTrans) {
			fascape.dehighlightTransitions();
			selTrans.setHighlighted(true);
		} else
			fascape.dehighlightTransitions();
	}

}