/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.util.Iterator;

/**
 * @author		Reto Lamprecht
 */
public class OrganicLayouter {

	static final int ITERATIONS = 5000;
	static final double P = 8;
	static final double T = 10;

	public static boolean layout(FSMView fsmView) {

		// turn off grid alignment but keep settings
		boolean alignbak = fsmView.isGridAligned();
		fsmView.alignWithGrid(false);

		double radius = fsmView.getStateDiameter() / 2;
		int n = fsmView.getNofStates();

		for (int i = 0; i < n * n + ITERATIONS; i++) {
			State s = fsmView.getState((int) (Math.random() * n));
			double dx = 0;
			double dy = 0;
			if (s.outTransitions != null) {
				for (Iterator it = s.outTransitions.iterator(); it.hasNext();) {
					State t = ((Transition) it.next()).to;
					if (t != s) {
						double x = t.centerX - s.centerX;
						double y = t.centerY - s.centerY;
						double dist = Math.sqrt(x * x + y * y);
						dx += 2 * x * Math.log(dist / (radius * (P / 2)));
						dy += 2 * y * Math.log(dist / (radius * (P / 2)));
					}
				}
			}
			if (s.inTransitions != null) {
				for (Iterator it = s.inTransitions.iterator(); it.hasNext();) {
					State t = ((Transition) it.next()).from;
					if (t != s) {
						double x = t.centerX - s.centerX;
						double y = t.centerY - s.centerY;
						double dist = Math.sqrt(x * x + y * y);
						dx += 2 * x * Math.log(dist / (radius * (P / 2)));
						dy += 2 * y * Math.log(dist / (radius * (P / 2)));
					}
				}
			}
			for (int j = 0; j < n; j++) {
				State t = fsmView.getState(j);
				if (s != t) {
					double x = t.centerX - s.centerX;
					double y = t.centerY - s.centerY;
					double dist = Math.sqrt(x * x + y * y);
					dx += x * Math.log(dist / (radius * (P + n)));
					dy += y * Math.log(dist / (radius * (P + n)));
				}
			}
			dx = dx / n;
			dy = dy / n;
			//double t = ((i/T)<2)?(2):(i/T); // abkhlfahrplan fr t berechen und T durch t ersetzen !!!
			s.setCenter(s.centerX + dx / T, s.centerY + dy / T);
			//	    s.model.setPosition(s.centerX+dx/T, s.centerY+dy/T);
		}

		// update model with found positions
		State current;
		for (int i = 0; i < n; i++) {
			current = fsmView.getState(i);
			current.model.setPosition(current.centerX, current.centerY);
		}
		fsmView.rearrangeStates();
		fsmView.alignWithGrid(alignbak);
		fsmView.fit();

		return true;
	} // layout

} // OrganicLayout