/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.fsmgui;

import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Point;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Shared;

/** 
 * Class to build the state contextmenu and to handle the selections.
 * 
 * @author		Reto Lamprecht
 */
public class StatePopup implements ActionListener {

	/** Needed only to determine size of window to place $#*&! StateLabelDialog. */
	protected FSMView fascape;
	/** Object that executes changes */
	protected FSMEditorOperations editorOps;
	/** getPopup(State) keeps the concerned state for the actionListener */
	protected FSMStateInterface state;
	/** The context menu and its menuitems are kept to avoid new instantiations. */
	protected JPopupMenu menu;
	protected JMenuItem deleteItem;
	protected JCheckBoxMenuItem acceptItem;
	protected JCheckBoxMenuItem startItem;
	protected JMenuItem changeItem;

	private Frame parent;
	protected StateLabelDialog stateLabelDialog;

	public StatePopup(FSMControllerInterface controller) {
		this(controller, new Frame());
	}

	public StatePopup(FSMControllerInterface controller, Frame parent) {
		this.fascape = controller.getView();
		this.editorOps = controller.getFSMEditorOperations();
		this.parent = parent;

		menu = new JPopupMenu();
		deleteItem = new JMenuItem(Shared.DELETE);
		deleteItem.addActionListener(this);
		acceptItem = new JCheckBoxMenuItem(Messages.getString("FSM.label.accepting")); //$NON-NLS-1$
		acceptItem.addActionListener(this);
		startItem = new JCheckBoxMenuItem(Messages.getString("FSM.label.start")); //$NON-NLS-1$
		startItem.addActionListener(this);
		changeItem = new JMenuItem(Messages.getString("FSM.label.edit_label")); //$NON-NLS-1$
		changeItem.addActionListener(this);
	}

	public JPopupMenu getPopup(FSMStateInterface s) {
		this.state = s;
		menu.removeAll();
		// "Delete"
		if (!s.isStartState()) {
			menu.add(deleteItem);
		}
		// "Accept"
		menu.add(acceptItem);
		acceptItem.setState(s.isAccepting());
		// "Start"
		menu.add(startItem);
		startItem.setState(s.isStartState());
		if (s.isStartState())
			startItem.setEnabled(false);
		else
			startItem.setEnabled(true);
		// "Change label"
		menu.add(changeItem);
		return menu;
	}

	public void actionPerformed(ActionEvent e) {
		String item = e.getActionCommand();

		if (Shared.DELETE==item) {
			editorOps.deleteState(state);
		} else if (Messages.getString("FSM.label.accepting").equals(item)) { //$NON-NLS-1$
			editorOps.setStateAccepting(state, !state.isAccepting());
		} else if (Messages.getString("FSM.label.start").equals(item)) { //$NON-NLS-1$
			editorOps.setStartState(state);
		} else if (Messages.getString("FSM.label.edit_label").equals(item)) { //$NON-NLS-1$
			if (null == stateLabelDialog) {
				stateLabelDialog = new StateLabelDialog(this.parent);
			}
			stateLabelDialog.setVisible(true);
		}
	}

	/** A Dialog containing a TextField and an ok-button to set state labels.
	 *  Instantiated by StatePopupListener on "change label".
	 */
	private class StateLabelDialog extends JDialog implements ActionListener {

		TextField t;

		StateLabelDialog(Frame parent) {
			super(parent, Messages.getString("FSM.label.edit_label"), true); //$NON-NLS-1$
			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dispose();
				}
			});
			getContentPane().setLayout(new FlowLayout());
			t = new TextField(15);
			getContentPane().add(t);
			t.addActionListener(this);
			JButton ok = new JButton(Shared.OKAY);
			getContentPane().add(ok);
			ok.addActionListener(this);
			pack();
			Point p = fascape.getLocationOnScreen();
			setLocation(
				p.x + fascape.getWidth() / 2 - getWidth() / 2,
				p.y + fascape.getHeight() / 2 - getHeight() / 2);
		}


		/* (non-Javadoc)
		 * @see java.awt.Component#setVisible(boolean)
		 */
		public void setVisible(boolean b) {
			this.t.setText(state.getLabel().toString());
			super.setVisible(b);
		}
		

		public void actionPerformed(ActionEvent e) {
			editorOps.setStateLabel(state, t.getText());
			dispose();
		}
	}

}