/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.fsmgui.State;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class AccNonAccLayouter {

	static double originX, originY;
	static double vdist = 60.0; // vertical distance between states.
	static double hdist = 200.0; // horizontal distance between state groups.
	static double hdist_i = vdist;
	// horizontal distance between state of one groups.

	static FSMModelInterface faModel;

	static List accepting = new ArrayList();
	static List nAccepting = new ArrayList();

	// used to transfer StateSets in calculations.
	private static HashSet hSet = new HashSet();
//	private static HashSet hSet2 = new HashSet();

	public static boolean layout(FSMView fsmView) {
		//return false;
		if (null == fsmView)
			return false;

		faModel = fsmView.getModel();

		accepting.clear();
		nAccepting.clear();

		for (int i = 0; i < fsmView.states.size(); i++) {
			State current = (State) fsmView.states.get(i);
			if (current.isAccepting()) {
				accepting.add(current);
			} else {
				nAccepting.add(current);
			}
		}

		////////////
		// algo 1 //		
		vdist = fsmView.minStateDist + 5.0;

		originX = fsmView.getStartState().getCenterX();
		originY = fsmView.getStartState().getCenterY();

		double offsetY = originY;
		double offsetX = originX;

		if (fsmView.getStartState().isAccepting()) {
			placeStates(accepting, offsetX, offsetY);
			offsetX += hdist;
			offsetY = originY;
			placeStates(nAccepting, offsetX, offsetY);
			moveAside(accepting, -vdist);
			moveAside(nAccepting, vdist);
		} else {
			placeStates(nAccepting, offsetX, offsetY);
			offsetX += hdist;
			offsetY = originY;
			placeStates(accepting, offsetX, offsetY);
			moveAside(nAccepting, -vdist);
			moveAside(accepting, vdist);
		}
		faModel = null;
		return true;
	}

	private static void placeStates(List vtStates, double offsetX, double offsetY) {
		for (int i = 0; i < vtStates.size(); i++) {
			State current = (State) vtStates.get(i);
			current.setCenter(offsetX, offsetY);
			offsetY += vdist;
		}
	}

	private static void moveAside(List vtStates, double offset) {
		for (int i = 1; i < vtStates.size(); i++) {
			State state = (State) vtStates.get(i);
			Set neigh = getNeighbourSet(state);
			for (int k = 0; k < vtStates.size(); k++) {
				if (k != i - 1 && k != i && k != i + 1) { // check all non-neighbours
					State s = (State) vtStates.get(k);
					if (neigh.contains(s.model)) {
						// moveAside
						state.setCenter(state.getCenterX() + offset, state.getCenterY());
						vtStates.remove(state);
						i--;
						break;
					}
				}
			}
		}

	}

	public static Object[][] sortStates(List vtStates) {
		int nof = vtStates.size();
		Object[][] cols = new Object[nof][nof];
		int[] index = new int[nof];
		for (int i = 0; i < nof; i++) {
			index[i] = 0;
		}

		// place each remaining state
		//for (int i = index; i < nof; i++) {
		while (vtStates.size() > 0) {

			// 1. try each ramaining state in vector
			for (int s = 0; s < vtStates.size(); s++) {
				State current = (State) vtStates.get(s);
				Set neighbours = getNeighbourSet(current);
				// try each column
				for (int columnIndex = 0; columnIndex < nof; columnIndex++) {

					if (index[columnIndex] == 0) {
						// first position in column is ok for all states.
						cols[index[columnIndex]++][columnIndex] = current;
						vtStates.remove(current);
						break;
					}

					// 2. check if has trans to upper in column
					if (neighbours.contains(((State) cols[index[columnIndex] - 1][columnIndex]).model)) {
						// second position in column doesnt need further checks.
						if (index[columnIndex] == 1) {
							cols[index[columnIndex]++][columnIndex] = current;
							vtStates.remove(current);
							break;
							// make sure no transition crosses upper states. 
						}
						boolean columnOk = true;
						for (int u = index[columnIndex] - 2; u >= 0; u--) {
							if (neighbours.contains(((State) cols[u][columnIndex]).model)) {
								//if yes, goto 2 with increased column.
								columnIndex++;
								columnOk = false;
								break;
							}
						}
						if (columnOk) {
							cols[index[columnIndex]++][columnIndex] = current;
							vtStates.remove(current);
							break;
						}
					}
					// if state has no transition to upper one, try next, unless it's the last one.
					else if (s == vtStates.size()) {
						cols[index[columnIndex]++][columnIndex] = current;
						vtStates.remove(current);
						break;
					}
				} // for (int columnIndex = 0; columnIndex < nof; columnIndex++)
			} // for (int s = 0; s < vtStates.size(); s++)
		} // while (vtStates.size() > 0)

		return cols;

	}

	/** collects the model.States with transitions from and to the specified State. */
	private static Set getNeighbourSet(State s) {
		hSet.clear();
		// add emanating transitions
		Iterator iter = s.model.getInputs().iterator();
		while (iter.hasNext()) {
			Object input = iter.next();
			hSet.addAll(s.model.getNextStateSet(input));
		}
		// add incoming transitions
		Iterator allStates = faModel.getAllStates().iterator();
		while (allStates.hasNext()) {
			ch.ethz.exorciser.ifa.State state = (ch.ethz.exorciser.ifa.State) allStates.next();
			if (state.getAllNeighbours().contains(s.model)) {
				hSet.add(state);
			}
		}
		return hSet;
	}

}