/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.fsmgui.Alphabet;

/**
 * Class to build the default context menu and to handle the selections.
 * 
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class AlphabetMenu {

    public static Character EPSILON = Alphabet.EPSILON;

    public static Character A = new Character('a');

    public static Character B = new Character('b');

    public static Character C = new Character('c');

    public static Character D = new Character('d');

    public static Character E = new Character('e');

    public static Character F = new Character('f');

    public static Character G = new Character('g');

    public static Character H = new Character('h');

    public static Character I = new Character('i');

    public static Character J = new Character('j');

    public static Character K = new Character('k');

    public static Character L = new Character('l');

    public static Character M = new Character('m');

    public static Character N = new Character('n');

    public static Character O = new Character('o');

    public static Character P = new Character('p');

    public static Character Q = new Character('q');

    public static Character R = new Character('r');

    public static Character S = new Character('s');

    public static Character T = new Character('t');

    public static Character U = new Character('u');

    public static Character V = new Character('v');

    public static Character W = new Character('w');

    public static Character X = new Character('x');

    public static Character Y = new Character('y');

    public static Character Z = new Character('z');

    public static Character N0 = new Character('0');

    public static Character N1 = new Character('1');

    public static Character N2 = new Character('2');

    public static Character N3 = new Character('3');

    public static Character N4 = new Character('4');

    public static Character N5 = new Character('5');

    public static Character N6 = new Character('6');

    public static Character N7 = new Character('7');

    public static Character N8 = new Character('8');

    public static Character N9 = new Character('9');

    public static Character DOT = new Character('.');

    public static Character PLUS = new Character('+');

    public static Character MINUS = new Character('-');

    public static List LETTERS_ALL;

    public static List LETTERS_71;

    public static List LETTERS_72;

    public static List LETTERS_73;

    public static List LETTERS_74;

    public static List NUMBERS;

    public static List SPECIAL;

    static {
        LETTERS_ALL = new ArrayList();
        LETTERS_71 = new ArrayList();
        LETTERS_72 = new ArrayList();
        LETTERS_73 = new ArrayList();
        LETTERS_74 = new ArrayList();
        LETTERS_71.add(A);
        LETTERS_71.add(B);
        LETTERS_71.add(C);
        LETTERS_71.add(D);
        LETTERS_71.add(E);
        LETTERS_71.add(F);
        LETTERS_71.add(G);
        LETTERS_72.add(H);
        LETTERS_72.add(I);
        LETTERS_72.add(J);
        LETTERS_72.add(K);
        LETTERS_72.add(L);
        LETTERS_72.add(M);
        LETTERS_72.add(N);
        LETTERS_73.add(O);
        LETTERS_73.add(P);
        LETTERS_73.add(Q);
        LETTERS_73.add(R);
        LETTERS_73.add(S);
        LETTERS_73.add(T);
        LETTERS_73.add(U);
        LETTERS_74.add(V);
        LETTERS_74.add(W);
        LETTERS_74.add(X);
        LETTERS_74.add(Y);
        LETTERS_74.add(Z);
        LETTERS_ALL.addAll(LETTERS_71);
        LETTERS_ALL.addAll(LETTERS_72);
        LETTERS_ALL.addAll(LETTERS_73);
        LETTERS_ALL.addAll(LETTERS_74);

        NUMBERS = new ArrayList();
        NUMBERS.add(N0);
        NUMBERS.add(N1);
        NUMBERS.add(N2);
        NUMBERS.add(N3);
        NUMBERS.add(N4);
        NUMBERS.add(N5);
        NUMBERS.add(N6);
        NUMBERS.add(N7);
        NUMBERS.add(N8);
        NUMBERS.add(N9);
        NUMBERS.add(DOT);
        NUMBERS.add(PLUS);
        NUMBERS.add(MINUS);

        SPECIAL = new ArrayList();
        SPECIAL.add(new Character('.'));
        SPECIAL.add(new Character(','));
        SPECIAL.add(new Character('('));
        SPECIAL.add(new Character(')'));
        SPECIAL.add(new Character('?'));
        SPECIAL.add(new Character('!'));
        SPECIAL.add(new Character('\''));
        SPECIAL.add(new Character('\"'));
        SPECIAL.add(new Character('/'));
        SPECIAL.add(new Character('\\'));
        SPECIAL.add(new Character('_'));
        SPECIAL.add(new Character(':'));
        SPECIAL.add(new Character(';'));
        SPECIAL.add(new Character('='));
        SPECIAL.add(new Character('<'));
        SPECIAL.add(new Character('>'));
        SPECIAL.add(new Character('*'));
        SPECIAL.add(new Character('#'));
        SPECIAL.add(new Character('@'));
        SPECIAL.add(new Character('&'));
        SPECIAL.add(new Character('~'));
        SPECIAL.add(new Character('%'));
        SPECIAL.add(new Character('$'));
    }

    public static JMenu getEditCustomMenu(IFAFSMController controller) {

        CustomAlphHandler menuHandler = new CustomAlphHandler(controller);

        Iterator i = controller.getModel().getAlphabet().iterator();
        String a = "{"; //$NON-NLS-1$

        if (i.hasNext()) {
            a = a + i.next();
        }
        while (i.hasNext()) {

            a = a + "," + i.next(); //$NON-NLS-1$
        }
        a = a + "}"; //$NON-NLS-1$
        JMenu customMenu = new JMenu("A = " + a); //$NON-NLS-1$

        customMenu.add(getCharMenu(LETTERS_71, controller, menuHandler, null));
        customMenu.add(getCharMenu(LETTERS_72, controller, menuHandler, null));
        customMenu.add(getCharMenu(LETTERS_73, controller, menuHandler, null));
        customMenu.add(getCharMenu(LETTERS_74, controller, menuHandler, null));
        customMenu.add(getCharMenu(NUMBERS, controller, menuHandler, Messages
                .getString("FSM.popup.numbers"))); //$NON-NLS-1$
        customMenu.add(getCharMenu(SPECIAL, controller, menuHandler, Messages
                .getString("FSM.popup.special"))); //$NON-NLS-1$

        customMenu.addSeparator();

        JCheckBoxMenuItem epscb = new JCheckBoxMenuItem(Alphabet.EPSILON
                .toString());
        epscb
                .setSelected(controller.getModel().getAlphabet()
                        .containsEpsilon());
        epscb.addActionListener(menuHandler);
        customMenu.add(epscb);
        customMenu.addSeparator();

        return customMenu;
    }

    /**
     * returns a JMenu containing an item for each Character in the specified
     * Vector, checked if the custom menu contains the Character, enabled if no
     * transition with this input exists in the model. If no name for the menu
     * is specified, the Character sequence is taken.
     */
    public static JMenu getCharMenu(List v, IFAFSMController controller,
            ActionListener handler, String name) {
        JMenu charMenu = new JMenu();
        String title = ""; //$NON-NLS-1$
        JCheckBoxMenuItem cmi;
        Character current;
        for (int i = 0; i < v.size(); i++) {
            current = (Character) v.get(i);
            cmi = new JCheckBoxMenuItem(current.toString());
            charMenu.add(cmi);
            title += current.toString();
            cmi.addActionListener(handler);
            if (controller.getModel().getAlphabet().contains(current)) {
                cmi.setState(true);
            }
            if (((BasicFA) controller.getModel())
                    .containsTransitionOnInput(current)) {
                cmi.setEnabled(false);
            } else {
                cmi.setEnabled(true);
            }
        }
        if ("".equalsIgnoreCase(name) || null == name) { //$NON-NLS-1$
            charMenu.setText(title);
        } else
            charMenu.setText(name);
        return charMenu;
    }

}

