/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.ethz.exorciser.fsmgui.Alphabet;

/** AlphabetStats takes an Alphabet as constructor argument and 
 *  (a) analyzes its number of letters / numbers / special characters
 *  (b) provides access to Vectors that contain all the letters / numbers / special characters.
 */

class AlphabetStats extends Object {

	List letters = new ArrayList();
	List numbers = new ArrayList();
	List special = new ArrayList();

	int letterCount = 0;

	boolean allLettersDone = false;

	public AlphabetStats(Alphabet alph) {
		Iterator iter = alph.iterator();
		Character current;
		while (iter.hasNext()) {
			current = (Character) iter.next();
			if (AlphabetMenu.LETTERS_71.contains(current)) {
				letters.add(current);
			} else if (AlphabetMenu.LETTERS_72.contains(current)) {
				letters.add(current);
			} else if (AlphabetMenu.LETTERS_73.contains(current)) {
				letters.add(current);
			} else if (AlphabetMenu.LETTERS_74.contains(current)) {
				letters.add(current);
			} else if (AlphabetMenu.NUMBERS.contains(current)) {
				numbers.add(current);
			} else if (AlphabetMenu.SPECIAL.contains(current)) {
				special.add(current);
			}
		}
	}

	public int nOfLetters() {
		return letters.size();
	}

	public int nOfNumbers() {
		return numbers.size();
	}

	public int nOfSpecial() {
		return special.size();
	}

	public List getLetters() {
		return letters;
	}

	public List getNumbers() {
		return numbers;
	}

	public List getSpecial() {
		return special;
	}

	public List getNext7Letters() {
		List next7 = new ArrayList();
		int i = letterCount;
		for (; i < letters.size() && i < letterCount + 7; i++) {
			next7.add(letters.get(i));
		}
		letterCount += i;
		if (i == letters.size())
			allLettersDone = true;
		return next7;
	}

	public void resetLetterCount() {
		letterCount = 0;
		allLettersDone = false;
	}

	public boolean allLettersDone() {
		return allLettersDone;
	}

}