/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.Iterator;
import java.util.Set;

import ch.ethz.exorciser.fsmgui.Alphabet;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class AlphabetUtil {

	private AlphabetUtil() {
	}

	public static Iterator getDetIterator(Alphabet alphabet) {
		if (alphabet.containsEpsilon()) {
			Alphabet res = (Alphabet) alphabet.clone();
			res.remove(Alphabet.EPSILON);
			return res.iterator();
		}
		return alphabet.iterator();
	}

	public Character getRandomInput(Alphabet alphabet) {
		Object c[] = alphabet.toArray();
		return ((Character) c[(int) (Math.random() * c.length)]);
	}

	/**
	  * Returns a random input - symbol if the Alphabet, but not ESPILON.
	  *
	  * @return   Returns a random <code>Character</code> symbol if the Alphabet.
	  */
	public static Character getRandomDetInput(Alphabet alphabet) {
		Set clone = (Set) alphabet.clone();
		Object c[] = clone.toArray();
		Character res;
		do {
			res = (Character) c[(int) (Math.random() * c.length)];
		} while (res == Alphabet.EPSILON);
		return res;
	}

}