/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;


/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class Cell extends Object {

	static final int NEUTRAL = 0;
	static final int SELECTED = 1;
	static final int REFERRED = 2;
	static final int ACTIVE = 4;
	static final int ACCEPTING = 8;
	static final int START = 16;

	public int selected = 0;
	public int referred = 0;
	public int accepting = 0;
	public int start = 0;
	public int active = 0;

	public State state;
	public Character input;

	/* space between cellborder and string */
	static Insets insets = new Insets(9, 14, 9, 14);

	static int height;

	/* when grab(x, y) is called, these vars get the offset of (x, y) from upper left corner. */
	int grabOffsX, grabOffsY;

	static int roundness = 14;

	String refString = ""; //$NON-NLS-1$
	Rectangle2D refStringBounds = new Rectangle2D.Double();
	RoundRectangle2D rect = new RoundRectangle2D.Double(0, 0, 1, height, roundness, roundness);

	static BasicStroke narrowStroke = new BasicStroke(1.6f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);
	static BasicStroke wideStroke = new BasicStroke(2.4f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);

	static Color[] boxCols;
	static Color[] borderCols;

	static int[] selectTint = { 120, 220, 120 }; // {50, 240, 220};
	static int[] referredTint = { 125, 165, 250 }; //{150, 150, 255};
	static int[] acceptTint = { 250, 200, 200 };

	static Color addTint(Color color, int[] tint, double amount) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		if (amount > 1.0)
			amount = 1.0;
		return new Color(
			(int) (r * (1.0 - amount) + amount * tint[0]),
			(int) (g * (1.0 - amount) + amount * tint[1]),
			(int) (b * (1.0 - amount) + amount * tint[2]));
	}

	static {
		boxCols = new Color[8];
		double sel = 0.7;
		double ref = 0.7;
		double accpt = 0.5;
		boxCols[NEUTRAL + NEUTRAL + NEUTRAL] = new Color(220, 220, 220);
		boxCols[SELECTED + NEUTRAL + NEUTRAL] = addTint(boxCols[NEUTRAL + NEUTRAL + NEUTRAL], selectTint, sel);
		boxCols[NEUTRAL + REFERRED + NEUTRAL] = addTint(boxCols[NEUTRAL + NEUTRAL + NEUTRAL], referredTint, ref);
		boxCols[SELECTED + REFERRED + NEUTRAL] = addTint(boxCols[NEUTRAL + REFERRED + NEUTRAL], selectTint, sel);
		boxCols[NEUTRAL + NEUTRAL + ACTIVE] = addTint(boxCols[NEUTRAL + NEUTRAL + NEUTRAL], acceptTint, accpt);
		//.brighter();;
		boxCols[SELECTED + NEUTRAL + ACTIVE] = addTint(boxCols[SELECTED + NEUTRAL + NEUTRAL], acceptTint, accpt);
		//.brighter();;
		boxCols[NEUTRAL + REFERRED + ACTIVE] = addTint(boxCols[NEUTRAL + REFERRED + NEUTRAL], acceptTint, accpt);
		//.brighter();;
		boxCols[SELECTED + REFERRED + ACTIVE] = addTint(boxCols[SELECTED + REFERRED + NEUTRAL], acceptTint, accpt);
		//.brighter();;

		borderCols = new Color[16];
		sel = 0.4;
		ref = 0.4;
		accpt = 0.7;
		borderCols[NEUTRAL + NEUTRAL + NEUTRAL] = new Color(90, 100, 110);
		borderCols[SELECTED + NEUTRAL + NEUTRAL] = addTint(borderCols[NEUTRAL + NEUTRAL + NEUTRAL], selectTint, sel);
		borderCols[NEUTRAL + REFERRED + NEUTRAL] = borderCols[NEUTRAL + NEUTRAL + NEUTRAL];
		//, referredTint, ref);
		borderCols[SELECTED + REFERRED + NEUTRAL] = addTint(borderCols[SELECTED + NEUTRAL + NEUTRAL], referredTint, ref);
		/*
			  borderCols[NEUTRAL + NEUTRAL + START] = borderCols[NEUTRAL + NEUTRAL + NEUTRAL].darker();
			  borderCols[SELECTED + NEUTRAL + START] = borderCols[SELECTED + NEUTRAL + NEUTRAL].darker();
			  borderCols[NEUTRAL + REFERRED + START] = borderCols[NEUTRAL + REFERRED + NEUTRAL].darker();
			  borderCols[SELECTED + REFERRED + START] = borderCols[SELECTED + REFERRED + NEUTRAL].darker();
		  */
	}

	public Cell(State state, Character input) {
		this.state = state;
		this.input = input;
	}

	public void setTempAppearance(int flags) {
		if ((flags & SELECTED) > 0)
			selected = 1;
		else
			selected = 0;
		if ((flags & REFERRED) > 0)
			referred = 1;
		else
			referred = 0;
		if ((flags & ACTIVE) > 0)
			active = 1;
		else
			active = 0;
	}

	public void setAppearance(int flags) {
		if (flags == 0) {
			selected = 0;
			referred = 0;
			accepting = 0;
			start = 0;
			active = 0;
		}
		if ((flags & SELECTED) > 0)
			selected = 1; //else selected = 0;
		if ((flags & REFERRED) > 0)
			referred = 1; //else referred = 0;
		if ((flags & ACCEPTING) > 0)
			accepting = 1; //else accepting = 0;
		if ((flags & START) > 0)
			start = 1; //else start = 0;
		if ((flags & ACTIVE) > 0)
			active = 1; //else start = 0;
	}
	public void setSelected(boolean b) {
		if (b)
			selected = 1;
		else
			selected = 0;
	}
	public void setReferred(boolean b) {
		if (b)
			referred = 1;
		else
			referred = 0;
	}
	public void setAccepting(boolean b) {
		if (b)
			accepting = 1;
		else
			accepting = 0;
	}
	public void setStart(boolean b) {
		if (b)
			start = 1;
		else
			start = 0;
	}
	public void setActive(boolean b) {
		if (b)
			active = 1;
		else
			active = 0;
	}

	public void setRect(int x, int y, int w, int h) {
		rect.setFrame(x, y, w, h);
	}

	public String getRefString() {
		refString = ""; //$NON-NLS-1$
		State[] refs = MatrixPanel.orderStates(state.getNextStateSet(input));
		if (refs.length > 0)
			refString = (String) refs[0].getLabel();
		for (int i = 1; i < refs.length; i++) {
			refString += ", " + refs[i].getLabel(); //$NON-NLS-1$
		}
		return refString;
	}

	public static Color getBoxColor(int selected, int referred, int active) {
		return boxCols[selected + referred * REFERRED + active * ACTIVE];
	}

	public void paint(Graphics2D g) {
		g.setColor(boxCols[selected + referred * REFERRED + active * ACTIVE]);
		g.fill(rect);
		g.setColor(borderCols[selected + referred * REFERRED]);
		if (start == 1)
			g.setStroke(wideStroke);
		g.draw(rect);
		if (start == 1)
			g.setStroke(narrowStroke);
		if (accepting == 1)
			g.draw(
				new RoundRectangle2D.Double(
					rect.getX() + 3,
					rect.getY() + 3,
					rect.getWidth() - 6,
					rect.getHeight() - 6,
					roundness - 6,
					roundness - 6));
		g.setColor(Color.black);
		// zum editieren einzeln zeichnen, selektiertes char highlighten
		g.drawString(
			getRefString(),
			(int) (rect.getX() + insets.left),
			(int) (rect.getY() + insets.top + g.getFontMetrics().getAscent()));
	}

	public boolean contains(int x, int y) {
		return rect.contains(x, y);
	}

	public void grab(int x, int y) {
		grabOffsX = (int) (x - rect.getX());
		grabOffsY = (int) (y - rect.getY());
	}
}