/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.Set;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class CellPopup implements ActionListener {

    FSMEditorOperations editorOps;

    JPopupMenu menu;

    Cell currCell;

    /* to get the selected state from selected menu item in actionPerformed. */
    Hashtable mi2state;

    public CellPopup(FSMEditorOperations editorOps) {
        this.editorOps = editorOps;
    }

    public void setEditTarget(FSMEditorOperations editorOps) {
        this.editorOps = editorOps;
    }

    public JPopupMenu getPopup(Cell cell, State[] states) {
        currCell = cell;

        menu = new JPopupMenu();
        JMenuItem mi;
        JCheckBoxMenuItem cmi;

        mi2state = new Hashtable();
        Set nextStates = cell.state.getNextStateSet(cell.input);
        //   State curr = null;
        for (int i = 0; i < states.length; i++) {
            if (!(cell.input.equals(Alphabet.EPSILON) && cell.state
                    .equals(states[i]))) {
                cmi = new JCheckBoxMenuItem((String) states[i].getLabel());
                mi2state.put(cmi, states[i]);
                menu.add(cmi);
                cmi.addActionListener(this);
                if (nextStates.contains(states[i])) {
                    cmi.setState(true);
                }
            }
        }

        mi = new JMenuItem(Messages.getString("FSM.popup.clear_cell")); //$NON-NLS-1$
        menu.add(mi);
        mi.addActionListener(this);

        menu.addSeparator();

        // boolean startState = (model.getStartState() == cell.state);

        if (!cell.state.isStartState()) {
            mi = new JMenuItem(Messages.getString("FSM.popup.delete_state")); //$NON-NLS-1$
            menu.add(mi);
            mi.addActionListener(this);
        }

        cmi = new JCheckBoxMenuItem(Messages.getString("FSM.label.accepting")); //$NON-NLS-1$
        menu.add(cmi);
        cmi.addActionListener(this);
        cmi.setState(cell.state.isAccepting());

        cmi = new JCheckBoxMenuItem(Messages.getString("FSM.label.start")); //$NON-NLS-1$
        menu.add(cmi);
        cmi.addActionListener(this);
        cmi.setState(cell.state.isStartState());
        if (cell.state.isStartState())
            cmi.setEnabled(false);
        else
            cmi.setEnabled(true);

        return menu;
    }

    public void actionPerformed(ActionEvent e) {
        String item = e.getActionCommand();

        if (Messages.getString("FSM.popup.clear_cell").equalsIgnoreCase(item)) { //$NON-NLS-1$
            editorOps.deleteTransitions(currCell.state, currCell.input);
            return;
        } else if (Messages.getString("FSM.popup.delete_state").equals(item)) { //$NON-NLS-1$
            editorOps.deleteState(currCell.state);
        } else if (Messages.getString("FSM.label.accepting").equalsIgnoreCase( //$NON-NLS-1$
                item)) {
            editorOps.setStateAccepting(currCell.state, !currCell.state
                    .isAccepting());
            return;
        } else if (Messages.getString("FSM.label.start").equals(item)) { //$NON-NLS-1$
            editorOps.setStartState(currCell.state);
            return;
        }

        State s = (State) mi2state.get(e.getSource());
        if (null != s) {
            if (currCell.state.getNextStateSet(currCell.input).contains(s)) {
                editorOps.deleteTransition(currCell.state, s, currCell.input);
            } else {
                editorOps.addTransition(currCell.state, s, currCell.input);
            }
        }
    }
}