/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class DefaultMatrixPopup implements ActionListener {

    IFACompleteEditor ifaCmplEd;

    IFAFSMController controller;

    FSMEditorOperations editorOps;

    public DefaultMatrixPopup(IFACompleteEditor ifaCmplEd) {
        this.ifaCmplEd = ifaCmplEd;
        this.controller = (IFAFSMController) ifaCmplEd.getController();
        this.editorOps = controller.getFSMEditorOperations();
    }

    public void setEditTarget(IFAFSMController controller) {
        this.controller = controller;
        this.editorOps = controller.getFSMEditorOperations();
    }

    public JPopupMenu getPopup() {
        JPopupMenu menu = new JPopupMenu();
        JMenuItem mi;
        JCheckBoxMenuItem cmi;

        mi = new JMenuItem(Shared.UNDO);
        menu.add(mi);
        mi.addActionListener(this);
        if (editorOps.undoListEmpty()) {
            mi.setEnabled(false);
        }

        mi = new JMenuItem(Shared.REDO);
        menu.add(mi);
        mi.addActionListener(this);
        if (editorOps.redoListEmpty()) {
            mi.setEnabled(false);
        }

        menu.addSeparator();

        mi = new JMenuItem(Messages.getString("FSM.popup.add_state")); //$NON-NLS-1$
        menu.add(mi);
        mi.addActionListener(this);

        mi = new JMenuItem(Shared.DELETE);
        menu.add(mi);
        mi.addActionListener(this);

        // !!!!!!!!!!!!!!!! menu.add(AlphabetMenu.getMenu(controller));

        menu.add(ViewMenu.getMenu(ifaCmplEd));

        cmi = new JCheckBoxMenuItem(Messages.getString("FSM.popup.track_input")); //$NON-NLS-1$
        menu.add(cmi);
        cmi.addActionListener(this);
        if (ifaCmplEd.isInputTrackerShowing()) {
            cmi.setState(true);
        }

        return menu;
    }
    
    public void actionPerformed(ActionEvent e) {
        String item = e.getActionCommand();
        if (Shared.UNDO.equals(item)) {
            editorOps.undo();
        } else if (Shared.REDO.equals(item)) {
            editorOps.redo();
        } else if (Messages.getString("FSM.popup.add_state").equals(item)) { //$NON-NLS-1$
            controller.addState(Double.NaN, Double.NaN);
        } else if (Shared.DELETE.equals(item)) {
            editorOps.clear();
            //? editorOps.setName("");
        } else if (Messages.getString("FSM.popup.track_input").equals(item)) { //$NON-NLS-1$
            ifaCmplEd.showInputTracker(!ifaCmplEd.isInputTrackerShowing());
        }
    }
}