/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.Iterator;
import java.util.Set;

import ch.ethz.exorciser.fsmgui.Alphabet;

/**
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class FAGenerator {

	public static FA getRandomDFA(int nrOS, double percentAcc, Alphabet alph) throws FAException {
		FAModel random = new FAModel("Random DFA"); //$NON-NLS-1$
		random.setAlphabet(alph);
		State tmpState = null;

		for (int i = 0; i < nrOS; i++) {
			tmpState = new State();
			random.addState(tmpState);
			if (Math.random() < percentAcc) {
				random.setAccepting(tmpState, true);
			}
		}
		tmpState = random.getRandomState();
		random.setStartState(tmpState);

		State nextState = null;
		boolean guided = true;

		int count = 0;
		while (guided && (count < nrOS * 8)) {
			count++;
			Iterator alphIter = AlphabetUtil.getDetIterator(alph);
			while (alphIter.hasNext()) {
				Character inp = (Character) alphIter.next();
				if (tmpState.getNextStateSet(inp).size() == 0) {
					Set nextStates = random.getNotReachableStates();
					if (nextStates.size() == 0) {
						nextStates = random.getTrapStates();
						if (nextStates.size() <= 1) {
							guided = false;
							break;
						}
					}
					Object[] posibles = nextStates.toArray();
					nextState = (State) posibles[(int) (Math.random() * posibles.length)];
					tmpState.addTransition(inp, nextState);
				}
			}
			tmpState = random.getRandomState();
		}

		Set states = random.getAllStates();
		if (states != null) {
			Iterator iter = states.iterator();
			while (iter.hasNext()) {
				tmpState = (State) iter.next();
				Iterator alphIter = AlphabetUtil.getDetIterator(alph);
				while (alphIter.hasNext()) {
					Character inp = (Character) alphIter.next();
					if (tmpState.getNextStateSet(inp).size() == 0) {
						nextState = random.getRandomState();
						tmpState.addTransition(inp, nextState);
					}
				}
			}
		}
		return random;
	}

	public static FA getRandomNotMinDFA(int nrOS, double percentAcc, Alphabet alph) throws FAException {
		FA res = getRandomDFA(nrOS, percentAcc, alph);
		FA resMin = (FA) res.clone();
		resMin.detMinimize();

		while (res.getStateSet().size() == resMin.getStateSet().size()) {
			res = getRandomDFA(nrOS, percentAcc, alph);
			resMin = (FA) res.clone();
			resMin.detMinimize();
		}
		return res;

	}

	public static FA getRandomGFA(int nrOS, int nrOfTrans, double percentAcc, Alphabet alph) throws FAException {
		FAModel random = new FAModel("Random FA"); //$NON-NLS-1$
		random.setAlphabet(alph);
		State tmpState = null;

		for (int i = 0; i < nrOS; i++) {
			tmpState = new State();
			random.addState(tmpState);
			if (Math.random() < percentAcc) {
				random.setAccepting(tmpState, true);
			}
		}
		tmpState = random.getRandomState();
		random.setStartState(tmpState);
		State nextState = null;

		for (int count = 0; count < nrOfTrans; count++) {
			Character inp = AlphabetUtil.getRandomDetInput(alph);
			if (Math.random() < 0.3) {
				inp = Alphabet.EPSILON;
			}

			Object[] notReachableStates = random.getNotReachableStates().toArray();
			if (notReachableStates.length > 0) {
				nextState = (State) notReachableStates[(int) (Math.random() * notReachableStates.length)];

			} else {
				nextState = random.getRandomState();
			}
			while (inp.equals(Alphabet.EPSILON) && (tmpState == nextState)) {
				nextState = random.getRandomState();
			}
			tmpState.addTransition(inp, nextState);
			tmpState = random.getRandomState();
		}
		return random;
	}

}