/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.fsmgui.Alphabet;

/**
 * @author Matthias Dreier
 * @author Vincent Tscherter
 */
public abstract class FASerializer {

    private static final Character A = new Character('a');

    private static final Character B = new Character('b');

    public static int NofStates(String fa) {
        return new StringTokenizer(fa, "#").nextToken().length(); //$NON-NLS-1$
    }

    public static String createDetABSerial(FA fa) {
        try {
            int i, idx, pos;
            State next, start;
            List states = new ArrayList();

            String text = ""; //$NON-NLS-1$
            if (fa == null) { return ""; //$NON-NLS-1$
            }
            fa.makeDetOpt();
            fa.detMinimize();
            //done = false;
            idx = 0;
            pos = 0;
            start = (State) fa.getStartState();
            states.add(pos, start);
            pos++;
            //  Breadth first search
            while (idx != pos) {
                next = ((State) (states.get(idx))).getNextState(A);
                if (states.indexOf(next) == -1) {
                    states.add(pos, next);
                    pos++;
                }
                next = ((State) (states.get(idx))).getNextState(B);
                if (states.indexOf(next) == -1) {
                    states.add(pos, next);
                    pos++;
                }
                idx++;
                //done = (idx == pos);
            }
            // code fa into string
            for (i = 0; i < states.size(); i++) {
                next = (State) states.get(i);
                if (next.isAccepting()) {
                    text += "1"; //$NON-NLS-1$
                } else {
                    text += "0"; //$NON-NLS-1$
                }
            }
            for (i = 0; i < states.size(); i++) {
                start = (State) states.get(i);
                next = start.getNextState(A);
                text += "#" + states.indexOf(next); //$NON-NLS-1$
                next = start.getNextState(B);
                text += "#" + states.indexOf(next); //$NON-NLS-1$
            }
            return text;
        } catch (FAException e) {
            Debug.showException(e);
            return null;
        }
    }

    public static String createSymetricDetABSerial(FA fa) {
        try {
            int i, idx, pos;
            State next, start;
            List states = new ArrayList();

            String text = ""; //$NON-NLS-1$
            if (fa == null) { return ""; //$NON-NLS-1$
            }
            fa.makeDetOpt();
            fa.detMinimize();
            //done = false;
            idx = 0;
            pos = 0;
            start = (State) fa.getStartState();
            states.add(pos, start);
            pos++;
            //  Breadth first search
            while (idx != pos) {
                next = ((State) (states.get(idx))).getNextState(B);
                if (states.indexOf(next) == -1) {
                    states.add(pos, next);
                    pos++;
                }
                next = ((State) (states.get(idx))).getNextState(A);
                if (states.indexOf(next) == -1) {
                    states.add(pos, next);
                    pos++;
                }
                idx++;
                //done = (idx == pos);
            }
            // code fa into string
            for (i = 0; i < states.size(); i++) {
                next = (State) states.get(i);
                if (next.isAccepting()) {
                    text += "1"; //$NON-NLS-1$
                } else {
                    text += "0"; //$NON-NLS-1$
                }
            }
            for (i = 0; i < states.size(); i++) {
                start = (State) states.get(i);
                next = start.getNextState(B);
                text += "#" + states.indexOf(next); //$NON-NLS-1$
                next = start.getNextState(A);
                text += "#" + states.indexOf(next); //$NON-NLS-1$
            }
            return text;
        } catch (FAException e) {
            Debug.showException(e);
            return null;
        }
    } //FA2String

    public static FA createDetABFA(String serial) {
        FA fa = new FAModel();
        fa.setAlphabet(Alphabet.createABeAlphabet());
        StringTokenizer tokenizer = new StringTokenizer(serial, "#"); //$NON-NLS-1$
        String acc = tokenizer.nextToken();
        List states = new ArrayList();
        int i = 0;
        while (i < acc.length()) {
            State s = new State("s" + i); //$NON-NLS-1$
            fa.addState(s);
            fa.setAccepting(s, acc.charAt(i) == '1');
            states.add(s);
            if (i == 0) {
                fa.setStartState(s);
            }
            i++;
        }
        i = 0;
        while (i < acc.length()) {
            int a = Integer.valueOf(tokenizer.nextToken()).intValue();
            int b = Integer.valueOf(tokenizer.nextToken()).intValue();
            ((State) states.get(i)).addTransition(A, (State) states.get(a));
            ((State) states.get(i)).addTransition(B, (State) states.get(b));
            i++;
        }
        return fa;
    } //String2FA

    public static final String createSerial(FA fa) {
        String s = ""; //$NON-NLS-1$
        Iterator i = fa.getAlphabet().iterator();
        if (i.hasNext()) {
            s += i.next();
        }
        while (i.hasNext()) {
            s += "." + i.next(); //$NON-NLS-1$
        }
        s += ":"; //$NON-NLS-1$
        List states = new ArrayList();
        states.add(fa.getStartState());
        int current = 0;
        while (current < states.size()) {
            State state = (State) states.get(current);
            i = fa.getAlphabet().iterator();
            while (i.hasNext()) {
                Iterator next = state.getNextStateSet((Character) i.next())
                        .iterator();
                while (next.hasNext()) {
                    Object o = next.next();
                    if (!states.contains(o)) states.add(o);
                }
            }
            current++;
        }
        i = states.iterator();
        if (i.hasNext()) {
            s += ((State) i.next()).isAccepting() ? "1" : "0"; //$NON-NLS-1$ //$NON-NLS-2$
        }
        while (i.hasNext()) {
            s += "." + (((State) i.next()).isAccepting() ? "1" : "0"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        }
        s += ":"; //$NON-NLS-1$
        i = states.iterator();
        while (i.hasNext()) {
            Iterator j = fa.getAlphabet().iterator();
            State q = (State) i.next();
            while (j.hasNext()) {
                Iterator k = q.getNextStateSet((Character) j.next()).iterator();
                if (k.hasNext()) {
                    s += states.indexOf(k.next());
                }
                while (k.hasNext()) {
                    s += "." + states.indexOf(k.next()); //$NON-NLS-1$

                }
                s += ":"; //$NON-NLS-1$
            }
        }
        return s;
    }

    public static FA createFA(String serial) {
        FA fa = new FAModel();
        String[] p = Pattern.compile(":").split(serial, -1); //$NON-NLS-1$
        // alphabet
        String[] a = Pattern.compile("\\.").split(p[0]); //$NON-NLS-1$
        Alphabet alpha = new Alphabet();
        for (int i = 0; i < a.length; i++) {
            alpha.add(new Character(a[i].charAt(0)));
        }
        fa.setAlphabet(alpha);
        // states: accepting?
        String[] s = Pattern.compile("\\.").split(p[1]); //$NON-NLS-1$
        List states = new ArrayList();
        for (int i = 0; i < s.length; i++) {
            State state = new State("q" + i); //$NON-NLS-1$
            fa.addState(state);
            fa.setAccepting(state, s[i].equals("1")); //$NON-NLS-1$
            states.add(state);
        }
        // start state
        fa.setStartState((State) states.get(0));
        // transitions
        for (int i = 0; i < s.length; i++) {
            for (int j = 0; j < a.length; j++) {
                String[] n = Pattern.compile("\\.").split( //$NON-NLS-1$
                        p[2 + i * a.length + j], -1); 
                if (n.length > 0 && !n[0].equals("")) { //$NON-NLS-1$
                    for (int k = 0; k < n.length; k++) {
                        ((State) states.get(i)).addTransition(new Character(
                                a[j].charAt(0)), (State) states.get(Integer
                                .parseInt(n[k])));
                    }
                } else {
                }
            }
        }
        return fa;
    }
}