/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.DefaultPopup;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class IFADefaultPopup extends DefaultPopup {

    private final class SaveAsActionListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            try {
                if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                    File file = fileChooser.getSelectedFile();
                    OutputStream os = new FileOutputStream(file);
                    FSMIO.saveFSM(ifaCmplEd.getModel(), os);
                    os.flush();
                    os.close();
                }
            } catch (FileNotFoundException e) {
                Debug.showException(e);
            } catch (IOException e) {
                Debug.showException(e);
            }
        }
    }

    private final class OpenActionListener implements ActionListener {

        public void actionPerformed(ActionEvent event) {
            try {
                if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                    File file = fileChooser.getSelectedFile();
                    InputStream is = new FileInputStream(file);
                    ifaCmplEd.setFA(FSMIO.loadFSM(is));
                    is.close();
                }
            } catch (FileNotFoundException e) {
                Debug.showException(e);
            } catch (IOException e) {
                Debug.showException(e);
            }
        }
    }

    IFACompleteEditor ifaCmplEd;

    IFAFSMController controller;

    protected JMenuItem accNaccItem;

    static JFileChooser fileChooser = new JFileChooser();

    public IFADefaultPopup(IFACompleteEditor ifaCmplEd, Frame parent) {
        super(ifaCmplEd.getController(), parent);
        this.ifaCmplEd = ifaCmplEd;
        this.controller = (IFAFSMController) ifaCmplEd.getController();
        accNaccItem = new JMenuItem(Messages.getString("FSM.popup.split")); //$NON-NLS-1$
        accNaccItem.addActionListener(this);
    }

    public void setEditTarget(IFAFSMController controller) {
        // reichts??
        this.controller = controller;
        this.editorOps = controller.getFSMEditorOperations();
    }

    public JPopupMenu getPopup(int x, int y) {
        this.lastClickX = x;
        this.lastClickY = y;

        menu.removeAll();

        menu.add(undoItem);
        undoItem.setEnabled(!editorOps.undoListEmpty());

        menu.add(redoItem);
        redoItem.setEnabled(!editorOps.redoListEmpty());

        menu.addSeparator();

        menu.add(copyItem);
        menu.add(pasteItem);
        pasteItem.setEnabled(controller.getModel().isPasteable());
        
        menu.addSeparator();
        //"Add state"
        menu.add(addStateItem);

        /*
         * JMenuItem mi = new JMenuItem("complete"); menu.add(mi);
         * mi.addActionListener(this); try { if
         * (((FA)controller.getModel()).isDeterministic() ||
         * ((FA)controller.getModel()).hasMultipleTransitions()) {
         * mi.setEnabled(false); } } catch (Exception e) {}
         */

        // "Clear"
        menu.add(clearItem);

        if (ifaCmplEd.alphabetEditable) {
            //			menu.add(AlphabetMenu.getMenu(controller));
            menu.add(AlphabetMenu.getEditCustomMenu(controller));
        }

        menu.add(ViewMenu.getMenu(ifaCmplEd));

        JCheckBoxMenuItem cmi = new JCheckBoxMenuItem(Messages
                .getString("FSM.popup.track_input")); //$NON-NLS-1$
        menu.add(cmi);
        cmi.addActionListener(this);
        if (ifaCmplEd.isInputTrackerShowing()) {
            cmi.setState(true);
        }
        menu.add(getLayoutMenu());
        menu.add(getStyleMenu());
        menu.add(changeNameItem);
        menu.addSeparator();
        JMenuItem mi = new JMenuItem(Shared.OPEN);
        mi.addActionListener(new OpenActionListener());
        menu.add(mi);
        mi = new JMenuItem(Shared.SAVE_AS);
        mi.addActionListener(new SaveAsActionListener());
        menu.add(mi);
        return menu;
    }

    protected JMenu getLayoutMenu() {

        layoutMenu.removeAll();

        // "Fit"
        layoutMenu.add(fitItem);

        layoutMenu.addSeparator();
        // "Organic"
        layoutMenu.add(organicItem);
        // "Circle"
        layoutMenu.add(circleItem);
        // "Accept/Non-Accept Split"
        layoutMenu.add(accNaccItem);

        layoutMenu.addSeparator();
        // "Align with grid"
        layoutMenu.add(alignItem);
        if (fsmView.isGridAligned()) {
            alignItem.setState(true);
        }

        return layoutMenu;
    }

    public void actionPerformed(ActionEvent e) {
         if (Messages.getString("FSM.popup.track_input") //$NON-NLS-1$
                .equals(e.getActionCommand())) {
            ifaCmplEd.showInputTracker(!ifaCmplEd.isInputTrackerShowing());
        } else if (Messages.getString(Messages.getString("FSM.popup.split")) //$NON-NLS-1$
                .equals(e.getActionCommand())) {
            AccNonAccLayouter.layout(fsmView);
            fsmView.fit();
            fsmView.repaint();
        } else {
            super.actionPerformed(e);
        }
    }

}