/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.Frame;

import javax.swing.event.MouseInputListener;

import ch.ethz.exorciser.fsmgui.BinaryCharTransitionPopup;
import ch.ethz.exorciser.fsmgui.FSMControllerInterface;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;
import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMMouseInputListener;
import ch.ethz.exorciser.fsmgui.FSMStateInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.fsmgui.State;
import ch.ethz.exorciser.fsmgui.StatePopup;
import ch.ethz.exorciser.fsmgui.Transition;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class IFAFSMController implements FSMControllerInterface {

	protected MouseInputListener mouseListener;
	public FSMEditorOperations editorOps;
	protected FSMModelInterface model;
	public FSMView fsmView;

	protected IFADefaultPopup defaultPopup;
	protected StatePopup statePopup;
	protected BinaryCharTransitionPopup binTransPopup;
	protected CustomTransitionPopup transPopup;

	//protected Alphabet customAlphabet;

	public IFAFSMController() {
	}

	public IFAFSMController(FSMModelInterface model, FSMView fsmView, Frame parent) {
		this.model = model;
		this.fsmView = fsmView;
		editorOps = new FSMEditorOperations(model, fsmView);
		mouseListener = new FSMMouseInputListener(this);
		fsmView.addMouseListener(mouseListener);
		fsmView.addMouseMotionListener(mouseListener);
		statePopup = new StatePopup(this, parent);
//		customAlphabet = new Alphabet(Alphabet.EPSILON);
	}

	void createDefaultPopup(IFACompleteEditor ifaCmplEd, Frame parent) {
		defaultPopup = new IFADefaultPopup(ifaCmplEd, parent);
	}

	public void setModel(FSMModelInterface model) {
		this.model = model;
		this.editorOps.setModel(model);

		if (mouseListener instanceof FSMMouseInputListener) {
			((FSMMouseInputListener) mouseListener).setEditTarget(this);
		}
		this.defaultPopup.setEditTarget(this);
		this.statePopup = new StatePopup(this);
		this.binTransPopup = null; // lazy new BinaryCharTransitionPopup(this);
		this.transPopup = null; // lazy new CustomTransitionPopup(this);
//		this.customAlphabet = model.getAlphabet();
	}

	public MouseInputListener getMouseInputListener() {
		return mouseListener;
	}

	public FSMEditorOperations getFSMEditorOperations() {
		return editorOps;
	}

	public FSMView getView() {
		return fsmView;
	}

	public FSMModelInterface getModel() {
		return model;
	}

	/** Use for clear, change name, change alphabet, add trapstate etc. */
	public void changeFSM(int x, int y) {
		defaultPopup.getPopup(x, y).show(getView(), x, y);
	}

	public FSMStateInterface addState(double x, double y) {
		ch.ethz.exorciser.ifa.State s = new ch.ethz.exorciser.ifa.State(fsmView.smallestFreeQLabel(), x, y);
		if (null == model.getStartState()) {
			editorOps.addStartState(s);
		} else {
			editorOps.addState(s);
		}
		return s;
	}

	public boolean addTransitionAllowed(FSMStateInterface state) {
		return true;
	}

	public void addTransition(Transition transition, int x, int y) {
		if (getModel().getAlphabet().isDualAlphabeh()) {
			if (null == binTransPopup)
				binTransPopup = new BinaryCharTransitionPopup(this);
			binTransPopup.getPopup(transition).show(getView(), x, y);
		} else {
			if (null == transPopup)
				transPopup = new CustomTransitionPopup(this);
			transPopup.getPopup(transition).show(getView(), x, y);
		}
	}

	/** use for setAccepting, setStart, setLabel */
	public void changeState(FSMStateInterface state, int x, int y) {
		statePopup.getPopup(state).show(getView(), x, y);
	}

	public void selectState(State state) {
	}

	/** use for set Label and move transition */
	public void changeTransition(Transition transition, int x, int y) {
//		if (getModel().getAlphabet() instanceof Character01Alphabet) {
		if (getModel().getAlphabet().isDualAlphabeh()) {
			if (null == binTransPopup)
				binTransPopup = new BinaryCharTransitionPopup(this);
			binTransPopup.getPopup(transition).show(getView(), x, y);
		} else {
			if (null == transPopup)
				transPopup = new CustomTransitionPopup(this);
			transPopup.getPopup(transition).show(getView(), x, y);
		}
	}

	/** use to move transition */
	public void changeTransition(Transition transition, State sOld, State sNew) {
		editorOps.changeTransition(transition, sOld, sNew);
	}

	public void selectTransition(Transition transition) {
	}

}