/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.Color;
import java.awt.Font;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.fsmgui.ComponentRenderer;
import ch.ethz.exorciser.fsmgui.FSMEvent;
import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMModelListener;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class InputTrackPanel
	extends JPanel
	implements CaretListener, FSMModelListener {

	public static boolean SMALL = true;
	private static int SMALL_TEXTFIELD_LENGTH = 20;
	private static int NORMAL_TEXTFIELD_LENGTH = 10;

	private boolean MUTE = false;
	/** If the InputTrackPanel is set to mute, it doesn't fire mark events to any diagram. */
	public void setMute(boolean b) {
		MUTE = b;
	}

	/** The Textfield to specify the input String.
	 */
	public JTextField input;

	public Font inputFont = new Font("SansSerif", Font.BOLD, 14); //$NON-NLS-1$
	public Font smallInputFont = new Font("SansSerif", Font.PLAIN, 12); //$NON-NLS-1$

	/** The FAUserInterface Object this AnimatorPanel is working with. */
	private IFACompleteEditor ifaCmplEd;
	private FSMModelInterface model;

	/* An inner class that displays a red or green light */
	AcceptSign acceptSign;

	/** The currently active states. */
	private Set activeStates;
	/** The active States after the whole input was read */
	private Set finalStates;
	private Iterator iterator;

	public InputTrackPanel(
		IFACompleteEditor ifaCmplEd,
		boolean small,
		int length) {
		this.ifaCmplEd = ifaCmplEd;
		this.model = ifaCmplEd.getModel();
		this.model.addFSMModelListener(this);

		//this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		//this.setLayout(new GridLayout(1, 3));

		JLabel label = new JLabel(Messages.getString("FSM.label.input")); //$NON-NLS-1$
		label.setFont(Font.getFont("sans")); //$NON-NLS-1$
		//label.setBackground(new Color(170, 170, 200));
		add(label);
		input = new JTextField(length);
		input.setFont((small) ? smallInputFont : inputFont);
		input.setForeground(new Color(35, 35, 55));
		add(input);
		input.setText(""); //$NON-NLS-1$
		input.addCaretListener(this);
		acceptSign = new AcceptSign();
		add(acceptSign);
		setBackground(Color.white);
	}

	public InputTrackPanel(IFACompleteEditor ifaCmplEd, boolean small) {
		this(
			ifaCmplEd,
			small,
			(small) ? SMALL_TEXTFIELD_LENGTH : NORMAL_TEXTFIELD_LENGTH);
	}

	public InputTrackPanel(IFACompleteEditor ifaCmplEd) {
		this(ifaCmplEd, false, NORMAL_TEXTFIELD_LENGTH);
	}

	public void setMyBackground(Color bgrColor) {
		this.setBackground(bgrColor);
		acceptSign.setBackground(bgrColor);
	}

	public void setEditTarget(FSMModelInterface model) {
		this.model.removeFSMModelListener(this);
		this.model = model;
		this.model.addFSMModelListener(this);
	}

	public void notify(FSMEvent e) {
		if (e.getId() != FSMEvent.UNMARK_ALL && e.getId() != FSMEvent.MARK && !MUTE)
			caretUpdate((javax.swing.event.CaretEvent) null);
	}

	public void caretUpdate(CaretEvent e) {
		model.notifyFSMModelListeners(FSMEvent.createUnMarkAllEvent());

		if (!ifaCmplEd.isInputTrackerShowing())
			return;
		State current;
		activeStates =
			model.getStatesAfter(input.getText().substring(0, input.getCaretPosition()));
		if (null != activeStates) {
			iterator = activeStates.iterator();
			while (iterator.hasNext()) {
				current = (State) iterator.next();
				Hashtable marks = new Hashtable();
				marks.put(current, ComponentRenderer.MARK_RED);
				model.notifyFSMModelListeners(FSMEvent.createMarkEvent(marks));
			}
			iterator = activeStates.iterator();
			if (input.getText().length() > input.getCaretPosition()) {
				Character i = new Character(input.getText().charAt(input.getCaretPosition()));
				while (null != i && iterator.hasNext()) {
					current = (State) iterator.next();
					if (null != current.getNextState(i)) {
						Hashtable marks = new Hashtable();
						Object[] trans = { current, current.getNextState(i)};
						marks.put(trans, ComponentRenderer.MARK_RED);
						model.notifyFSMModelListeners(FSMEvent.createMarkEvent(marks));
					}
				}
			}
		}
		finalStates = model.getStatesAfter(input.getText());

		try {
			if (null != finalStates
				&& ((BasicFA) model).isOneOfTheseAccepting(finalStates)) {
				acceptSign.setAccept(true);
				repaint();
			} else {
				acceptSign.setAccept(false);
				repaint();
			}
		} catch (FAException e1) {
			Debug.showException(e1);
		}
	}

//	private Set statesAfter(String inputString) {
//		if (null == model)
//			return null;
//		State currState = (State) model.getStartState();
//		if (null == currState) {
//			return null;
//		}
//
//		Set currSet = new HashSet();
//		currSet.add(currState);
//		// next line adds all states reachable from the startstate with epsilon to currSet.
//		currSet.addAll(currState.getEPSReachableStates(null));
//		Character currChar;
//		Set newSet;
//		for (int i = 0; i < inputString.length(); i++) {
//			currChar = new Character(inputString.charAt(i));
//			newSet = State.getNextStateSet(currSet, currChar);
//			// newSet now contains the states reachable from currSet with currChar) 
//			currSet.clear();
//			currSet.addAll(newSet);
//			// next loop calculates the states reachable from newSet with EPSILON.
//			Iterator iter = newSet.iterator();
//			while (iter.hasNext()) {
//				State s = (State) iter.next();
//				// the method getEPSReachableStates searches rekursively for states reachable by epsilon-paths.
//				// it puts reached states into the specified sets and stops, if a reached state is already contained in the first set.
//				s.getEPSReachableStates(currSet, currSet, s);
//			}
//			//currSet = newSet;
//		}
//		return currSet;
//	}

}