/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;


/**
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class StatePairs extends HashSet {

	public StatePairs() {
		super();
	}

	public boolean containsStatePair(State first, State second) {
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			StatePair stp = (StatePair) iter.next();
			if ((stp.first == first && stp.second == second) || (stp.first == second && stp.second == first)) {
				return true;
			}
		}
		return false;
	}

	public StatePair getStatePair(State first, State second) {
		StatePair res = null;
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			StatePair stp = (StatePair) iter.next();
			if ((stp.first == first && stp.second == second) || (stp.first == second && stp.second == first)) {
				res = stp;
				break;
			}
		}
		return res;
	}

	public void addStatePair(State first, State second, State newState) {
		StatePair newStatePair = new StatePair(first, second, newState);
		add(newStatePair);
	}

	public State getCorrespondingState(State first, State second) {
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			StatePair stp = (StatePair) iter.next();
			if ((stp.first == first && stp.second == second) || (stp.first == second && stp.second == first)) {
				return stp.state;
			}
		}
		return null;
	}

	public void mark(State thisState, State otherState) {
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			StatePair stp = (StatePair) iter.next();
			if ((stp.first == thisState && stp.second == otherState) || stp.first == otherState && stp.second == thisState) {
				stp.marked = true;
			}
		}
	}

	public State[] getUnMarkedStatePair() {
		State[] res = null;
		Iterator iter = this.iterator();
		while (iter.hasNext()) {
			StatePair stp = (StatePair) iter.next();
			if (!stp.marked) {
				res = new State[] { stp.first, stp.second, stp.state };
				break;
			}
		}
		return res;
	}

	public static StatePairs getStatePairs(FA fa) {
		List states = new ArrayList();
		Iterator iter = fa.getAllStates().iterator();
		while (iter.hasNext()) {
			State state = (State) iter.next();
			states.add(state);
		}
		StatePairs res = new StatePairs();
		for (int i = 0; i < states.size() - 1; i++) {
			State st1 = (State) states.get(i);
			for (int k = i + 1; k < states.size(); k++) {
				State st2 = (State) states.get(k);
				res.addStatePair(st1, st2, new State());
			}

		}
		return res;
	}
}