/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.ifa;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;

import ch.ethz.exorciser.Messages;

/**
 * @author Reto Lamprecht
 * @author Christian Hfeli
 * @author Vincent Tscherter
 */
public class ViewMenu {

    public static JMenu getMenu(IFACompleteEditor ifaCmplEd) {

        ViewMenuHandler menuHandler = new ViewMenuHandler(ifaCmplEd);

        JMenu viewMenu = new JMenu(Messages.getString("FSM.popup.views")); //$NON-NLS-1$
        JRadioButtonMenuItem rmi;
        //	JCheckBoxMenuItem cmi;

        ButtonGroup viewGroup = new ButtonGroup();
        rmi = new JRadioButtonMenuItem(Messages
                .getString("FSM.popup.state_diagram")); //$NON-NLS-1$
        viewMenu.add(rmi);
        viewGroup.add(rmi);
        rmi.addActionListener(menuHandler);
        if (ifaCmplEd.isDiagramShowing()) {
            rmi.setSelected(true);
        }

        rmi = new JRadioButtonMenuItem(Messages.getString("FSM.popup.table")); //$NON-NLS-1$
        viewMenu.add(rmi);
        viewGroup.add(rmi);
        rmi.addActionListener(menuHandler);
        if (ifaCmplEd.isMatrixShowing()) {
            rmi.setSelected(true);
        }

        rmi = new JRadioButtonMenuItem(Messages.getString("FSM.popup.both")); //$NON-NLS-1$
        viewMenu.add(rmi);
        viewGroup.add(rmi);
        rmi.addActionListener(menuHandler);
        if (ifaCmplEd.isMatrixAndDiagramShowing()) {
            rmi.setSelected(true);
        }

        return viewMenu;
    }
}

class ViewMenuHandler implements ActionListener {

    IFACompleteEditor ifaCmplEd;

    public ViewMenuHandler(IFACompleteEditor ifaCmplEd) {
        this.ifaCmplEd = ifaCmplEd;
    }

    public void actionPerformed(ActionEvent e) {

        String item = e.getActionCommand();

        if (Messages.getString("FSM.popup.state_diagram").equals(item)) { //$NON-NLS-1$
            ifaCmplEd.showDiagram();
        } else if (Messages.getString("FSM.popup.table").equals(item)) { //$NON-NLS-1$
            ifaCmplEd.showMatrix();
        } else if (Messages.getString("FSM.popup.both").equals(item)) { //$NON-NLS-1$
            ifaCmplEd.showMatrixAndDiagram();
        }
    }

}