/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.GrammarChange;
import ch.ethz.exorciser.markov.grammar.GrammarIO;
import ch.ethz.exorciser.markov.runtime.Constraints;

/**
 * @author Vincent Tscherter
 */
public class Exercise {

   public String title;

   public String text;

   public String example;

   public Grammar checker;

   public Grammar template;

   public Grammar solution;

   public Constraints constraints;

   private Exercise (Grammar solution) {
      constraints = new Constraints();
      this.solution = solution;
      template = new Grammar(); // the grammar the user works on
      checker = new Grammar();
      Object[] alpha = solution.getAlphabet().toArray();
      for (int i = 0; i < alpha.length; i++) {
         template.executeGrammarChange(GrammarChange.createAddTerminal(
               template, alpha[i]));
         checker.executeGrammarChange(GrammarChange.createAddTerminal(checker,
               alpha[i]));
         //			template.addAlphabetSymbol(alpha[i]);
         //			checker.addAlphabetSymbol(alpha[i]);
      }
   }

   static String textvalue;

   static String examplevalue;

   static String titlevalue;

   static List stepcoef;

   static boolean step;

   static List stringcoef;

   public static Exercise createExercise (String id) {
      Exercise ex = new Exercise(GrammarIO.loadGrammar(ClassLoader
            .getSystemResourceAsStream(id + ".markov"))); //$NON-NLS-1$
      InputStream isc = ClassLoader.getSystemResourceAsStream(id + ".checker"); //$NON-NLS-1$
      if (isc != null) {
         ex.checker = GrammarIO.loadGrammar(isc);
         // is there an inputchecker?
      }
      try {
         String language = "_"+Locale.getDefault().getLanguage(); //$NON-NLS-1$
         InputStream stream = ClassLoader.getSystemResourceAsStream(id
               + language + ".exercise"); //$NON-NLS-1$
         if (stream == null) {
            stream = ClassLoader.getSystemResourceAsStream(id + ".exercise"); //$NON-NLS-1$
         }
         BufferedInputStream is = new BufferedInputStream(stream);
         SAXParser saxParser = SAXParserFactory.newInstance().newSAXParser();
         saxParser.parse(is, new DefaultHandler() {

            public void startElement (String namespaceURI, String localName,
                  String qName, Attributes atts) {
               String name = localName;
               if ("".equals(name)) { //$NON-NLS-1$
                  name = qName;
               }
               if (name.equals("exercise")) { //$NON-NLS-1$
                  stepcoef = new ArrayList();
                  stringcoef = new ArrayList();
               } else if (name.equals("text")) { //$NON-NLS-1$
                  textvalue = atts.getValue("value"); //$NON-NLS-1$
               } else if (name.equals("title")) { //$NON-NLS-1$
                  titlevalue = atts.getValue("value"); //$NON-NLS-1$
               } else if (name.equals("example")) { //$NON-NLS-1$
                  examplevalue = atts.getValue("value"); //$NON-NLS-1$
               } else if (name.equals("steplimit")) { //$NON-NLS-1$
                  step = true;
               } else if (name.equals("stringlimit")) { //$NON-NLS-1$
                  step = false;
               } else if (name.equals("coef") && step) { //$NON-NLS-1$
                  stepcoef.add(atts.getValue("value")); //$NON-NLS-1$
               } else if (name.equals("coef") && !step) { //$NON-NLS-1$
                  stringcoef.add(atts.getValue("value")); //$NON-NLS-1$
               }
            }
         });
         ex.title = titlevalue;
         ex.text = textvalue; //$NON-NLS-1$ //$NON-NLS-2$
         ex.example = examplevalue;
         if (stepcoef.size() > 0) {
            ex.constraints.relativeStepLimit = new int[stepcoef.size()];
            int i = 0;
            while (i < stepcoef.size()) {
               ex.constraints.relativeStepLimit[i] = Integer.parseInt(stepcoef
                     .get(i)
                     + ""); //$NON-NLS-1$
               i++;
            }
         }
         if (stringcoef.size() > 0) {
            ex.constraints.relativeStringLimit = new int[stringcoef.size()];
            int i = 0;
            while (i < stringcoef.size()) {
               ex.constraints.relativeStringLimit[i] = Integer
                     .parseInt(stringcoef.get(i) + ""); //$NON-NLS-1$
               i++;
            }
         }
      } catch (ParserConfigurationException e) {
         Debug.showException(e);
      } catch (SAXException e) {
         Debug.showException(e);
      } catch (FactoryConfigurationError e) {
         e.printStackTrace();
      } catch (IOException e) {
         Debug.showException(e);
      }
      return ex;
   }
}