/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.regex.Pattern;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import tools.GifEncoder;
import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.GrammarChange;
import ch.ethz.exorciser.markov.grammar.GrammarIO;
import ch.ethz.exorciser.markov.grammar.Production;
import ch.ethz.exorciser.markov.grammar.view.ProductionEditor;
import ch.ethz.exorciser.markov.runtime.Constraints;
import ch.ethz.exorciser.markov.runtime.ResetOnGrammarChange;
import ch.ethz.exorciser.markov.runtime.Visualisation;
import ch.ethz.exorciser.markov.runtime.ui.ApplicationPanel;
import ch.ethz.exorciser.tools.Filter;

/**
 * @author Vincent Tscherter
 * @author Thomas Briner
 */
public class Markov extends JPanel implements ch.ethz.exorciser.Exercise {

    static JFileChooser fileChooser = new JFileChooser();
    static {
        fileChooser.setFileFilter(new Filter("markov", "Markov Grammar")); //$NON-NLS-1$ //$NON-NLS-2$
    }

    private static int CENTER = GridBagConstraints.CENTER;

    private static int BOTH = GridBagConstraints.BOTH;

    static boolean FORWARD = true;

    static boolean BACKWARD = false;

    public static int MAXIMUM_INPUT_LENGTH = Integer.parseInt(Messages
            .getString("Markov.MaxInputLength")); //$NON-NLS-1$

    File file;

    protected Notifications notifications = new Notifications();

    JTextField input = new JTextField();

    JCheckBox exTextCheck, grammarCheck, executeCheck;

    private JSlider speedSlider;

    private String text = ""; //$NON-NLS-1$

    private Constraints c;

    protected Visualisation visualisation;

    protected Exercise exercise;

    protected ProductionEditor editor = new ProductionEditor(new Grammar());

    protected ApplicationPanel sequence;

    ResetOnGrammarChange resetOnGrammarChange;

    public Markov(String query) {
        this(Exercise.createExercise(Pattern.compile("=").split(query)[1])); //$NON-NLS-1$
    }

    public Markov() {
        this((Exercise) null);
    }

    public Markov(Exercise ex) {
        exercise = ex;
        if (ex != null && ex.template != null) {
            editor.setGrammar(ex.template);
        }
        resetOnGrammarChange = new ResetOnGrammarChange(this);
        editor.getGrammar().addGrammarListener(resetOnGrammarChange);
        // model for runtime for graphical execution of the algorithm
        visualisation = new Visualisation(editor.getGrammar(), "", "", //$NON-NLS-1$ //$NON-NLS-2$
                new Constraints(), notifications, new ArrayList());
        // view for runtime
        sequence = new ApplicationPanel(visualisation);
        input.addKeyListener(new KeyAdapter() {

            // changes of input string by user
            public void keyReleased(KeyEvent event) {
                notifications.removeAll();
                String s = input.getText();
                if (s.length() > MAXIMUM_INPUT_LENGTH) {
                    JOptionPane.showMessageDialog(null, Messages.getString(
                            "Markov.notification.string_limit") //$NON-NLS-1$
                            .replaceAll("\\$STRINGLIMIT", "" //$NON-NLS-1$ //$NON-NLS-2$
                                    + Markov.MAXIMUM_INPUT_LENGTH), Messages
                            .getString("Markov.warning.string_limit"), //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                    s = s.substring(0, MAXIMUM_INPUT_LENGTH);
                    input.setText(s);
                }
                visualisation.setGivenInput(input.getText());
                executeReset(); // terminates execution & resets runtime
                checkGivenInput(input.getText());
                sequence.resized();
            }
        });
        input.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
        visualisation.addSequenceListener(new Visualisation.Listener.Adapter() {

            public void handleReset() {
                if (!input.getText().equals(visualisation.getActString())) {
                    input.setText(visualisation.getActString());
                }
            }
        });
        //
        JPanel filePanel = new JPanel(new FlowLayout()); // UI for Template
        // Grammar
        filePanel.setBackground(Color.white);
        filePanel.add(createButton(open));
        filePanel.add(createButton(save));
        filePanel.add(createButton(saveAs));
        filePanel.add(createButton(export));
        //
        JPanel grammar = new JPanel(new BorderLayout());
        grammar.add(filePanel, BorderLayout.NORTH);
        grammar.add(new JScrollPane(editor), BorderLayout.CENTER);
        //
        speedSlider = new JSlider(0, 100, 100); // UI for execution handling
        speedSlider.setToolTipText(Messages.getString("Markov.label.speed")); //$NON-NLS-1$
        speedSlider.setPaintLabels(true);
        speedSlider.setBackground(Color.white);
        visualisation.speedChanged(100);
        Hashtable labelTable = new Hashtable();
        labelTable.put(new Integer(10), new JLabel(Messages
                .getString("Markov.label.slow"))); //$NON-NLS-1$
        labelTable.put(new Integer(90), new JLabel(Messages
                .getString("Markov.lable.fast"))); //$NON-NLS-1$
        speedSlider.setLabelTable(labelTable);
        speedSlider.setPreferredSize(new Dimension(200, 40));
        speedSlider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                visualisation
                        .speedChanged(((JSlider) e.getSource()).getValue());
            }
        });
        JPanel slider = new JPanel();
        slider.add(speedSlider, BorderLayout.CENTER);
        slider.setBackground(Color.white);
        JPanel ctrl = new JPanel();
        ctrl.setBackground(Color.white);
        ctrl.add(createButton(runBackward, Shared.RUN_BACKWARD)); //$NON-NLS-1$
        ctrl.add(createButton(stepBackward, Shared.STEP_BACK)); //$NON-NLS-1$
        ctrl.add(createButton(pause, Shared.PAUSE)); //$NON-NLS-1$
        ctrl.add(createButton(stepForward, Shared.STEP_FORWARD)); //$NON-NLS-1$
        ctrl.add(createButton(runForward, Shared.RUN_FORWARD)); //$NON-NLS-1$
        ctrl.add(createButton(reset, Shared.STOP_AND_RESET)); //$NON-NLS-1$
        JPanel control = new JPanel(new BorderLayout());
        control.add(slider, BorderLayout.NORTH);
        control.add(ctrl, BorderLayout.CENTER);
        control.setBackground(Color.white);
        JPanel execution = new JPanel(new BorderLayout());
        execution.add(input, BorderLayout.NORTH);
        //		JViewport view = new JViewport();
        //		view.setView(sequence);
        execution.add(new JScrollPane(sequence), BorderLayout.CENTER);
        execution.add(control, BorderLayout.SOUTH);
        //
        JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, grammar,
                execution);
        sp.setDividerLocation(0.5);
        //
        //		String text = "";
        if (ex != null && ex.text != null) {
            text = ex.text;
        }
        //		Constraints c;
        if (ex != null && ex.constraints != null) {
            c = ex.constraints;
        } else {
            c = new Constraints();
        }
        // Exercise Text
        JEditorPane task = ExerciseUIFactory.createTaskPane(this);
        //
        if (ex != null && ex.example != null) {
            //			givenInput = ex.example;
            visualisation.setGivenInput(ex.example);
            sequence.resized();
        }
        //
        JPanel main = new JPanel();
        main = new JPanel(new GridBagLayout());
        main.setBackground(Color.white);
        int i;
        i = 0;
        Insets ins = new Insets(0, 0, 0, 0);
        main.add(task, new GridBagConstraints(0, i++, 1, 1, 1, 0, CENTER, BOTH,
                ins, 0, 0));
        main.add(sp, new GridBagConstraints(0, i++, 1, 1, 1, 1, CENTER, BOTH,
                ins, 0, 0));
        JEditorPane p = new JEditorPane();
        p.setEditable(false);
        p.addHyperlinkListener(new ExtendedHyperlinkListener());
        try {
            p.setPage(ClassLoader.getSystemResource((ex != null) ? (Messages
                    .getString("Markov.file.control")) //$NON-NLS-1$
                    : (Messages.getString("Markov.file.help")))); //$NON-NLS-1$
        } catch (IOException e) {
            Debug.showException(e);
        }
        ExerciseUIFactory.createExerciseUI(main, p, notifications, this);
        //		}
        sp.setDividerLocation(300);
        visualisation.reset();
        //		stopChecking.setEnabled(false);
        //  notifications.addHintNotification(Messages
        //          .getString("Markov.feedback.hint.title"), //$NON-NLS-1$
        //         Messages.getString("Markov.feedback.hint")); //$NON-NLS-1$
    }

    //// BUTTONS /////////////////////////////////////////////////////////////
    protected Thread t;

    public void instructions() {
        notifications.removeAll();
        notifications.addHintNotification(Messages
                .getString("Markov.feedback.hint.title"), //$NON-NLS-1$
                Messages.getString("Markov.feedback.hint")); //$NON-NLS-1$
    }

    public void check(String query) {
        if (t == null || !t.isAlive()) {
            t = new Checker(Markov.this, Query.getValue(query, "mode").equals( //$NON-NLS-1$
                    "random")); //$NON-NLS-1$
            t.start();
        } else {
            stop_checking();
        }
    }

    public void stop_checking() {
        if (t != null) {
            t.interrupt();
            t = null;
        }
    }

    public void permute() {
        notifications.removeAll();
        repaint();
        Grammar a = ((Grammar) editor.getGrammar().clone());
        int n = a.getProductions().size();
        for (int i = 0; i < n; i++) {
            int l = (int) (Math.random() * n);
            a.executeGrammarChange(GrammarChange.createMoveProduction(a,
                    (Production) a.getProductions().get(l), l, (int) (Math
                            .random() * n)));
        }
        editor.setGrammar(a);
        editor.getGrammar().addGrammarListener(resetOnGrammarChange);
        visualisation.changeGrammar(editor.getGrammar());
        // notify visualisation without
        visualisation.reset(); // changing input & constraints
    }

    public void solve() {
        notifications.removeAll();
        if (exercise != null && exercise.solution != null) {
            Grammar a = (Grammar) exercise.solution.clone();
            editor.setGrammar(a);
            editor.getGrammar().addGrammarListener(resetOnGrammarChange);
            visualisation.changeGrammar(editor.getGrammar());
            visualisation.reset();
        }
    }

    // file handling
    Action open = new AbstractAction(Shared.OPEN, Shared.loadIcon(Messages
            .getString("Markov.image.open"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                try {
                    file = fileChooser.getSelectedFile();
                    InputStream is = new FileInputStream(file);
                    Grammar g = GrammarIO.loadGrammar(is);
                    is.close();
                    if (g != null) {
                        notifications.removeAll();
                        editor.setGrammar(g);
                        visualisation.changeGrammar(editor.getGrammar());
                        visualisation.reset();
                        repaint();
                    }
                } catch (FileNotFoundException e) {
                    Debug.showException(e);
                } catch (IOException e) {
                    Debug.showException(e);
                }
            }
        }
    };

    Action save = new AbstractAction(Shared.SAVE, Shared.loadIcon(Messages
            .getString("Markov.image.save"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            try {
                OutputStream os = null;
                if (file == null) {
                    if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                        os = new FileOutputStream(fileChooser.getSelectedFile());
                        file = fileChooser.getSelectedFile();
                    }
                } else {
                    os = new FileOutputStream(file);
                }
                if (os != null) {
                    GrammarIO.saveGrammar(editor.getGrammar(), os);
                    os.flush();
                    os.close();
                }
            } catch (FileNotFoundException e) {
                Debug.showException(e);
            } catch (IOException e) {
                Debug.showException(e);
            }
        }
    };

    Action saveAs = new AbstractAction(Shared.SAVE_AS, Shared.loadIcon(Messages
            .getString("Markov.image.save_as"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            try {
                fileChooser.setSelectedFile(file);
                if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                    file = fileChooser.getSelectedFile();
                    OutputStream os = new FileOutputStream(file);
                    GrammarIO.saveGrammar(editor.getGrammar(), os);
                    os.flush();
                    os.close();
                }
            } catch (FileNotFoundException e) {
                Debug.showException(e);
            } catch (IOException e) {
                Debug.showException(e);
            }
        }
    };

    Action export = new AbstractAction(Shared.EXPORT, Shared.loadIcon(Messages
            .getString("Markov.image.export"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            // Dialog to choose, which Components to Export
            final JDialog compToPrint = new JDialog();
            compToPrint.setModal(true);
            compToPrint.setTitle(Messages.getString("Markov.export.title")); //$NON-NLS-1$
            Container contentPane = compToPrint.getContentPane();
            contentPane.setBackground(Color.white);
            contentPane.setLayout(new BorderLayout());

            JPanel checkbox = new JPanel();
            exTextCheck = new JCheckBox(Messages
                    .getString("Markov.export.selection.text")); //$NON-NLS-1$
            exTextCheck.setSelected(true);
            exTextCheck.setBackground(Color.white);
            grammarCheck = new JCheckBox(Messages
                    .getString("Markov.export.selection.grammar")); //$NON-NLS-1$
            grammarCheck.setSelected(true);
            grammarCheck.setBackground(Color.white);
            executeCheck = new JCheckBox(Messages
                    .getString("Markov.export.selection.trace")); //$NON-NLS-1$
            executeCheck.setSelected(true);
            executeCheck.setBackground(Color.white);
            checkbox.add(exTextCheck);
            checkbox.add(grammarCheck);
            checkbox.add(executeCheck);
            checkbox.setBackground(Color.white);
            Action ok = new AbstractAction() {

                public void actionPerformed(ActionEvent event) {
                    compToPrint.dispose();
                    export();
                }
            };
            Action cancel = new AbstractAction() {

                public void actionPerformed(ActionEvent event) {
                    compToPrint.dispose();
                }
            };
            JPanel buttons = new JPanel(new GridLayout());
            JButton okButton = new JButton(ok);
            okButton.setText(Shared.OKAY); //$NON-NLS-1$
            buttons.add(okButton);
            JButton cancelButton = new JButton(cancel);
            cancelButton.setText(Shared.CANCEL);
            buttons.add(cancelButton);
            //contentPane.add(label);
            contentPane.add(checkbox, BorderLayout.CENTER);
            contentPane.add(buttons, BorderLayout.SOUTH);
            compToPrint.pack();
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
            compToPrint.setLocation(
                    (screen.width - compToPrint.getWidth()) / 2,
                    (screen.height - compToPrint.getHeight()) / 2);
            compToPrint.setVisible(true);
        }
    };

    Action reset = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.stop"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            executeReset();
        }
    };

    Action stepForward = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.step_forward"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            notifications.removeAll();
            visualisation.setDirection(FORWARD);
            visualisation.setStep(true);
            visualisation.run();
        }
    };

    Action stepBackward = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.step_back"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            notifications.removeAll();
            visualisation.setDirection(BACKWARD);
            visualisation.setStep(true);
            visualisation.run();
        }
    };

    Action runForward = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.run_forward"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            notifications.removeAll();
            initThread();
            visualisation.clearHighlight();
            visualisation.setDirection(FORWARD);
            visualisation.setStep(false);
            t.start();
        }
    };

    Action runBackward = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.run_backward"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            notifications.removeAll();
            initThread();
            visualisation.clearHighlight();
            visualisation.setDirection(BACKWARD);
            visualisation.setStep(false);
            t.start();
        }
    };

    Action pause = new AbstractAction("", Shared.loadIcon(Messages //$NON-NLS-1$
            .getString("Markov.image.pause"))) { //$NON-NLS-1$

        public void actionPerformed(ActionEvent arg0) {
            visualisation.pause();
            sequence.repaint();
        }
    };

    void initThread() {
        Constraints constr;
        if (exercise == null || exercise.constraints == null) {
            constr = new Constraints();
        } else {
            constr = exercise.constraints;
        }
        String string;
        if (t != null) {
            t.interrupt();
            t = null;
        }
        string = visualisation.getActString(); // get the actual string
        List history = visualisation.getHistory(); // an history
        boolean finished = visualisation.isFinished();
        t = new Visualisation(editor.getGrammar(), string, input.getText(),
                notifications, history);
        visualisation = (Visualisation) t; // init a new thread with old data
        visualisation.setConstraints(constr);
        visualisation.addSequenceListener(sequence);
        visualisation.speedChanged(speedSlider.getValue());
        visualisation.setFinished(finished);
        sequence.setVisualisation(visualisation);
    }

    public void executeReset() {
        //remove marks
        List prods = editor.getGrammar().getProductions();
        for (int i = 0; i < prods.size(); i++) {
            ((Production) prods.get(i)).removeAttribute("MARK"); //$NON-NLS-1$
        }
        //
        notifications.removeAll();
        if (t != null) t.interrupt();
        t = null;
        visualisation.reset();
    }

    void export() {
        // Printerpanel, where the chosen components are being prepared
        JPanel pp = new JPanel();
        pp.setLayout(new BoxLayout(pp, BoxLayout.Y_AXIS));
        int exDimY = 0, exDimX = 0, grDimY = 0, grDimX = 0, seqDimY = 0, seqDimX = 0;
        if (exTextCheck.isSelected()) {
            JEditorPane taskPrint = ExerciseUIFactory.createTaskPane(this);
            taskPrint.setEditable(false);
            taskPrint.setAlignmentX(Component.CENTER_ALIGNMENT);
            exDimY = taskPrint.getPreferredSize().height;
            exDimX = taskPrint.getPreferredSize().width;
            pp.add(taskPrint);
        }
        if (grammarCheck.isSelected()) {
            ProductionEditor edClone = (ProductionEditor) editor.clone();
            List v = edClone.getGrammar().getProductions();
            for (int i = 0; i < v.size(); i++) {
                ((Production) v.get(i)).removeAttribute("MARK"); //$NON-NLS-1$
            }
            edClone.setAlignmentX(Component.CENTER_ALIGNMENT);
            pp.add(edClone);
            grDimY = editor.getSize().height;
            grDimX = editor.getSize().width;
        }
        ApplicationPanel seqClone = new ApplicationPanel(sequence.getLines());
        if (executeCheck.isSelected()) {
            seqClone = (ApplicationPanel) sequence.clone();
            seqClone.resized();
            //			seqClone.setAlignmentX(Component.CENTER_ALIGNMENT);
            seqDimY = sequence.getSizeForPrint().height;
            seqDimX = sequence.getSizeForPrint().width;
            pp.add(seqClone);
        }
        int maxDimX = Math.max(Math.max(grDimX, exDimX), seqDimX);
        // compute size for GIF
        JFrame pf = new JFrame();
        pf.getContentPane().add(pp);
        pf.pack();
        if (maxDimX > 0) {
            final Image img = new BufferedImage(maxDimX,
                    (exDimY + grDimY + seqDimY),
                    BufferedImage.TYPE_BYTE_INDEXED);
            pp.setPreferredSize(new Dimension(maxDimX,
                    (exDimY + grDimY + seqDimY)));
            pp.setSize(new Dimension(maxDimX, (exDimY + grDimY + seqDimY)));
            pp.paint(img.getGraphics());
            fileChooser.setFileFilter(new Filter("gif", "GIF Image")); //$NON-NLS-1$ //$NON-NLS-2$
            try {
                fileChooser.setSelectedFile(file);
                if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                    file = fileChooser.getSelectedFile();
                    OutputStream out = new FileOutputStream(file);
                    // save Image to GIF, using ACME ImageEncoder
                    GifEncoder encoder = new GifEncoder(img, out);
                    encoder.encode();
                    out.flush();
                    out.close();
                }
            } catch (FileNotFoundException e2) {
                Debug.showException(e2);
            } catch (IOException ioe) {
                Debug.showException(ioe);
            }
            fileChooser.setFileFilter(new Filter("markov", "Markov Grammar")); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    void checkGivenInput(String i) {
        boolean illegalSymbol = false;
        for (int k = 0; k < i.length(); k++) {
            illegalSymbol = illegalSymbol
                    || !editor.getGrammar().getAlphabet().contains(
                            "" + i.charAt(k)); //$NON-NLS-1$
        }
        if ((illegalSymbol) && (!i.equals(""))) { //$NON-NLS-1$
            notifications.addWarningNotification(Messages
                    .getString("Markov.feedback.illegal_input.title"), //$NON-NLS-1$
                    Messages.getString("Markov.feedback.illegal_input")); //$NON-NLS-1$
        }
    }

    private JButton createButton(Action a) {
        JButton b = new JButton(a);
        b.setToolTipText(a.getValue(Action.NAME).toString());
        b.setMargin(new Insets(0, 0, 0, 0));
        return b;
    }

    private JButton createButton(Action a, String tooltip) {
        JButton b = new JButton(a);
        b.setToolTipText(tooltip);
        b.setMargin(new Insets(0, 0, 0, 0));
        return b;
    }

    public JComponent getPageUI() {
        return this;
    }

    public String getTitle() {
        if (exercise != null) {
            return Messages.getString("Markov.titel") + ": " + exercise.title; //$NON-NLS-1$//$NON-NLS-2$
        }
		return Messages.getString("Markov.titel") + ": " //$NON-NLS-1$ //$NON-NLS-2$
		        + Messages.getString("Markov.lab"); //$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see ch.ethz.exorciser.Exercise#getTask()
     */
    public String getTask() {
        return Messages.getString("Markov.exercise.template").replaceAll( //$NON-NLS-1$
                "\\$TEXT", text).replaceAll( //$NON-NLS-1$
                "\\$STEPLIMIT", //$NON-NLS-1$
                c.getRelativeStepLimitAsHTML()).replaceAll("\\$STRINGLIMIT", //$NON-NLS-1$
                c.getRelativeStringLimitAsHTML());
    }
}