/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.grammar;

/**
 * @author Reto Lamprecht
 * @author Vincent Tscherter
 */
public class GrammarChange {

   public static final int CHANGE_PRODUCTION = 119;

   public static final int CHANGE_PRODUCTION_LEFT = 120;

   public static final int CHANGE_PRODUCTION_RIGHT = 121;

   public static final int ADD_PRODUCTION = 122;

   public static final int REMOVE_PRODUCTION = 123;

   public static final int MOVE_PRODUCTION = 124;

   public static final int CLEAR_ALL = 125;

   public static final int ADD_VAR = 130;

   public static final int REMOVE_VAR = 131;

   public static final int ADD_TERM = 132;

   public static final int REMOVE_TERM = 133;

   public static final int RENAME_SYM = 135;

   public static final int SET_STOP_PRODUCTION = 150;

   public static final int SET_NONSTOP_PRODUCTION = 151;

   private int id;

   private String left, right;

   private int from, to;

   private Object item1, item2;

   private Grammar grammar;

   private Production production;

   public int getId () {
      return id;
   }

   public String getLeft () {
      return left;
   }

   public String getRight () {
      return right;
   }

   public int getFrom () {
      return from;
   }

   public int getTo () {
      return to;
   }

   public Object getVariable () {
      return item1;
   } // used with ADD_VAR, REMOVE_VAR

   public Object getTerminal () {
      return item1;
   } // used with ADD_TERM, REMOVE_TERM

   public Object getOldSym () {
      return item1;
   } // used with RENAME_SYM

   public Object getNewSym () {
      return item2;
   } // used with RENAME_SYM

   public Grammar getGrammar () {
      return grammar;
   }

   public Production getProduction () {
      return production;
   } // used with CHANGE_PRODUCTION_LEFT/RIGHT, ADD/REMOVE_PRODUCTION

   public int getProdIndex () {
      return to;
   }

   /**
    * Used for GrammarChanges of type CHANGE_PRODUCTION_LEFT/RIGHT.
    */
   private GrammarChange (int id, Grammar g, Production p, String l, String r,
         int from, int to) {
      this.id = id;
      this.grammar = g;
      this.production = p;
      this.left = l;
      this.right = r;
      this.from = from;
      this.to = to;
   }

   /**
    * Used for GrammarChanges of type ADD/REMOVE_VAR/TERM.
    */
   private GrammarChange (int id, Grammar g, Object item1, Object item2) {
      this.id = id;
      this.grammar = g;
      this.item1 = item1;
      this.item2 = item2;
   }

   public static GrammarChange createChangeProductionLeft (Grammar grammar,
         Production prod, String left) {
      return new GrammarChange(CHANGE_PRODUCTION_LEFT, grammar, prod, left,
            (String) null, -1, -1);
   }

   public static GrammarChange createChangeProduction (Grammar grammar,
         Production prod, String left, String right) {
      return new GrammarChange(CHANGE_PRODUCTION, grammar, prod, left, right,
            -1, -1);
   }

   public static GrammarChange createChangeProductionRight (Grammar grammar,
         Production prod, String right) {
      return new GrammarChange(CHANGE_PRODUCTION_RIGHT, grammar, prod,
            (String) null, right, -1, -1);
   }

   public static GrammarChange createMoveProduction (Grammar grammar,
         Production prod, int from, int to) {
      return new GrammarChange(MOVE_PRODUCTION, grammar, prod, (String) null,
            (String) null, from, to);
   }

   public static GrammarChange createAddProduction (Grammar grammar,
         Production prod, int index) {
      return new GrammarChange(ADD_PRODUCTION, grammar, prod, (String) null,
            (String) null, -1, index);
   }

   public static GrammarChange createRemoveProduction (Grammar grammar,
         Production prod) {
      return new GrammarChange(REMOVE_PRODUCTION, grammar, prod, (String) null,
            (String) null, -1, -1);
   }

   public static GrammarChange createClearAll (Grammar grammar) {
      return new GrammarChange(CLEAR_ALL, grammar, null, null);
   }

   public static GrammarChange createAddVariable (Grammar grammar,
         Object variable) {
      return new GrammarChange(ADD_VAR, grammar, variable, null);
   }

   public static GrammarChange createRemoveVariable (Grammar grammar,
         Object variable) {
      return new GrammarChange(REMOVE_VAR, grammar, variable, null);
   }

   public static GrammarChange createAddTerminal (Grammar grammar,
         Object terminal) {
      return new GrammarChange(ADD_TERM, grammar, terminal, null);
   }

   public static GrammarChange createRemoveTerminal (Grammar grammar,
         Object terminal) {
      return new GrammarChange(REMOVE_TERM, grammar, terminal, null);
   }

   public static GrammarChange createRenameSymbol (Grammar grammar,
         Object oldSym, Object newSym) {
      return new GrammarChange(RENAME_SYM, grammar, oldSym, newSym);
   }

   public static GrammarChange createSetStopProduction (Grammar grammar,
         Production prod) {
      return new GrammarChange(SET_STOP_PRODUCTION, grammar, prod,
            (String) null, (String) null, -1, -1);
   }

   public static GrammarChange createSetNonStopProduction (Grammar grammar,
         Production prod) {
      return new GrammarChange(SET_NONSTOP_PRODUCTION, grammar, prod,
            (String) null, (String) null, -1, -1);
   }
}