/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.grammar.view;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.util.List;

import ch.ethz.exorciser.Messages;

/**
 * @author Reto Lamprecht
 * @author Vincent Tscherter
 */
public class ContrastRenderer extends DefaultRenderer {

   ///////////
   // Colors //
   private static Color varBGR = new Color(240, 240, 240);

   private static Color varBorder = new Color(210, 210, 210);

   private static Color varText = new Color(0, 0, 0);

   private static Color termBGR = Color.white;

   private static Color termBorder = new Color(230, 230, 230);

   private static Color termText = new Color(0, 0, 0);

   //	private static Color startBGR = new Color(255, 150, 150);
   //	private static Color startBorder = startBGR;
   //	private static Color startText = varText;
   //	private static Color symlistBGR = Color.white;
   private static Color varListBorder = new Color(50, 50, 50);

   private static Color termListBorder = new Color(50, 50, 50);

   private static Color prodBGR = Color.white; //new Color(240, 240, 240);

   private static Color prodEmptyBorder = new Color(50, 80, 180);

   private static Color prodBorder = Color.white;

   private static Color prodText = new Color(0, 0, 0);

   //	private static Color prodmarkedBGR = new Color(130, 60, 60);
   //	private static Color prodmarkedBorder = new Color(70, 00, 0);
   private static Color draggedSym = new Color(0, 0, 255, 80);

   //	private static Color draggedProdBGR = new Color(0, 240, 0, 130);
   private static Color draggedProdBorder = new Color(0, 140, 0, 130);

   //	private static Color highlightSymList = new Color(200, 200, 200, 130);
   ////////////
   // Strokes //
   static BasicStroke ProdBoundsStroke = new BasicStroke(1.4f);

   static BasicStroke listStroke = new BasicStroke(1.4f);

   static BasicStroke symStroke = new BasicStroke(Math.min(1,
         ProductionEditor.fontsize / 16), BasicStroke.CAP_ROUND,
         BasicStroke.JOIN_ROUND);

   public ContrastRenderer () {}

   public void configureEditor (ProductionEditor ped) {
      ProductionUI.insets = new Insets(2, 7, 3, 7);
      SymbolList.insets = new Insets(2, 6, 2, 3);
      ProductionUI.left_arrow_hgap = 7;
      ProductionUI.arrow_right_hgap = 7;
   }

   public void paintSymbol (Symbol sym, Graphics2D g) {
      boolean v = sym.isVar();
      boolean customMark = false;
      Color customMarkCol = null;
      if (null != sym.getParentProduction()
            && sym.getParentProduction().hasAttribute(
                  ProductionEditor.ATTR_MARK_SINGLE_SYM)) {
         List value = (List) sym.getParentProduction().getAttribute(
               ProductionEditor.ATTR_MARK_SINGLE_SYM);
         if (sym.getIndexInProduction() == ((Integer) value.get(0)).intValue()) {
            customMark = true;
            customMarkCol = (Color) value.get(1);
         }
      }
      if (sym != epsilon) {
         if (customMark) {
            g.setColor(mixColor(customMarkCol, (v ? varBGR : termBGR), 0.5));
         } else {
            g.setColor((v ? varBGR : termBGR));
         }
         g.fill(sym.rrect);
         if (sym.dragged) {
            g.setColor(draggedSym);
            g.fill(sym.rrect);
         }
      }
      if (sym != epsilon) {
         if (customMark) {
            g.setColor(mixColor(customMarkCol, (v ? varBorder : termBorder),
                  0.25));
         } else {
            g.setColor((v ? varBorder : termBorder));
            //				g.setColor((start) ? startBorder : (v ? varBorder :
            // termBorder));
         }
         Stroke backupStroke = g.getStroke();
         g.setStroke(symStroke);
         g.draw(sym.rrect);
         if (sym.dragged) {
            g.setColor(draggedSym);
            g.draw(sym.rrect);
         }
         g.setStroke(backupStroke);
      }
      g.setColor((v ? varText : termText));
      g.drawString(sym.getModel().toString(),
            sym.bounds.x + Symbol.insets.left, sym.getTextLineY());
   }

   public void paintSymbolList (SymbolList syml, Graphics2D g) {
      if (syml.isVariableList()) {
         g.setColor(varListBorder);
         g.drawString(Messages.getString("Markov.label.marker") + " = {", //$NON-NLS-1$ //$NON-NLS-2$
               syml.openingBracketsX, syml.getTextLineY());
         g.drawString("}", //$NON-NLS-1$
               syml.closingBracketsX, syml.getTextLineY());
      } else if (syml.isTerminalList()) {
         g.setColor(termListBorder);
         g.drawString(Messages.getString("Markov.label.alphabet") + " = {", //$NON-NLS-1$ //$NON-NLS-2$
               syml.openingBracketsX, syml.getTextLineY());
         g.drawString("}", //$NON-NLS-1$
               syml.closingBracketsX, syml.getTextLineY());
      } else if (syml.symbols.isEmpty()) {
         epsilon.updateBounds(syml.getX() + SymbolList.insets.left, syml.getY()
               + SymbolList.insets.top);
         epsilon.paint(this, g);
      }
   }

   public void paintProduction (ProductionUI prod, Graphics2D g) {
      Composite backupComposit = g.getComposite();
      Stroke backupStroke = g.getStroke();
      g
            .setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
                  0.20f));
      if (prod.getModel().hasAttribute(ProductionEditor.ATTR_MARK)) {
         g.setColor(mixColor(prodBGR, (Color) prod.getModel().getAttribute(
               ProductionEditor.ATTR_MARK), 0.5));
      } else {
         g.setColor(prodBGR);
      }
      g.fill(prod.rrect);
      g.setComposite(backupComposit);
      g.setStroke(ProdBoundsStroke);
      if (prod.dragged) {
         g.setColor(draggedProdBorder);
         g.draw(prod.rrect);
      } else {
         if (prod.getModel().hasAttribute(ProductionEditor.ATTR_MARK)) {
            g.setColor(mixColor(prodBorder, (Color) prod.getModel()
                  .getAttribute(ProductionEditor.ATTR_MARK), 0.25));
         } else if (prod.getModel().isEmpty()) {
            g.setColor(prodEmptyBorder);
         } else {
            g.setColor(prodBorder);
         }
         g.draw(prod.rrect);
      }
      g.setStroke(backupStroke);
      g.setColor(prodText);
      g.drawString(prod.index + ": ", //$NON-NLS-1$
            prod.indexX, prod.left.getTextLineY());
      prod.left.paint(this, g);
      g.setColor(prodText);
      if (prod.getModel().hasAttribute(ProductionEditor.ATTR_STOP)) paintStop(
            g, prod, prod.arrowX, prod.left.getTextLineY());
      else paintArrow(g, prod, prod.arrowX, prod.left.getTextLineY());
      prod.right.paint(this, g);
   }

   public void paintInsertIcon (InsertProductionIcon icon, Graphics2D g) {
      icon.paint(g);
   }

   public void paintProductionEditor (ProductionEditor ped, Graphics2D g) {
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      g.setFont(ProductionEditor.font);
   }
}