/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.grammar.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;

import ch.ethz.exorciser.Messages;

/**
 * @author Reto Lamprecht
 */
public abstract class DefaultRenderer implements Renderer {

   // not the best idea to have this layout member in this class, could pass it
   // as argument in paintArrow...
   static BasicStroke arrowStroke = new BasicStroke(
         ProductionEditor.fontsize / 9, BasicStroke.CAP_ROUND,
         BasicStroke.JOIN_ROUND);

   /**
    * Mixes two colors. The third argument specifies the composition. 1 produces
    * pure first Color, 0.5 an equal mix, 0 pure second Color.
    */
   public static Color mixColor (Color c1, Color c2, double rel) {
      if (rel > 1 || rel < 0) return null;
      float[] cc1 = new float[3];
      float[] cc2 = new float[3];
      c1.getColorComponents(cc1);
      c2.getColorComponents(cc2);
      Color mix = new Color((float) (cc1[0] * rel + cc2[0] * (1 - rel)),
            (float) (cc1[1] * rel + cc2[1] * (1 - rel)),
            (float) (cc1[2] * rel + cc2[2] * (1 - rel)));
      return mix;
   }

   protected Symbol epsilon = new Symbol(
         (ch.ethz.exorciser.markov.grammar.Grammar) null, new Character(
               ch.ethz.exorciser.markov.grammar.Grammar.EPSILON),
         (SymbolList) null);

   public abstract void configureEditor (ProductionEditor ped);

   public abstract void paintSymbol (Symbol sym, Graphics2D g);

   public void updateSymListBounds (SymbolList syml, int x, int y) {
      syml.bounds.x = x;
      syml.bounds.y = y;
      int width = SymbolList.insets.left;
      int height = ProductionEditor.fm.getHeight();
      syml.openingBracketsX = x + width;
      if (syml.isVariableList()) {
         width += ProductionEditor.fm.stringWidth(Messages
               .getString("Markov.label.marker") //$NON-NLS-1$
               + " = {"); //$NON-NLS-1$ 
      } else if (syml.isTerminalList()) {
         width += ProductionEditor.fm.stringWidth(Messages
               .getString("Markov.label.alphabet") //$NON-NLS-1$
               + " = {"); //$NON-NLS-1$
      }
      Symbol current = null;
      for (int i = 0; i < syml.symbols.size(); i++) {
         current = (Symbol) syml.symbols.get(i);
         current.updateBounds(x + width, y + SymbolList.insets.top);
         width += current.getWidth() + syml.symbols_hgap;
         height = Math.max(height, current.getHeight());
      }
      if (syml.symbols.size() == 0) {
         width += syml.empty_hgap;
      }
      syml.closingBracketsX = x + width;
      width += ProductionEditor.fm.stringWidth("}"); //$NON-NLS-1$
      syml.bounds.width = width + SymbolList.insets.right - syml.symbols_hgap;
      syml.bounds.height = height + SymbolList.insets.top
            + SymbolList.insets.bottom;
      if (current != null) {
         syml.textLineY = current.getTextLineY();
      } else {
         syml.textLineY = y + SymbolList.insets.top + Symbol.insets.top
               + ProductionEditor.fm.getAscent();
      }
      syml.rrect.setFrame(syml.bounds);
   }

   public abstract void paintSymbolList (SymbolList syml, Graphics2D g);

   public abstract void paintProduction (ProductionUI prod, Graphics2D g);

   public abstract void paintInsertIcon (InsertProductionIcon icon, Graphics2D g);

   public abstract void paintProductionEditor (ProductionEditor ped,
         Graphics2D g);

   protected void paintArrow (Graphics2D g, ProductionUI prod, int x, int y) {
      int mid = (int) (y - ProductionEditor.fm.getAscent() * 0.35);
      int topx = x + prod.arrowW;
      Stroke backup = g.getStroke();
      g.setStroke(arrowStroke);
      g.drawLine(x, mid, topx, mid);
      g.drawLine(topx, mid, topx - 5, mid - 5);
      g.drawLine(topx, mid, topx - 5, mid + 5);
      g.setStroke(backup);
   }

   protected void paintStop (Graphics2D g, ProductionUI prod, int x, int y) {
      int mid = (int) (y - ProductionEditor.fm.getAscent() * 0.35);
      int topx = x + prod.arrowW;
      Stroke backup = g.getStroke();
      g.setStroke(arrowStroke);
      g.drawLine(x, mid, topx - 5, mid);
      g.drawLine(topx - 5, mid + 5, topx - 5, mid - 5);
      g.setStroke(backup);
   }
}