/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.markov.grammar.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.util.List;

/**
 * @author		Reto Lamprecht
 */
public class InsertProductionIcon {

	ProductionEditor prodEd;
	public InsertProductionIcon(ProductionEditor prodEd) {
		this.prodEd = prodEd;
	}

	private boolean visible = false;
	public void setVisible(boolean v) {
		visible = v;
	}
	public boolean isVisible() {
		return visible;
	}

	int x, y;
	int w = 14;
	int h = 18;
	int plusW = 3;
	int plusH = 3;
	/** distance from left edge of productions. */
	int hDist = 4;
	BasicStroke plusStroke = new BasicStroke(1.6f);

	double[] iconData = { w >> 1, 0, -w >> 1, -h >> 1, -w >> 1, h >> 1 };
	float[] res = new float[iconData.length];
	GeneralPath iconPath = new GeneralPath();
	AffineTransform at = new AffineTransform();

	public void setLocation(int index) {
		ProductionUI prod = null;
		List productions = prodEd.productions;
		if (productions.isEmpty()) {
			visible = false;
			return;
		}
		if (index < productions.size()) {
			prod = (ProductionUI) prodEd.productions.get(index);
			x = prod.getX() - hDist;
			y = prod.getY() - (prodEd.productions_vgap / 2);
		} else if (index == productions.size()) {
			prod = (ProductionUI) prodEd.productions.get(prodEd.productions.size()-1);
			x = prod.getX() - hDist;
			y = prod.getY() + prod.getHeight() + (prodEd.productions_vgap / 2);
		}
		at.setToIdentity();
		at.translate(x, y);
		at.transform(iconData, 0, res, 0, iconData.length >> 1);
		iconPath = makeIconPath(res);
	}
	public boolean contains(int x_, int y_) {
		return iconPath.contains(x_, y_);
	}
	public void paint(Graphics g_) {
		if (!visible)
			return;
		g_.setColor(Color.black);
		((Graphics2D) g_).fill(iconPath);
		g_.setColor(Color.white);
		((Graphics2D) g_).setStroke(plusStroke);
		int left = x - (w / 2) + 1; // one pixel from left edge;
		g_.drawLine(left, y, left + plusW * 2, y);
		g_.drawLine(left + plusW, y - plusH, left + plusW, y + plusH);

	}

	private GeneralPath makeIconPath(float[] coords) {
		iconPath.reset();
		iconPath.moveTo(coords[0], coords[1]);
		for (int i = 2; i < coords.length - 1; i += 2) {
			iconPath.lineTo(coords[i], coords[i + 1]);
		}
		iconPath.closePath();
		return iconPath;
	}
}