/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.grammar.view;

import java.awt.event.ActionEvent;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.GrammarChange;
import ch.ethz.exorciser.markov.grammar.GrammarIO;
import ch.ethz.exorciser.markov.grammar.Production;

/**
 * @author Reto Lamprecht
 */
public class Popups {

   public static JFileChooser fileChooser = new JFileChooser();

   private static String Variables = ""; //$NON-NLS-1$
   static {
      for (int i = 0; i < 25; i++) {
         if (i == 4) continue;
         Variables += (char) ('\u03b1' + i);
      }
   }

   public final static String Terminals = Messages.getString("Markov.symbols"); //$NON-NLS-1$

   protected ProductionEditor prodEd;

   protected JPopupMenu popup = new JPopupMenu();

   protected JMenu addVarMenu = new JMenu(Messages
         .getString("Markov.popup.add_marker")); //$NON-NLS-1$

   protected JMenu addTermMenu = new JMenu(Messages
         .getString("Markov.popup.add_symbol")); //$NON-NLS-1$

   protected JMenu renameVarMenu = new JMenu();

   protected JMenu renameTermMenu = new JMenu();

   /** An Action for each Symbol */
   protected Hashtable addActions;

   protected Hashtable renameActions;

   protected static final int RENAME = 200;

   protected static final int ADD_VAR = 201;

   protected static final int ADD_TERM = 202;

   protected VarRemoveAction varRemoveAction = new VarRemoveAction();

   protected TermRemoveAction termRemoveAction = new TermRemoveAction();

   protected CopyProdAction copyProdAction = new CopyProdAction();

   protected ProdRemoveAction removeProdAction = new ProdRemoveAction();

   protected StopProdAction stopAction = new StopProdAction();

   protected NonStopProdAction nonstopAction = new NonStopProdAction();

   public Popups (ProductionEditor prodEd) {
      this.prodEd = prodEd;
      initSymbolActions();
   }

   public void setProductionEditor (ProductionEditor prodEd) {
      this.prodEd = prodEd;
   }

   public JPopupMenu getVarListMenu () {
      popup.removeAll();
      popup.add(getVarMenu(addVarMenu, null));
      return popup;
   }

   public JPopupMenu getSingleVarMenu (Object variable) {
      popup.removeAll();
      if (this.prodEd.varList.size() < this.prodEd.MAX_NOF_VARS) {
         popup.add(getVarMenu(addVarMenu, (Object) null));
      }
      popup.add(varRemoveAction.setVariable(variable));
      popup.add(getVarMenu(renameVarMenu, variable));
      //renameVarMenu.setText("Rename '" + variable + "' to");
      renameVarMenu.setText(Messages.getString("Markov.popup.rename") //$NON-NLS-1$
            .replaceAll("\\$X", variable.toString())); //$NON-NLS-1$
      //		popup.add(setStartVarAction.setVariable(variable));
      return popup;
   }

   public JPopupMenu getTermListMenu () {
      popup.removeAll();
      popup.add(getTermMenu(addTermMenu, null));
      return popup;
   }

   public JPopupMenu getSingleTermMenu (Object terminal) {
      popup.removeAll();
      if (this.prodEd.termList.size() < this.prodEd.MAX_NOF_TERMS) {
         popup.add(getTermMenu(addTermMenu, (Object) null));
      }
      popup.add(termRemoveAction.setTerminal(terminal));
      popup.add(getTermMenu(renameTermMenu, terminal));
      renameTermMenu.setText(Messages.getString("Markov.popup.rename") //$NON-NLS-1$
            .replaceAll("\\$X", terminal.toString())); //$NON-NLS-1$
      //renameTermMenu.setText("Rename '" + terminal + "' to");
      return popup;
   }

   public JPopupMenu getProductionMenu (Production production) {
      popup.removeAll();
      if (production.hasAttribute("STOP")) { //$NON-NLS-1$
         nonstopAction.setProduction(production);
         popup.add(nonstopAction);
      } else {
         stopAction.setProduction(production);
         popup.add(stopAction);
      }
      copyProdAction.setProduction(production);
      popup.add(copyProdAction);
      removeProdAction.setProduction(production);
      popup.add(removeProdAction);
      return popup;
   }

   public JPopupMenu getDefaultMenu () {
      popup.removeAll();
      popup.add(new ClearAllAction(Messages
            .getString("Markov.popup.clear_schema"), this.prodEd//$NON-NLS-1$
            .getGrammar()));
      popup.add(new SaveToLaTexAction(Messages
            .getString("Markov.popup.export_to_latex"), this.prodEd//$NON-NLS-1$
            .getGrammar()));
      return popup;
   }

   // not used because of epsilon
   public JPopupMenu getInsertSymbolMenu (Symbol sym) {
      popup.removeAll();
      int symIndex = sym.getParentSymbolList().indexOf(sym);
      // upon a right click on the first symbol, provide option to add new
      // symbol in front...
      if (symIndex == 0) {
         createNewSides(sym, "", sym.getParentSymbolList().toString()); //$NON-NLS-1$
         popup.addSeparator();
      }
      // ... else add new symbol after clicked symbol.
      symIndex += 1;
      String oldSide = sym.getParentSymbolList().toString();
      String prefix = oldSide.substring(0, symIndex);
      String suffix = oldSide.substring(symIndex, oldSide.length());
      createNewSides(sym, prefix, suffix);
      return popup;
   }

   protected void createNewSides (Symbol sym, String prefix, String suffix) {
      boolean left = sym.getParentSymbolList().isLeft();
      Grammar grammar = prodEd.getGrammar();
      Production prod = sym.getParentProduction();
      List vtTerm = prodEd.getGrammar().getAlphabet();
      for (int i = 0; i < vtTerm.size(); i++) {
         String sTerm = vtTerm.get(i).toString();
         popup.add(new InsertSymbolAction(left, grammar, prod, prefix + sTerm
               + suffix));
      }
      popup.addSeparator();
      List vtVar = prodEd.getGrammar().getMarkers();
      for (int i = 0; i < vtVar.size(); i++) {
         String sVar = vtVar.get(i).toString();
         popup.add(new InsertSymbolAction(left, grammar, prod, prefix + sVar
               + suffix));
      }
   }

   /**
    * Adds Menus with max 10 Variables (Variables that are already added are not
    * shown) to the specified JMenu. If the JMenu is the member renameVarMenu,
    * the second Argument is the Symbol to be renamed
    */
   protected JMenu getVarMenu (JMenu menu, Object old) {
      menu.removeAll();
      List v = chopString(10, Variables);
      Iterator enVars = v.iterator();
      while (enVars.hasNext()) {
         JMenu jm = new JMenu();
         String s = (String) enVars.next();
         for (int i = 0; i < s.length(); i++) {
            String sTmp = s.substring(i, i + 1);
            if (!prodEd.getGrammar().isInMarkers(sTmp)) {
               if (menu == renameVarMenu) {
                  SymbolAction a = (SymbolAction) renameActions.get(sTmp);
                  a.setID(RENAME);
                  a.setOldSym(old);
                  jm.add(a);
               } else {
                  SymbolAction a = (SymbolAction) addActions.get(sTmp);
                  a.setID(ADD_VAR);
                  jm.add(a);
               }
            }
         }
         jm.setText(s.substring(0, 1) + " .. "//$NON-NLS-1$
               + s.substring(s.length() - 1, s.length()));
         menu.add(jm);
      }
      return menu;
   }

   /**
    * Adds Menus with max 10 Terminals (Terminals that are already added are not
    * shown) to the specified JMenu. If the JMenu is the member renameTermMenu,
    * the second Argument is the Symbol to be renamed
    */
   protected JMenu getTermMenu (JMenu menu, Object old) {
      menu.removeAll();
      List v = chopString(10, Terminals);
      Iterator enTerms = v.iterator();
      while (enTerms.hasNext()) {
         JMenu jm = new JMenu();
         String s = (String) enTerms.next();
         for (int i = 0; i < s.length(); i++) {
            String sTmp = s.substring(i, i + 1);
            if (!prodEd.getGrammar().isInAlphabeth(sTmp)) {
               if (menu == renameTermMenu) {
                  SymbolAction a = (SymbolAction) renameActions.get(sTmp);
                  a.setID(RENAME);
                  a.setOldSym(old);
                  jm.add(a);
               } else {
                  SymbolAction a = (SymbolAction) addActions.get(sTmp);
                  a.setID(ADD_TERM);
                  jm.add(a);
               }
            }
         }
         jm.setText(s.substring(0, 1) + " .. "//$NON-NLS-1$
               + s.substring(s.length() - 1, s.length()));
         menu.add(jm);
      }
      return menu;
   }

   /**
    * Chops the specified String into substrings of max. length len and returns
    * them stored in a Vector.
    */
   private List chopString (int len, String s) {
      List vt = new ArrayList();
      String sTmp = ""; //$NON-NLS-1$
      for (int j = 0; j < s.length(); j++) {
         sTmp += s.substring(j, j + 1);
         if (j % len == 0 && j != 0) {
            vt.add(sTmp);
            sTmp = ""; //$NON-NLS-1$
         }
      }
      if (sTmp.length() > 0) {
         vt.add(sTmp);
      }
      return vt;
   }

   protected void initSymbolActions () {
      renameActions = new Hashtable();
      addActions = new Hashtable();
      fillEmUp(Variables);
      fillEmUp(Terminals);
   }

   private void fillEmUp (String s) {
      for (int i = 0; i < s.length(); i++) {
         String sTmp = s.substring(i, i + 1);
         addActions.put(sTmp, new SymbolAction(sTmp));
         renameActions.put(sTmp, new SymbolAction(sTmp));
      }
   }

   /**
    * The Action used for Add and Rename of Symbols. They only get instantiatied
    * in initSymbolActions for all Variables and Terminals.
    */
   public class SymbolAction extends AbstractAction {

      int id = 0;

      Object s1 = null; // serves as new Object with rename.

      Object s2 = null; // serves as old Object with rename.

      public SymbolAction (Object s1) {
         super(s1.toString());
         this.s1 = s1;
      }

      public void setID (int id) {
         this.id = id;
      }

      public void setOldSym (Object o) {
         this.s2 = o;
      }

      public void actionPerformed (ActionEvent e) {
         switch (id) {
         case RENAME:
            prodEd.getGrammar()
                  .executeGrammarChange(
                        GrammarChange.createRenameSymbol(prodEd.getGrammar(),
                              s2, s1));
            break;
         case ADD_VAR:
            prodEd.getGrammar().executeGrammarChange(
                  GrammarChange.createAddVariable(prodEd.getGrammar(), s1));
            break;
         case ADD_TERM:
            prodEd.getGrammar().executeGrammarChange(
                  GrammarChange.createAddTerminal(prodEd.getGrammar(), s1));
            break;
         }
      }
   }

   public class VarRemoveAction extends AbstractAction {

      Object variable = null;

      public VarRemoveAction () {
         super(""); //$NON-NLS-1$
      }

      public Action setVariable (Object variable) {
         this.putValue(Action.NAME, Messages.getString("Markov.popup.remove") //$NON-NLS-1$
               .replaceAll("\\$X", //$NON-NLS-1$//$NON-NLS-2$
                     variable.toString()));
         this.variable = variable;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange
                     .createRemoveVariable(prodEd.getGrammar(), variable));
      }
   }

   public class TermRemoveAction extends AbstractAction {

      Object terminal = null;

      public TermRemoveAction () {
         super(""); //$NON-NLS-1$
      }

      public Action setTerminal (Object terminal) {
         this.putValue(Action.NAME, Messages.getString("Markov.popup.remove") //$NON-NLS-1$
               .replaceAll("\\$X", //$NON-NLS-1$//$NON-NLS-2$
                     terminal.toString()));
         this.terminal = terminal;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange
                     .createRemoveTerminal(prodEd.getGrammar(), terminal));
      }
   }

   public class ProdRemoveAction extends AbstractAction {

      Production production = null;

      public ProdRemoveAction () {
         super(Messages.getString("Markov.popup.delete_production")); //$NON-NLS-1$
      }

      public Action setProduction (Production p) {
         this.production = p;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange.createRemoveProduction(prodEd.getGrammar(),
                     production));
      }
   }

   /*
    * private class RenameAction extends AbstractAction{ Object oldSym = null;
    * Object newSym = null; public RenameAction(Object oldSym, Object newSym){
    * super(newSym.toString()); this.oldSym = oldSym; this.newSym = newSym; }
    * public void actionPerformed(ActionEvent e){
    * prodEd.getGrammar().executeGrammarChange(GrammarChange.createRenameSymbol(prodEd.getGrammar(),
    * oldSym, newSym)); } }
    */
   public class StopProdAction extends AbstractAction {

      Production production = null;

      public StopProdAction () {
         super(Messages.getString("Markov.popup.make_stop")); //$NON-NLS-1$
      }

      public Action setProduction (Production p) {
         this.production = p;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange.createSetStopProduction(prodEd.getGrammar(),
                     production));
      }
   }

   public class NonStopProdAction extends AbstractAction {

      Production production = null;

      public NonStopProdAction () {
         super(Messages.getString("Markov.popup.make_non_stop")); //$NON-NLS-1$
      }

      public Action setProduction (Production p) {
         this.production = p;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange.createSetNonStopProduction(prodEd.getGrammar(),
                     production));
      }
   }

   public class CopyProdAction extends AbstractAction {

      Production production = null;

      public CopyProdAction () {
         super(Messages.getString("Markov.popup.copy_production")); //$NON-NLS-1$
      }

      public Action setProduction (Production p) {
         this.production = p;
         return this;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange.createAddProduction(prodEd.getGrammar(),
                     (Production) production.clone(), prodEd
                           .getIndexOf(production)));
         prodEd.updateBounds();
         prodEd.repaint();
      }
   }

   public class ClearAllAction extends AbstractAction {

      Grammar grammar = null;

      public ClearAllAction (String msg, Grammar g) {
         super(msg);
         this.grammar = g;
      }

      public void actionPerformed (ActionEvent e) {
         prodEd.getGrammar().executeGrammarChange(
               GrammarChange.createClearAll(grammar));
         prodEd.updateBounds();
         prodEd.repaint();
      }
   }

   public class SaveToLaTexAction extends AbstractAction {

      Grammar grammar = null;

      public SaveToLaTexAction (String msg, Grammar g) {
         super(msg);
         this.grammar = g;
      }

      public void actionPerformed (ActionEvent e) {
         if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
            OutputStream os = null;
            try {
               os = new FileOutputStream(fileChooser.getSelectedFile());
               GrammarIO.saveGrammarToLatex(grammar, os);
               os.close();
            } catch (Exception ex) {
               ex.printStackTrace();
            } finally {
               if (os != null) {
                  try {
                     os.close();
                  } catch (IOException e1) {
                     e1.printStackTrace();
                  }
               }
            }
         }
      }
   }

   public class InsertSymbolAction extends AbstractAction {

      boolean left;

      Grammar grammar;

      Production prod;

      String newSide;

      public InsertSymbolAction (boolean left, Grammar grammar,
            Production prod, String newSide) {
         super(newSide);
         this.left = left;
         this.grammar = grammar;
         this.prod = prod;
         this.newSide = newSide;
      }

      public void actionPerformed (ActionEvent e) {
         if (left) {
            Popups.this.prodEd.getGrammar().executeGrammarChange(
                  GrammarChange.createChangeProductionLeft(grammar, prod,
                        newSide));
         } else {
            Popups.this.prodEd.getGrammar().executeGrammarChange(
                  GrammarChange.createChangeProductionRight(grammar, prod,
                        newSide));
         }
      }
   }
}