/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package ch.ethz.exorciser.markov.grammar.view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.GrammarChange;
import ch.ethz.exorciser.markov.grammar.GrammarListener;
import ch.ethz.exorciser.markov.grammar.Production;

/**
 * @author Reto Lamprecht
 */
public class ProductionEditor extends JPanel implements MouseInputListener,
		GrammarListener, Printable {

	private static Color dragProdColor = new Color(120, 120, 180);

	static float fontsize = 16f;

	static Font font;

	static FontMetrics fm;

	static boolean ANTIALIASED = true;
	static { // get suitable FontMetrics
		BufferedImage buf = new BufferedImage(10, 10,
				BufferedImage.TYPE_4BYTE_ABGR);
		Graphics2D g = (Graphics2D) buf.getGraphics();
		if (ANTIALIASED)
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
		font = g.getFont().deriveFont(fontsize);
		g.setFont(font);
		fm = g.getFontMetrics();
	}

	// ////////////////////////
	// Production Attributes //
	public static String ATTR_STOP = "STOP"; //$NON-NLS-1$

	public static String ATTR_MARK = "MARK"; //$NON-NLS-1$

	public static String ATTR_MARK_SINGLE_SYM = "MARK_SINGLE_SYM"; //$NON-NLS-1$

	// /////////////////////////
	// ////////////////////////////
	// Editability Configuration //
	public boolean VARLIST_EDITABLE = true;

	public boolean TERMLIST_EDITABLE = true;

	// public boolean SYMBOLS_DRAGABLE = false;
	public boolean PRODUCTIONS_DRAGABLE = true;

	public boolean PRODUCTIONS_EDITABLE = true;

	public boolean SHOW_VAR_LIST = true;

	public boolean SHOW_TERM_LIST = true;

	public int MAX_LEN_OF_PRODUCTION = Integer.parseInt(Messages
			.getString("MarkovMaxProductionLength")); //$NON-NLS-1$

	public int MAX_NOF_PRODUCTIONS = Integer.parseInt(Messages
			.getString("MarkovMaxNOfProductions")); //$NON-NLS-1$

	public int MAX_NOF_VARS = Integer.parseInt(Messages
			.getString("MarkovMaxNOfMarkers")); //$NON-NLS-1$

	public int MAX_NOF_TERMS = Integer.parseInt(Messages
			.getString("MarkovMaxNOfSymbols")); //$NON-NLS-1$

	// /////////////////////////////
	// /////////////////////
	// layout information //
	private Rectangle bounds = new Rectangle(0, 0, 1, 1);

	private Insets insets = new Insets(0, 0, 0, 0);

	Renderer renderer = new ContrastRenderer();

	// public Renderer metalRenderer = new MetalRenderer();
	// public Renderer contrastRenderer = new ContrastRenderer();
	int varList_termList_vgap = 3;

	int termList_productionList_vgap = 10;

	int productions_vgap = 8;

	// //////////////////////
	// //////////////
	// Drag & Drop //
	private Object hitObject;

	private Symbol draggedSymbol;

	private boolean paintDraggedSym = false;

	// false if dragged symbol temporary inserted somewhere.
	private ProductionUI draggedProduction;

	private ProductionUI ProdWhich_L_Dragged, ProdWhich_R_Dragged;

	private ProductionUI ProdWhich_L_Dropped, ProdWhich_R_Dropped;

	private int indexOfDraggedProd = -1;

	private int indexOfDroppedProd = -1;

	private ProductionUI insertedProduction;

	private int indexOfInsertedProd = -1;

	// ///////////////
	SymbolList varList;

	SymbolList termList;

	List productions = new ArrayList();

	private ProductionUI lastAdded, lastRemoved;

	// to avoid duplicate removes and adds
	public Object clone() {
		ProductionEditor clone = new ProductionEditor(grammar);
		return clone;
	}

	private Grammar grammar;

	public Grammar getGrammar() {
		return this.grammar;
	}

	public boolean FORCE_CONSTRAINTS = true;

	Popups popups;

	/**
	 * The method whoIs() of this inner class analyses an object. It's used in
	 * MouseListener methods.
	 */
	private class HitLocation {

		Symbol symbol;

		SymbolList symbolList;

		ProductionUI productionUI;

		Production production;

		private boolean bIsSymbol_;

		private boolean bIsSymbolList_;

		private boolean bIsProdUI_;

		private boolean bIsArrow_;

		private boolean bIsInsIcon_;

		boolean isSymbol() {
			return bIsSymbol_;
		}

		boolean isSymbolList() {
			return bIsSymbolList_;
		}

		boolean isProductionUI() {
			return bIsProdUI_;
		}

		boolean isArrow() {
			return bIsArrow_;
		}

		boolean isInsIcon() {
			return bIsInsIcon_;
		}

		boolean insideProduction() {
			return null != production;
		}

		boolean insideSymSelection() {
			return null != symbolList && null == production;
		}

		void whoIs(Object obj) {
			initFields();
			if (obj instanceof Symbol) {
				bIsSymbol_ = true;
				symbol = (Symbol) obj;
				symbolList = symbol.getParentSymbolList();
				productionUI = symbol.getParentProductionUI();
				production = symbol.getParentProduction();
			} else if (obj instanceof SymbolList) {
				bIsSymbolList_ = true;
				symbolList = (SymbolList) obj;
				productionUI = symbolList.getParentProductionUI();
				production = symbolList.getParentProduction();
			} else if (obj instanceof ProductionUI.Arrow) {
				bIsArrow_ = true;
				productionUI = ((ProductionUI.Arrow) obj).getProductionUI();
				production = ((ProductionUI.Arrow) obj).getProduction();
			} else if (obj instanceof InsertProductionIcon) {
				bIsInsIcon_ = true;
			} else if (obj instanceof ProductionUI) {
				bIsProdUI_ = true;
				productionUI = (ProductionUI) obj;
				production = ((ProductionUI) obj).getModel();
			}
		}

		private void initFields() {
			symbol = null;
			symbolList = null;
			productionUI = null;
			production = null;
			bIsSymbol_ = false;
			bIsSymbolList_ = false;
			bIsProdUI_ = false;
			bIsArrow_ = false;
			bIsInsIcon_ = false;
		}
	}

	private HitLocation hitLocation_ = new HitLocation();

	private InsertProductionIcon insIcon;

	public ProductionEditor(Grammar g) {
		addMouseListener(this);
		addMouseMotionListener(this);
		popups = new MarkovPopups(this);
		insIcon = new InsertProductionIcon(this);
		setBackground(Color.white);
		renderer.configureEditor(this);
		setGrammar(g);
	}

	/*
	 * public void disableChanges() { removeMouseListener(this);
	 * removeMouseMotionListener(this); // addMouseListener(disabledML);
	 * this.setToolTipText("Stop execution before changing the grammar!"); }
	 */
	/*
	 * public void enableChanges() { MouseListener ml[] = getMouseListeners();
	 * boolean isListener = false; for (int i = 0; i < ml.length; i++) { if
	 * (ml[i] == this) { isListener = true; break; } } if (!isListener) {
	 * addMouseListener(this); addMouseMotionListener(this); } //
	 * removeMouseListener(disabledML); this.setToolTipText(""); }
	 */
	public void setGrammar(Grammar grammar) {
		if (this.grammar != null) {
			this.grammar.removeGrammarListener(this);
		}
		grammar.addGrammarListener(this);
		this.grammar = grammar;
		varList = new SymbolList(grammar, SymbolList.VARIABLE_LIST, grammar
				.getMarkers());
		termList = new SymbolList(grammar, SymbolList.TERMINAL_LIST, grammar
				.getAlphabet());
		productions.clear();
		List prod = grammar.getProductions();
		for (int i = 0; i < prod.size(); i++) {
			productions
					.add(new ProductionUI(grammar, (Production) prod.get(i)));
		}
		if (MAX_NOF_VARS < grammar.getMarkers().size()) {
			MAX_NOF_VARS = grammar.getMarkers().size();
		}
		if (MAX_NOF_TERMS < grammar.getAlphabet().size()) {
			MAX_NOF_TERMS = grammar.getAlphabet().size();
		}
		if (MAX_NOF_PRODUCTIONS < prod.size()) {
			MAX_NOF_PRODUCTIONS = prod.size();
		}
		updateProdIndices();
		updateBounds();
		resized();
		repaint();
	}

	void updateBounds() {
		int x = 20, y = 20;
		int width = 0;
		int height = insets.top;
		if (SHOW_TERM_LIST) {
			termList.updateBounds(renderer, x + insets.left, y + height);
			width = Math.max(width, termList.getWidth());
			height += termList.getHeight() + varList_termList_vgap;
		}
		if (SHOW_VAR_LIST) {
			varList.updateBounds(renderer, x + insets.left, y + height);
			width = Math.max(width, varList.getWidth());
			height += varList.getHeight() + termList_productionList_vgap;
		}
		ProductionUI current;
		for (int i = 0; i < productions.size(); i++) {
			current = (ProductionUI) productions.get(i);
			current.updateBounds(renderer, x + insets.left, y + height);
			width = Math.max(width, current.getWidth());
			height += current.getHeight() + productions_vgap;
		}
		bounds.width = width + insets.left + insets.right + 30;
		bounds.height = height + insets.bottom + 20;
	}

	private boolean paintFirstTime = true;

	public void paintComponent(Graphics _g) {
		super.paintComponent(_g);
		Graphics2D g = (Graphics2D) _g;
		renderer.paintProductionEditor(this, g);
		if (paintFirstTime) {
			updateBounds();
			paintFirstTime = false;
		}
		if (SHOW_TERM_LIST)
			termList.paint(renderer, g);
		if (SHOW_VAR_LIST)
			varList.paint(renderer, g);
		ProductionUI current;
		for (int i = 0; i < productions.size(); i++) {
			current = (ProductionUI) productions.get(i);
			current.paint(renderer, g);
		}
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				0.70f));
		if (null != draggedSymbol && paintDraggedSym)
			draggedSymbol.paint(renderer, g);
		renderer.paintInsertIcon(insIcon, g);
	}

	private Object objectHitBy(int x, int y) {
		Object obj = varList.objectHitBy(x, y);
		if (null != obj) {
			return obj;
		}
		obj = termList.objectHitBy(x, y);
		if (null != obj) {
			return obj;
		}
		if (insIcon.contains(x, y)) {
			return insIcon;
		}
		ProductionUI current;
		for (int i = 0; i < productions.size(); i++) {
			current = (ProductionUI) productions.get(i);
			obj = current.objectHitBy(x, y);
			if (null != obj) {
				return obj;
			}
		}
		return this;
	}

	public Dimension getPreferredSize() {
		updateBounds();
		return new Dimension(bounds.width, bounds.height);
	}

	public int print(Graphics g, PageFormat p, int pageIndex) {
		if (pageIndex > 0) {
			return (NO_SUCH_PAGE);
		}
		Graphics2D g2D = (Graphics2D) g;
		g2D.translate(p.getImageableX(), p.getImageableY());
		this.paint(g2D);
		// g2D.drawString("Execution of the grammar:",0,0);
		return (PAGE_EXISTS);
	}

	public Dimension getMinimumSize() {
		updateBounds();
		return new Dimension(bounds.width, bounds.height);
	}

	public void notifyAnythingChanged() {
	}

	/**
	 * Grammar model must be updated already!
	 */
	public void notify(GrammarChange gc) {
		unmarkAllProductions();
		unmarkAllSymbols();
		// no constraint checks needed. grammar only propagates legal changes.
		switch (gc.getId()) {
		case GrammarChange.CHANGE_PRODUCTION_LEFT:
			getProductionUI(gc.getProduction()).setLeft(gc.getLeft());
			resized();
			break;
		case GrammarChange.CHANGE_PRODUCTION_RIGHT:
			getProductionUI(gc.getProduction()).setRight(gc.getRight());
			resized();
			break;
		case GrammarChange.ADD_PRODUCTION:
			if (!productions.contains(getProductionUI(gc.getProduction()))) {
				addProduction(gc.getProdIndex(), new ProductionUI(grammar, gc
						.getProduction()));
			}
			updateProdIndices();
			resized();
			break;
		case GrammarChange.REMOVE_PRODUCTION:
			removeProduction(getProductionUI(gc.getProduction()));
			updateProdIndices();
			resized();
			break;
		case GrammarChange.MOVE_PRODUCTION:
			break;
		case GrammarChange.CLEAR_ALL:
			productions.clear();
			resized();
			break;
		case GrammarChange.ADD_VAR:
			addVariable(gc.getVariable());
			break;
		case GrammarChange.REMOVE_VAR:
			removeVariable(gc.getVariable());
			resized();
			break;
		case GrammarChange.ADD_TERM:
			addTerminal(gc.getTerminal());
			break;
		case GrammarChange.REMOVE_TERM:
			removeTerminal(gc.getTerminal());
			resized();
			break;
		case GrammarChange.RENAME_SYM:
			renameSym(gc.getOldSym(), gc.getNewSym());
			break;
		case GrammarChange.SET_STOP_PRODUCTION:
			// do nothing
			break;
		case GrammarChange.SET_NONSTOP_PRODUCTION:
			// do nothing
			break;
		default:
			break;
		}
		updateBounds();
		repaint();
	}

	private void addVariable(Object o) {
		varList.addSymbol(new Symbol(grammar, o, varList));
	}

	private void removeVariable(Object o) {
		varList.removeObject(o);
		// updateProductionUIFromModel();
		removeSymbolFromProductions(o);
	}

	private void addTerminal(Object o) {
		termList.addSymbol(new Symbol(grammar, o, termList));
	}

	private void removeTerminal(Object o) {
		termList.removeObject(o);
		removeSymbolFromProductions(o);
		// updateProductionUIFromModel();
	}

	private void renameSym(Object oldS, Object newS) {
		varList.renameObject(oldS, newS);
		termList.renameObject(oldS, newS);
		renameSymbolsInProductions(oldS, newS);
	}

	/**
	 * Renames all occurrences of oldO with newO in ProductionUIs.
	 */
	private void renameSymbolsInProductions(Object oldS, Object newS) {
		for (int i = 0; i < productions.size(); i++) {
			ProductionUI current = (ProductionUI) productions.get(i);
			current.renameOccurrencesOf(oldS, newS);
		}
	}

	private void removeSymbolFromProductions(Object o) {
		for (int i = 0; i < productions.size(); i++) {
			ProductionUI current = (ProductionUI) productions.get(i);
			current.removeOccurrencesOf(o);
		}
		removeEmptyProductions();
	}

	private void removeProduction(ProductionUI prod) {
		if (prod == lastRemoved) {
			return;
		}
		if (productions.remove(prod)) {
			lastRemoved = prod;
			lastAdded = null;
		}
	}

	private void addProduction(int i, ProductionUI prod) {
		if (prod == lastAdded) {
			return;
		}
		productions.add(i, prod);
		updateProdIndices();
		lastAdded = prod;
		lastRemoved = null;
	}

	public void removeEmptyProductions() {
		for (int i = 0; i < productions.size(); i++) {
			ProductionUI current = (ProductionUI) productions.get(i);
			if ("".equals(current.getLeftFromView()) //$NON-NLS-1$
					&& "".equals(current.getRightFromView())) { //$NON-NLS-1$
				grammar.executeGrammarChange(GrammarChange
						.createRemoveProduction(getGrammar(), current
								.getModel()));
				i--;
			}
		}
	}

	public void resized() {
		updateBounds();
		this.setPreferredSize(new Dimension(bounds.width, bounds.height));
		this.setMinimumSize(getPreferredSize());
		if (getRootPane() != null) {
			((JComponent) getParent()).revalidate();
			((JComponent) getParent().getParent()).revalidate();
			getRootPane().repaint();
		}
	}

	public Dimension getSize() {
		updateBounds();
		return new Dimension(bounds.width, bounds.height);
	}

	private boolean ANYTHING_MARKED = false;

	public void markProduction(int i, Color markColor) {
		if (i > productions.size())
			return;
		markProduction((ProductionUI) productions.get(i), markColor);
		ANYTHING_MARKED = true;
		repaint();
	}

	public void markProduction(ProductionUI production, Color markColor) {
		if (markColor == null) {
			production.getModel().removeAttribute(ATTR_MARK);
		} else {
			production.getModel().setAttribute(ATTR_MARK, markColor);
			ANYTHING_MARKED = true;
		}
		repaint();
	}

	public void unmarkAllProductions() {
		for (int i = 0; i < productions.size(); i++) {
			Production pui = ((ProductionUI) productions.get(i)).getModel();
			pui.removeAttribute(ATTR_MARK);
		}
		ANYTHING_MARKED = false;
	}

	public void markSingleSymbol(int prodIndex, int symIndex, Color markColor) {
		if (prodIndex > productions.size())
			return;
		Production pui = ((ProductionUI) productions.get(prodIndex)).getModel();
		List value = new ArrayList();
		value.add(0, new Integer(symIndex));
		value.add(1, markColor);
		pui.setAttribute(ATTR_MARK_SINGLE_SYM, value);
		ANYTHING_MARKED = true;
		repaint();
	}

	public void unmarkAllSymbols() {
		for (int i = 0; i < productions.size(); i++) {
			Production pui = ((ProductionUI) productions.get(i)).getModel();
			pui.removeAttribute(ATTR_MARK_SINGLE_SYM);
		}
		ANYTHING_MARKED = false;
	}

	private ProductionUI getProductionUI(Production prod) {
		for (int i = 0; i < productions.size(); i++) {
			ProductionUI current = (ProductionUI) productions.get(i);
			if (current.getModel() == prod) {
				return current;
			}
		}
		return null;
	}

	private void updateProdIndices() {
		for (int i = 0; i < productions.size(); i++) {
			((ProductionUI) productions.get(i)).index = i;
		}
	}

	private int indexAt(int y) {
		ProductionUI current = null;
		for (int i = 0; i < productions.size(); i++) {
			current = (ProductionUI) productions.get(i);
			if (y < current.getY() + current.getHeight())
				return i;
		}
		return productions.size();
	}

	public int getIndexOf(Production prod) {
		return getProductionUI(prod).index;
	}

	// ///////////////////////////////////
	// MouseMotionListener methods //
	// ///////////////////////////////////
	public void mouseMoved(MouseEvent e) {
	}

	private boolean PRODUCTION_COPIED = false;

	private int lastHandledX, lastHandledY;

	public void mousePressed(MouseEvent e) {
		lastHandledX = e.getX();
		lastHandledY = e.getY();
		if (ANYTHING_MARKED) {
			unmarkAllProductions();
			unmarkAllSymbols();
		}
		// //////////////
		// right click //
		if ((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0) { // ||
			hitObject = objectHitBy(e.getX(), e.getY());
			hitLocation_.whoIs(hitObject);
			if (hitLocation_.insideSymSelection()) {

				if (hitLocation_.symbolList == varList && VARLIST_EDITABLE
				// && varList.size() < MAX_NOF_VARS
				) {
					if (hitLocation_.isSymbol()) {
						popups.getSingleVarMenu(hitLocation_.symbol.getModel())
								.show(this, e.getX(), e.getY());
					} else {
						popups.getVarListMenu().show(this, e.getX(), e.getY());
					}
				} else if (hitLocation_.symbolList == termList
						&& TERMLIST_EDITABLE
				// && termList.size() < MAX_NOF_TERMS
				) {
					if (hitLocation_.isSymbol()) {
						popups
								.getSingleTermMenu(
										hitLocation_.symbol.getModel()).show(
										this, e.getX(), e.getY());
					} else {
						popups.getTermListMenu().show(this, e.getX(), e.getY());
					}
				}
			} else if (hitLocation_.insideProduction() && PRODUCTIONS_EDITABLE) {
				popups.getProductionMenu(hitLocation_.production).show(this,
						e.getX(), e.getY());
			} else if (hitObject instanceof ProductionEditor) {
				popups.getDefaultMenu().show(this, e.getX(), e.getY());
			}
		}
		// /////////////
		// left click //
		else {
			hitObject = objectHitBy(e.getX(), e.getY());
			if (hitObject instanceof Symbol && PRODUCTIONS_EDITABLE) { // SYMBOLS_DRAGABLE)
				// {
				draggedSymbol = (Symbol) hitObject;
				if (draggedSymbol.belongsToVariableList()
						|| draggedSymbol.belongsToTerminalList()
						|| (e.getModifiers() & InputEvent.CTRL_MASK) > 0) {
					draggedSymbol = (Symbol) draggedSymbol.clone();
				} else {
					if (FORCE_CONSTRAINTS) {
						int index = draggedSymbol.getParentSymbolList().symbols
								.indexOf(draggedSymbol);
						draggedSymbol.getParentSymbolList().removeSymbol(
								draggedSymbol);
						draggedSymbol.getParentSymbolList().addSymbol(index,
								draggedSymbol);
					}
					if (draggedSymbol.belongsToLeftSide()) {
						ProdWhich_L_Dragged = draggedSymbol
								.getParentSymbolList().getParentProductionUI();
					}
					if (draggedSymbol.belongsToRightSide()) {
						ProdWhich_R_Dragged = draggedSymbol
								.getParentSymbolList().getParentProductionUI();
					}
				}
				draggedSymbol.grab(e.getX(), e.getY());
				draggedSymbol.drag(true);
				paintDraggedSym = true;
				draggedSymbol.setLocation(e.getX(), e.getY());
			} else if ((hitObject instanceof ProductionUI || (hitObject instanceof SymbolList
					&& (SymbolList) hitObject != varList && (SymbolList) hitObject != termList))
					&& PRODUCTIONS_DRAGABLE) {
				if (hitObject instanceof ProductionUI) {
					draggedProduction = (ProductionUI) hitObject;
				} else if (hitObject instanceof SymbolList) {
					draggedProduction = ((SymbolList) hitObject)
							.getParentProductionUI();
				}
				indexOfDraggedProd = productions.indexOf(draggedProduction);
				if ((e.getModifiers() & InputEvent.CTRL_MASK) > 0
						&& productions.size() < MAX_NOF_PRODUCTIONS) {
					draggedProduction = (ProductionUI) draggedProduction
							.clone();
					addProduction(indexOfDraggedProd, draggedProduction);
					updateBounds();
					PRODUCTION_COPIED = true;
				}
				draggedProduction.drag(true);
				markProduction(draggedProduction, dragProdColor);
			}
		}
		repaint();
	}

	Object lastHitObject;

	public void mouseDragged(MouseEvent e) {
		lastHandledX = e.getX();
		lastHandledY = e.getY();
		hitObject = objectHitBy(e.getX(), e.getY());
		hitLocation_.whoIs(hitObject);
		if (draggedSymbol != null && hitObject != draggedSymbol) {
			handleSymbolDrag(e);
		} else if (hitObject != lastHitObject) {
			lastHitObject = hitObject;
			if (draggedProduction != null
					&& hitLocation_.productionUI != draggedProduction) {
				removeProduction(draggedProduction);
				updateBounds();
				indexOfDroppedProd = indexAt(e.getY());
				addProduction(indexOfDroppedProd, draggedProduction);
				updateBounds();
			}
		}
		repaint();
		this.scrollRectToVisible(new Rectangle(new Point(e.getX(),
				e.getY() + 20)));
	}

	public void mouseReleased(MouseEvent e) {
		if (Math.abs(lastHandledX - e.getX()) > 4
				|| Math.abs(lastHandledY - e.getY()) > 4) {
			mouseDragged(e);
		}
		insIcon.setVisible(false);
		if (insertedProduction != null) {
			// why this check?? && indexOfInsertedProd > 0) {
			if ("".equals(insertedProduction.getLeftFromView()) //$NON-NLS-1$
					&& "".equals(insertedProduction.getRightFromView())) { //$NON-NLS-1$
				removeProduction(insertedProduction);
			} else {
				getGrammar().executeGrammarChange(
						GrammarChange.createAddProduction(getGrammar(),
								insertedProduction.getModel(),
								indexOfInsertedProd));
			}
			insertedProduction = null;
			indexOfInsertedProd = -1;
		}
		// /////////////////////////
		// handle Symbol DragDrop //
		if (ProdWhich_L_Dropped != null) {
			if (ProdWhich_L_Dropped == ProdWhich_L_Dragged
					|| ProdWhich_L_Dropped == ProdWhich_R_Dragged) {
				// dropped into same production?
				getGrammar().executeGrammarChange(
						GrammarChange.createChangeProduction(getGrammar(),
								ProdWhich_L_Dropped.getModel(),
								ProdWhich_L_Dropped.getLeftFromView(),
								ProdWhich_L_Dropped.getRightFromView()));
				ProdWhich_L_Dragged = null;
				ProdWhich_R_Dragged = null;
			} else {
				getGrammar().executeGrammarChange(
						GrammarChange.createChangeProductionLeft(getGrammar(),
								ProdWhich_L_Dropped.getModel(),
								ProdWhich_L_Dropped.getLeftFromView()));
			}
			ProdWhich_L_Dropped = null;
		}
		if (ProdWhich_R_Dropped != null) {
			if (ProdWhich_R_Dropped == ProdWhich_L_Dragged
					|| ProdWhich_R_Dropped == ProdWhich_R_Dragged) {
				// dropped into same production?
				getGrammar().executeGrammarChange(
						GrammarChange.createChangeProduction(getGrammar(),
								ProdWhich_R_Dropped.getModel(),
								ProdWhich_R_Dropped.getLeftFromView(),
								ProdWhich_R_Dropped.getRightFromView()));
				ProdWhich_L_Dragged = null;
				ProdWhich_R_Dragged = null;
			} else {
				getGrammar().executeGrammarChange(
						GrammarChange.createChangeProductionRight(getGrammar(),
								ProdWhich_R_Dropped.getModel(),
								ProdWhich_R_Dropped.getRightFromView()));
			}
			ProdWhich_R_Dropped = null;
		}
		if (ProdWhich_L_Dragged != null) {
			getGrammar().executeGrammarChange(
					GrammarChange.createChangeProductionLeft(getGrammar(),
							ProdWhich_L_Dragged.getModel(), ProdWhich_L_Dragged
									.getLeftFromView()));
			ProdWhich_L_Dragged = null;
		}
		if (ProdWhich_R_Dragged != null) {
			getGrammar().executeGrammarChange(
					GrammarChange.createChangeProductionRight(getGrammar(),
							ProdWhich_R_Dragged.getModel(), ProdWhich_R_Dragged
									.getRightFromView()));
			ProdWhich_R_Dragged = null;
		}
		if (draggedSymbol != null)
			draggedSymbol.drag(false);
		draggedSymbol = null;
		// /////////////////////////////
		// handle Production DragDrop //
		if (PRODUCTION_COPIED) {
			getGrammar().executeGrammarChange(
					GrammarChange.createAddProduction(getGrammar(),
							draggedProduction.getModel(), productions
									.indexOf(draggedProduction)));
			PRODUCTION_COPIED = false;
		}
		if (indexOfDraggedProd != -1 && indexOfDroppedProd != -1) {
			getGrammar().executeGrammarChange(
					GrammarChange.createMoveProduction(grammar,
							draggedProduction.getModel(), indexOfDraggedProd,
							indexOfDroppedProd));
			updateProdIndices();
			indexOfDraggedProd = -1;
			indexOfDroppedProd = -1;
		}
		if (draggedProduction != null) {
			draggedProduction.drag(false);
			markProduction(draggedProduction, null);
			draggedProduction = null;
		}
		updateBounds();
		repaint();
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseClicked(MouseEvent e) {
		if ((e.getModifiers() & InputEvent.BUTTON1_MASK) > 0) {
			hitObject = objectHitBy(e.getX(), e.getY());
			if (hitObject instanceof ProductionUI.Arrow) {
				if (((ProductionUI.Arrow) hitObject).isStop()) {
					grammar.executeGrammarChange(GrammarChange
							.createSetNonStopProduction(grammar,
									((ProductionUI.Arrow) hitObject)
											.getProduction()));
				} else {
					grammar.executeGrammarChange(GrammarChange
							.createSetStopProduction(grammar,
									((ProductionUI.Arrow) hitObject)
											.getProduction()));
				}
			}
		}
	}

	private void handleSymbolDrag(MouseEvent e) {
		// duplicate removes don't harm.
		draggedSymbol.getParentSymbolList().removeSymbol(draggedSymbol);
		paintDraggedSym = true;
		ProdWhich_R_Dropped = null;
		ProdWhich_L_Dropped = null;
		updateBounds();
		draggedSymbol.setLocation(e.getX(), e.getY());
		// insIcon.setVisible(false);
		if (hitObject != null) {
			hitLocation_.whoIs(hitObject);
			if (productions.size() < MAX_NOF_PRODUCTIONS // &&
															// indexAt(e.getY())
					// ==
					// productions.size()
					&& insertedProduction == null) {
				if (indexAt(e.getY()) == productions.size()) {
					insertEmptyProduction_(indexAt(e.getY()));
				} else {
					updateBounds();
					insIcon.setLocation(indexAt(e.getY())); // indexOfInsertedProd);
					insIcon.setVisible(true);
					repaint();
					// return;
				}
			}
			// ////////////////////////////////
			// Drag Symbol over other Symbol //
			if ((hitLocation_.isSymbol() || hitLocation_.isSymbolList())
					&& !hitLocation_.insideSymSelection()) {
				hitLocation_.symbolList.addSymbol(hitLocation_.symbolList
						.indexAt(e.getX()), draggedSymbol);
				// ///////////////////////////////////////
				// Check Constraints: add Symbol legal? //
				paintDraggedSym = false;
				updateBounds();
				if (hitLocation_.symbolList.isLeft()) {
					ProdWhich_L_Dropped = hitLocation_.productionUI;
				} else if (hitLocation_.symbolList.isRight()) {
					ProdWhich_R_Dropped = hitLocation_.productionUI;
				}
			} else if (hitLocation_.isInsIcon() && insertedProduction == null) {
				insertEmptyProduction_(indexAt(e.getY()));
			}
			// ///////////////////////////////
			// Drag Symbol over empty space //
			else if (hitObject instanceof ProductionEditor) {
				/*
				 * // add empty production at the bottom bzw. inbetween if
				 * (productions.size() < MAX_NOF_PRODUCTIONS //&&
				 * indexAt(e.getY()) == productions.size() && insertedProduction ==
				 * null) { if (indexAt(e.getY()) == productions.size()) {
				 * insertEmptyProduction_(indexAt(e.getY())); } else {
				 * updateBounds(); insIcon.setLocation(indexAt(e.getY()));
				 * //indexOfInsertedProd); insIcon.setVisible(true); repaint();
				 * return; } }
				 */
			}
			// remove inserted Production if necessary
			// if (indexAt(e.getY()) < indexOfInsertedProd && insertedProduction
			// != null) {
			if (insertedProduction != null
					&& hitLocation_.productionUI != insertedProduction
					&& hitObject != insIcon
					&& indexAt(e.getY()) < productions.size()
					&& indexAt(e.getY()) != insertedProduction.index) {
				removeProduction(insertedProduction);
				insertedProduction = null;
				indexOfInsertedProd = -1;
				updateBounds();
			}
		}
	}

	private void insertEmptyProduction_(int index) {
		insIcon.setVisible(false);
		insertedProduction = new ProductionUI(grammar, new Production(" -> ")); //$NON-NLS-1$
		indexOfInsertedProd = index;
		// indexOfInsertedProd = productions.size();
		addProduction(indexOfInsertedProd, insertedProduction);
		updateBounds();
		repaint();
	}

	class DisabledML implements MouseListener {

		public void mouseClicked(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}
	}
}