/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.grammar.view;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.RoundRectangle2D;

import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.Production;

/**
 * this class displays an object of a grammar in a symbollist.
 * 
 * @author Reto Lamprecht
 */
public class Symbol {

	boolean dragged;

	public void drag(boolean b) {
		dragged = b;
	}

	/**
	 * insets between symbolbounds and bounds of parent symbollist.
	 */
	static Insets insets = new Insets(0, 3, 0, 2);

	static int roundness = 5;

	RoundRectangle2D rrect = new RoundRectangle2D.Double(0, 0, 1, 1, roundness,
			roundness);

	/**
	 * this Rectangle contains position and size of this symbol, calculated in
	 * method updateBounds().
	 */
	Rectangle bounds = new Rectangle(0, 0, 1, 1);

	public int getWidth() {
		return bounds.width;
	}

	public int getHeight() {
		return bounds.height;
	}

	public int getX() {
		return bounds.x;
	}

	public int getY() {
		return bounds.y;
	}

	// public void setPosition(int x, int y) {}
	/**
	 * y-coord of the text in this symbol. used in ProductionUI to align index
	 * and arrow (->).
	 */
	private int textLineY;

	public int getTextLineY() {
		return textLineY;
	}

	/**
	 * stores the mouse position relative to this symbolbounds where this symbol
	 * was grabbed.
	 */
	private Point mouseRef = new Point(0, 0);

	public Point getMouseRef() {
		return mouseRef;
	}

	public void grab(int x, int y) {
		mouseRef.setLocation(x - bounds.x, y - bounds.y);
	}

	/**
	 * the grammar this symbol belongs to. used in method paint() to determine
	 * if its the startvariable.
	 */
	private Grammar grammar;

	void setGrammar(Grammar g) {
		this.grammar = g;
	}

	/**
	 * the symbollist this symbol belongs to.
	 */
	private SymbolList parent;

	public SymbolList getParentSymbolList() {
		return parent;
	}

	public void setParentSymbolList(SymbolList s) {
		this.parent = s;
	}

	/**
	 * the object this symbol displays.
	 */
	private Object model = ""; //$NON-NLS-1$

	public Object getModel() {
		return model;
	}

	void setModel(Object o) {
		this.model = o;
	}

	public Symbol(Grammar grammar, Object model, SymbolList symbolList) {
		this.grammar = grammar;
		this.model = model;
		this.parent = symbolList;
	}

	void setLocation(int x, int y) {
		updateBounds(x - mouseRef.x, y - mouseRef.y);
	}

	void updateBounds(int x, int y) {
		bounds.x = x;
		bounds.y = y;
		String asString = model.toString();
		FontMetrics fm = ProductionEditor.fm;
		bounds.width = insets.left + fm.stringWidth(asString) + insets.right;
		bounds.height = insets.top + fm.getHeight() + insets.bottom;
		textLineY = y + insets.top + fm.getAscent();
		rrect.setFrame(bounds);
	}

	public void paint(Renderer r, Graphics2D g) {
		r.paintSymbol(this, g);
	}

	public boolean contains(int x, int y) {
		return bounds.contains(x, y);
	}

	public boolean belongsToLeftSide() {
		return parent.isLeft();
	}

	public boolean belongsToRightSide() {
		return parent.isRight();
	}

	public boolean belongsToVariableList() {
		return parent.isVariableList();
	}

	public boolean belongsToTerminalList() {
		return parent.isTerminalList();
	}

	// public boolean isStart() {
	// if (null == grammar)
	// return false;
	// else
	// return (grammar.getStartVariable().equals(this.model));
	// }
	public boolean isVar() {
		if (null == grammar) {
			return false;
		}
		return grammar.isInMarkers(this.model);
	}

	public ProductionUI getParentProductionUI() {
		return this.parent.getParentProductionUI();
	}

	public Production getParentProduction() {
		if (null != parent && null != parent.getParentProductionUI()) {
			return this.parent.getParentProductionUI().getModel();
		}
		return null;
	}

	/**
	 * Returns the index of this symbol in its parent production, counted from
	 * left to right.
	 */
	public int getIndexInProduction() {
		if (parent.isLeft()) {
			return parent.indexOf(this);
		}
		return getParentProduction().getLeft().length + parent.indexOf(this);
	}

	public String toString() {
		return model.toString();
	}

	public Object clone() {
		Symbol clone = new Symbol(grammar, model, parent);
		clone.bounds.setBounds(bounds.x, bounds.y, bounds.width, bounds.height);
		return clone;
	}
	/*
	 * public boolean equals(Object obj) { if (!(obj instanceof Symbol)) {
	 * return super.equals(obj); } else return ((Symbol)
	 * obj).getModel().equals(model); }
	 */
}