/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.runtime;

import java.util.List;

import ch.ethz.exorciser.markov.grammar.Grammar;
import ch.ethz.exorciser.markov.grammar.Production;

// does the execution of a grammar for the background checker
public class CheckerExecution {

   private List productions;

   private String string;

   private Constraints constraints;

   private int stepLimit;

   private int stringLimit;

   private boolean finished = false;

   private boolean stop;

   //	private boolean step;
   private int prodNr = 0;

   private int stepCount = 0;

   public CheckerExecution (Grammar grammar, String input, Constraints constr) {
      this.string = input;
      this.constraints = constr;
      productions = grammar.getProductions();
      stringLimit = this.constraints.getRelativeStringLimit(string.length());
      stepLimit = this.constraints.getRelativeStepLimit(string.length());
   }

   public String run () {
      while (!finished && !stop) {
         int pos;
         if (stepCount > stepLimit) {
            stop = true;
         } else if (string.length() > stringLimit) {
            stop = true;
         } else {
            pos = applicable(prodNr);
            if (pos > -1) {
               applicateProduction(prodNr, pos);
               if (((Production) productions.get(prodNr)).hasAttribute("STOP")) { //$NON-NLS-1$
                  finished = true;
                  stop = true;
               }
               prodNr = 0;
               stepCount++;
            } else {
               prodNr++; // nchste Produktion versuchen
            }
            if (prodNr >= productions.size()) {
               finished = true;
               stop = true;
            }
         }
      }
      return string;
   }

   // checks if there is a match for the pattern on the
   // lefthand side of the actual production
   // in the string
   private int applicable (int productionNumber) {
      if (productions.size() > productionNumber) {
         int i = string
               .indexOf(((Production) productions.get(productionNumber))
                     .leftAsString());
         return i;
      }
	return -1;
   }

   private void applicateProduction (int productionNumber, int pos) {
      String leftPart, rightPart, oldMiddlePart, newMiddlePart;
      oldMiddlePart = ((Production) productions.get(productionNumber))
            .leftAsString();
      newMiddlePart = ((Production) productions.get(productionNumber))
            .rightAsString();
      leftPart = string.substring(0, pos);
      rightPart = string.substring(pos + oldMiddlePart.length(), string
            .length());
      string = leftPart.concat(newMiddlePart.concat(rightPart));
   }
}