/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.runtime;

import ch.ethz.exorciser.Messages;

/**
 * @author Thomas Briner
 */
public class Constraints {

   public int[] relativeStepLimit = { Integer.parseInt(Messages.getString("Markov.RelativeStepLimit"))}; //$NON-NLS-1$

   public int[] relativeStringLimit = { Integer.parseInt(Messages.getString("Markov.RelativeStringLimit"))}; //$NON-NLS-1$

   public final int getRelativeStepLimit (int l) {
      int v = relativeStepLimit[0];
      int p = l;
      int i = 1;
      while (i < relativeStepLimit.length) {
         v += p * relativeStepLimit[i];
         p *= l;
         i++;
      }
      return v;
   }

   public final int getRelativeStringLimit (int l) {
      int v = relativeStringLimit[0];
      int p = l;
      int i = 1;
      while (i < relativeStringLimit.length) {
         v += p * relativeStringLimit[i];
         p *= l;
         i++;
      }
      return v;
   }

   public final String getRelativeStepLimitAsHTML () {
      String v = ""; //$NON-NLS-1$
      if (relativeStepLimit[0] != 0) {
         v += relativeStepLimit[0];
      }
      int i = 1;
      while (i < relativeStepLimit.length) {
         if (relativeStepLimit[i] != 0) {
            if (!v.equals("")) { //$NON-NLS-1$
               v = " + " + v; //$NON-NLS-1$
            }
            v = ((i != 1) ? ("n<sup>" + i + "</sup>") : ("n")) + v; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            v = ((relativeStepLimit[i] != 1) ? (relativeStepLimit[i] + "") //$NON-NLS-1$
                  : ("")) //$NON-NLS-1$
                  + v;
         }
         i++;
      }
      return v;
   }

   public final String getRelativeStringLimitAsHTML () {
      String v = ""; //$NON-NLS-1$
      if (relativeStringLimit[0] != 0) {
         v += relativeStringLimit[0];
      }
      int i = 1;
      while (i < relativeStringLimit.length) {
         if (relativeStringLimit[i] != 0) {
            if (!v.equals("")) { //$NON-NLS-1$
               v = " + " + v; //$NON-NLS-1$
            }
            v = ((i != 1) ? ("n<sup>" + i + "</sup>") : ("n")) + v; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            v = ((relativeStringLimit[i] != 1) ? (relativeStringLimit[i] + "") //$NON-NLS-1$
                  : ("")) //$NON-NLS-1$
                  + v; //$NON-NLS-1$ //$NON-NLS-2$
         }
         i++;
      }
      return v;
   }
   //	public String toString() {
   //		String constr = "stepLimit ";
   //		for (int i=0;i<relativeStepLimit.length;i++){
   //			constr = constr + relativeStepLimit[i];
   //		}
   //		constr = constr + " stringLimit ";
   //		for (int i=0;i<relativeStringLimit.length;i++){
   //			constr = constr + relativeStringLimit[i];
   //		}
   //		
   //		return constr;
   //	}
}