/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.runtime.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;

import ch.ethz.exorciser.markov.runtime.Visualisation;

/**
 * @author Thomas Briner
 * @author Daniel Mller
 */
public class ApplicationPanel extends JPanel implements Visualisation.Listener,
      Printable {

   private List vLine = new ArrayList();

   int maxlength = 0;

   private Renderer renderer;

   private Visualisation s;

   private Dimension dimPref, dimReal;

   public ApplicationPanel (Visualisation s) {
      this.s = s;
      s.addSequenceListener(this);
      renderer = new ContrastRenderer();
      setBackground(Color.white);
      setFont(Renderer.PLAIN_FONT);
   }

   public void setVisualisation (Visualisation s) {
      this.s = s;
   }

   private ApplicationPanel () {}

   public ApplicationPanel (List lines) {
      vLine = lines;
      renderer = new ContrastRenderer();
      setBackground(Color.white);
      setFont(Renderer.PLAIN_FONT);
   }

   public Object clone () {
      ApplicationPanel clone = new ApplicationPanel();
      clone.vLine = new ArrayList();
      for (int i = 0; i < this.vLine.size(); i++) {
         clone.vLine.add(this.vLine.get(i));
      }
      clone.setBackground(Color.white);
      clone.renderer = new ContrastRenderer();
      clone.setFont(Renderer.PLAIN_FONT);
      clone.maxlength = maxlength;
      return clone;
   }

   public List getLines () {
      return vLine;
   }

   public void handleReset () {
      vLine.clear();
      maxlength = 0;
      handleNewLine(s.getActString());
   }

   public void handleNewLine (String str) {
      maxlength = Math.max(maxlength, str.length());
      if (vLine.size() > 0) {
         vLine.add(new Line(str, vLine.size(), (Line) vLine
               .get(vLine.size() - 1)));
      } else {
         vLine.add(new Line(str, 0, null));
      }
      resized();
   }

   public void resized () {
      this.setPreferredSize(new Dimension((maxlength) * Renderer.X_WIDTH + 60,
            vLine.size() * Renderer.Y_WIDTH));
      this.setMinimumSize(getPreferredSize());
      if (getRootPane() != null) {
         ((JComponent) getParent()).revalidate();
         ((JComponent) getParent().getParent()).revalidate();
         getRootPane().repaint();
      }
   }

   public Dimension getSizeForPrint () {
      return new Dimension((maxlength) * Renderer.X_WIDTH + 60, vLine.size()
            * Renderer.Y_WIDTH);
   }

   public void paintComponent (Graphics _g) {
      super.paintComponent(_g);
      Graphics2D g = (Graphics2D) _g;
      renderer.paintPanel(this, g);
      for (int i = 0; i < vLine.size(); i++) {
         ((Line) vLine.get(i)).paint(renderer, g);
      }
   }

   public String getLineToString (int line) {
      return ((Line) vLine.get(line)).toString();
   }

   public int print (Graphics g, PageFormat p, int pageIndex) {
      if (pageIndex > 0) {
         return (NO_SUCH_PAGE);
      }
	Graphics2D g2D = (Graphics2D) g;
	 g2D.translate(p.getImageableX(), p.getImageableY());
	 this.paint(g2D);
	 return (PAGE_EXISTS);
   }

   public void handleIn (int begin, int end) {
      ((Line) vLine.get(vLine.size() - 1)).setIn(begin, end);
   }

   public void handleOut (int begin, int end) {
      ((Line) vLine.get(vLine.size() - 1)).setOut(begin, end);
   }

   public void handleProductionNumber (int nr) {
      ((Line) vLine.get(vLine.size() - 1)).setProdNr(nr);
   }

   public void handleRemoveLastLine () {
      vLine.remove(vLine.size() - 1);
      resized();
   }

   public void handleSetViewDown () {
      resized();
      dimPref = this.getPreferredSize();
      dimReal = this.getSize();
      if (dimPref.height > dimReal.height) {
         this.setSize(dimReal.width, dimPref.height);
      }
      this.scrollRectToVisible(new Rectangle(new Point(1, dimPref.height)));
      resized();
   }

   public int lineNr () {
      return vLine.size();
   }
} // ApplicationLogPanel
