/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.runtime.ui;

import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JComponent;

import ch.ethz.exorciser.markov.runtime.Symbol;

/**
 * @author Daniel Mueller
 */
public class ContrastRenderer implements Renderer {

   /**
    * Creates a new <code>ContrastRenderer</code> instance.
    */
   public ContrastRenderer () {}

   /**
    * <code>paintPanel</code>
    * 
    * @param panel
    *           a <code>JPanel</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintPanel (JComponent panel, Graphics2D g) {
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
   }

   /**
    * <code>writeText</code> Write a text in the panel which called it
    * 
    * @param string
    *           a <code>String</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    * @param i
    *           an <code>int</code> value = from LineNumber
    */
   public void writeText (String string, Graphics2D g, int i) {
      g.setColor(TEXTCOLOR);
      int val = 0;
      int idx;
      String tmpstring;
      while ((idx = string.indexOf("/nl", val)) != -1) { //$NON-NLS-1$
         tmpstring = string.substring(val, idx);
         g.drawString(tmpstring, 5, 18 * i + 18);
         val = idx + 3;
         i++;
      }
      g.drawString(string.substring(val, string.length()), 5, 18 * i + 18);
   }

   /**
    * <code>paintLineNr</code> Write LineNr in ApplicationLogPanel
    * 
    * @param nr
    *           an <code>int</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintLineNr (int nr, Graphics2D g) {
      g.drawString(Integer.toString(nr) + ":", BORDER, nr * Y_WIDTH + BORDER); //$NON-NLS-1$
   }

   /**
    * <code>paintSymbol</code>
    * 
    * @param sym
    *           a <code>Symbol</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintSymbol (Symbol sym, Graphics2D g) {
      g.drawString(sym.toString(), sym.x * X_WIDTH + SPACE + 5, sym.y * Y_WIDTH
            + BORDER);
   }

   /**
    * <code>paintConnection</code>
    * 
    * @param line
    *           a <code>Line</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintConnection (Line line, Graphics2D g) {
      g.setColor(HIGHLIGHT);
      if (line.out_begin > -1) {
         g.drawLine(line.out_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + OUT_UP, line.out_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + OUT_DOWN);
         g.drawLine(line.out_end * X_WIDTH + SPACE, line.nr * Y_WIDTH + OUT_UP,
               line.out_end * X_WIDTH + SPACE, line.nr * Y_WIDTH + OUT_DOWN);
         g.drawLine(line.out_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + OUT_DOWN, line.out_end * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + OUT_DOWN);
      }
      if (line.in_begin > -1) {
         g.drawLine(line.in_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH + IN_UP,
               line.in_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH + IN_DOWN);
         g.drawLine(line.in_end * X_WIDTH + SPACE, line.nr * Y_WIDTH + IN_UP,
               line.in_end * X_WIDTH + SPACE, line.nr * Y_WIDTH + IN_DOWN);
         g.drawLine(line.in_begin * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + IN_DOWN, line.in_end * X_WIDTH + SPACE, line.nr * Y_WIDTH
               + IN_DOWN);
      }
      if (line.in_begin > -1) {
         g.drawLine((line.getPrev().out_end - line.getPrev().out_begin)
               * (X_WIDTH / 2) + SPACE + (line.getPrev().out_begin * X_WIDTH),
               (line.nr - 1) * Y_WIDTH + OUT_DOWN,
               (line.in_end - line.in_begin) * (X_WIDTH / 2) + SPACE
                     + (line.in_begin * X_WIDTH), line.nr * Y_WIDTH + IN_DOWN);
         g
               .drawString(
                     Integer.toString(line.prodNr),
                     ((line.in_end - line.in_begin)
                           * (X_WIDTH / 2)
                           + SPACE
                           + 2
                           + (line.in_begin * X_WIDTH)
                           + (line.getPrev().out_end - line.getPrev().out_begin)
                           * (X_WIDTH / 2) + SPACE + (line.getPrev().out_begin * X_WIDTH)) / 2 + 5,
                     (line.nr) * Y_WIDTH);
      }
      g.setColor(TEXTCOLOR);
   }
   /*
    * public String asString() { return "contrast"; //$NON-NLS-1$ }
    */
} // ContrastRenderer
