/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.markov.runtime.ui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;

import javax.swing.JComponent;

import ch.ethz.exorciser.markov.runtime.Symbol;

/**
 * @author Daniel Mueller
 */
public interface Renderer {

   /**
    * <code>FONTSIZE</code> set to 18
    */
   public static int FONTSIZE = 14;

   /**
    * <code>BOLD_FONT</code>
    */
   public static Font BOLD_FONT = new Font("Dialog", Font.BOLD, FONTSIZE); //$NON-NLS-1$

   /**
    * <code>ITALIC_FONT</code>
    */
   public static Font ITALIC_FONT = new Font("Dialog", Font.ITALIC, FONTSIZE); //$NON-NLS-1$

   /**
    * <code>PLAIN_FONT</code>
    */
   public static Font PLAIN_FONT = new Font("Dialog", Font.PLAIN, FONTSIZE); //$NON-NLS-1$

   /**
    * <code>BUTTON_FONT</code>
    */
   public static Font BUTTON_FONT = new Font("Dialog", Font.BOLD, 12); //$NON-NLS-1$

   /**
    * <code>TEXTCOLOR</code>
    */
   public static Color TEXTCOLOR = new Color(0, 0, 0);

   /**
    * <code>HIGHLIGHT</code> set to red
    */
   public static Color HIGHLIGHT = new Color(250, 0, 0);

   /**
    * <code>BACKGROUND</code>
    */
   public static Color BACKGROUND = new Color(240, 240, 240);

   /**
    * <code>MARK</code> set to gray
    */
   public static Color MARK = new Color(100, 100, 100);

   /**
    * <code>BORDER</code>
    */
   public static int BORDER = 20;

   /**
    * <code>X_WIDTH</code>
    */
   public static int X_WIDTH = 20;

   /**
    * <code>Y_WIDTH</code>
    */
   public static int Y_WIDTH = 40;

   /**
    * <code>SPACE</code>
    */
   public static int SPACE = 50;

   /**
    * <code>IN_UP</code>
    */
   public static int IN_UP = 15;

   /**
    * <code>IN_DOWN</code>
    */
   public static int IN_DOWN = 5;

   /**
    * <code>OUT_UP</code>
    */
   public static int OUT_UP = 15;

   /**
    * <code>OUT_DOWN</code>
    */
   public static int OUT_DOWN = 25;

   /**
    * <code>paintPanel</code>
    * 
    * @param panel
    *           a <code>JPanel</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintPanel (JComponent panel, Graphics2D g);

   /**
    * <code>writeText</code>
    * 
    * @param string
    *           a <code>String</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    * @param i
    *           an <code>int</code> value
    */
   public void writeText (String string, Graphics2D g, int i);

   /**
    * <code>paintSymbol</code>
    * 
    * @param sym
    *           a <code>Symbol</code> value
    * @param g
    *           a <code>Graphics2D</code> value
    */
   public void paintSymbol (Symbol sym, Graphics2D g);

   public void paintConnection (Line line, Graphics2D g);

   public void paintLineNr (int nr, Graphics2D g);
   /*
    * public String asString();
    */
} // Renderer
