/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.nondet.npda.Generator;
import ch.ethz.exorciser.nondet.npda.GeneratorParam;
import ch.ethz.exorciser.nondet.npda.NPDAEngine;
import ch.ethz.exorciser.treebrowser.TreeBrowser;
import ch.ethz.exorciser.treebrowser.TreeBrowserDetails;

/**
 * status: rc1 description: the GUI responsible for the setup of the ndpa and
 * the corresponding treebrowser as well as creating and handling all the
 * controls for user interaction this is a very specific class and therefore
 * isn't very reusable but serves as an example how to put the different
 * components together nevertheless the GUI is completely independent of
 * concrete npda implementations
 * 
 * @author Philipp Schlegel
 */
final public class GUI implements ActionListener, ChangeListener {

   final static Color bg_color = Color.white; //common background color

   final static int border = 5; //common border size

   final static int width = 225; //max width of the control panel

   final static String title_text = Messages.getString("NPDA.exercise"); //$NON-NLS-1$

   //different max sizes to achieve desired window behaviour
   final static Dimension tbbd_maxSize = new Dimension(width, width);

   final static Dimension npda_maxSize = new Dimension(width, 55);

   final static Dimension wc_maxSize = new Dimension(width, 85);

   final static Dimension mc_maxSize = new Dimension(width, 85);

   //	final static Dimension tt_maxSize=new Dimension(width,85);
   final static Dimension sc_maxSize = new Dimension(width, 85);

   final static Dimension i_maxSize = new Dimension(width, 100);

   private JComboBox npda_combobox = new JComboBox();

   private JTextField wc_textfield = new JTextField(16);

   private JButton wc_go = new JButton(Messages.getString("NPDA.label.go")); //$NON-NLS-1$

   private JButton wc_button = new JButton(Messages
         .getString("NPDA.label.generate")); //$NON-NLS-1$

   private JButton mc_start = new JButton(Shared.START);

   private JButton mc_stop = new JButton(Shared.STOP);

   private JButton s_all = new JButton(Messages.getString("NPDA.label.expand")); //$NON-NLS-1$

   //	private JButton s_highlights=new JButton("show solution if found");
   private Notifications notifications = new Notifications();

   private List genparams = NPDAEngine.getParams();

   //get information about available npdas
   private GeneratorParam param = (GeneratorParam) genparams.get(0);

   //just init with first npda
   private Generator generator = NPDAEngine.create(param.defaultWord(), param);

   private TreeBrowser treebrowser = new TreeBrowser(generator.getTree());

   //init treebrowser with the tree
   private TreeBrowserDetails tbbd = new TreeBrowserDetails(treebrowser);

   //this is a REAL UGLY hack, but I've been FORCED to to that
   private int maxMovieDelay = 2000;

   private int movieSpeed = 1000; //init intervall for the movie mode

   private Timer timer = new Timer(movieSpeed, this);

   //timer for the movie mode
   //	private GUI() {
   //	}
   public GUI (Container c) {
      tbbd.setPreferredSize(tbbd_maxSize); //ugly hack
      tbbd.setMaximumSize(tbbd_maxSize); //ugly hack
      timer.setInitialDelay(0); //no delay when starting the movie mode
      c.setLayout(new BorderLayout());
      c.add(ExerciseUIFactory.createTaskPane((Exercise) c), BorderLayout.NORTH); //add
      // the
      // title
      // to
      // the
      // gui
      c.add(control(), BorderLayout.EAST); //add the control list to the gui
      c.add(treebrowser, BorderLayout.CENTER);
      //add the treebrowser to the gui
      //		generator.start(); //start calculating the npda
   }

   private JComponent control () { //just creates a JComponent that contains
      // the whole control area
      JPanel control = new JPanel();
      control.setLayout(new BoxLayout(control, BoxLayout.Y_AXIS));
      control.setBackground(bg_color);
      control.setPreferredSize(new Dimension(width, 0));
      control.setBorder(BorderFactory.createEmptyBorder(border, border, border,
            border));
      JPanel npda_choice = new JPanel(new BorderLayout());
      //full automatic generation of npda combobox -> GUI is independent of
      // concrete npda implementations
      for (int i = 0; i < genparams.size(); i++)
         npda_combobox.addItem(((GeneratorParam) genparams.get(i)).name());
      npda_combobox.setActionCommand("npda_choice"); //$NON-NLS-1$
      npda_combobox.addActionListener(this);
      npda_choice.setBackground(bg_color);
      npda_choice.setMaximumSize(npda_maxSize);
      npda_choice.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("NPDA.label.chosse_npda"))); //$NON-NLS-1$
      npda_choice.add(npda_combobox, BorderLayout.CENTER);
      JPanel word_choice = new JPanel(new BorderLayout());
      wc_button.setActionCommand("wc_button"); //$NON-NLS-1$
      wc_button.addActionListener(this);
      wc_go.setActionCommand("wc_go"); //$NON-NLS-1$
      wc_go.addActionListener(this);
      wc_textfield.setText(param.defaultWord());
      word_choice.setBackground(bg_color);
      word_choice.setMaximumSize(wc_maxSize);
      word_choice.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("NPDA.label.input_word"))); //$NON-NLS-1$
      word_choice.add(wc_textfield, BorderLayout.CENTER);
      word_choice.add(wc_button, BorderLayout.SOUTH);
      word_choice.add(wc_go, BorderLayout.EAST);
      JPanel movie_control = new JPanel(new BorderLayout());
      JSlider movie_speed = new JSlider(SwingConstants.HORIZONTAL, 0,
            maxMovieDelay, movieSpeed);
      mc_stop.setEnabled(false);
      mc_stop.setActionCommand("mc_stop"); //$NON-NLS-1$
      mc_stop.addActionListener(this);
      mc_start.setActionCommand("mc_start"); //$NON-NLS-1$
      mc_start.addActionListener(this);
      movie_speed.setBackground(bg_color);
      movie_speed.setBorder(BorderFactory.createTitledBorder(BorderFactory
            .createEmptyBorder(), Messages.getString("NPDA.label.speed"))); //$NON-NLS-1$
      movie_speed.setMajorTickSpacing(maxMovieDelay / 4);
      Hashtable labelTable = new Hashtable();
      labelTable.put(new Integer(0), new JLabel(Messages
            .getString("NPDA.label.slow"))); //$NON-NLS-1$
      labelTable.put(new Integer(maxMovieDelay), new JLabel(Messages
            .getString("NPDA.label.fast"))); //$NON-NLS-1$
      movie_speed.setLabelTable(labelTable);
      movie_speed.setPaintTicks(true);
      movie_speed.setPaintLabels(true);
      movie_speed.addChangeListener(this);
      movie_control.setBackground(bg_color);
      movie_control.setMaximumSize(mc_maxSize);
      movie_control.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("NPDA.label.movie_mode_control"))); //$NON-NLS-1$
      movie_control.add(mc_stop, BorderLayout.EAST);
      movie_control.add(mc_start, BorderLayout.CENTER);
      movie_control.add(movie_speed, BorderLayout.NORTH);
      JPanel solver = new JPanel(new BorderLayout());
      s_all.setActionCommand("s_all"); //$NON-NLS-1$
      s_all.addActionListener(this);
      solver.setBackground(bg_color);
      solver.setMaximumSize(sc_maxSize);
      solver.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("NPDA.label.solver"))); //$NON-NLS-1$
      solver.add(s_all, BorderLayout.CENTER);
      JScrollPane information_pane = new JScrollPane(notifications);
      notifications.setBackground(bg_color);
      notifications.addNotification(param.name(), param.description());
      information_pane.setBackground(bg_color);
      information_pane.setPreferredSize(i_maxSize);
      information_pane.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("NPDA.label.information"))); //$NON-NLS-1$
      control.add(npda_choice);
      control.add(word_choice);
      control.add(solver);
      control.add(movie_control);
      control.add(tbbd); //ugly hack
      control.add(information_pane);
      return (control);
   }

   /**
    * @see java.awt.event.ActionListener#actionPerformed(ActionEvent) main
    *      action handling
    */
   public void actionPerformed (ActionEvent e) {
      String action = e.getActionCommand();
      if (action != null) {
         if (action.equals("npda_choice")) { //new npda has been chosen //$NON-NLS-1$
                                             // //$NON-NLS-1$
            JComboBox npc = (JComboBox) e.getSource();
            int i = npc.getSelectedIndex();
            notifications.removeAll();
            param = (GeneratorParam) genparams.get(i);
            notifications.addNotification(param.name(), param.description());
            generator.reset(param.defaultWord(), param);
            treebrowser.resetTree(generator.getTree());
            wc_textfield.setText(param.defaultWord());
         } else if (action.equals("wc_go")) { //$NON-NLS-1$
            generator.reset(wc_textfield.getText());
            treebrowser.resetTree(generator.getTree());
         } else if (action.equals("wc_button")) { //generate random //$NON-NLS-1$
                                                  // word//$NON-NLS-1$
            String random = param.randomWord();
            wc_textfield.setText(random);
         } else if (action.equals("s_all")) { //show all calculated //$NON-NLS-1$
                                              // nodes//$NON-NLS-1$
            treebrowser.getTree().showAll();
            treebrowser.updateTB();
         } else if (action.equals("mc_start")) { //start the movie //$NON-NLS-1$
                                                 // mode//$NON-NLS-1$
            if (!timer.isRunning()) {
               npda_combobox.setEnabled(false);
               wc_textfield.setEnabled(false);
               wc_go.setEnabled(false);
               wc_button.setEnabled(false);
               mc_start.setEnabled(false);
               mc_stop.setEnabled(true);
               s_all.setEnabled(false);
               timer.setDelay(maxMovieDelay - movieSpeed);
               timer.start();
               generator.startMovie((float) movieSpeed / maxMovieDelay);
            }
         } else if (action.equals("mc_stop")) { //stop the movie mode //$NON-NLS-1$
            // //$NON-NLS-1$
            if (timer.isRunning()) {
               npda_combobox.setEnabled(true);
               wc_textfield.setEnabled(true);
               wc_go.setEnabled(true);
               wc_button.setEnabled(true);
               mc_start.setEnabled(true);
               mc_stop.setEnabled(false);
               s_all.setEnabled(true);
               generator.stopMovie();
               timer.stop();
            }
         }
      } else {
         treebrowser.updateTB(); //just update the whole thing
      }
   }

   /**
    * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent) handles
    *      the case when the movie mode speed slider has been used
    */
   public void stateChanged (ChangeEvent e) {
      JSlider movie_speed = (JSlider) e.getSource();
      if (!movie_speed.getValueIsAdjusting()) {
         movieSpeed = movie_speed.getValue();
         if (timer.isRunning()) {
            generator.stopMovie();
            timer.stop();
            timer.setDelay(maxMovieDelay - movieSpeed);
            timer.start();
            generator.startMovie((float) movieSpeed / maxMovieDelay);
         }
      }
   }
}