/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda;

import ch.ethz.exorciser.treebrowser.Edge;
import ch.ethz.exorciser.treebrowser.Node;
import ch.ethz.exorciser.treebrowser.Tree;

/**
 * Abstract class representing a generic generator for Trees. This class was
 * meant to be as generic as possible and abstract of what the generated
 * represents.
 * 
 * Creation is done depending of a String parameter and customizable
 * GeneratorParams. Provide your own GeneratorParam class together with
 * subclasses of Generator.
 * 
 * @author Oliver Biderbost
 */
public abstract class Generator {

    protected Tree _tree;

    public abstract void reset(String word);

    public abstract void reset(String word, GeneratorParam genParam);

    public abstract Tree getTree();

    public abstract String defaultWord();

    public abstract String randomWord();

    public abstract void startMovie(float speed);

    public abstract void stopMovie();

    public abstract Edge getTip(Node n);
    
    //public abstract String getSolutionInfo();
    
}