/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda;

import java.awt.Graphics;
import java.awt.Rectangle;

import ch.ethz.exorciser.treebrowser.Edge;

/**
 * Transition between execution states of an NPDA This is almost a pure data
 * object without logic
 * 
 * @author Oliver Biderbost
 */
public class NPDATransition extends Edge {

   protected Rule _rule;

   /**
    * Constructor NPDATransition.
    * 
    * @param from
    * @param to
    * @param r
    */
   public NPDATransition (NPDAConfiguration from, NPDAConfiguration to, Rule r) {
      _source = from;
      _target = to;
      _rule = r;
   }

   /**
    * Method toString.
    * 
    * @return String returns String of the form "state,read,pop->state,push"
    *         \u03B5 is epsilon
    */
   public String toString () {
      String read = _rule.read().equals("") ? "\u03B5" : _rule.read(); //$NON-NLS-1$ //$NON-NLS-2$
      String pop = _rule.pop().equals("") ? "\u03B5" : _rule.pop(); //$NON-NLS-1$ //$NON-NLS-2$
      String push = _rule.push().equals("") ? "\u03B5" : _rule.push(); //$NON-NLS-1$ //$NON-NLS-2$
      //		char s1 = (char) _rule.from();
      //		char s2 = (char) _rule.to();
      String s = "" + _rule.from() + "," + read + "," + pop + " \u2192 "; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
      s += "" + _rule.to() + "," + push + ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      return s;
   }

   /**
    * @see nondeterministic.npda.Paintable
    */
   public void paint (Graphics g, Rectangle r) {
      Painter.paint(this, g, r);
   }

   /**
    * @see nondeterministic.npda.Paintable
    */
   public void paintDetails (Graphics g, Rectangle r) {
      Painter.paintDetails(this, g, r);
   }

   /**
    * @see nondeterministic.npda.Node#hide(boolean)
    */
   public void setHide (boolean yes) {
      _hidden = yes;
      if (_hidden) _highlighted = false;
   }

   /**
    * @see nondeterministic.npda.Node#highlight(boolean)
    */
   public void setHighlight (boolean yes) {
      _highlighted = yes;
      if (_highlighted) _hidden = false;
   }
}