/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import ch.ethz.exorciser.treebrowser.Node;
import ch.ethz.exorciser.treebrowser.Tree;

/**
 * Implementation of a Tree representing the execution states of a
 * nondeterministic pushdown automaton
 * 
 * @author Oliver Biderbost
 */
public class NPDATree extends Tree {

   protected List _accepting = null;

   protected int _depth = -1;

   protected NPDAEngine _engine;

   /**
    * Constructor NPDATree
    * 
    * @param root
    */
   public NPDATree (Node root, NPDAEngine e) {
      _root = root;
      _engine = e;
      //		init();
      _nodes = new ArrayList();
      _accepting = new LinkedList(); // list of accepting nodes
      ((NPDAConfiguration) _root).init(this); // init node as root node
      register(_root);
   }

   public void register (Node n) {
      _nodes.add(n);
      if (((NPDAConfiguration) n).isAccepting() && !_accepting.contains(n))
            _accepting.add(n); // add to list of accepting nodes
   }

   public int numNodes () {
      return _nodes.size();
   }

   public int numVisibleNodes () {
      int i = 0;
      Iterator iter = _nodes.iterator();
      while (iter.hasNext()) {
         if (!((Node) iter.next()).isHidden()) {
            i++;
         }
      }
      return i;
   }

   //	public void init() {
   //		_nodes = new ArrayList();
   //		_accepting = new LinkedList(); // list of accepting nodes
   //		 ((NPDAConfiguration) _root).init(this); // init node as root node
   //		register(_root);
   //	}
   public void focus (Node n) {
      n.focus();
   }

   public void showAll () {
      _root.showAll();
      //		showHighlights();
   }

   //	public void showHighlights() {
   //		Iterator path;
   //		Iterator highlights = _accepting.iterator();
   //		while (highlights.hasNext()) { // for all highlights
   //			path = ((Node) highlights.next()).getEdgePath().iterator();
   //			while (path.hasNext()) // for all edges in path
   //				 ((Edge) path.next()).setHighlight(true);
   //		}
   //		{
   //			Iterator nodepath;
   //			Iterator nodehighlights = _accepting.iterator();
   //			while (nodehighlights.hasNext()) { // for all highlights
   //				nodepath = ((Node) nodehighlights.next()).getNodePath().iterator();
   //				while (nodepath.hasNext()) // for all edges in path
   //					 ((NPDAConfiguration) ((Node) nodepath.next())).unhide();
   //			}
   //		}
   //	}
   /**
    * Method accepting
    * 
    * @return List Returns accepting nodes
    */
   public List accepting () {
      return _accepting;
   }
}