/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.font.TextAttribute;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.AttributedString;

import ch.ethz.exorciser.Messages;

/**
 * status: rc1 description: class that paints node and transition content the
 * painting is done for a master size and then - with the help of
 * Graphics2D.scale(double,double) - scaled to the actual size the calculations
 * for the scaling seem to be a bit complicated but the good results justify
 * them afternode: after a design decision the scaling isn't used at the moment
 * but it may helpfull for the future
 * 
 * @author Philipp Schlegel
 */
public class Painter {

   final static float width = 203;

   final static float height = 56;

   final static float transWidth = 101;

   final static float transHeight = 22;

   final static int offset = 40;

   final static Rectangle2D.Float accepting = new Rectangle2D.Float(0, 0, 203,
         56);

   final static Rectangle2D.Float upper = new Rectangle2D.Float(10, 15, 183, 16);

   final static Rectangle2D.Float lower = new Rectangle2D.Float(10, 35, 183, 16);

   final static Point2D.Float wordPos = new Point2D.Float(12, 28);

   final static Point2D.Float stackPos = new Point2D.Float(12, 48);

   final static Rectangle2D.Float stateRect = new Rectangle2D.Float(183, 0, 20,
         20);

   final static Point2D.Float statePos = new Point2D.Float(189, 16);

   final static Point2D.Float transPos = new Point2D.Float(5, 15);

   /**
    * Method paint.
    * 
    * @param conf
    * @param g
    * @param r
    *           paints the content of the big node boxes
    */
   public static void paint (NPDAConfiguration conf, Graphics g, Rectangle r) {
      Graphics2D g2d = (Graphics2D) g;
      Rectangle oldClip = g2d.getClipBounds();
      Font oldFont = g2d.getFont(); //save all the old states
      Color oldColor = g2d.getColor();
      float xfact = r.width / width; //calculate the relations
      float yfact = r.height / height;
      float invxfact = width / r.width;
      float invyfact = height / r.height;
      //transform the design to the new coordinates
      Rectangle2D.Float oldClipTrans = new Rectangle2D.Float(invxfact
            * oldClip.x, invyfact * oldClip.y, invxfact * oldClip.width,
            invyfact * oldClip.height);
      Rectangle2D.Float rTrans = new Rectangle2D.Float(invxfact * r.x, invyfact
            * r.y, invxfact * r.width, invyfact * r.height);
      Rectangle2D.Float acceptingTrans = new Rectangle2D.Float(rTrans.x
            + accepting.x, rTrans.y + accepting.y, accepting.width,
            accepting.height);
      Rectangle2D.Float upperTrans = new Rectangle2D.Float(rTrans.x + upper.x,
            rTrans.y + upper.y, upper.width, upper.height);
      Rectangle2D.Float lowerTrans = new Rectangle2D.Float(rTrans.x + lower.x,
            rTrans.y + lower.y, lower.width, lower.height);
      //create word/stack output and highlight necessary letters
      AttributedString word = new AttributedString(conf.word());
      word.addAttribute(TextAttribute.FONT, oldFont.deriveFont((float) 12));
      if (conf.wordPos() < conf.word().length())
            word.addAttribute(TextAttribute.BACKGROUND, Color.red, conf
                  .wordPos(), conf.wordPos() + 1);
      String stack = conf.getStackString();
      Rectangle2D.Float stateRectTrans = new Rectangle2D.Float(rTrans.x
            + stateRect.x, rTrans.y + stateRect.y, stateRect.width,
            stateRect.height);
      g2d.scale(xfact, yfact); //scale the whole thing
      g2d.setClip(rTrans.createIntersection(oldClipTrans)); //note: clipping
                                                            // has to be
                                                            // calculated
                                                            // carefully
      if (conf.isAccepting()) { //if it's an accepting state
         g2d.setColor(Color.green);
         g2d.fill(acceptingTrans);
         g2d.setColor(oldColor);
      }
      g2d.setFont(oldFont.deriveFont((float) 12));
      g2d.setClip(upperTrans.createIntersection(oldClipTrans));
      g2d.drawString(Messages.getString("NPDA.label.word"), rTrans.x //$NON-NLS-1$
            + wordPos.x, rTrans.y + wordPos.y); //$NON-NLS-1$
      g2d.drawString(word.getIterator(), rTrans.x + wordPos.x + offset,
            rTrans.y + wordPos.y);
      g2d.setClip(lowerTrans.createIntersection(oldClipTrans));
      g2d.drawString(Messages.getString("NPDA.label.stack"), rTrans.x //$NON-NLS-1$
            + stackPos.x, rTrans.y + stackPos.y); //$NON-NLS-1$
      g2d.drawString(stack, rTrans.x + stackPos.x + offset, rTrans.y
            + stackPos.y);
      g2d.setClip(rTrans.createIntersection(oldClipTrans));
      g2d.draw(upperTrans);
      g2d.draw(lowerTrans);
      if (conf.isAccepting()) g2d.setColor(Color.green);
      else g2d.setColor(Color.white);
      g2d.fill(stateRectTrans);
      g2d.setColor(oldColor);
      g2d.draw(stateRectTrans);
      g2d.drawString(Messages.getString("NPDA.label.state"), rTrans.x //$NON-NLS-1$
            + statePos.x - offset, rTrans.y + statePos.y - 3); //$NON-NLS-1$
      g2d.setFont(oldFont.deriveFont(Font.BOLD, 14));
      g2d.drawString(String.valueOf(conf.state()), rTrans.x + statePos.x,
            rTrans.y + statePos.y);
      g2d.scale(invxfact, invyfact); //rescale
      g2d.setFont(oldFont); //and restore the states
      g2d.setClip(oldClip);
   }

   /**
    * Method paint.
    * 
    * @param trans
    * @param g
    * @param r
    *           paints the content of the small transition boxes
    */
   public static void paint (NPDATransition trans, Graphics g, Rectangle r) {
      Graphics2D g2d = (Graphics2D) g; //about the same as above
      Rectangle oldClip = g2d.getClipBounds();
      Font oldFont = g2d.getFont();
      float xfact = r.width / transWidth;
      float yfact = r.height / transHeight;
      float invxfact = transWidth / r.width;
      float invyfact = transHeight / r.height;
      Rectangle2D.Float oldClipTrans = new Rectangle2D.Float(invxfact
            * oldClip.x, invyfact * oldClip.y, invxfact * oldClip.width,
            invyfact * oldClip.height);
      Rectangle2D.Float rTrans = new Rectangle2D.Float(invxfact * r.x, invyfact
            * r.y, invxfact * r.width, invyfact * r.height);
      g2d.scale(xfact, yfact);
      g2d.setClip(rTrans.createIntersection(oldClipTrans));
      g2d.setFont(new Font("Lucida Sans", Font.PLAIN, 12)); //explicit font //$NON-NLS-1$
                                                            // choice because of
                                                            // troubles with the
                                                            // arrow
                                                            // //$NON-NLS-1$
      g2d.drawString(trans.toString(), rTrans.x + transPos.x, rTrans.y
            + transPos.y);
      g2d.scale(invxfact, invyfact);
      g2d.setFont(oldFont);
      g2d.setClip(oldClip);
   }

   /**
    * Method paintDetails.
    * 
    * @param c
    * @param g
    * @param r
    *           paints the optional detailed view of a node
    */
   public static void paintDetails (NPDAConfiguration c, Graphics g, Rectangle r) {
      Graphics2D g2d = (Graphics2D) g;
      Shape clip = g2d.getClip();
      g2d.setClip(r);
      g2d.drawImage(c._npda.image(), r.x, r.y, r.width, r.height, null);
      g2d.setClip(clip);
   }

   /**
    * Method paintDetails.
    * 
    * @param t
    * @param g
    * @param r
    *           paints the optional detailed view of a transition not used at
    *           the moment
    */
   public static void paintDetails (NPDATransition t, Graphics g, Rectangle r) {
      paint(t, g, r);
   }
}