/*
 * Exorciser - www.educeth.ch/exorciser  Copyright 2000-2004 Swiss Federal
 * Institue of Technology, Zrich Switzerland
 */
package ch.ethz.exorciser.nondet.npda;

/**
 * The rules of a PDA Provides the logic for matching configurations and
 * creating new configurations by applying this rule to a given configuration.
 * 
 * @author Oliver Biderbost
 */
public class Rule {

   protected int _from; // source state

   protected int _to; // tartet state

   protected String _read; // match on input word,

   protected String _pop; // match on stack, first char is top of stack

   protected String _push; // push to stack, first char is on top of stack

   public Rule (int from, int to, String read, String pop, String push) {
      _from = from;
      _to = to;
      _read = read;
      _pop = pop;
      _push = push;
   }

   /**
    * Method from.
    * 
    * @return int source state
    */
   public int from () {
      return _from;
   }

   /**
    * Method to.
    * 
    * @return int tartet state
    */
   public int to () {
      return _to;
   }

   /**
    * Method read.
    * 
    * @return String match on input word,
    */
   public String read () {
      return _read;
   }

   /**
    * Method pop.
    * 
    * @return String match on stack, first char is top of stack
    */
   public String pop () {
      return _pop;
   }

   /**
    * Method push.
    * 
    * @return String push to stack, last char is on top of stack
    */
   public String push () {
      return _push;
   }

   public boolean matchConfiguration (NPDAConfiguration c) {
      if (c.word().startsWith(read(), c.wordPos()) //  input match
            && c.stack().startsWith(pop())) //  stack match
            return true; // stack and input match
      return false;
   }

   /**
    * Method applyToConfiguration.
    * 
    * @param c
    *           pops pop() from a the Stack char by char and then pushes push()
    *           onto it last char is on top of the stack assumes
    *           matchConfiguration(c) is true
    */
   public void applyToConfiguration (NPDAConfiguration c) {
      // fixme: add check if really matches
      StringBuffer sb = new StringBuffer(c.stack());
      sb.delete(0, pop().length());
      sb.insert(0, push());
      c.stack(sb.toString());
      c.wordPos(c.wordPos() + read().length());
   }
}