/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda.automata;

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.JLabel;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.nondet.npda.Rule;

/**
 * NPDA class for parsing palindromes (e.g. 00100 or 1001)
 * 
 * @author Oliver Biderbost
 */
public class PalindromeNPDA extends NPDA {

	protected static Image _palImg = null;
	static {
		MediaTracker mt = new MediaTracker(new JLabel());
		ClassLoader cl = ParenthesisByGrammarNPDA.class.getClassLoader();
		URL url = cl.getResource(Messages.getString("NPDA.Palindrome.image")); //$NON-NLS-1$
		_palImg = Toolkit.getDefaultToolkit().getImage(url);
		mt.addImage(_palImg, 0);
		try {
			mt.waitForID(0);
		} catch (InterruptedException e) {
		}
	}

	public PalindromeNPDA() {
		super(3);
		startState(0);
		addAccept(2);
		// from,to,read,pop,push
		addRule(new Rule(0, 0, "0", "", "0")); // push input to stack
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(0, 0, "1", "", "1")); // push input to stack
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(0, 1, "", "", "")); // decide even palyndrome centre
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(0, 1, "1", "", "")); // decide odd palyndrome centre
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(0, 1, "0", "", "")); // decide odd palyndrome centre
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(1, 1, "0", "0", "")); // match stack and input
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(1, 1, "1", "1", "")); // match stack and input
												// //$NON-NLS-1$ //$NON-NLS-2$
												// //$NON-NLS-3$
		addRule(new Rule(1, 2, "$", "$", "")); // accept on empty //$NON-NLS-1$
												// //$NON-NLS-2$ //$NON-NLS-3$
		// //$NON-NLS-2$ //$NON-NLS-3$
	}

	public String name() {
		return Messages.getString("NPDA.Palindrome.title"); //$NON-NLS-1$
	}

	public String description() {
		return Messages.getString("NPDA.Palindrome.description"); //$NON-NLS-1$
	}

	public Image image() {
		return _palImg;
	}

	public String defaultWord() {
		return Messages.getString("NPDA.Palindrome.default_word"); //$NON-NLS-1$
	}

	public String randomWord() {
		String s = ""; //$NON-NLS-1$
		int i;
		if (coin()) // odd palindrome
			s += coinZeroOne();
		int curlen = 0;
		while (curlen < MAX_GEN_WORD_LEN - 1) {
			i = coinZeroOne();
			s = i + s + i;
			if (coin())
				return s;
		}
		return s;
	}

	protected int coinZeroOne() {
		if (coin()) {
			return 0;
		}
		return 1;
	}
}