/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda.automata;

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.JLabel;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.nondet.npda.NPDAConfiguration;
import ch.ethz.exorciser.nondet.npda.Rule;

/**
 * NPDA class for parsing parenthesis expressions (e.g. ((()[])) or []()) the
 * grammar is used directly to parse
 * 
 * @author Oliver Biderbost
 */
public class ParenthesisByGrammarNPDA extends ParenthesisNPDA {

   protected static Image _pbgImg = null;
   static {
      MediaTracker mt = new MediaTracker(new JLabel());
      ClassLoader cl = ParenthesisByGrammarNPDA.class.getClassLoader();
      URL url = cl.getResource(Messages
            .getString("NPDA.ParenthesisByGrammar.image")); //$NON-NLS-1$
      _pbgImg = Toolkit.getDefaultToolkit().getImage(url);
      mt.addImage(_pbgImg, 0);
      try {
         mt.waitForID(0);
      } catch (InterruptedException e) {}
   }

   /**
    * Constructor for ParenthesisNPDA.
    */
   public ParenthesisByGrammarNPDA () {
      super(3);
      startState(1);
      addAccept(2);
      // from,to,read,pop,push
      addRule(new Rule(1, 1, "", "S", "")); // push grammar to stack //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "", "S", "(S)")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "", "S", "[S]")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "", "S", "SS")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "(", "(", "")); // match stack and input //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, ")", ")", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "[", "[", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 1, "]", "]", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 2, "$", "$", "")); // accept on empty //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
   }

   /**
    * Method start
    * 
    * @param word
    * @return NPDAConfiguration creates a starting configuration for the given
    *         input string invalidates all Configurations already created
    */
   public NPDAConfiguration start (String word) {
      _word = word + '$';
      return new NPDAConfiguration(this, "S$", 0, _startState); //$NON-NLS-1$
   }

   public String name () {
      return Messages.getString("NPDA.ParenthesisByGrammar.title"); //$NON-NLS-1$
   }

   public String description () {
      return Messages.getString("NPDA.ParenthesisByGrammar.description"); //$NON-NLS-1$
   }

   public Image image () {
      return _pbgImg;
   }
}