/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda.automata;

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.JLabel;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.nondet.npda.NPDAConfiguration;
import ch.ethz.exorciser.nondet.npda.Rule;

/**
 * NPDA class for parsing parenthesis expressions (e.g. ((()[])) or []())
 * 
 * @author Oliver Biderbost
 */
public class ParenthesisNPDA extends NPDA {

    protected static Image _pImg = null;
    static {
        MediaTracker mt = new MediaTracker(new JLabel());
        ClassLoader cl = ParenthesisByGrammarNPDA.class.getClassLoader();
        URL url = cl.getResource(Messages.getString("NPDA.Parenthesis.image")); //$NON-NLS-1$
        _pImg = Toolkit.getDefaultToolkit().getImage(url);
        mt.addImage(_pImg, 0);
        try {
            mt.waitForID(0);
        } catch (InterruptedException e) {
        }
    }

    protected ParenthesisNPDA(int numstates) {
        super(numstates);
    }

    public ParenthesisNPDA() {
        super(2);
        startState(0);
        addAccept(1);
        // from,to,read,pop,push
        addRule(new Rule(0, 0, "(", "", ")")); // push input to stack
                                               // //$NON-NLS-1$ //$NON-NLS-2$
                                               // //$NON-NLS-3$
        addRule(new Rule(0, 0, "[", "", "]")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        addRule(new Rule(0, 0, ")", ")", "")); // match stack and input
                                               // //$NON-NLS-1$ //$NON-NLS-2$
                                               // //$NON-NLS-3$
        addRule(new Rule(0, 0, "]", "]", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        addRule(new Rule(0, 1, "$", "$", "")); // accept on empty //$NON-NLS-1$
                                               // //$NON-NLS-2$ //$NON-NLS-3$
    }

    public NPDAConfiguration start(String word) {
        _word = word + "$"; //$NON-NLS-1$
        return new NPDAConfiguration(this, "$", 0, _startState); //$NON-NLS-1$
    }

    public String name() {
        return Messages.getString("NPDA.Parenthesis.title"); //$NON-NLS-1$
    }

    public String description() {
        return Messages.getString("NPDA.Parenthesis.description"); //$NON-NLS-1$
    }

    public String defaultWord() {
        return Messages.getString("NPDA.Parenthesis.default_word"); //$NON-NLS-1$
    }

    public String randomWord() {
        return genWord(MAX_GEN_WORD_LEN);
    }

    protected String genWord(int maxlen) {
        String s = ""; //$NON-NLS-1$
        if (maxlen < 1)
            return s;
        char c1 = '(';
        char c2 = ')';
        if (coin()) {
            c1 = '[';
            c2 = ']';
        }
        int l1 = (int) Math.floor(Math.random() * maxlen / 2);
        //		int l2 = (int) Math.floor(Math.random() * maxlen / 2);
        s = c1 + genWord(l1) + genWord(l1) + c2;
        return s;
    }

    public Image image() {
        return _pImg;
    }
}