/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.nondet.npda.automata;

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.JLabel;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.nondet.npda.Rule;

/**
 * NPDA class for parsing a sample regular expression (ab|(a|ab)(ab)*(a|ab))*
 * 
 * @author Oliver Biderbost
 */
public class RegexExampleNPDA extends NPDA {

   protected static Image _reeImg = null;
   static {
      MediaTracker mt = new MediaTracker(new JLabel());
      ClassLoader cl = ParenthesisByGrammarNPDA.class.getClassLoader();
      URL url = cl.getResource(Messages.getString("NPDA.RegexExample.image")); //$NON-NLS-1$
      _reeImg = Toolkit.getDefaultToolkit().getImage(url);
      mt.addImage(_reeImg, 0);
      try {
         mt.waitForID(0);
      } catch (InterruptedException e) {}
   }

   public RegexExampleNPDA () {
      super(4);
      startState(0);
      addAccept(3);
      // from,to,read,pop,push
      addRule(new Rule(0, 1, "a", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(0, 2, "a", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 0, "a", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(1, 2, "a", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(2, 0, "b", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(2, 1, "b", "", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      addRule(new Rule(0, 3, "$", "$", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
   }

   public String name () {
      return Messages.getString("NPDA.RegexExample.title"); //$NON-NLS-1$
   }

   public String description () {
      return Messages.getString("NPDA.RegexExample.description"); //$NON-NLS-1$
   }

   public String defaultWord () {
      return Messages.getString("NPDA.RegexExample.default_word"); //$NON-NLS-1$
   }

   public String randomWord () {
      return genWord(MAX_GEN_WORD_LEN);
   }

   protected String genWord (int maxlen) { // fixme: maxlen not sharp
      String s = ""; //$NON-NLS-1$
      int len = (int) Math.floor(Math.random() * maxlen);
      while (len > s.length() && (coin() || coin()))
         // prob. 0.75
         if (coin()) s += "ab"; //$NON-NLS-1$
         else {
            if (coin() || coin()) s += "a"; //$NON-NLS-1$
            else s += "ab"; //$NON-NLS-1$
            while (coin())
               s += "ab"; //$NON-NLS-1$
            if (coin()) s += "a"; //$NON-NLS-1$
            else s += "ab"; //$NON-NLS-1$
         }
      return s;
   }

   public Image image () {
      return _reeImg;
   }
}