/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.fsmgui.CircleLayouter;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;
import ch.ethz.exorciser.fsmgui.FSMEvent;
import ch.ethz.exorciser.fsmgui.FSMStateInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FAException;
import ch.ethz.exorciser.ifa.FASerializer;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.ifa.State;
import ch.ethz.exorciser.rl.fa2re.Fa2ReEditor;
import ch.ethz.exorciser.rl.fa2re.Fa2ReFSMController;
import ch.ethz.exorciser.rl.re.ABRegExpParser;
import ch.ethz.exorciser.rl.re.Epsilon;
import ch.ethz.exorciser.rl.re.REParserException;
import ch.ethz.exorciser.rl.re.RegularExpression;
import ch.ethz.exorciser.rl.re.Symbol;
import ch.ethz.exorciser.rl.re.dictionary.REPrefixDictionary;

/**
 * @author Vincent Tscherter
 */
public class FA2RE extends JPanel implements Exercise {

   static final Insets nullInsets = new Insets(0, 0, 0, 0);

   public void instructions () {
      notifications.removeAll();
      notifications.addHintNotification(Messages
            .getString("FA2RE.instructions.title"), //$NON-NLS-1$
            Messages.getString("FA2RE.instructions")); //$NON-NLS-1$
   }

   private final class CheckAction implements ActionListener {

      public void actionPerformed (ActionEvent event) {
         check();
      }
   }

   private final class OkAction implements ActionListener {

      private final Alphabet alphabet;

      private OkAction (Alphabet alphabet) {
         super();
         this.alphabet = alphabet;
      }

      public void actionPerformed (ActionEvent event) {
         FA _fa = faeditor.getFA();
         try {
            boolean ok = true;
            if (_fa.getNotReachableStates().size() > 0) {
               if (JOptionPane.showConfirmDialog(null, Messages
                     .getString("FA2RE.fa_contains_unreachable_states"), //$NON-NLS-1$
                     Messages.getString("FA2RE.unreachable_states"), //$NON-NLS-1$
                     JOptionPane.CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                  _fa.removeNotReachableStates();
               } else {
                  ok = false;
               }
            }
            if (!_fa.getAlphabet().equals(alphabet)) {
               JOptionPane.showMessageDialog(null, Messages.getString(
                     "FA2RE.alphabet_should_be") //$NON-NLS-1$
                     .replaceFirst("\\$ALPHABET", //$NON-NLS-1$
                           alphabet.toString()), Messages
                     .getString("FA2RE.wrong_alphabet"), //$NON-NLS-1$
                     JOptionPane.OK_OPTION);
               ok = false;
            }
            if (ok) {
               setFA(_fa);
               dialog.dispose();
            }
         } catch (Exception e) {
            Debug.showException(e);
         }
      }
   }

   JButton cancel = new JButton(new AbstractAction(Shared.CANCEL) {//$NON-NLS-1$

            public void actionPerformed (ActionEvent e) {
               dialog.dispose();
            }
         });

   public void edit () {
      notifications.removeAll();
      repaint();
      dialog = new JDialog((Frame) null, Messages.getString("FA2RE.edit_fa"), //$NON-NLS-1$
            true);
      dialog.getContentPane().setLayout(new BorderLayout());
      JButton ok = new JButton(Shared.OKAY); //$NON-NLS-1$
      ok.addActionListener(new OkAction(ALPHABETH));
      JPanel buttons = new JPanel(new GridLayout(1, 2));
      buttons.add(ok);
      buttons.add(cancel);
      dialog.getContentPane().add(buttons, BorderLayout.SOUTH);
      dialog.getContentPane().add(faeditor, BorderLayout.CENTER);
      Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
      dialog.setBounds((screen.width - 500) / 2, (screen.height - 400) / 2,
            500, 400);
      faeditor.setFA((FA) originalFA.clone());
      dialog.setVisible(true);
   }

   private final class KeyListener extends KeyAdapter {

      public void keyReleased (KeyEvent event) {
         FA2RE.this.setInstantSolution(FA2RE.this.refield.getText());
      }
   }

   public void random (String query) {
      notifications.removeAll();
      String level = Query.getValue(query, "level"); //$NON-NLS-1$
      if (level.equals("beginner")) { //$NON-NLS-1$
         setFA(REPrefixDictionary.getInstance().getRandomFA(3));
      } else if (level.equals("advanced")) { //$NON-NLS-1$
         setFA(REPrefixDictionary.getInstance().getRandomFA(5));
      } else if (level.equals("expert")) { //$NON-NLS-1$
         setFA(REPrefixDictionary.getInstance().getRandomFA(9));
      }
   }

   public void next () {
      refield.setText(""); //$NON-NLS-1$
      fmscontroller.refine();
      notifications.removeAll();
   }

   public void reset () {
      refield.setText(""); //$NON-NLS-1$
      fa = (FA) originalFA.clone();
      newTask();
      notifications.removeAll();
   }

   public void solve () {
      refield.setText(""); //$NON-NLS-1$
      notifications.removeAll();
      fmscontroller.solve();
   }

   static final Alphabet ALPHABETH = Symbol.createASymbol().getFA()
         .getAlphabet();

   //	static int[] levelvalues = { 3, 5, 9 };
   JDialog dialog;
   static {
      REPrefixDictionary.getInstance().load(12);
   }

   FSMEditorOperations editorOps;

   FA fa;

   FA faDet;

   IFACompleteEditor faeditor = new IFACompleteEditor();

   Notifications notifications = new Notifications();

   Fa2ReFSMController fmscontroller;

   FSMView fsmView;

   int index = 0;

   private IFACompleteEditor input = new IFACompleteEditor(
         IFACompleteEditor.VIEWER);

   private String instantSolution;

   FA originalFA;

   protected JTextField refield = new JTextField();

   private Fa2ReEditor viewer = new Fa2ReEditor();

   public FA2RE (String query) {
      this();
      String f = Query.getValue(query, "fa"); //$NON-NLS-1$
      if (f != null) {
         setFA(FASerializer.createFA(f));
      }
   }

   public FA2RE () {
      setFA(REPrefixDictionary.getInstance().getRandomFA(5));
      JEditorPane task = ExerciseUIFactory.createTaskPane(this);
      int CENTER = GridBagConstraints.CENTER;
      int BOTH = GridBagConstraints.BOTH;
      JPanel main = new JPanel(new GridBagLayout());
      main.setBackground(Color.white);
      int i = 0;
      main.add(task, new GridBagConstraints(0, i++, 1, 1, 1, 0, CENTER, BOTH,
            nullInsets, 0, 0));
      main.add(input, new GridBagConstraints(0, i++, 1, 1, 1, 0.4, CENTER,
            BOTH, nullInsets, 0, 0));
      main.add(new JSeparator(SwingConstants.HORIZONTAL),
            new GridBagConstraints(0, i++, 1, 1, 1, 0, CENTER, BOTH,
                  nullInsets, 0, 0));
      main.add(viewer, new GridBagConstraints(0, i++, 1, 1, 1, 0.6, CENTER,
            BOTH, nullInsets, 0, 0));
      JPanel r = new JPanel(new BorderLayout());
      r.setBackground(Color.white);
      refield.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
      r.add(new JLabel(" R = "), BorderLayout.WEST); //$NON-NLS-1$
      r.add(refield, BorderLayout.CENTER);
      main.add(r, new GridBagConstraints(0, i++, 1, 1, 1, 0, CENTER, BOTH,
            nullInsets, 0, 0));
      JEditorPane p = new JEditorPane();
      p.setEditable(false);
      p.addHyperlinkListener(new ExtendedHyperlinkListener());
      try {
         p.setPage(ClassLoader.getSystemResource(Messages
               .getString("FA2RE.file.control"))); //$NON-NLS-1$
      } catch (IOException e) {
         Debug.showException(e);
      }
      ExerciseUIFactory.createExerciseUI(main, p, notifications, this);
      refield.addKeyListener(new KeyListener());
      refield.addActionListener(new CheckAction());
   }

   public void check () {
      notifications.removeAll();
      if (instantSolution == null || instantSolution.equals("")) { //$NON-NLS-1$
         notifications.addErrorNotification(Shared.ERROR, Messages
               .getString("FA2RE.no_expression")); //$NON-NLS-1$
         return;
      }
      try {
         RegularExpression regexp = ABRegExpParser.createRE(instantSolution);
         try {
            if (faDet.isEquivalent(regexp.getFA())) {
               RegularExpression reop = REPrefixDictionary.getInstance().getRE(
                     regexp.getFA());
               if (reop != null
                     && regexp.getPrefixForm().length() > reop.getPrefixForm()
                           .length()) {
                  notifications.addCorrectNotification(Shared.CORRECT, Messages
                        .getString("FA2RE.equivalent_but") //$NON-NLS-1$
                        .replaceAll("\\$ANSWER", //$NON-NLS-1$
                              regexp.getInfixForm()).replaceAll("\\$SOLUTION", //$NON-NLS-1$
                              reop.getInfixForm()));
               } else {
                  notifications.addCorrectNotification(Shared.CORRECT, Messages
                        .getString("FA2RE.switch_to") //$NON-NLS-1$
                        .replaceAll("\\$EXPRESSION", //$NON-NLS-1$
                              regexp.getInfixForm()).replaceAll("\\$CLASS", //$NON-NLS-1$
                              RE2FA.class.getName()));
               }
            } else {
               notifications.addErrorNotification(Shared.ERROR, Messages
                     .getString("FA2RE.check_word") //$NON-NLS-1$
                     .replaceFirst(
                           "\\$WORD",//$NON-NLS-1$
                           
                           regexp.getFA().findWitnessForNonEquivalence(
                                 originalFA)));
            }
         } catch (FAException ex) {
            Debug.showException(ex);
         }
      } catch (REParserException ex) {
         notifications.addErrorNotification(Messages
               .getString("REParserException"), ex.getMessage()); //$NON-NLS-1$
      }
   }

   public FA getOriginalFA () {
      return originalFA;
   }

   protected void newTask () {
      Iterator iter;
      FSMStateInterface state;
      try {
         input.setFA(originalFA);
         viewer.setFA(fa);
         fmscontroller = (Fa2ReFSMController) viewer.getController();
         fmscontroller.getModel().addFSMModelListener(fmscontroller);
         fsmView = fmscontroller.getView();
         editorOps = new FSMEditorOperations(fmscontroller.getModel(),
               fmscontroller.getView());
         // add a new accepting end state E
         State endState = new State("E"); //$NON-NLS-1$
         editorOps.addState(endState);
         // add new epsilon transitions between all accepting states and the
         // new end state
         iter = fa.getAcceptingStates().iterator();
         while (iter.hasNext()) {
            state = (FSMStateInterface) iter.next();
            fa.notifyFSMModelListeners(FSMEvent.createAddTransitionEvent(state,
                  endState, new Epsilon()));
            state.setAccepting(false);
         }
         endState.setAccepting(true);
         // add a new start state S
         State startState = new State("S"); //$NON-NLS-1$
         FSMStateInterface oldStartState = fa.getStartState();
         editorOps.addStartState(startState);
         // add a epsilon transition from the new to the old start state
         fa.notifyFSMModelListeners(FSMEvent.createAddTransitionEvent(
               startState, oldStartState, new Epsilon()));
         startState.setStart(true);
         // arrange the fsmView again
         CircleLayouter.layout(fsmView);
         fsmView.fit();
         fsmView.repaint();
         fmscontroller.initForNewTask(notifications);
      } catch (Exception ex) {
         Debug.showException(ex);
      }
   }

   public void setFA (FA fa) {
      this.fa = fa;
      fa.setName(""); //$NON-NLS-1$
      fa.reLabelAllStates("q"); //$NON-NLS-1$
      try {
         fa.removeNotReachableStates();
         fa.removeTrapStates();
      } catch (FAException e) {}
      originalFA = (FA) fa.clone();
      originalFA.setName(""); //$NON-NLS-1$
      faDet = (FA) fa.clone();
      try {
         faDet.makeDetOpt();
      } catch (FAException e) {
         Debug.showException(e);
      }
      newTask();
   }

   public void setInstantSolution (String instantSolution) {
      this.instantSolution = instantSolution;
   }

   public String getTitle () {
      return Messages.getString("FA2RE.title"); //$NON-NLS-1$
   }

   public JComponent getPageUI () {
      return this;
   }

   public String getTask () {
      return Messages.getString("FA2RE.exercise"); //$NON-NLS-1$
   }
}