/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JPanel;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.rl.facts.FACT;
import ch.ethz.exorciser.rl.facts.FACTFactory;
import ch.ethz.exorciser.rl.re.ABRegExpParser;
import ch.ethz.exorciser.rl.re.REParserException;
import ch.ethz.exorciser.rl.re.RegularExpression;

/**
 * @author Vincent Tscherter
 * 
 * $Log: FACTExercise.java,v $
 * Revision 1.4  2004/09/23 16:08:08  root
 * *** empty log message ***
 *
 */
public class FACTExercise extends JPanel implements Exercise {

	Notifications notifications = new Notifications();

	FACT fact;

	IFACompleteEditor working = new IFACompleteEditor();

	RegularExpression re;

	String tasktext;

	String feedback;

	public void check() {
		notifications.removeAll();
		FA solution = re.getFA();
		FA answer = working.getFA();
		FAFAChecker.check(solution, answer, feedback, notifications);
	}

	public void solve() {
		notifications.removeAll();
		working.setFA((FA) re.getFA().clone());
	}

	FACTFactory factory;

	public void instructions() {
		notifications.removeAll();
		notifications.addHintNotification(Messages
				.getString("FAEDITOR.instructions.title"), //$NON-NLS-1$
				Messages.getString("FAEDITOR.instructions")); //$NON-NLS-1$
	}
	
	public void random(String query) throws IOException {
		if (factory == null) {
			factory = FACTFactory.createFactory(Messages
					.getString("FACTGenerator.file.templates")); //$NON-NLS-1$
		}
		notifications.removeAll();
		working.getFA().clear();
		String level = Query.getValue(query, "level"); //$NON-NLS-1$
		try {
			String type = Query.getValue(query, "type");
			if (level.equals("beginner")) { //$NON-NLS-1$
				fact = factory.createBeginnerFACT(type);
				feedback = Messages
						.getString("FACTExercise.file.feedback.beginner"); //$NON-NLS-1$
				tasktext = Messages
						.getString("FACTExercise.text.exercise.beginner").replaceAll("\\$TEXT", fact.getText()); //$NON-NLS-1$ //$NON-NLS-2$
				re = ABRegExpParser.createPrefixFormRE(fact.getLanguage());

			} else if (level.equals("advanced")) { //$NON-NLS-1$
				fact = factory.createAdvancedFACT();
				feedback = Messages
						.getString("FACTExercise.file.feedback.advanced"); //$NON-NLS-1$
				tasktext = Messages
						.getString("FACTExercise.text.exercise.advanced").replaceAll("\\$TEXT", fact.getText()); //$NON-NLS-1$ //$NON-NLS-2$

				re = ABRegExpParser.createPrefixFormRE(fact.getLanguage());
			} else if (level.equals("expert")) { //$NON-NLS-1$
				// TODO
				feedback = Messages
						.getString("FACTExercise.file.feedback.advanced"); //$NON-NLS-1$
				tasktext = "<p>not implemented yet ..."; //$NON-NLS-1$
			}
		} catch (REParserException e) {
			Debug.showException(e);
		}
		if (task == null) {
			task = ExerciseUIFactory.createTaskPane(this);
		} else {
			ExerciseUIFactory.resetTaskText(task, this);
		}
	}

	public FACTExercise() throws IOException {
		this("level=beginner"); //$NON-NLS-1$
	}

	JEditorPane task;

	public FACTExercise(String query) throws IOException {
		random(query);
		working.getFA().setAlphabet(re.getFA().getAlphabet());
		JPanel main = new JPanel(new BorderLayout());
		main.add(task, BorderLayout.NORTH);
		main.add(working, BorderLayout.CENTER);
		JEditorPane ctrl = new JEditorPane();
		ctrl.setEditable(false);
		ctrl.addHyperlinkListener(new ExtendedHyperlinkListener());
		try {
			ctrl.setPage(ClassLoader.getSystemResource(Messages
					.getString("FACTExercise.file.ctrl"))); //$NON-NLS-1$
		} catch (IOException e) {
			Debug.showException(e);
		}
		ExerciseUIFactory.createExerciseUI(main, ctrl, notifications, this);
	}

	public JComponent getPageUI() {
		return this;
	}

	public String getTitle() {
		return Messages.getString("FACTExercise.text.titel"); //$NON-NLS-1$
	}

	public String getTask() {
		return tasktext;//
	}

}