/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.fsmgui.ComponentRenderer;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;
import ch.ethz.exorciser.fsmgui.FSMEvent;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FAException;
import ch.ethz.exorciser.ifa.FAGenerator;
import ch.ethz.exorciser.ifa.FASerializer;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.ifa.IFAFSMController;
import ch.ethz.exorciser.ifa.State;
import ch.ethz.exorciser.rl.re.dictionary.REPrefixDictionary;

/**
 * @author Vincent Tscherter
 */
public class Generic extends JPanel implements Exercise {

    static {
        REPrefixDictionary.getInstance().load(12);
    }

	public void instructions() {
		notifications.removeAll();
		notifications.addHintNotification(Messages
				.getString("FAEDITOR.instructions.title"), //$NON-NLS-1$
				Messages.getString("FAEDITOR.instructions")); //$NON-NLS-1$
	}
	
    public void check() {
        notifications.removeAll();
        try {
            FA foo = (FA) A.getFA().clone();
            if (Generic.this.type == KLEENE_STAR) {
                foo.nDetStar();
            } else if (Generic.this.type == UNION) {
                foo.nDetUnion(B.getFA());
            } else if (Generic.this.type == CONCATENATION) {
                foo.nDetConcatenation(B.getFA());
            } else if (Generic.this.type == NFA2DFA) {
                check(foo, working.getFA(), DETERMINISTIC);
                return;
            } else if (Generic.this.type == REMOVE_EPSILON) {
                check(foo, working.getFA(), EPSILONFREE);
                return;
            }
            check(foo, working.getFA(), DEFAULT);
        } catch (FAException e) {
            Debug.showException(e);
        }

    }

    private final class OKListener implements ActionListener {

        public void actionPerformed(ActionEvent event) {
            if (Generic.this.type == UNION
                    || Generic.this.type == CONCATENATION) {
                FA a = editorA.getFA();
                FA b = editorB.getFA();
                boolean ok = true;
                if (a.getStateSet().size() < 1) {
                    JOptionPane.showMessageDialog(null, Messages
                            .getString("Generic.editfa.more_a"), //$NON-NLS-1$
                            Messages.getString("Generic.editfa.more.title"), //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                    ok = false;
                }
                if (ok && b.getStateSet().size() < 1) {
                    JOptionPane.showMessageDialog(null, Messages
                            .getString("Generic.editfa.more_b"), //$NON-NLS-1$
                            Messages.getString("Generic.editfa.more.title"), //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                    ok = false;
                }
                if (ok && !b.getAlphabet().equals(a.getAlphabet())) {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    Messages
                                            .getString("Generic.edit.inconsitent"), //$NON-NLS-1$
                                    //$NON-NLS-1$
                                    Messages
                                            .getString("Generic.edit.inconsitent.title"), //$NON-NLS-1$
                                    JOptionPane.OK_OPTION);
                    ok = false;
                }
                if (ok) {
                    setFA(a, b);
                    jd.dispose();
                }
            } else {
                FA fa = editorA.getFA();
                if (fa.getStateSet().size() < 1) {
                    JOptionPane.showMessageDialog(null, Messages
                            .getString("Generic.editfa.more"), //$NON-NLS-1$
                            Messages.getString("Generic.editfa.more.title"), //$NON-NLS-1$
                            JOptionPane.OK_OPTION);
                } else {
                    setFA(fa);
                    jd.dispose();
                }
            }
        }
    }

    public void edit() {
        notifications.removeAll();
        repaint();
        jd = new JDialog((Frame) null, Messages
                .getString("Generic.label.edit_fa"), true); //$NON-NLS-1$
        jd.getContentPane().setLayout(new BorderLayout());
        JButton ok = new JButton(Shared.OKAY);
        ok.addActionListener(new OKListener());
        JButton cancel = new JButton(new AbstractAction(Shared.CANCEL) {

            public void actionPerformed(ActionEvent e) {
                jd.dispose();
            }
        });
        JPanel buttons = new JPanel(new GridLayout(1, 2));
        buttons.add(ok);
        buttons.add(cancel);
        jd.getContentPane().add(buttons, BorderLayout.SOUTH);
        editorA.setFA((FA) A.getFA().clone());
        if (Generic.this.type == UNION || Generic.this.type == CONCATENATION) {
            editorB.setFA((FA) B.getFA().clone());
            int CENTER = GridBagConstraints.CENTER;
            int BOTH = GridBagConstraints.BOTH;
            JPanel x = new JPanel(new GridBagLayout());
            x.setBackground(Color.white);
            x.add(new JLabel("A"), //$NON-NLS-1$
                    new GridBagConstraints(0, 0, 1, 1, 0, 0, CENTER, BOTH,
                            new Insets(0, 0, 0, 0), 0, 0));
            x.add(editorA, new GridBagConstraints(1, 0, 1, 1, 1, 1, CENTER,
                    BOTH, new Insets(0, 0, 0, 0), 0, 0));
            x.add(new JLabel("B"), //$NON-NLS-1$
                    new GridBagConstraints(2, 0, 1, 1, 0, 0, CENTER, BOTH,
                            new Insets(0, 0, 0, 0), 0, 0));
            x.add(editorB, new GridBagConstraints(3, 0, 1, 1, 1, 1, CENTER,
                    BOTH, new Insets(0, 0, 0, 0), 0, 0));
            jd.getContentPane().add(x, BorderLayout.CENTER);
        } else {
            jd.getContentPane().add(editorA, BorderLayout.CENTER);
        }
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        jd.setBounds((screen.width - 500) / 2, (screen.height - 400) / 2, 500,
                400);

        jd.setVisible(true);
    }

    public void random(String query) {
        String level = Query.getValue(query, "level"); //$NON-NLS-1$
        size = 3;
        if (level.equals("beginner")) { //$NON-NLS-1$
            size = 3;
        } else if (level.equals("advanced")) { //$NON-NLS-1$
            size = 5;
        } else if (level.equals("expert")) { //$NON-NLS-1$
            size = 8;
        }
        if (Generic.this.type == NFA2DFA) {
            setFA(randomNFA(size, Alphabet.createABeAlphabet()));
        } else if (Generic.this.type == REMOVE_EPSILON) {
            setFA(randomEpsilonNFA());
        } else if (Generic.this.type == KLEENE_STAR) {
            setFA(REPrefixDictionary.getInstance().getRandomFA(size));
        } else {
            setFA(REPrefixDictionary.getInstance().getRandomFA(size),
                    REPrefixDictionary.getInstance().getRandomFA(size));
        }
        notifications.removeAll();

    }

    public void solve() {
        notifications.removeAll();
        revalidate();
        repaint();
        try {
            FA foo = (FA) A.getFA().clone();
            foo.getAlphabet().add(Alphabet.EPSILON);
            if (Generic.this.type == KLEENE_STAR) {
                foo.nDetStar();
            } else if (Generic.this.type == UNION) {
                foo.nDetUnion(B.getFA());
            } else if (Generic.this.type == CONCATENATION) {
                foo.nDetConcatenation(B.getFA());
            } else if (Generic.this.type == NFA2DFA) {
                foo.makeDetOpt();
                foo.reLabelAllStates("s"); //$NON-NLS-1$
            } else if (Generic.this.type == REMOVE_EPSILON) {
                foo.removeEpsilon();
                //foo.reLabelAllStates("s");
            }
            working.setFA(foo);
        } catch (FAException e) {
            Debug.showException(e);
        }
        //		}
    }

    public static final int CONCATENATION = 2;

    static final int DEFAULT = 0;

    static final int DETERMINISTIC = 1;

    static final int EPSILONFREE = 2;

    public static final int KLEENE_STAR = 0;

    //// static int[] levelvalues = { 3, 5, 8 };
    public static final int NFA2DFA = 3;

    public static final int REMOVE_EPSILON = 4;

    public static final int UNION = 1;

    IFACompleteEditor A, B, working;

    IFACompleteEditor editorA = new IFACompleteEditor(),
            editorB = new IFACompleteEditor();

    Notifications notifications;

    int size = 3;

    JDialog jd;

    int type;

    public Generic(String query) {
        String t = Query.getValue(query, "type"); //$NON-NLS-1$
        if (t != null) {
            if (t.equals("concatenation")) { //$NON-NLS-1$
                init(CONCATENATION);
                String faa = Query.getValue(query, "faA"); //$NON-NLS-1$
                String fab = Query.getValue(query, "faB"); //$NON-NLS-1$
                if (faa != null && fab != null) {
                    setFA(FASerializer.createFA(faa), FASerializer
                            .createFA(fab));
                }
            } else if (t.equals("union")) { //$NON-NLS-1$
                init(UNION);
                String faa = Query.getValue(query, "faA"); //$NON-NLS-1$
                String fab = Query.getValue(query, "faB"); //$NON-NLS-1$
                if (faa != null && fab != null) {
                    setFA(FASerializer.createFA(faa), FASerializer
                            .createFA(fab));
                }
            } else if (t.equals("kleene_star")) { //$NON-NLS-1$
                init(KLEENE_STAR);
                String fa = Query.getValue(query, "fa"); //$NON-NLS-1$
                if (fa != null) {
                    setFA(FASerializer.createFA(fa));
                }
            } else if (t.equals("nfa2dfa")) { //$NON-NLS-1$
                init(NFA2DFA);
                String fa = Query.getValue(query, "fa"); //$NON-NLS-1$
                if (fa != null) {
                    setFA(FASerializer.createFA(fa));
                }
            } else if (t.equals("remove_epsilon")) { //$NON-NLS-1$
                init(REMOVE_EPSILON);
                String fa = Query.getValue(query, "fa"); //$NON-NLS-1$
                if (fa != null) {
                    setFA(FASerializer.createFA(fa));
                }
            } else {
                init(Generic.DEFAULT);
            }
        } else {
            init(Generic.DEFAULT);
        }
    }

    public Generic(int type) {
        init(type);
    }

    private void init(int type_) {
        this.type = type_;
        working = new IFACompleteEditor();
        working.getFA().setAlphabet(
                REPrefixDictionary.getInstance().getRandomRE().getFA()
                        .getAlphabet());
        A = new IFACompleteEditor(IFACompleteEditor.VIEWER);
        B = new IFACompleteEditor(IFACompleteEditor.VIEWER);
        notifications = new Notifications();

        int CENTER = GridBagConstraints.CENTER;
        int BOTH = GridBagConstraints.BOTH;
        JEditorPane task = ExerciseUIFactory.createTaskPane(this);
        JPanel ab = new JPanel(new GridBagLayout());
        ab.setBackground(Color.white);
        ab.add(new JLabel("A"), //$NON-NLS-1$
                new GridBagConstraints(0, 0, 1, 1, 0, 0, CENTER, BOTH,
                        new Insets(0, 0, 0, 0), 0, 0));
        ab.add(A, new GridBagConstraints(1, 0, 1, 1, 1, 1, CENTER, BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        if (type_ == UNION || type_ == CONCATENATION) {
            ab.add(new JLabel("B"), //$NON-NLS-1$
                    new GridBagConstraints(2, 0, 1, 1, 0, 0, CENTER, BOTH,
                            new Insets(0, 0, 0, 0), 0, 0));
            ab.add(B, new GridBagConstraints(3, 0, 1, 1, 1, 1, CENTER, BOTH,
                    new Insets(0, 0, 0, 0), 0, 0));
        }
        JPanel main = new JPanel(new GridBagLayout());
        main.setBackground(Color.white);
        main.add(task, new GridBagConstraints(0, 0, 1, 1, 1, 0, CENTER, BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        main.add(ab, new GridBagConstraints(0, 1, 1, 1, 1, 0.4, CENTER, BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        main.add(new JSeparator(SwingConstants.HORIZONTAL),
                new GridBagConstraints(0, 2, 1, 1, 1, 0, CENTER, BOTH,
                        new Insets(0, 0, 0, 0), 0, 0));
        main.add(working, new GridBagConstraints(0, 3, 1, 1, 1, 0.6, CENTER,
                BOTH, new Insets(0, 0, 0, 0), 0, 0));

        JEditorPane p = new JEditorPane();
        p.setEditable(false);
        p.addHyperlinkListener(new ExtendedHyperlinkListener());

        try {
            p.setPage(ClassLoader.getSystemResource(Messages
                    .getString("Generic.file.control"))); //$NON-NLS-1$
        } catch (IOException e) {
            Debug.showException(e);
        }

        ExerciseUIFactory.createExerciseUI(main, p, notifications, this);

        if (type_ == NFA2DFA) {
            setFA(randomNFA(size, Alphabet.createABeAlphabet()));
        } else if (type_ == REMOVE_EPSILON) {
            setFA(randomEpsilonNFA());
        } else if (type_ == KLEENE_STAR) {
            setFA(REPrefixDictionary.getInstance().getRandomFA(size));
        } else {
            setFA(REPrefixDictionary.getInstance().getRandomFA(size),
                    REPrefixDictionary.getInstance().getRandomFA(size));
        }
    }

    void check(FA solution, FA answer, int requires) {
        String s = answer.findWitnessForNonEquivalence(solution);
        if (s == null) {
            if (answer.getAllStates().size() != 0) {
                try {
                    FA foo = (FA) answer.clone();
                    if (foo.getFAType().equals(FA.UFA)) {
                        foo.complete();
                    }
                    if (requires == EPSILONFREE
                            && answer.getFAType().equals(FA.GFA)) {
                        notifications.addWarningNotification(Messages
                                .getString("Generic.feedback.incomplete"), //$NON-NLS-1$
                                Messages.getString("Generic.feedback.eps") //$NON-NLS-1$
                                        .replaceAll("\\$EPSILON", //$NON-NLS-1$
                                                Alphabet.EPSILON.toString())); //$NON-NLS-1$ //$NON-NLS-2$
                        // -- start mark epsilon transitions [vt 20040114]
                        Hashtable ht = new Hashtable();
                        Iterator stateit = answer.getStateSet().iterator();
                        while (stateit.hasNext()) {
                            State from = (State) stateit.next();
                            Iterator to = from
                                    .getNextStateSet(Alphabet.EPSILON)
                                    .iterator();
                            while (to.hasNext()) {
                                Object[] trans = { from, (State) to.next()};
                                ht.put(trans, ComponentRenderer.MARK_RED);
                            }
                        }
                        answer.notifyFSMModelListeners(FSMEvent
                                .createMarkEvent(ht));
                        // -- end mark epsiolon transitions
                    } else if (requires == EPSILONFREE
                            && answer.getFAType().equals(FA.NFA)) {
                        notifications
                                .addCorrectNotification(Shared.CORRECT, ""); //$NON-NLS-2$ //$NON-NLS-1$ //$NON-NLS-1$
                    } else if (requires == DETERMINISTIC
                            && !foo.isDeterministic()) {
                        notifications
                                .addWarningNotification(
                                        Messages
                                                .getString("Generic.feedback.incomplete"), //$NON-NLS-1$
                                        //$NON-NLS-1$
                                        Messages
                                                .getString("Generic.feedback.equiv_but")); //$NON-NLS-1$
                    } else {

                        notifications
                                .addCorrectNotification(Shared.CORRECT, ""); //$NON-NLS-1$
                    }
                } catch (FAException e) {
                    Debug.showException(e);
                }
            } else {
                notifications.addWarningNotification(Messages
                        .getString("Generic.feedback.nostates.title"), //$NON-NLS-1$
                        Messages.getString("Generic.feedback.nostates")); //$NON-NLS-1$
            }
        } else {
            notifications.addErrorNotification(Messages
                    .getString("Generic.feedback.wrong"), //$NON-NLS-1$
                    Messages.getString("Generic.feedback.notequiv").replaceAll( //$NON-NLS-1$
                            "\\$WORD", s)); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    public void rewind() {
        if (working.getController() instanceof IFAFSMController) {
            FSMEditorOperations eo = ((IFAFSMController) working
                    .getController()).editorOps;
            while (!eo.undoListEmpty()
                    && working.getFA().findWitnessForNonEquivalence(A.getFA()) != null) {
                {
                    ((IFAFSMController) working.getController()).editorOps
                            .undo();

                }
                notifications.removeAll();
            }
        }
    }

    protected FA randomEpsilonNFA() {
        int nofState = size;
        FA rnd = null;
        do {
            try {
                rnd = FAGenerator.getRandomGFA(nofState + 2,
                        (int) (2.5 * nofState), 0.4, Alphabet
                                .createABeAlphabet());
                rnd.removeNotReachableStates();
                rnd.removeTrapStates();
            } catch (FAException e) {
                Debug.showException(e);
            }
        } while (rnd == null || rnd.getAllStates().size() != nofState
                || !rnd.getFAType().equals(FA.GFA)
                || rnd.getAcceptingStates().size() == rnd.getStateSet().size()
                || rnd.getAcceptingStates().size() == 0
                || rnd.isLanguageEmpty());
        rnd.reLabelAllStates("q"); //$NON-NLS-1$
        rnd.setName(""); //$NON-NLS-1$
        return rnd;
    }

    public static FA randomNFA(int size, Alphabet ab) {
        int nofState = size;
        FA rnd = null;
        try {
            int loopLimit = 0;
            do {
                loopLimit++;
                try {
                    rnd = FAGenerator.getRandomGFA(nofState + 2,
                            (int) (2.5 * nofState), 0.4, ab);
                    rnd.removeEpsilon();
                    rnd.removeNotReachableStates();
                    rnd.removeTrapStates();
                } catch (FAException e) {
                    Debug.showException(e);
                }
            } while (loopLimit < 1000
                    && (rnd == null
                            || rnd.getAllStates().size() != nofState
                            || (!rnd.getFAType().equals(FA.GFA) && !rnd
                                    .getFAType().equals(FA.NFA))
                            || rnd.isLanguageFinite() || rnd.isLanguageEmpty() || rnd
                            .isLanguageAll()));
            rnd.reLabelAllStates("q"); //$NON-NLS-1$
            rnd.setName(""); //$NON-NLS-1$
        } catch (FAException e) {
            Debug.showException(e);
        }
        return rnd;

    }

    protected void setFA(FA a) {
        A.setFA((FA) a.clone());
        if (type == REMOVE_EPSILON || type == KLEENE_STAR) {
            working.setFA((FA) a.clone());
        } else if (type == NFA2DFA) {
            FA foo = (FA) a.clone();
            foo.clear();
            working.setFA(foo);
        }
    }

    protected void setFA(FA a, FA b) {
        A.setFA((FA) a.clone());
        B.setFA((FA) b.clone());
        FA foo = (FA) b.clone();
        foo.clear();
        working.setFA(foo);
    }

    public JComponent getPageUI() {
        return this;
    }

    public String getTitle() {
        switch (type) {
        case REMOVE_EPSILON:
            return Messages.getString("Generic.exercise.epsilon.title") //$NON-NLS-1$
                    .replaceAll("\\$EPSILON", Alphabet.EPSILON.toString()); //$NON-NLS-1$ //$NON-NLS-2$
        case UNION:
            return Messages.getString("Generic.exercise.union.title"); //$NON-NLS-1$
        case KLEENE_STAR:
            return Messages.getString("Generic.exercise.kleene.title"); //$NON-NLS-1$
        case CONCATENATION:
            return Messages.getString("Generic.exercise.catenation.title"); //$NON-NLS-1$
        case NFA2DFA:
            return Messages.getString("Generic.exercise.nfa2dfa.title"); //$NON-NLS-1$
        default:
            return ""; //$NON-NLS-1$
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ch.ethz.exorciser.Exercise#getTask()
     */
    public String getTask() {
        switch (type) {
        case REMOVE_EPSILON:
            return Messages.getString("Generic.exercise.epsilon").replaceAll( //$NON-NLS-1$
                    "\\$EPSILON", Alphabet.EPSILON.toString()); //$NON-NLS-1$ //$NON-NLS-2$
        case UNION:
            return Messages.getString("Generic.exercise.union"); //$NON-NLS-1$
        case KLEENE_STAR:
            return Messages.getString("Generic.exercise.kleene"); //$NON-NLS-1$
        case CONCATENATION:
            return Messages.getString("Generic.exercise.catenation"); //$NON-NLS-1$
        case NFA2DFA:
            return Messages.getString("Generic.exercise.nfa2dfa"); //$NON-NLS-1$
        default:
            return ""; //$NON-NLS-1$
        }
    }

}