/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.fsmgui.ComponentRenderer;
import ch.ethz.exorciser.fsmgui.FSMEvent;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FAException;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.ifa.NonMinFAGenerator;
import ch.ethz.exorciser.ifa.State;

/**
 * @author Vincent Tscherter
 */
public class LowerBoundExercise extends JPanel implements Exercise {

	Notifications notifications = new Notifications();

	IFACompleteEditor faviewer = new IFACompleteEditor(IFACompleteEditor.VIEWER);

	Notifications working = new Notifications();

	JTextField ctf;

	static Border DEFAULT_BORDER = (new JTextField()).getBorder();

	boolean solving = false;

	MouseAdapter ma = new MouseAdapter() {
		public void mouseEntered(MouseEvent e) {
			JTextField tf = (JTextField) e.getSource();
			ctf = tf;
			if (words.contains(tf)) {
				tf.setBorder(BorderFactory.createLineBorder(Color.red, 3));
			}
			updateFAmarks();
		}

		public void mouseExited(MouseEvent e) {
			JTextField tf = (JTextField) e.getSource();
			ctf = null;
			tf.setBorder(DEFAULT_BORDER);
			updateFAmarks();
		}
	};

	CaretListener cl = new CaretListener() {
		public void caretUpdate(CaretEvent e) {
			JTextField tf = (JTextField) e.getSource();
			if (ctf == tf) {
				// tf.setBorder(BorderFactory.createLineBorder(Color.red, 3));
				updateFAmarks();
			}
		}
	};

	void updateFAmarks() {
		faviewer.getFA().notifyFSMModelListeners(
				FSMEvent.createUnMarkAllEvent());
		Hashtable marks = new Hashtable();
		if (words.contains(ctf)) {
			Iterator iterator = faviewer.getFA().getStatesAfter(ctf.getText())
					.iterator();
			while (iterator.hasNext()) {
				marks.put(iterator.next(), ComponentRenderer.MARK_RED);
			}
		}
		faviewer.getFA().notifyFSMModelListeners(
				FSMEvent.createMarkEvent(marks));
	}

	FA minfa;

	List words = new ArrayList();

	// List zeugen = new ArrayList();

	String tasktext;

	public void check() {
		notifications.removeAll();
		// int empty = 0;
		// check pairwise different
		for (int i = 0; i < words.size(); i++) {
			for (int j = i + 1; j < words.size(); j++) {
				String wi = ((JTextField) words.get(i)).getText();
				String wj = ((JTextField) words.get(j)).getText();
				if (wi.equals(wj)) {
					notifications
							.addErrorNotification(
									Messages
											.getString("LowerBoundExercise.feedback.doubeltte.title"), //$NON-NLS-1$
									Messages
											.getString(
													"LowerBoundExercise.feedback.doubeltte").replaceAll("\\$I", i + "").replaceAll("\\$J", j + "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$
					return;
				}
			}
		}
		// check alphabet
		for (int i = 0; i < words.size(); i++) {
			String w = ((JTextField) words.get(i)).getText();
			if (minfa.getStatesAfter(w).size() == 0) {
				notifications
						.addErrorNotification(
								Messages
										.getString("LowerBoundExercise.feedback.malformed.title"), //$NON-NLS-1$
								Messages
										.getString(
												"LowerBoundExercise.feedback.malformed").replaceAll("\\$I", i + "")); //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
				return;
			}
		}
		// check pairwise non-equivalence
		for (int i = 0; i < words.size(); i++) {
			for (int j = i + 1; j < words.size(); j++) {
				String wi = ((JTextField) words.get(i)).getText();
				String wj = ((JTextField) words.get(j)).getText();
				Set si = minfa.getStatesAfter(wi);
				Set sj = minfa.getStatesAfter(wj);
				if (si.equals(sj)) {
					notifications
							.addErrorNotification(
									Messages
											.getString("LowerBoundExercise.feedback.illegeal_matching.title"), //$NON-NLS-1$
									Messages
											.getString(
													"LowerBoundExercise.feedback.illegeal_matching").replaceAll("\\$I", i + "").replaceAll("\\$J", j + "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
					return;
				}
			}
		}
		notifications.addCorrectNotification(Messages
				.getString("LowerBoundExercise.feedback.well_done"), ""); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public void solve() {
		solve(false);
	}

	public void solve(boolean stepbystep) {
		solving = true;
		notifications.removeAll();

		// remove malformed words
		for (int i = 0; i < words.size(); i++) {
			JTextField tf = (JTextField) words.get(i);
			if (minfa.getStatesAfter(tf.getText()).size() == 0) {
				tf.setText("<new>"); //$NON-NLS-1$
				// tf.setText("");
			}
		}
		// remove illegal maching pairs
		for (int i = 0; i < words.size(); i++) {
			for (int j = i + 1; j < words.size(); j++) {
				JTextField tfi = ((JTextField) words.get(i));
				JTextField tfj = ((JTextField) words.get(j));
				Set si = minfa.getStatesAfter(tfi.getText());
				Set sj = minfa.getStatesAfter(tfj.getText());
				if (si.equals(sj)) {
					tfj.setText("<new>"); //$NON-NLS-1$
				}
			}
		}
		// add new words

		Map map = new HashMap();
		map.put(minfa.getStartState(), ""); //$NON-NLS-1$
		List l = new ArrayList();
		l.add(minfa.getStartState());
		int pos = 0;

		while (pos < l.size()) {

			State s = (State) l.get(pos);
			Iterator iterator = minfa.getAlphabet().iterator();
			while (iterator.hasNext()) {
				Character a = (Character) iterator.next();
				if (!a.equals(Alphabet.EPSILON)) {
					State next = s.getNextState(a);
					if (!l.contains(next)) {
						l.add(next);
						map.put(next, map.get(s).toString() + a);
					}
				}

			}
			pos++;
		}
		// filling in
		List list = new ArrayList();
		for (int i = 0; i < words.size(); i++) {
			JTextField tf = ((JTextField) words.get(i));
			if (!tf.getText().equals("<new>")) { //$NON-NLS-1$
				list.add(minfa.getStatesAfter(tf.getText()).iterator().next());
			}
		}
		Iterator keys = map.keySet().iterator();
		for (int i = 0; i < words.size(); i++) {
			JTextField tf = ((JTextField) words.get(i));
			if (tf.getText().equals("<new>")) { //$NON-NLS-1$
				Object key = keys.next();
				while (list.contains(key)) {
					key = keys.next();
				}
				tf.setText(map.get(key).toString());
				list.add(key);
			}
		}
		solving = false;
	}

	public void random(String query) {
		notifications.removeAll();
		String level = Query.getValue(query, "level"); //$NON-NLS-1$
		// int x = 0;
		try {
			if (level.equals("advanced")) { //$NON-NLS-1$
				NonMinFAGenerator.minimalNofStates = 5;
				faviewer.setFA(NonMinFAGenerator.generate());
			} else if (level.equals("expert")) { //$NON-NLS-1$
				faviewer.setFA(Generic.randomNFA((int) (Math.random() * 3) + 4,
						Alphabet.create01eAlphabet()));
			} else {
				NonMinFAGenerator.minimalNofStates = 5;
				faviewer.setFA(NonMinFAGenerator.generate());
				faviewer.getFA().detMinimize();
			}
			init();
		} catch (FAException e) {
			Debug.showException(e);
		}
	}

	/**
	 * @throws FAException
	 */
	void init() throws FAException {
		int x;
		faviewer.getFA().setName("A"); //$NON-NLS-1$
		minfa = (FA) faviewer.getFA().clone();
		minfa.makeDetOpt();
		minfa.detMinimize();
		x = minfa.nOfStates();
		tasktext = Messages
				.getString("LowerBoundExercise.exercise.text").replaceAll("\\$X", x + "") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				.replaceAll("\\$MINUSONE", (x - 1) + "") //$NON-NLS-1$ //$NON-NLS-2$
				.replaceAll("\\$NOFWORDS", (x * (x - 1) / 2) + ""); //$NON-NLS-1$ //$NON-NLS-2$

		if (task == null) {
			task = ExerciseUIFactory.createTaskPane(this);
		} else {
			ExerciseUIFactory.resetTaskText(task, this);
		}
		// words
		words.clear();
		working.removeAll();
		working.setLayout(new GridBagLayout());
		for (int i = 0; i < x; i++) {
			JEditorPane ep = new JEditorPane("text/html", "<html><body>w<sub>" //$NON-NLS-1$ //$NON-NLS-2$
					+ i + "</sub></body></html>"); //$NON-NLS-1$
			ep.setEditable(false);
			JTextField tf = new JTextField();
			tf.addMouseListener(ma);
			tf.addCaretListener(cl);
			words.add(tf);
			working.add(ep, new GridBagConstraints(0, i, 1, 1, 0, 0,
					GridBagConstraints.CENTER, GridBagConstraints.BOTH,
					new Insets(0, 0, 0, 0), 0, 0));
			working.add(tf, new GridBagConstraints(1, i, 1, 1, 1, 0,
					GridBagConstraints.CENTER, GridBagConstraints.BOTH,
					new Insets(0, 0, 0, 0), 0, 0));
		}

		revalidate();
		repaint();
	}

	public LowerBoundExercise() {
		this("level=beginner"); //$NON-NLS-1$
	}

	JEditorPane task;

	public LowerBoundExercise(String query) {
		random(query);
		JPanel main = new JPanel(new BorderLayout());
		main.add(task, BorderLayout.NORTH);

		working.setBorder(new EmptyBorder(5, 5, 5, 5));
		working.setOpaque(false);
		JScrollPane srp = new JScrollPane(working);
		srp.getViewport().setOpaque(true);
		srp.getViewport().setBackground(Color.white);

		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, faviewer,
				srp);
		sp.setResizeWeight(0.8);
		main.add(sp, BorderLayout.CENTER);

		JEditorPane ctrl = new JEditorPane();
		ctrl.setEditable(false);
		ctrl.addHyperlinkListener(new ExtendedHyperlinkListener());
		try {
			ctrl.setPage(ClassLoader.getSystemResource(Messages
					.getString("LowerBoundExercise.file.ctrl"))); //$NON-NLS-1$
		} catch (IOException e) {
			Debug.showException(e);
		}
		ExerciseUIFactory.createExerciseUI(main, ctrl, notifications, this);
	}

	public JComponent getPageUI() {
		return this;
	}

	public String getTitle() {
		return Messages.getString("LowerBoundExercise.exercise.title"); //$NON-NLS-1$
	}

	public String getTask() {
		return tasktext;//
	}

	JDialog jd;

	IFACompleteEditor editor = new IFACompleteEditor();

	public void edit() {
		try {
			notifications.removeAll();
			jd = new JDialog((Frame) null, Messages
					.getString("LowerBoundExercise.edit.title"), true); //$NON-NLS-1$
			jd.getContentPane().setLayout(new BorderLayout());
			JButton ok = new JButton(Shared.OKAY);
			ok.addActionListener(new OKListener());
			JButton cancel = new JButton(new AbstractAction(Shared.CANCEL) {
				public void actionPerformed(ActionEvent e) {
					jd.dispose();
				}
			});
			JPanel buttons = new JPanel(new GridLayout(1, 2));
			buttons.add(ok);
			buttons.add(cancel);
			jd.getContentPane().add(buttons, BorderLayout.SOUTH);
			editor.setFA((FA) faviewer.getFA().clone());
			jd.getContentPane().add(editor, BorderLayout.CENTER);
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			jd.setBounds((screen.width - 500) / 2, (screen.height - 400) / 2,
					500, 400);
			jd.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}



	private final class OKListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			if (editor.getFA().getStateSet().size() < 2) {
				JOptionPane
						.showMessageDialog(
								null,
								Messages
										.getString("LowerBoundExercise.edit.error.nostates"), //$NON-NLS-1$
								Messages
										.getString("LowerBoundExercise.edit.error.nofa"), JOptionPane.OK_OPTION); //$NON-NLS-1$
			} else {
				faviewer.setFA(editor.getFA());
				try {
					init();
				} catch (FAException e) {
					Debug.showException(e);
				}
				jd.dispose();
			}
		}
	}

}