/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FAException;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.ifa.NonMinFAGenerator;
import ch.ethz.exorciser.rl.minfa.TableChecker;
import ch.ethz.exorciser.rl.minfa.TableFAs;
import ch.ethz.exorciser.rl.minfa.TableModel;
import ch.ethz.exorciser.rl.minfa.TableView;
import ch.ethz.exorciser.rl.minfa.TwinFAView;


/**
 * @author Vincent Tscherter
 */
public class MinFA extends JPanel implements Exercise {

    public JComponent getPageUI() {
        return this;
    }

    public void solve() {
        checker.solveTable();
    }

    public void next() {
        checker.refineTable();
    }

    public void reset() {
        tablemodel.clear();
    }

    public void check() {
        notifications.removeAll();
        checker.checkTable();
    }

    public void edit() {
        notifications.removeAll();
        jd = new JDialog((Frame) null, Messages
                .getString("MinFA.label.edit_dfa"), true); //$NON-NLS-1$
        JPanel buttons = new JPanel(new GridLayout(1, 2));
        buttons.add(new JButton(okAction));
        buttons.add(new JButton(cancelAction));
        jd.getContentPane().setLayout(new BorderLayout());
        jd.getContentPane().add(buttons, BorderLayout.SOUTH);
        jd.getContentPane().add(faeditor, BorderLayout.CENTER);
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        jd.setBounds((screen.width - 500) / 2, (screen.height - 400) / 2, 500,
                400);
        faeditor.setFA((FA) tablemodel.getFA().clone());
        jd.setVisible(true);
    }

    private Action cancelAction = new AbstractAction(Shared.CANCEL) {//$NON-NLS-1$

        public void actionPerformed(ActionEvent e) {
            jd.dispose();
        }
    };

    private Action okAction = new AbstractAction(Shared.OKAY) {//$NON-NLS-1$

        public void actionPerformed(ActionEvent event) {
            FA fa = faeditor.getFA();
            try {
                boolean ok = true;
                if (ok && fa.getStateSet().size() < 2) {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    Messages
                                            .getString("MinFA.feedback.fa_contains_less_than_two_states"), //$NON-NLS-1$
                                    Messages
                                            .getString("MinFA.feedback.title.fa_contains_less_than_two_states"), //$NON-NLS-1$
                                    JOptionPane.OK_OPTION);
                    ok = false;
                }
                if (ok && !fa.isDeterministic()) {
                    if (JOptionPane
                            .showConfirmDialog(
                                    null,
                                    Messages
                                            .getString("MinFA.feedback.fa_nondet"), //$NON-NLS-1$
                                    Messages
                                            .getString("MinFA.feedback.title.fa_nondet"), //$NON-NLS-1$
                                    JOptionPane.CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                        fa.makeDetOpt();
                        fa.mergeTrapStates();
                        fa.removeNotReachableStates();
                    } else {
                        ok = false;
                    }
                }
                if (ok && fa.getNotReachableStates().size() > 0) {
                    if (JOptionPane
                            .showConfirmDialog(
                                    null,
                                    Messages
                                            .getString("MinFA.feeback.unreachable_states"), //$NON-NLS-1$

                                    Messages
                                            .getString("MinFA.feeback.title.unreachable_states"), //$NON-NLS-1$
                                    JOptionPane.CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                        fa.removeNotReachableStates();
                    } else {
                        ok = false;
                    }
                }
                if (ok && fa.getStateSet().size() < 2) {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    Messages
                                            .getString("MinFA.feedback.fa_contains_less_than_two_states"), //$NON-NLS-1$

                                    Messages
                                            .getString("MinFA.feedback.title.fa_contains_less_than_two_states"), //$NON-NLS-1$
                                    JOptionPane.OK_OPTION);
                    ok = false;
                }

                if (ok) {
                    fa.getAlphabet().remove(Alphabet.EPSILON);
                    tablemodel.setFA(fa);
                    jd.dispose();
                }
            } catch (FAException e) {
                Debug.showException(e);
            }
        }
    };

    Notifications notifications = new Notifications();

    protected TableModel tablemodel = new TableModel();

    protected TableChecker checker;

    protected IFACompleteEditor faeditor = new IFACompleteEditor();

    protected JDialog jd;

    public MinFA() {
        NonMinFAGenerator.minimalNofStates = 5;
        JEditorPane task = ExerciseUIFactory.createTaskPane(this);

        int CENTER = GridBagConstraints.CENTER;
        int BOTH = GridBagConstraints.BOTH;

        JPanel main = new JPanel(new GridBagLayout());
        main.setBackground(Color.white);
        main.add(task, new GridBagConstraints(0, 0, 1, 1, 1, 0, CENTER, BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        main.add(new TwinFAView(new TableFAs(tablemodel)),
                new GridBagConstraints(0, 1, 1, 1, 1, 1, CENTER, BOTH,
                        new Insets(0, 0, 0, 0), 0, 0));
        main.add(new TableView(tablemodel), new GridBagConstraints(0, 2, 1, 1,
                1, 0, CENTER, BOTH, new Insets(0, 0, 0, 0), 0, 0));
        //
        checker = new TableChecker(tablemodel, notifications);
        //
        JEditorPane p = new JEditorPane();
        p.setEditable(false);
        p.addHyperlinkListener(new ExtendedHyperlinkListener());
        try {
            p.setPage(ClassLoader.getSystemResource(Messages
                    .getString("MinFA.file.control"))); //$NON-NLS-1$
        } catch (IOException e) {
            Debug.showException(e);
        }
        ExerciseUIFactory.createExerciseUI(main, p, notifications, this);
        //
        tablemodel.setFA(NonMinFAGenerator.generate());
        tablemodel.setFA(NonMinFAGenerator.generate());
    }

    public String getTitle() {
        return Messages.getString("MinFA.exercise.title"); //$NON-NLS-1$
    }

    public void instructions() {
        notifications.removeAll();
        notifications.addHintNotification(Messages
                .getString("MinFA.instructions.title"), //$NON-NLS-1$
                Messages.getString("MinFA.instructions")); //$NON-NLS-1$
    }

    public void random(String query) {
        notifications.removeAll();
        String level = Query.getValue(query, "level"); //$NON-NLS-1$
        if (level.equals("beginner")) { //$NON-NLS-1$
            NonMinFAGenerator.minimalNofStates = 5;
        } else if (level.equals("advanced")) { //$NON-NLS-1$
            NonMinFAGenerator.minimalNofStates = 6;
        } else if (level.equals("expert")) { //$NON-NLS-1$
            NonMinFAGenerator.minimalNofStates = 8;
        }
        tablemodel.setFA(NonMinFAGenerator.generate());
    }

    /*
     * (non-Javadoc)
     * 
     * @see ch.ethz.exorciser.Exercise#getTask()
     */
    public String getTask() {
        return Messages.getString("MinFA.exercise"); //$NON-NLS-1$
    }

}