/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Exercise;
import ch.ethz.exorciser.ExerciseUIFactory;
import ch.ethz.exorciser.ExtendedHyperlinkListener;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Query;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FAException;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.rl.re.ABRegExpParser;
import ch.ethz.exorciser.rl.re.REParserException;
import ch.ethz.exorciser.rl.re.RegularExpression;
import ch.ethz.exorciser.rl.re.dictionary.REPrefixDictionary;

/**
 * @author Vincent Tscherter
 */
public class RE2FA extends JPanel implements Exercise {

	public JComponent getPageUI() {
		return this;
	}

	protected IFACompleteEditor working;

	protected Notifications notifications;

	protected JTextField re;

	protected int index = 0;

	static {
		REPrefixDictionary.getInstance().load(11);
	}

	public void random(String query) {
		String level = Query.getValue(query, "level"); //$NON-NLS-1$
		int l = 7;
		if (level.equals("beginner")) { //$NON-NLS-1$
			l = 7;
		} else if (level.equals("advanced")) { //$NON-NLS-1$
			l = 9;
		} else if (level.equals("expert")) { //$NON-NLS-1$
			l = 11;
		}
		RegularExpression r = REPrefixDictionary.getInstance().getRandomRE(l);
		int i = 0;
		int size = r.getFA().getStateSet().size();
		try {
			while ((size > l - 2 || r.getFA().isLanguageFinite()) && i < 100) {
				r = REPrefixDictionary.getInstance().getRandomRE(l);
				size = r.getFA().getStateSet().size();
				i++;
			}
		} catch (FAException e) {
			Debug.showException(e);
		}
		re.setText(r.getInfixForm());
		if (working != null) {
			working.getFA().clear();
		}
		if (notifications != null) {
			notifications.removeAll();
		}
	}

	private String feedbackscheme;

	public RE2FA() {
		feedbackscheme = Messages.getString("RE2FA.file.feedback"); //$NON-NLS-1$
		re = new JTextField();
		random("level=beginner"); //$NON-NLS-1$
		re.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
		working = new IFACompleteEditor();
		working.getFA().setAlphabet(
				REPrefixDictionary.getInstance().getRandomRE().getFA()
						.getAlphabet());
		notifications = new Notifications();

		JEditorPane task = ExerciseUIFactory.createTaskPane(this);
		JPanel repanel = new JPanel(new BorderLayout());
		repanel.setBackground(Color.white);
		repanel.add(new JLabel("R = "), BorderLayout.WEST); //$NON-NLS-1$
		repanel.add(re, BorderLayout.CENTER);

		int CENTER = GridBagConstraints.CENTER;
		int BOTH = GridBagConstraints.BOTH;
		JPanel main = new JPanel(new GridBagLayout());
		main.setBackground(Color.white);
		main.add(task, new GridBagConstraints(0, 0, 1, 1, 1, 0, CENTER, BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		main.add(repanel, new GridBagConstraints(0, 1, 1, 1, 1, 0, CENTER,
				BOTH, new Insets(0, 10, 0, 0), 0, 0));
		main.add(working, new GridBagConstraints(0, 2, 1, 1, 1, 1, CENTER,
				BOTH, new Insets(0, 0, 0, 0), 0, 0));
		//
		JPanel side = new JPanel(new GridBagLayout());
		side.setBackground(Color.white);

		JEditorPane p = new JEditorPane();
		p.setEditable(false);
		p.addHyperlinkListener(new ExtendedHyperlinkListener());
		try {
			p.setPage(ClassLoader.getSystemResource(Messages
					.getString("RE2FA.file.control"))); //$NON-NLS-1$
		} catch (IOException e) {
			Debug.showException(e);
		}
		ExerciseUIFactory.createExerciseUI(main, p, notifications, this);
	}

	public RE2FA(String query) {
		this();
		String r = Query.getValue(query, "re"); //$NON-NLS-1$
		if (r != null) {
			try {
				re.setText(ABRegExpParser.createRE(r).getInfixForm());
			} catch (REParserException e) {
				Debug.showException(e);
			}
		}
		r = Query.getValue(query, "feedback"); //$NON-NLS-1$
		if (r != null) {
			feedbackscheme = r;
		} else {
			feedbackscheme = Messages.getString("RE2FA.file.feedback"); //$NON-NLS-1$
		}
	}

	public void check() {
		notifications.removeAll();
		try {
			RegularExpression expr = ABRegExpParser.createRE(re.getText());
			re.setText(expr.getInfixForm());
			FAFAChecker.check(expr.getFA(), working.getFA(), feedbackscheme,
					notifications);
		} catch (REParserException e) {
			notifications.addErrorNotification(Messages
					.getString("REParserException"), e.getMessage()); //$NON-NLS-1$
		}
	}

	public void solve() {
		notifications.removeAll();
		revalidate();
		repaint();
		try {
			RegularExpression regexp = ABRegExpParser.createRE(re.getText());
			re.setText(regexp.getInfixForm());
			FA f = regexp.getFA();
			f.reLabelAllStates("q"); //$NON-NLS-1$
			f.removeTrapStates();
			working.setFA(f);
		} catch (FAException e) {
			Debug.showException(e);
		} catch (REParserException e) {
			notifications.addErrorNotification(Messages
					.getString("RE2FA.feedback.parse_error"), e.getMessage()); //$NON-NLS-1$
		}
	}

	public void instructions() {
		notifications.removeAll();
		notifications.addHintNotification(Messages
				.getString("FAEDITOR.instructions.title"), //$NON-NLS-1$
				Messages.getString("FAEDITOR.instructions")); //$NON-NLS-1$
	}

	public String getTitle() {
		return Messages.getString("RE2FA.exercise.title"); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.ethz.exorciser.Exercise#getTask()
	 */
	public String getTask() {
		return Messages.getString("RE2FA.exercise"); //$NON-NLS-1$
	}

}