/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.fa2re;

import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import ch.ethz.exorciser.fsmgui.FSMControllerInterface;
import ch.ethz.exorciser.fsmgui.FSMMouseInputListener;
import ch.ethz.exorciser.fsmgui.FSMView;

/**
 * @author Pascal Erni
 * @author Vincent Tscherter
 */
public class Fa2ReDiagramMouseListener2 extends FSMMouseInputListener { //implements
                                                                        // MouseInputListener
                                                                        // {

   private FSMView fascape;

   private FSMControllerInterface controller;

   private Point2D modCoords = new Point2D.Double();

   public Fa2ReDiagramMouseListener2 (FSMControllerInterface controller) {
      super(controller);
      this.controller = controller;
      this.fascape = controller.getView();
   }

   public void setEditTarget (FSMControllerInterface ctrl) {
      this.controller = ctrl;
   }

   int sampleCount = 0;

   public void mousePressed (MouseEvent e) {
      lastClickX = e.getX();
      lastClickY = e.getY();
      fascape.getModelCoords(modCoords, e.getX(), e.getY());
      double modX = modCoords.getX();
      double modY = modCoords.getY();
      concernedFAState = fascape.getStateAt(modX, modY);
      concernedState = null;
      if (null != concernedFAState) {
         concernedState = concernedFAState.model;
      }
      concernedTransLabel = fascape.getTransitionLabelAt(modX, modY);
      int modif = e.getModifiers();
      // handle right mouseclicks resp. Ctrl-clicks
      if ((modif & InputEvent.BUTTON3_MASK) > 0
            || (((modif & InputEvent.BUTTON1_MASK) > 0) & ((modif & InputEvent.CTRL_MASK) > 0))) {
         if ((null == concernedTransLabel) && (null == concernedTransHead)
               && (null == concernedState)) {
            controller.changeFSM(e.getX(), e.getY());
         }
      }
      // handle middle mouseclicks resp. Shift-clicks
      else if ((modif & InputEvent.BUTTON2_MASK) > 0
            || (((modif & InputEvent.BUTTON1_MASK) > 0) & ((modif & InputEvent.SHIFT_MASK) > 0))) {
         if (e.getClickCount() > 1) {
            fascape.fit();
            fascape.repaint();
         } else {
            fascape.setScaleRef(e.getX(), e.getY());
            status = ZOOM_PLANE;
         }
      }
      // handle left mouseclicks
      else {
         if (e.getClickCount() > 1) {
            if (null != concernedState) {
               if (controller instanceof Fa2ReFSMController) {
                  ((Fa2ReFSMController) controller)
                        .prepareToRemoveState(concernedState);
               }
            }
         }
         if (null != concernedState) {
            controller.selectState(concernedFAState);
            fascape.grabState(concernedFAState, modX, modY);
            status = ABOUT_TO_MOVE_STATE;
         } else if (null != concernedTransLabel) {
            if (controller instanceof Fa2ReFSMController) {
               ((Fa2ReFSMController) controller)
                     .editTransitionLabel(concernedTransLabel);
            }
         } else {
            fascape.setTranslationRef(e.getX(), e.getY());
            status = TRANSLATE_PLANE;
         }
      }
   } // mousePressed

   public void mouseDragged (MouseEvent e) {
      int modif = e.getModifiers();
      fascape.getModelCoords(modCoords, e.getX(), e.getY());
      double modX = modCoords.getX();
      double modY = modCoords.getY();
      if ((modif & InputEvent.BUTTON2_MASK) > 0) {
         if (status == ZOOM_PLANE) {
            fascape.scale(e.getY());
            fascape.repaint();
         }
      } else if ((modif & InputEvent.BUTTON1_MASK) > 0) {
         limitMousePos(e);
         fascape.getModelCoords(modCoords, e.getX(), e.getY());
         modX = modCoords.getX();
         modY = modCoords.getY();
         // last mouseclick was a single left click into the empty plane:
         if (status == TRANSLATE_PLANE) {
            // e.translatePoint(fascape.dx, fascape.dy);
            fascape.translate(e.getX(), e.getY());
            fascape.repaint();
         }
         // last mousePressed occured in center of a state:
         else if (status == ABOUT_TO_MOVE_STATE) {
            fascape.moveState(concernedFAState, modX, modY);
         }
         fascape.repaint();
      }
   } // mouseDragged

   public void mouseReleased (MouseEvent e) {
      limitMousePos(e);
      fascape.getModelCoords(modCoords, e.getX(), e.getY());
      double modX = modCoords.getX();
      double modY = modCoords.getY();
      if (status == ABOUT_TO_MOVE_STATE) {
         Point p = fascape.getNearestFreePoint(concernedFAState,
               (int) (modX - concernedFAState.getMouseRefX()),
               (int) (modY - concernedFAState.getMouseRefY()));
         if (null != p) {
            concernedState.setPosition(p.x, p.y);
         }
         concernedFAState.release();
         status = JUST_LOOKING;
      }
   } // mouseReleased
   //	public void mouseMoved(MouseEvent e) {
   //	}
   //	public void mouseClicked(MouseEvent e) {
   //	}
   //	public void mouseEntered(MouseEvent e) {
   //	}
   //	public void mouseExited(MouseEvent e) {
   //	}
}