/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.fa2re;

import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.Notifications;
import ch.ethz.exorciser.Shared;
import ch.ethz.exorciser.fsmgui.ComponentRenderer;
import ch.ethz.exorciser.fsmgui.DefaultViewerPopup;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;
import ch.ethz.exorciser.fsmgui.FSMEvent;
import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMModelListener;
import ch.ethz.exorciser.fsmgui.FSMMouseInputListener;
import ch.ethz.exorciser.fsmgui.FSMStateInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.fsmgui.StatePopup;
import ch.ethz.exorciser.fsmgui.Transition;
import ch.ethz.exorciser.ifa.IFAFSMController;
import ch.ethz.exorciser.rl.re.ABRegExpParser;
import ch.ethz.exorciser.rl.re.Concatenation;
import ch.ethz.exorciser.rl.re.Empty;
import ch.ethz.exorciser.rl.re.KleeneStar;
import ch.ethz.exorciser.rl.re.REParserException;
import ch.ethz.exorciser.rl.re.RegularExpression;
import ch.ethz.exorciser.rl.re.Union;
import ch.ethz.exorciser.rl.re.dictionary.REPrefixDictionary;

/**
 * @author Pascal Erni
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class Fa2ReFSMController extends IFAFSMController implements
      FSMModelListener {

   private DefaultViewerPopup defaultviewerPopup;

   EditTransitionLabelDialog editTransitionLabelDialog;

   List reToFaElements = new ArrayList();

   Frame parent;

   FSMStateInterface currStateToRemove;

   Notifications notifications;

   boolean autosolve = false;

   public Fa2ReFSMController (FSMModelInterface model, FSMView fsmView,
         Frame parent) {
      this.model = model;
      this.fsmView = fsmView;
      this.editorOps = new FSMEditorOperations(model, fsmView);
      this.parent = parent;
      mouseListener = new Fa2ReDiagramMouseListener2(this);
      fsmView.addMouseListener(mouseListener);
      fsmView.addMouseMotionListener(mouseListener);
      //[R.L. 15.1.2002]
      //defaultPopup = nw DefaultPopup(this, parent);
      defaultviewerPopup = new DefaultViewerPopup(this, parent);
      //[R.L. 15.1.2002]
      //		reToFaStatePopup = new Fa2ReStatePopup(this, parent);
      //		customAlphabet = new Alphabet(Alphabet.EPSILON);
   }

   public void changeFSM (int x, int y) {
      defaultviewerPopup.getPopup(x, y).show(getView(), x, y);
   }

   public void setModel (FSMModelInterface model) {
      this.model = model;
      this.editorOps.setModel(model);
      if (mouseListener instanceof FSMMouseInputListener) {
         ((FSMMouseInputListener) mouseListener).setEditTarget(this);
      }
      //		} else if (mouseListener instanceof PenPanel) {
      //			((PenPanel) mouseListener).setEditTarget(this);
      //		}
      //this.defaultPopup.setEditTarget(this);
      this.statePopup = new StatePopup(this);
      this.binTransPopup = null; // lazy new BinaryCharTransitionPopup(this);
      this.transPopup = null; // lazy new CustomTransitionPopup(this);
   }

   //[R.L. 15.1.2002]
   public void initForNewTask (Notifications list) {
      reToFaElements.clear();
      this.notifications = list;
   }

   public void prepareToRemoveState (FSMStateInterface state) {
      Transition t;
      FSMStateInterface s;
      Iterator i;
      Iterator i2;
      List tempV = new ArrayList();
      Fa2ReElement e;
      Fa2ReElement e2;
      Fa2ReElement eNew;
      //String solution;
      RegularExpression reA;
      RegularExpression reB;
      RegularExpression reC;
      RegularExpression reD;
      if (!state.isAccepting() && !state.isStartState()) {
         boolean abort = false;
         i = reToFaElements.iterator();
         while (i.hasNext()) {
            e = (Fa2ReElement) i.next();
            if (e.status == Fa2ReElement.DELETE) {
               abort = true;
            }
         }
         if (!abort) {
            currStateToRemove = state;
            // solution
            model.notifyFSMModelListeners(FSMEvent.createUnMarkAllEvent());
            reToFaElements.clear();
            reToFaElements.add(new Fa2ReElement(state, Fa2ReElement.DELETE));
            i = getModel().getAllStates().iterator();
            //elements to remove
            while (i.hasNext()) {
               s = (FSMStateInterface) i.next();
               if (s != state) {
                  t = fsmView.getTransition(state, s);
                  // incoming transitions -> delete
                  if (t != null) {
                     reToFaElements.add(new Fa2ReElement(state, s,
                           Fa2ReElement.DELETE));
                  }
                  t = fsmView.getTransition(s, state);
                  // outgoing transitions -> delete
                  if (t != null) {
                     reToFaElements.add(new Fa2ReElement(s, state,
                           Fa2ReElement.DELETE));
                  }
               } else {
                  t = fsmView.getTransition(s, s);
                  // 
                  if (t != null) {
                     reToFaElements.add(new Fa2ReElement(s, s,
                           Fa2ReElement.DELETE));
                  }
               }
            }
            //elements to change and create
            i = reToFaElements.iterator();
            while (i.hasNext()) {
               e = (Fa2ReElement) i.next();
               // e is Transition und endet bei state, aber nicht auf sich
               // selbst
               if (e.isTransition() && (e.to == state) && (e.from != state)) {
                  i2 = reToFaElements.iterator();
                  while (i2.hasNext()) {
                     e2 = (Fa2ReElement) i2.next();
                     if (e2.isTransition() && (e2.from == state)
                           && (e2.to != state)) {
                        if (null == fsmView.getTransition(e.from, e2.to)) {
                           fsmView.addTransition("?", e.from, e2.to); //$NON-NLS-1$
                           eNew = new Fa2ReElement(e.from, e2.to,
                                 Fa2ReElement.NEW);
                           eNew.beginRe = "?"; //$NON-NLS-1$
                           // vt solution = "";
                           reA = new Empty();
                        } else {
                           eNew = new Fa2ReElement(e.from, e2.to,
                                 Fa2ReElement.CHANGE);
                           try {
                              reA = ABRegExpParser.createRE(fsmView
                                    .getTransition(e.from, e2.to).label
                                    .getLabelString());
                           } catch (REParserException ex) {
                              reA = new Empty();
                              Debug.showException(ex);
                           }
                           eNew.beginRe = fsmView.getTransition(e.from, e2.to).label
                                 .getLabelString();
                        }
                        try {
                           reB = ABRegExpParser.createRE(fsmView.getTransition(
                                 e.from, e.to).label.getLabelString());
                        } catch (REParserException ex) {
                           reB = new Empty();
                           Debug.showException(ex);
                        }
                        if (null != fsmView.getTransition(e.to, e.to)) {
                           try {
                              reC = ABRegExpParser.createRE(fsmView
                                    .getTransition(e.to, e.to).label
                                    .getLabelString());
                           } catch (REParserException ex) {
                              reC = new Empty();
                              Debug.showException(ex);
                           }
                        } else {
                           reC = new Empty();
                        }
                        try {
                           reD = ABRegExpParser.createRE(fsmView.getTransition(
                                 e2.from, e2.to).label.getLabelString());
                        } catch (REParserException ex) {
                           reD = new Empty();
                           Debug.showException(ex);
                        }
                        eNew.solution = Union.union(reA, Concatenation
                              .concatenation(reB, Concatenation.concatenation(
                                    KleeneStar.kleen(reC), reD)));
                        tempV.add(eNew);
                     }
                  }
               }
            }
            i = tempV.iterator();
            while (i.hasNext()) {
               reToFaElements.add(i.next());
            }
            mark();
            tryToRemoveState();
         }
      }
   }

   protected Fa2ReElement getReToFaElement (Transition t) {
      if (t != null) {
         for (int i = 0; i < reToFaElements.size(); i++) {
            if (((Fa2ReElement) reToFaElements.get(i)).isTransition()) {
               if ((((Fa2ReElement) reToFaElements.get(i)).to == t.to.model)
                     && (((Fa2ReElement) reToFaElements.get(i)).from == t.from.model)) { return (Fa2ReElement) reToFaElements
                     .get(i); }
            }
         }
      }
      return null;
   }

   public boolean refine () {
      int i = 0;
      Transition tc = null, tw = null;
      Fa2ReElement ec = null, ew = null;
      while (i < fsmView.getNofTransitions()) {
         Transition t = fsmView.getTransition(i++);
         Fa2ReElement e = getReToFaElement(t);
         if (e != null && !e.wrongAnser.equals("") //$NON-NLS-1$
               && e.status != Fa2ReElement.DONE) {
            tw = t;
            ew = e;
         } else if (e != null
               && (e.status == Fa2ReElement.NEW || e.status == Fa2ReElement.CHANGE)) {
            tc = t;
            ec = e;
         }
      }
      if (tw != null) {
         tc = tw;
         ec = ew;
      }
      if (tc != null) {
         FSMStateInterface from = tc.from.model;
         FSMStateInterface to = tc.to.model;
         fsmView.removeTransition(tc);
         RegularExpression re = REPrefixDictionary.getInstance().getRE(
               ec.solution.getFA());
         if (re != null) {
            ec.solution = re;
         }
         fsmView.addTransition(ec.solution, from, to);
         ec.status = Fa2ReElement.DONE;
         autosolve = true;
         tryToRemoveState();
         autosolve = false;
         mark();
         fsmView.repaint();
         return true;
      }
      Object[] o = fsmView.getModel().getAllStates().toArray();
      i = 0;
      while (i < o.length
            && (((FSMStateInterface) o[i]).isStartState() || ((FSMStateInterface) o[i])
                  .isAccepting())) {
         i++;
      }
      if (i >= o.length || ((FSMStateInterface) o[i]).isStartState()
            || ((FSMStateInterface) o[i]).isAccepting()) {
         return false;
      }
	prepareToRemoveState((FSMStateInterface) o[i]);
	 return true;
   }

   public void solve () {
      while (refine()) {
         // do noting
      }
   }

   void tryToRemoveState () {
      boolean done = true;
      // sind alle done oder delete
      for (int i = 0; i < reToFaElements.size(); i++) {
         if ((((Fa2ReElement) reToFaElements.get(i)).status == Fa2ReElement.NEW)
               || (((Fa2ReElement) reToFaElements.get(i)).status == Fa2ReElement.CHANGE)) {
            done = false;
         }
      }
      // falls ja, weg damit
      if (done) {
         for (int i = 0; i < reToFaElements.size(); i++) {
            if (((Fa2ReElement) reToFaElements.get(i)).status == Fa2ReElement.DELETE) {
               if (!((Fa2ReElement) reToFaElements.get(i)).isTransition()) {
                  editorOps.deleteState(((Fa2ReElement) reToFaElements.get(i))
                        .getState());
               }
            }
         }
         unmark();
         reToFaElements.clear();
         if (model.getAllStates().size() == 2 && !autosolve) {
            notifications.removeAll();
            notifications.addCorrectNotification(Shared.CORRECT, ""); //$NON-NLS-1$//$NON-NLS-2$
         }
      }
   }

   public void editTransitionLabel (Transition trans) {
      if (null == editTransitionLabelDialog) {
         editTransitionLabelDialog = new EditTransitionLabelDialog(parent);
      }
      Fa2ReElement e = getReToFaElement(trans);
      if (e != null) {
         if ((e.status == Fa2ReElement.NEW)
               || (e.status == Fa2ReElement.CHANGE)) {
            editTransitionLabelDialog.show(trans);
         }
      }
   }

   public void notify (FSMEvent e) {
      if (e.getId() != FSMEvent.UNMARK_ALL && e.getId() != FSMEvent.MARK) {
         mark();
      }
   }

   void mark () {
      Iterator i = reToFaElements.iterator();
      while (i.hasNext()) {
         Fa2ReElement element = (Fa2ReElement) i.next();
         if (element.isState()) {
            getFSMEditorOperations().markState(element.getState(),
                  element.getMark());
         }
         if (element.isTransition()) {
            getFSMEditorOperations().markTransition(element.getFromState(),
                  element.getToState(), element.getMark());
         }
      }
   }

   private void unmark () {
      Iterator i = reToFaElements.iterator();
      while (i.hasNext()) {
         Fa2ReElement element = (Fa2ReElement) i.next();
         if (element.isState()) {
            getFSMEditorOperations().markState(element.getState(),
                  ComponentRenderer.MARK_NOT_MARKED);
         }
         if (element.isTransition()) {
            getFSMEditorOperations().markTransition(element.getFromState(),
                  element.getToState(), ComponentRenderer.MARK_NOT_MARKED);
         }
      }
   }

   /**
    * A Dialog containing a TextField and an ok-button to set the FA's name.
    * Instantiated on "change name".
    */
   public class EditTransitionLabelDialog extends Dialog implements
         ActionListener {

      JTextField t;

      JLabel l;

      Transition trans;

      EditTransitionLabelDialog (Frame parent) {
         super(parent, Messages.getString("FA2RE.controller.edit.transition"), //$NON-NLS-1$
               true);
         addWindowListener(new WindowAdapter() {

            public void windowClosing (WindowEvent e) {
               dispose();
            }
         });
         Point p = fsmView.getLocationOnScreen();
         setLocation(p.x + fsmView.getWidth() / 2 - getWidth() / 2, p.y
               + fsmView.getHeight() / 2 - getHeight() / 2);
         setLayout(new FlowLayout());
         l = new JLabel(""); //$NON-NLS-1$
         add(l);
         t = new JTextField(15);
         add(t);
         t.addActionListener(this);
         JButton ok = new JButton(Shared.OKAY); //$NON-NLS-1$
         add(ok);
         ok.addActionListener(this);
      }

      public void show (Transition transition) {
         pack();
         this.trans = transition;
         Fa2ReElement element = getReToFaElement(transition);
         if (element.beginRe != "?") { //$NON-NLS-1$
            l.setText(element.beginRe + "|"); //$NON-NLS-1$
         } else {
            l.setText(""); //$NON-NLS-1$
         }
         if (element.wrongAnser != "") { //$NON-NLS-1$
            t.setText(element.wrongAnser);
         } else {
            t.setText(""); //$NON-NLS-1$
         }
         pack();
         t.grabFocus();
         this.setVisible(true);
      }

      public void actionPerformed (ActionEvent e) {
         RegularExpression re;
         if (!t.getText().equals("")) { //$NON-NLS-1$
            FSMStateInterface from = trans.from.model;
            FSMStateInterface to = trans.to.model;
            Fa2ReElement element = getReToFaElement(trans);
            try {
               if (element.beginRe != "?") { //$NON-NLS-1$
                  re = ABRegExpParser.createRE(element.beginRe + "|" //$NON-NLS-1$
                        + t.getText()); //$NON-NLS-1$
               } else {
                  re = ABRegExpParser.createRE(t.getText());
               }
               if (element.solution.equals(re)) {
                  notifications.removeAll();
                  RegularExpression reop = REPrefixDictionary.getInstance()
                        .getRE(re.getFA());
                  if (reop != null
                        && re.getPrefixForm().length() > reop.getPrefixForm()
                              .length()) {
                     notifications.addWarningNotification(Messages
                           .getString("FA2RE.controller.simplify.title"), //$NON-NLS-1$
                           Messages.getString("FA2RE.controller.simplify") //$NON-NLS-1$
                                 .replaceAll("\\$ANSWER", //$NON-NLS-1$
                                       re.toString()).replaceAll("\\$SOLUTION", //$NON-NLS-1$
                                       reop.toString()));
                     element.status = Fa2ReElement.DONE;
                     element.wrongAnser = ""; //$NON-NLS-1$
                     fsmView.removeTransition(trans);
                     fsmView.addTransition(reop, from, to);
                     tryToRemoveState();
                  } else {
                     element.status = Fa2ReElement.DONE;
                     element.wrongAnser = ""; //$NON-NLS-1$
                     fsmView.removeTransition(trans);
                     fsmView.addTransition(re, from, to);
                     tryToRemoveState();
                  }
               } else {
                  //Fehler Feedback
                  String w = element.solution.getFA()
                        .findWitnessForNonEquivalence(re.getFA());
                  notifications.removeAll();
                  notifications.addErrorNotification(Shared.ERROR, Messages
                        .getString("FA2RE.controller.error") //$NON-NLS-1$
                        .replaceAll("\\$WORD", w) //$NON-NLS-1$
                        .replaceAll("\\$FROM", //$NON-NLS-1$
                              trans.from.getLabel()).replaceAll("\\$TO", //$NON-NLS-1$
                              trans.to.getLabel()).replaceAll("\\$VIA", //$NON-NLS-1$
                              currStateToRemove.getLabel().toString()));
                  element.wrongAnser = t.getText();
                  fsmView.removeTransition(trans);
                  fsmView.addTransition(re, from, to);
               }
            } catch (REParserException ex) {
               notifications.removeAll();
               notifications.addErrorNotification(Messages
                     .getString("REParserException"), ex //$NON-NLS-1$
                     .getMessage());
               element.wrongAnser = t.getText();
               fsmView.removeTransition(trans);
               if (element.beginRe != "?") { //$NON-NLS-1$
                  fsmView.addTransition(element.beginRe + "|" //$NON-NLS-1$
                        + t.getText(), //$NON-NLS-1$
                        from, to);
               } else {
                  fsmView.addTransition(t.getText(), from, to);
               }
            }
            mark();
         }
         dispose();
      }
   }
}