/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zürich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.facts;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import ch.ethz.exorciser.Messages;

/**
 * @author Vincent Tscherter
 */
public class FACTFactory {

	private List textTemplates, languageTemplates;

	static final String[] numbers = {
			Messages.getString("FACTGenerator.zero"), Messages.getString("FACTGenerator.one"), Messages.getString("FACTGenerator.two"), Messages.getString("FACTGenerator.three"), Messages.getString("FACTGenerator.four"), //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
			Messages.getString("FACTGenerator.five"), Messages.getString("FACTGenerator.six"), Messages.getString("FACTGenerator.seven"), Messages.getString("FACTGenerator.eight"), Messages.getString("FACTGenerator.nine"), Messages.getString("FACTGenerator.ten"), Messages.getString("FACTGenerator.eleven"), Messages.getString("FACTGenerator.twelve") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$

	private int t=-1;
	
	public FACT createBeginnerFACT(String type) {
		
		if (type!=null) {
			if (type.equals("current") && t!=-1)
				return createBeginnerFACT(t);
			try {
				t = Integer.parseInt(type);
				return createBeginnerFACT(t);
			} catch (Exception e) {
				//  nothing to do
			}
		}
		t = -1;
		return createBeginnerFACT((int) (textTemplates.size() * Math.random()));
		//return createBeginnerFACT(17);
	}

	private FACTFactory() {
		// empty constructor
	}
	
	public FACT createBeginnerFACT(int nr) {
		String alphabet = Messages.getString("FACTGenerator.alphabet"); //$NON-NLS-1$
		// generate word
		String word = ""; //$NON-NLS-1$
		int l = (int) (Math.random() * 3) + 2;
		for (int i = 0; i < l; i++) {
			word += (Math.random() > 0.5) ? ('a') : ('b');
		}
		// random symbol
		char alpha = (Math.random() > 0.5) ? ('a') : ('b');
		char beta = (alpha == 'a') ? ('b') : ('a');
		// random x & y

		int y = (int) (Math.random() * 2) + 2;
		int x = (int) (Math.random() * 3) + 2;
		x = (x <= y) ? (y + 1) : (x);
		// 
		String text = createText(textTemplates.get(nr).toString(), 
				alpha, beta, word, x, y);
		String language = createExpression(
				languageTemplates.get(nr).toString(), alphabet, alpha, beta,
				word, x, y);
		return new FACT(text, language, alphabet);
	}

	public FACT createAdvancedFACT() {
		int nr1 = (int) (textTemplates.size() * Math.random());
		int nr2 = (int) ((textTemplates.size() - 1) * Math.random());
		nr2 = (nr2 == nr1) ? (nr2 + 1) : (nr2);
		return createAdvancedFACT(nr1, nr2);
	}

	public FACT createAdvancedFACT(int nr1, int nr2) {
		// part 1
		String alphabet = Messages.getString("FACTGenerator.alphabet"); //$NON-NLS-1$
		String word = ""; //$NON-NLS-1$
		int l = (int) (Math.random() * 3) + 2;
		for (int i = 0; i < l; i++) {
			word += (Math.random() > 0.5) ? ('a') : ('b');
		}
		char alpha = (Math.random() > 0.5) ? ('a') : ('b');
		char beta = (alpha == 'a') ? ('b') : ('a');

		int y = (int) (Math.random() * 2) + 2;
		int x = (int) (Math.random() * 3) + 2;
		x = (x <= y) ? (y + 1) : (x);
		String text1 = createText(textTemplates.get(nr1).toString(), 
				alpha, beta, word, x, y);
		String language1 = createExpression(languageTemplates.get(nr1)
				.toString(), alphabet, alpha, beta, word, x, y);

		// part 2
		word = ""; //$NON-NLS-1$
		l = (int) (Math.random() * 3) + 2;
		for (int i = 0; i < l; i++) {
			word += (Math.random() > 0.5) ? ('a') : ('b');
		}
		alpha = (Math.random() > 0.5) ? ('a') : ('b');
		beta = (alpha == 'a') ? ('b') : ('a');

		// random x & y

		y = (int) (Math.random() * 2) + 2;
		x = (int) (Math.random() * 3) + 2;
		x = (x <= y) ? (y + 1) : (x);
		// 
		String text2 = createText(textTemplates.get(nr2).toString(), 
				alpha, beta, word, x, y);
		String language2 = createExpression(languageTemplates.get(nr2)
				.toString(), alphabet, alpha, beta, word, x, y);
		if (Math.random() < .5) {
			return new FACT(
					text1 + Messages.getString("FACTGenerator.and") + text2, "&" + language1 + language2, alphabet); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return new FACT(
				text1 + Messages.getString("FACTGenerator.or") + text2, "|" + language1 + language2, alphabet); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private static String createText(String template, 
			char alpha, char beta, String word, int x, int y) {
		String t = template.replaceAll("ω", "\"" + word.replaceAll("\\.", "") //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				+ "\""); //$NON-NLS-1$
		if (x < numbers.length) {
			t = t.replaceAll("ξ", numbers[x] + ""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			t = t.replaceAll("ξ", x + ""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (y < numbers.length) {
			t = t.replaceAll("ζ", numbers[y] + ""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			t = t.replaceAll("ζ", y + ""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		t = t.replaceAll("α", "'" + alpha + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		t = t.replaceAll("β", "'" + beta + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		return t;
	}

	private static String createExpression(String template, String alphabet,
			char alpha, char beta, String word, int x, int y) {
		StringBuffer sb = new StringBuffer();
		int i = 0;
		while (i < template.length()) {
			char c = template.charAt(i);
			if (c == 'Σ') {
				sb.append(alphabet);
			} else if (c == 'α') {
				sb.append(alpha);
			} else if (c == 'β') {
				sb.append(beta);
			} else if (c == 'ω') {
				for (int j = 0; j < word.length() - 1; j++) {
					sb.append('.');
				}
				sb.append(word);
			} else if (c == 'ξ' || c == 'ζ') {
				// get subexpression
				int counter = 1;
				int l = 1;
				while (counter > 0) {
					char p = template.charAt(i + l);
					if (p == '-' || p == '.' || p == '|') {
						counter++;
					} else if (p == '*' || p == '¬') {
						// do nothing
					} else {
						counter--;
					}
					l++;
				}
				int t = (c == 'ξ') ? (x) : (y);
				for (int j = 0; j < t - 1; j++) {
					sb.append('.');
				}
				for (int j = 0; j < t; j++) {
					sb.append(createExpression(
							template.substring(i + 1, i + l), alphabet, alpha,
							beta, word, x, y));
				}
				i += l - 1;
			} else {
				sb.append(c);
			}
			i++;
		}
		return sb.toString();
	}

	public static  FACTFactory createFactory(String resource) throws IOException {
		FACTFactory factory = new FACTFactory();
		factory.textTemplates = new ArrayList();
		factory.languageTemplates = new ArrayList();
		BufferedReader r = new BufferedReader(new InputStreamReader(ClassLoader
				.getSystemResourceAsStream(resource), "UTF-8")); //$NON-NLS-1$
		String line = r.readLine();
		while (line != null) {
			factory.textTemplates.add(line.substring(0, line.indexOf(';')).trim());
			factory.languageTemplates.add(line.substring(line.indexOf(';') + 1).trim());
			line = r.readLine();
		}
		return factory;
	}


}