/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

/**
 * Operations on a cell that can be saved for undo/redo
 * 
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class CellOperation {

   public static final int CELL_SET = 0;

   public static final int CELL_CLEAR = 1;

   public static final int CELL_EQUIV_SET = 2;

   public static final int CELL_EQUIV_CLEAR = 3;

   public static final int CELL_CLEAR_ALL = 4;

   private int type_;

   private TableCell cell_;

   private Object arg_;

   /**
    * Constructor.
    * 
    * @param type
    *           Type of the operation
    * @param cell
    *           DropTarget (TableCell) of operation
    * @param arg
    *           Argument to the operation (e.g. content of cell)
    */
   public CellOperation (int type, TableCell cell, Object arg) {
      type_ = type;
      cell_ = cell;
      arg_ = arg;
   }

   /**
    * Getter for the type of the operation
    */
   public int getType () {
      return type_;
   }

   /**
    * Getter for the TableCell the operation was performed on.
    */
   public TableCell getCell () {
      return cell_;
   }

   /**
    * Getter for the argument to the operation
    */
   public Object getArgument () {
      return arg_;
   }

   public String toString () {
      return new String("Type: " + type_ + " Cell: " + cell_ + " Arg: " + arg_); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
   }
}