/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Stroke;

/**
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class ContrastRenderer extends Renderer {

   ///////////
   // Colors //
   private static Color BGRColor = Color.white;

   private static Color symTextColor = Color.black;

   private static Color symDrawColor = new Color(210, 210, 210);

   private static Color symFillColor = symDrawColor;

   private static Color cellDrawColor = Color.black;

   private static Color cellFillColor = Color.white;

   private static Color labelTextColor = Color.black;

   private static Color labelDrawColor = Color.black;

   private static Color labelFillColor = BGRColor;

   private static Color cursorColor = symDrawColor; //new Color(180, 150, 170);

   private static Color paintSymColor = cursorColor;

   private static Color dragSymColor = cursorColor;

   private static Color headSymColor = symDrawColor;

   private static Color tailSymColor = Color.white;

   ////////////
   // Strokes //
   static double labelStrokeWidth = 23 * (2.0 / 30.0);

   static BasicStroke labelStroke = new BasicStroke((float) labelStrokeWidth,
         BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER);

   static BasicStroke symListStroke = new BasicStroke(1.2f);

   static BasicStroke symStroke = new BasicStroke(Math.min(1,
         TableView.fontsize / 16), BasicStroke.CAP_ROUND,
         BasicStroke.JOIN_ROUND);

   //	public ContrastRenderer() {
   //	}
   public void configureView (TableView tv) {
      SymbolList.insets = new Insets(4, 9, 4, 9);
      RowUI.hgap = 7;
   }

   public Color getCursorColor () {
      return cursorColor;
   }

   public Color getPaintSymColor () {
      return paintSymColor;
   }

   public Color getDragSymColor () {
      return dragSymColor;
   }

   public Color getHeadSymColor () {
      return headSymColor;
   }

   public Color getTailSymColor () {
      return tailSymColor;
   }

   /** Unmark by passing null as Color argument. */
   public void markSymbol (Symbol sym, Color color) {
      if (color == null) {
         sym.drawColor = headSymColor;
         sym.fillColor = symFillColor;
      } else {
         sym.drawColor = color;
         sym.fillColor = color;
      }
   }

   /** Unmark by passing null as Color argument. */
   public void markSymbolList (SymbolList syml, Color color) {
      if (color == null) {
         syml.drawColor = cellDrawColor;
         syml.fillColor = cellFillColor;
      } else {
         syml.drawColor = color;
         syml.fillColor = mixColor(color, cellFillColor, 0.40);
      }
   }

   /** Unmark by passing null as Color argument. */
   public void markRowLabel (RowLabel l, Color color) {
      if (color == null) {
         l.drawColor = labelDrawColor;
         l.fillColor = labelFillColor;
      } else {
         l.drawColor = color;
         l.fillColor = mixColor(color, labelFillColor, 0.40);
      }
   }

   public void paintSymbol (Symbol sym, Graphics _g) {
      Graphics2D g = (Graphics2D) _g;
      if (FIRST_TIME || sym.drawColor == null || sym.fillColor == null)
            markSymbol(sym, null);
      if (sym.fillBGR()) {
         g.setColor(sym.fillColor);
         g.fill(sym.rrect);
         g.setColor(sym.drawColor);
         Stroke backupStroke = g.getStroke();
         g.setStroke(symStroke);
         g.draw(sym.rrect);
         g.setStroke(backupStroke);
      }
      g.setColor(symTextColor);
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      g.drawString(sym.getModel().toString(),
            sym.bounds.x + Symbol.insets.left, sym.getTextLineY());
   }

   public void paintSymbolList (SymbolList syml, Graphics _g) {
      Graphics2D g = (Graphics2D) _g;
      if (FIRST_TIME || syml.drawColor == null || syml.fillColor == null)
            markSymbolList(syml, null);
      g.setColor(syml.fillColor);
      g.fill(syml.rrect);
      g.setColor(syml.drawColor);
      Stroke backupStroke = g.getStroke();
      g.setStroke(symListStroke);
      g.draw(syml.rrect);
      g.setStroke(backupStroke);
   }

   public void paintRowLabel (RowLabel l, Graphics _g) {
      Graphics2D g = (Graphics2D) _g;
      if (FIRST_TIME || l.drawColor == null || l.fillColor == null)
            markRowLabel(l, null);
      Stroke backupStroke = g.getStroke();
      g.setStroke(labelStroke);
      g.setColor(l.fillColor);
      g.fillOval(l.getX(), l.getY(), l.getWidth(), l.getHeight());
      g.setColor(l.drawColor);
      g.drawOval(l.getX(), l.getY(), l.getWidth(), l.getHeight());
      if (l.isAccepting())
            g.drawOval(l.getX() + l.acceptBorder, l.getY() + l.acceptBorder, l
                  .getWidth()
                  - l.acceptBorder * 2, l.getHeight() - l.acceptBorder * 2);
      g.setStroke(backupStroke);
      Font backupFont = g.getFont();
      g.setFont(TableView.labelFont);
      g.setColor(labelTextColor);
      g.drawString(l.getLabel(), l.getLabel_X(), l.getLabel_Y());
      g.setFont(backupFont);
   }

   public void paintTableView (TableView tv, Graphics _g) {
      _g.setColor(BGRColor);
      _g.fillRect(0, 0, tv.getX() + tv.getWidth(), tv.getY() + tv.getHeight());
   }
}