/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Color;
import java.awt.Graphics;

/**
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht

 */

public abstract class Renderer {

	public abstract Color getCursorColor();
	public abstract Color getPaintSymColor();
	public abstract Color getDragSymColor();
	public abstract Color getHeadSymColor();
	public abstract Color getTailSymColor();

	/**
	 *  lets you set gaps, insets, fonts. they have to be set in the editor because update Bounds is done there.
	 */
	public abstract void configureView(TableView tv);

	public abstract void paintSymbol(Symbol sym, Graphics g);

	public abstract void markSymbol(Symbol sym, Color color);

	public abstract void paintSymbolList(SymbolList syml, Graphics g);

	public abstract void markSymbolList(SymbolList syml, Color color);

	public abstract void paintRowLabel(RowLabel l, Graphics g);

	public abstract void markRowLabel(RowLabel l, Color color);

	public abstract void paintTableView(TableView tv, Graphics g);

	public void prolog() {
	}
	public void epilog() {
		if (FIRST_TIME)
			FIRST_TIME = false;
	}

	boolean FIRST_TIME = true;
	public void reset() {
		FIRST_TIME = true;
	}

	/** Mixes two colors. The third argument specifies the composition. 1 produces pure first Color, 0.5 an equal mix, 0 pure second Color.*/
	public static Color mixColor(Color c1, Color c2, double rel) {
		if (rel > 1 || rel < 0)
			return null;
		float[] cc1 = new float[3];
		float[] cc2 = new float[3];
		c1.getColorComponents(cc1);
		c2.getColorComponents(cc2);
		Color mix =
			new Color(
				(float) (cc1[0] * rel + cc2[0] * (1 - rel)),
				(float) (cc1[1] * rel + cc2[1] * (1 - rel)),
				(float) (cc1[2] * rel + cc2[2] * (1 - rel)));
		return mix;
	}
}