/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Color;
import java.awt.Rectangle;

import ch.ethz.exorciser.ifa.State;

/**
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class RowLabel {

   /*
    * spaceX/Y specify the space labels take. It is not the size they are
    * painted. They are painted with size diameter - gap.
    */
   static int spaceX = 44;

   static int spaceY = 38;

   /* gap between space-bounds and labelcircle. */
   private static int gap = -3;

   public int acceptBorder = 6;

   private State state;

   public State getState () {
      return state;
   }

   public String getLabel () {
      return ((String) state.getLabel()).trim();
   }

   public boolean isAccepting () {
      return state.isAccepting();
   }

   Color drawColor;

   Color fillColor;

   Rectangle bounds = new Rectangle(gap, gap, spaceX - 2 * gap, spaceY - 2
         * gap);

   public int getWidth () {
      return bounds.height;
   }

   public int getHeight () {
      return bounds.height;
   }

   public int getX () {
      return bounds.x + (spaceX - spaceY) / 2 - 1;
   }

   public int getY () {
      return bounds.y + 1;
   }

   public int getLabel_X () {
      return (bounds.x + spaceX / 2 - gap - TableView.labelFm
            .stringWidth(getLabel()) / 2);
   }

   public int getLabel_Y () {
      return (int) (bounds.y + spaceY / 2 - gap + TableView.labelFm.getHeight() * 0.3);
   }

   public RowLabel (State s) {
      this.state = s;
   }

   public void updateOrigin (int x, int y) {
      bounds.x = x + gap;
      bounds.y = y + gap;
   }

   public boolean contains (int x, int y) {
      int radius = (spaceY - gap) / 2;
      int centerX = bounds.x + radius;
      int centerY = bounds.y + radius;
      int dx = x - centerX;
      int dy = y - centerY;
      if ((dx * dx + dy * dy) < radius * radius) {
		return true;
	}
	return false;
   }
}