/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import java.util.List;

import ch.ethz.exorciser.ifa.State;



/**
 * this class displays a production of a grammar.
 * 
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class RowUI {

	List items = new ArrayList();

	static int roundness = 8;
	RoundRectangle2D rrect =
		new RoundRectangle2D.Double(0, 0, 1, 1, roundness, roundness);
	Rectangle bounds = new Rectangle(0, 0, 1, 1);
	public int getWidth() {
		return bounds.width;
	}
	public int getHeight() {
		return bounds.height;
	}
	public int getX() {
		return bounds.x;
	}
	public int getY() {
		return bounds.y;
	}

	/** space between items. */
	static int hgap = 47;
	/** space between labe and first item. */
	//static int label_hgap = 47;

	int indexX;

	State state;
	public State getState() {
		return state;
	}

	RowLabel label;
	public RowLabel getRowLabel() {
		return label;
	}

	TableView parent;

	int index;

	boolean marked = false;
	public void mark(boolean b) {
		marked = b;
	}

	public RowUI(TableCell[] cells, State state, TableView parent, int index) {
		SymbolList syml;
		for (int i = 0; i < cells.length; i++) {
			syml = new SymbolList(cells[i].getContent());
			items.add(syml); // aimfeld 21.12.2001
			parent.s2c.put(syml, cells[i]);
			parent.c2s.put(cells[i], syml);
		}
		this.state = state;
		this.label = new RowLabel(state);
		this.parent = parent;
		this.index = index;
	}

	void unmarkAllItems(Renderer renderer) {
		for (int i = 0; i < items.size(); i++) {
			SymbolList current = (SymbolList) items.get(i);
			renderer.markSymbolList(current, null);
		}
	}

	/** used to calc max width & height in TableView */
	void calcSpreads() {
		for (int i = 0; i < items.size(); i++) {
			SymbolList current = (SymbolList) items.get(i);
			current.calcSpread();
		}
	}

	/** used to calc max width in TableView */
	int getWidth(int i) {
		SymbolList current = (SymbolList) items.get(i);
		return Math.max(RowLabel.spaceX, current.getWidth());
	}

	/** used to calc max height in TableView */
	int getHeight(int i) {
		SymbolList current = (SymbolList) items.get(i);
		return Math.max(RowLabel.spaceY, current.getHeight());
	}

	void updateBounds(int x, int y, int h) {
		bounds.x = x;
		bounds.y = y;
		bounds.height = h;

		// align labels with center of preceeding row
		if (index > 0 && parent.getColWidth(index - 1) > RowLabel.spaceX) {
			label.updateOrigin(
				x - (parent.getColWidth(index - 1) - RowLabel.spaceX) / 2,
				y);
		} else {
			label.updateOrigin(x, y);
		}

		// arrange Cells from left to right
		int width = RowLabel.spaceX + hgap;
		for (int i = 0; i < items.size(); i++) {
			SymbolList current = (SymbolList) items.get(i);
			current.updateBounds(x + width, y, parent.getColWidth(i + index), h);
			width += parent.getColWidth(i + index) + hgap;
		}
		bounds.width = width - hgap; //width + insets.right - hgap;
	}

	public void paint(Renderer r, Graphics2D g) {
		r.paintRowLabel(label, g);
		for (int i = 0; i < items.size(); i++) {
			SymbolList current = (SymbolList) items.get(i);
			current.paint(r, g);
		}
	}

	public Object objectHitBy(int x, int y) {
		// check label
		if (label.contains(x, y))
			return label;

		// check cells
		for (int i = 0; i < items.size(); i++) {
			SymbolList current = (SymbolList) items.get(i);
			Object obj = current.objectHitBy(x, y);
			if (null != obj) {
				return obj;
			}
		}
		if (bounds.contains(x, y)) {
			return this;
		}
		return null;
	}

}