/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.RoundRectangle2D;

/**
 * this class displays an object of a grammar in a symbollist. *
 * 
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class Symbol {

   boolean dragged;

   public void drag (boolean b) {
      dragged = b;
   }

   boolean dragable = true;

   public boolean isDragable () {
      return dragable;
   }

   public void setDragable (boolean b) {
      dragable = b;
   }

   Color drawColor;

   Color fillColor;

   boolean fillBGR = true;

   public boolean fillBGR () {
      return fillBGR;
   }

   boolean visible;

   public void setVisible (boolean b) {
      visible = b;
   }

   /**
    * insets between symbolbounds and bounds of parent symbollist.
    */
   static Insets insets = new Insets(0, 3, 0, 2);

   static int roundness = 7;

   RoundRectangle2D rrect = new RoundRectangle2D.Double(0, 0, 1, 1, roundness,
         roundness);

   /**
    * this Rectangle contains position and size of this symbol, calculated in
    * method updateBounds().
    */
   Rectangle bounds = new Rectangle(0, 0, 1, 1);

   public int getWidth () {
      return bounds.width;
   }

   public int getHeight () {
      return bounds.height;
   }

   public int getX () {
      return bounds.x;
   }

   public int getY () {
      return bounds.y;
   }

   public int getCenterX () {
      return bounds.x + (bounds.width / 2);
   }

   public int getCenterY () {
      return bounds.y + (bounds.height / 2);
   }

   /**
    * y-coord of the text in this symbol. used in ProductionUI to align index
    * and arrow (->).
    */
   private int textLineY;

   public int getTextLineY () {
      return textLineY;
   }

   /**
    * stores the mouse position relative to this symbolbounds where this symbol
    * was grabbed.
    */
   private Point mouseRef = new Point(0, 0);

   public Point getMouseRef () {
      return mouseRef;
   }

   public void grab (int x, int y) {
      mouseRef.setLocation(x - bounds.x, y - bounds.y);
   }

   /**
    * the symbollist this symbol belongs to.
    */
   private SymbolList parent;

   public SymbolList getParentSymbolList () {
      return parent;
   }

   public void setParentSymbolList (SymbolList s) {
      this.parent = s;
   }

   /**
    * the object this symbol displays.
    */
   private Object model;

   public Object getModel () {
      return model;
   }

   public Character getChar () {
      return new Character(this.toString().charAt(0));
   }

   void setModel (Object o) {
      this.model = o;
   }

   public Symbol (Object model, SymbolList symbolList) {
      this.model = model;
      this.parent = symbolList;
   }

   void setLocation (int x, int y) {
      updateOrigin(x - mouseRef.x, y - mouseRef.y);
   }

   void updateSpread () {
      String asString = model.toString();
      FontMetrics fm = TableView.fm;
      bounds.width = insets.left + fm.stringWidth(asString) + insets.right;
      bounds.height = insets.top + fm.getHeight() + insets.bottom;
      rrect.setFrame(bounds);
   }

   void updateOrigin (int x, int y) {
      bounds.x = x;
      bounds.y = y;
      textLineY = y + insets.top + TableView.fm.getAscent();
      rrect.setFrame(bounds);
   }

   public void paint (Renderer r, Graphics g) {
      r.paintSymbol(this, g);
   }

   public boolean contains (int x, int y) {
      return bounds.contains(x, y);
   }

   public String toString () {
      return model.toString();
   }

   public Object clone () {
      Symbol clone = new Symbol(model, parent);
      clone.bounds.setBounds(bounds.x, bounds.y, bounds.width, bounds.height);
      return clone;
   }
}