/*
 * Exorciser - www.educeth.ch/exorciser  Copyright 2000-2004 Swiss Federal
 * Institue of Technology, Zrich Switzerland
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class SymbolList {

   /* The Symbols in this SymbolList. */
   public List symbols = new ArrayList();

   public int size () {
      return symbols.size();
   }

   /*
    * Backup Vector for the symbols Vector, used with temporary insertion of
    * Symbols.
    */
   private List backup = new ArrayList();

   static int roundness = 8;

   RoundRectangle2D rrect = new RoundRectangle2D.Double(0, 0, 1, 1, roundness,
         roundness);

   private Rectangle bounds = new Rectangle(0, 0, 1, 1);

   public int getWidth () {
      return bounds.width;
   }

   public int getHeight () {
      return bounds.height;
   }

   public int getX () {
      return bounds.x;
   }

   public int getY () {
      return bounds.y;
   }

   private int textLineY;

   public int getTextLineY () {
      return textLineY;
   }

   public boolean marked;

   Color drawColor;

   Color fillColor;

   static Insets insets;

   /** space between symbols. */
   int symbols_hgap = 3;

   /** space inside symbolList if its empty */
   int empty_hgap = 18;

   /**
    * if true, no new size is calculated in calcSpread. Set to true after
    * temporary insert by calling method backupContentAndInsert().
    */
   boolean USE_OLD_SPREAD = false;

   /** to avoid duplicate removes and adds. */
   private Symbol lastRemoved, lastAdded;

   public SymbolList (List symbols) {
      for (int i = 0; i < symbols.size(); i++) {
         this.symbols.add(new Symbol(symbols.get(i), this));
      }
   }

   public SymbolList (Object[] symbols) {
      for (int i = 0; i < symbols.length; i++) {
         this.symbols.add(new Symbol(symbols[i], this));
      }
   }

   public SymbolList (Object symbol) {
      if (symbol != null) {
         this.symbols.add(new Symbol(symbol, this));
      }
   }

   void applySymbol (Object symbol) {
      if (symbol != null) {
         applySymbols(new Object[] { symbol});
      } else {
         applySymbols(""); //$NON-NLS-1$
      }
   }

   void addSymbol (Object symbol, boolean dragable, boolean transparent) {
      if (symbol != null) {
         addSymbols(new Object[] { symbol});
         Symbol sym = ((Symbol) symbols.get(symbols.size() - 1));
         sym.setDragable(dragable);
         sym.fillBGR = !transparent;
      }
   }

   void applySymbols (Object[] symbol) {
      int i;
      for (i = 0; i < symbol.length && i < this.symbols.size(); i++) {
         Symbol current = (Symbol) this.symbols.get(i);
         current.setModel(symbol[i]);
      }
      for (; i < symbol.length; i++) {
         this.symbols.add(new Symbol(symbol[i], this));
      }
      for (; i < this.symbols.size(); i++) {
         this.symbols.remove(i);
      }
   }

   void applySymbols (String symbole) {
      int i;
      for (i = 0; i < symbole.length() && i < this.symbols.size(); i++) {
         Symbol current = (Symbol) this.symbols.get(i);
         current.setModel(symbole.substring(i, i + 1));
      }
      for (; i < symbole.length(); i++) {
         this.symbols.add(new Symbol(symbole.substring(i, i + 1), this));
      }
      for (; i < this.symbols.size(); i++) {
         this.symbols.remove(i);
      }
   }

   public void addSymbols (Object[] symbole) {
      for (int i = 0; i < symbole.length; i++) {
         this.symbols.add(new Symbol(symbole[i], this));
      }
   }

   public Character[] getSymbols () {
      Character[] array = new Character[symbols.size()];
      for (int i = 0; i < array.length; i++) {
         array[i] = new Character(symbols.get(i).toString().charAt(0));
      }
      return array;
   }

   public void addSymbol (Symbol s) {
      addSymbol(symbols.size(), s);
   }

   public void addSymbol (int i, Symbol s) {
      if (s == lastAdded) return;
      s.setParentSymbolList(this);
      this.symbols.add(i, s);
      lastAdded = s;
      lastRemoved = null;
   }

   public void removeSymbol (Symbol s) {
      if (s == lastRemoved) return;
      if (this.symbols.remove(s)) {
         // causes exception! s.setParentSymbolList(null);
         lastRemoved = s;
         lastAdded = null;
      }
   }

   public void removeObject (Object o) {
      for (int i = 0; i < symbols.size(); i++) {
         Symbol current = (Symbol) symbols.get(i);
         if (current.getModel().equals(o)) {
            symbols.remove(current);
            i--;
         }
      }
   }

   public Character getFirstChar () {
      if (!symbols.isEmpty()) {
         return new Character(symbols.get(0).toString().charAt(0));
      }
	return null;
   }

   public boolean isEmpty () {
      return symbols.size() == 0;
   }

   public void backupContentAndInsert (Symbol s) {
      if (s == lastAdded) { return; }
      backup.clear();
      backup.addAll(symbols);
      symbols.clear();
      addSymbol(s);
      USE_OLD_SPREAD = true;
   }

   public void removeAndRestoreContent (Symbol s) {
      if (s == lastRemoved) return;
      if (symbols.contains(s)) {
         removeSymbol(s);
         symbols.clear();
         symbols.addAll(backup);
         backup.clear();
      }
      USE_OLD_SPREAD = false;
   }

   public void clearBackup () {
      backup.clear();
      USE_OLD_SPREAD = false;
   }

   public void clearLastAdded () {
      lastAdded = null;
   }

   /**
    * Sets the model of all Symbols in this list with model == oldO to newO.
    * Called from ProducitonUI-method renameOccurrencesOf() (for productions)
    * and ProductionEditor-method renameSym() (for varList and termList)
    */
   public void renameObject (Object oldO, Object newO) {
      for (int i = 0; i < symbols.size(); i++) {
         Symbol current = (Symbol) symbols.get(i);
         if (current.getModel().equals(oldO)) {
            current.setModel(newO);
         }
      }
   }

   public int indexAt (int x) {
      Symbol current = null;
      for (int i = 0; i < symbols.size(); i++) {
         current = (Symbol) symbols.get(i);
         if (x < current.getX() + current.getWidth()) return i;
      }
      return symbols.size();
   }

   void calcSpread () {
      if (symbols.size() > 1 && USE_OLD_SPREAD) USE_OLD_SPREAD = false;
      if (USE_OLD_SPREAD && !symbols.isEmpty()) {
         Symbol sym = (Symbol) symbols.get(0);
         sym.updateSpread();
         return;
      }
      int width = insets.left;
      int height = TableView.fm.getHeight();
      for (int i = 0; i < symbols.size(); i++) {
         Symbol current = (Symbol) symbols.get(i);
         current.updateSpread();
         width += current.getWidth() + symbols_hgap;
         height = Math.max(height, current.getHeight());
      }
      if (symbols.size() == 0) width += empty_hgap;
      bounds.width = width + insets.right - symbols_hgap;
      bounds.height = height + insets.top + insets.bottom;
   }

   void updateBounds (int x, int y, int w, int h) {
      if (symbols.size() > 1 && USE_OLD_SPREAD) USE_OLD_SPREAD = false;
      bounds.x = x;
      bounds.y = y;
      // uncomment following lines for expanded-to-labelsize cells
      //bounds.width = w;
      //bounds.height = h;
      bounds.x = x + (w - bounds.width) / 2;
      bounds.y = y + (h - bounds.height) / 2;
      // arrange symbols from left to right
      int width = insets.left;
      Symbol current = null;
      for (int i = 0; i < symbols.size(); i++) {
         current = (Symbol) symbols.get(i);
         current.updateOrigin(bounds.x + width, bounds.y + insets.top);
         if (USE_OLD_SPREAD) {
            current.updateOrigin(bounds.x + bounds.width / 2
                  - current.getWidth() / 2, bounds.y + insets.top);
         }
         width += current.getWidth() + symbols_hgap;
      }
      if (current != null) {
         textLineY = current.getTextLineY();
      } else {
         textLineY = y + insets.top + Symbol.insets.top
               + TableView.fm.getAscent();
      }
      rrect.setFrame(bounds);
   }

   public void paint (Renderer r, Graphics g) {
      r.paintSymbolList(this, g);
      Symbol current;
      for (int i = 0; i < symbols.size(); i++) {
         current = (Symbol) symbols.get(i);
         current.paint(r, g);
      }
   }

   public Object objectHitBy (int x, int y) {
      Symbol current;
      for (int i = 0; i < symbols.size(); i++) {
         current = (Symbol) symbols.get(i);
         if (current.contains(x, y)) { return current; }
      }
      if (bounds.contains(x, y)) {
         return this;
      }
	return null;
   }
}