/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;

import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.ifa.State;

/**
 * Man kann: - Symbole von Zelle zu Zelle verschieben. - mit Ctrl down werden
 * die Symbole kopiert. mit dem flag SYMBOLS_DRAGABLE ist drag&drop
 * ausgeschaltet.
 * 
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class TableView extends JPanel implements MouseInputListener,
      TableModelListener {

   public static final Color MARK_COL_RED = Color.red;

   public static final Color MARK_COL_GREEN = new Color(40, 160, 40);

   public static final Color MARK_COL_BLUE = Color.blue;

   /** Turns antialiasing on and off. */
   public static boolean ANTIALIASED = true;

   /** Symbols can only be drag&dropped if this flag is true. */
   public boolean SYMBOLS_DRAGABLE = true;

   protected TableModel model;

   /** Hashtable that maps SymbolLists to TableCells. */
   protected Hashtable s2c;

   /** Hashtable that maps TableCells to SymbolLists. */
   protected Hashtable c2s;

   //String equivSymbol = "{}";
   protected String equivSymbol = new Character('\u00f8').toString();

   Object[] availableSymbols = { "0", "1"}; //$NON-NLS-1$ //$NON-NLS-2$

   SymbolList charList = new SymbolList(availableSymbols);

   SymbolList epsilonList = new SymbolList(Alphabet.EPSILON);

   //SymbolList equivList = new SymbolList(equivSymbol);
   SymbolList equivList = new SymbolList(new Character(equivSymbol.charAt(0)));

   /**
    * Hashtable that maps the Symbols in the symbol lists to cursors for paint
    * mode. The Hashtable is created the first time paintComponent is called.
    */
   Hashtable cursors;

   private Cursor currentCursor;

   // put clones of available Symbols here to avoid incessant Object creation
   // for paintSymbolCursors.
   //Hashtable symbolCursors = new Hashtable();
   /** The RowUIs of the table. */
   List rows = new ArrayList();

   public int nOfRows () {
      return rows.size();
   }

   public int nOfCols () {
      return rows.size();
   }

   /** The label at the bottom, not part of a row... */
   RowLabel lastRowLabel;

   /** Font used for Symbols. */
   static float fontsize = 18f;

   static Font font;

   static FontMetrics fm;

   /** Font used for RowLabels. */
   static float labelFontsize = 12f;

   static Font labelFont;

   static FontMetrics labelFm;
   static {
      // get suitable FontMetrics for symbols ...
      BufferedImage buf = new BufferedImage(10, 10,
            BufferedImage.TYPE_4BYTE_ABGR);
      Graphics2D g = (Graphics2D) buf.getGraphics();
      configureGraphics(g);
      fm = g.getFontMetrics();
      // ... and for labels
      labelFont = g.getFont().deriveFont(labelFontsize);
      g.setFont(labelFont);
      labelFm = g.getFontMetrics();
   }

   /**
    * Turns antialiasing on, if the flag TableView.ANTIALIASED is true and sets
    * the font for Symbols.
    */
   private static void configureGraphics (Graphics2D g) {
      if (ANTIALIASED)
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                  RenderingHints.VALUE_ANTIALIAS_ON);
      font = g.getFont().deriveFont(fontsize);
      g.setFont(font);
   }

   /** coords of upper left corner of table (with charList) */
   int origin_X = 20;

   int origin_Y = 20;

   /** horizontal gap between rows in the table. */
   int rows_vgap = 8;

   /** vertical gap between charList and table. */
   int charList_table_vgap = 14;

   /** size of labels including gaps. */
   static int label_diameter = 46;

   /** space between TableView bounds and the cells. */
   static Insets insets = new Insets(3, 4, 3, 4);

   private Rectangle bounds = new Rectangle(0, 0, 1, 1);

   public int getWidth_ () {
      return bounds.width;
   }

   public int getHeight_ () {
      return bounds.height;
   }

   public int getX_ () {
      return bounds.x;
   }

   public int getY_ () {
      return bounds.y;
   }

   /** the array containing the height of each row. */
   int[] rowHeights;

   public int getRowHeight (int i) {
      if (i < rowHeights.length) {
		return rowHeights[i];
	}
	return -1;
   }

   /** the array containing the width of each column. */
   int[] colWidths;

   public int getColWidth (int i) {
      if (i < colWidths.length) {
		return colWidths[i];
	}
	return -1;
   }

   /** the Renderer that will draw all visible elements. */
   Renderer renderer;

   public Renderer getRenderer () {
      return renderer;
   }

   public Renderer contrastRenderer = new ContrastRenderer();

   //public Renderer screenRenderer = new ScreenRenderer();
   public void setRenderer (Renderer r) {
      this.renderer = r;
      this.renderer.reset();
      cursors = null;
      exitPaintMode();
      this.renderer.configureView(this);
   }

   /**
    * The method whoIs() of this inner class analyses an object. It's used in
    * MouseListener methods.
    */
   private class HitLocation {

      Symbol symbol;

      SymbolList symbolList;

      RowLabel rowLabel;

      private boolean bSymbol_;

      private boolean bSymbolList_;

      private boolean bRowLabel_;

      private boolean bCharList_;

      private boolean bEpsilonList_;

      private boolean bEquivList_;

      boolean isSymbol () {
         return bSymbol_;
      }

      boolean isSymbolList () {
         return bSymbolList_;
      }

      boolean isRowLabel () {
         return bRowLabel_;
      }

      boolean insideTableCell () {
         return null != symbolList && !bCharList_ && !bEpsilonList_
               && !bEquivList_;
      }

      boolean insideSpecialSymbolList () {
         return bCharList_ || bEpsilonList_ || bEquivList_;
      }

      boolean insideCharList () {
         return bCharList_;
      }

      boolean insideEpsilonList () {
         return bEpsilonList_;
      }

      boolean insideEquivList () {
         return bEquivList_;
      }

      void whoIs (Object hit) {
         initFields();
         if (hit instanceof Symbol) {
            bSymbol_ = true;
            symbol = (Symbol) hit;
            symbolList = symbol.getParentSymbolList();
         } else if (hit instanceof SymbolList) {
            bSymbolList_ = true;
            symbolList = (SymbolList) hit;
         } else if (hit instanceof RowLabel) {
            bRowLabel_ = true;
            rowLabel = (RowLabel) hit;
         }
         if (symbolList == TableView.this.charList) {
            bCharList_ = true;
         } else if (symbolList == TableView.this.epsilonList) {
            bEpsilonList_ = true;
         } else if (symbolList == TableView.this.equivList) {
            bEquivList_ = true;
         }
      }

      private void initFields () {
         symbol = null;
         symbolList = null;
         rowLabel = null;
         bSymbol_ = false;
         bSymbolList_ = false;
         bRowLabel_ = false;
         bCharList_ = false;
         bEpsilonList_ = false;
         bEquivList_ = false;
      }
   }

   private HitLocation hitLocation_ = new HitLocation();

   private Popups popups;

   public TableView () {
      this(null);
   }

   public TableView (TableModel model) {
      setModel(model);
      addMouseListener(this);
      addMouseMotionListener(this);
      setRenderer(contrastRenderer);
      popups = new Popups(this);
   }

   /**
    * adds this view as TableModelListener to the TableModel. creates visible
    * cells for each model-cell.
    */
   public void setModel (TableModel model) {
      rows = new ArrayList();
      if (null != this.model) this.model.removeTableModelListener(this);
      if (null == model) return;
      this.model = model;
      model.addTableModelListener(this);
      if (model.getFA() == null) { return; }
      if (null != model.getFA().getAlphabet()) {
         availableSymbols = model.getFA().getAlphabet().toArray();
         charList.applySymbols(availableSymbols);
      }
      s2c = new Hashtable();
      c2s = new Hashtable();
      State[] states = model.getStates();
      for (int i = 0; i < model.getNumRows() && i < states.length; i++) {
         TableCell[] curr = model.getRow(i);
         rows.add(new RowUI(curr, states[i], this, i));
      }
      lastRowLabel = new RowLabel(states[states.length - 1]);
      updateBounds();
   }

   public void markRowLabel (State state, Color color) {
      if (state.equals(lastRowLabel.getState())) {
         renderer.markRowLabel(lastRowLabel, color);
         return;
      }
      for (int i = 0; i < rows.size(); i++) {
         RowUI curr = (RowUI) rows.get(i);
         if (state.equals(curr.getState())) {
            renderer.markRowLabel(curr.getRowLabel(), color);
            return;
         }
      }
   }

   public void markRowLabel (int i, Color color) {
      if (i == rows.size()) renderer.markRowLabel(lastRowLabel, color);
      else if (i < rows.size()) {
         RowUI curr = (RowUI) rows.get(i);
         if (null != curr) renderer.markRowLabel(curr.getRowLabel(), color);
      }
   }

   public void unmarkRowLabels () {
      for (int i = 0; i < rows.size(); i++) {
         RowUI curr = (RowUI) rows.get(i);
         renderer.markRowLabel(curr.getRowLabel(), null);
      }
      if (lastRowLabel != null) {
         renderer.markRowLabel(lastRowLabel, null);
      }
   }

   public void notify (TableEvent e) {
      TableCell cell;
      SymbolList sl;
      switch (e.getType()) {
      case TableEvent.CELL_NOTOK:
         if (e.getArgument() instanceof Pair) {
            cell = (TableCell) ((Pair) e.getArgument()).getFirst();
            sl = (SymbolList) c2s.get(cell);
            if (null != sl) {
               renderer.markSymbolList(sl, MARK_COL_RED);
            }
         }
         break;
      case TableEvent.CELL_ACTIVATE:
         cell = (TableCell) e.getArgument();
         TableCell nextCell = null;
         sl = (SymbolList) c2s.get(cell);
         if (null != sl) {
            renderer.markSymbolList(sl, MARK_COL_BLUE);
         }
         if (draggedSymbol != null) {
            nextCell = cell.computeRef(draggedSymbol.getChar());
            if (nextCell == null) {
               markRowLabel(cell.getRowState().getNextState(
                     draggedSymbol.getChar()), MARK_COL_GREEN);
            }
         } else if (paintSymbol != null) {
            nextCell = cell.computeRef(paintSymbol.getChar());
            if (nextCell == null) {
               markRowLabel(cell.getRowState().getNextState(
                     paintSymbol.getChar()), MARK_COL_GREEN);
            }
         } else {
            nextCell = cell.getRef();
         }
         if (nextCell != null) {
            sl = (SymbolList) c2s.get(nextCell);
            if (sl != null) {
               renderer.markSymbolList(sl, MARK_COL_GREEN);
            }
         }
         break;
      case TableEvent.TABLE_CLEAR:
      case TableEvent.TABLE_DEACTIVATE:
         unmarkAllCells();
         unmarkRowLabels();
         break;
      case TableEvent.CELL_RESET:
      case TableEvent.CELL_SET:
         cell = (TableCell) e.getArgument();
         sl = (SymbolList) c2s.get(cell);
         sl.applySymbol(cell.getContent());
         cell = cell.getRef();
         if ((cell != null) && (!cell.isEmpty()) && (!cell.containsEpsilon())) {
            sl.addSymbol(cell.getWordString(), false, true);
         }
         updateBounds();
         break;
      case TableEvent.CELL_CLEAR:
         cell = (TableCell) e.getArgument();
         sl = (SymbolList) c2s.get(cell);
         sl.applySymbols(""); //$NON-NLS-1$
         updateBounds();
         break;
      case TableEvent.CELL_EQUIV_SET:
         cell = (TableCell) e.getArgument();
         sl = (SymbolList) c2s.get(cell);
         sl.applySymbols(equivSymbol);
         updateBounds();
         break;
      case TableEvent.CELL_EQUIV_CLEAR:
         cell = (TableCell) e.getArgument();
         sl = (SymbolList) c2s.get(cell);
         sl.applySymbols(""); //$NON-NLS-1$
         updateBounds();
         break;
      case TableEvent.TABLE_SET:
         setModel(model);
         updateBounds();
         revalidate();
         break;
      }
      repaint();
   }

   /////////////////////////////////////
   //   MouseMotionListener methods //
   /**
    * The symbol in the charList that is a clone of the Symbol that currently
    * sticks on the mousepointer.
    */
   Symbol paintSymbol;

   /** the Symbol that currently sticks on the mousepointer. */
   Symbol paintSymbolCursor;

   /** the symbol that is currently dragged. */
   Symbol draggedSymbol;

   /** the SymbolList from which the draggedSymbol was dragged. */
   SymbolList draggedFromSymL;

   /** the SymbolList into which the draggedSymbol was dropped. */
   SymbolList droppedIntoSymL;

   Object hitObject;

   Object oldHitSymL;

   boolean cellContentEqualsPaintSym = false;

   /**
    * assigns Object that the mouse pointer is currently over to variable
    * hitObject. fires CELL_ACTIVATE and TABLE_DEACTIVATE events.
    */
   public void mouseMoved (MouseEvent e) {
      if (model == null) return;
      hitObject = objectHitBy(e.getX(), e.getY());
      hitLocation_.whoIs(hitObject);
      // leaved old cell -> remove temporary symbol
      if (oldHitSymL != hitLocation_.symbolList) {
         oldHitSymL = hitLocation_.symbolList;
         model.createEvent(this, TableEvent.TABLE_DEACTIVATE);
         if (paintSymbol != null) {
            removeTemporaryInsertedSym_(paintSymbolCursor, e);
         }
         // inside tablecell -> add temporary symbol
         if (hitLocation_.insideTableCell()) {
            model.createEvent(this, TableEvent.TABLE_DEACTIVATE);
            model.createEvent(this, TableEvent.CELL_ACTIVATE, s2c
                  .get(hitLocation_.symbolList));
            if (paintSymbol != null) {
               cellContentEqualsPaintSym = paintSymbol.getChar().equals(
                     hitLocation_.symbolList.getFirstChar());
               if (!cellContentEqualsPaintSym) {
                  temporaryInsertSym_(paintSymbolCursor);
               }
            }
         }
      }
   } // mouseMoved

   public void mousePressed (MouseEvent e) {
      if ((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0) {
         popups.getPopup().show(this, e.getX(), e.getY());
      } else if (paintSymbol == null) {
         hitObject = objectHitBy(e.getX(), e.getY());
         hitLocation_.whoIs(hitObject);
         if (hitLocation_.isSymbol() && SYMBOLS_DRAGABLE
               && ((Symbol) hitObject).isDragable()) {
            draggedSymbol = hitLocation_.symbol;
            if (hitLocation_.insideSpecialSymbolList()
                  || (e.getModifiers() & InputEvent.CTRL_MASK) > 0) {
               draggedSymbol = (Symbol) draggedSymbol.clone();
            }
            draggedSymbol.grab(e.getX(), e.getY());
            draggedSymbol.drag(true);
            //renderer.markSymbol(draggedSymbol, renderer.getDragSymColor());
            if (!equivSymbol.toString().equals(
                  draggedSymbol.getModel().toString())) {
               model.createEvent(this, TableEvent.CHAR_SELECT, draggedSymbol
                     .getChar());
            }
         }
      }
   } // mousePressed

   public void mouseDragged (MouseEvent e) {
      mouseMoved(e);
      if (paintSymbol == null && draggedSymbol != null
            && hitObject != draggedSymbol) {
         // dragged a Symbol away from its location
         if (draggedFromSymL == null
               && !((e.getModifiers() & InputEvent.CTRL_MASK) > 0)) {
            draggedFromSymL = draggedSymbol.getParentSymbolList();
         }
         removeTemporaryInsertedSym_(draggedSymbol, e);
         if (hitObject != null) {
            temporaryInsertSym_(draggedSymbol);
         }
      }
      repaint();
   } // mouseDragged

   public void mouseReleased (MouseEvent e) {
      if (paintSymbol == null && draggedSymbol != null) {
         mouseDragged(e);
         model.createEvent(this, TableEvent.CHAR_DESELECT_ALL);
         if (draggedFromSymL != charList && draggedFromSymL != epsilonList
               && draggedFromSymL != equivList && draggedFromSymL != null
               && draggedFromSymL != droppedIntoSymL) {
            // the symbol was dragged from a cell
            removeFromCell_(draggedSymbol, draggedFromSymL);
         }
         if (droppedIntoSymL != null) {
            // the symbol was dropped into a cell, update content of the cell
            addToCell_(draggedSymbol, droppedIntoSymL);
         }
         draggedSymbol.drag(false);
         renderer.markSymbol(draggedSymbol, null);
         draggedSymbol = null;
         droppedIntoSymL = null;
         draggedFromSymL = null;
      }
      updateBounds();
   }

   public void mouseEntered (MouseEvent e) {}

   public void mouseExited (MouseEvent e) {}

   public void mouseClicked (MouseEvent e) {
      hitObject = objectHitBy(e.getX(), e.getY());
      hitLocation_.whoIs(hitObject);
      // enter & exit paintmode9
      if (paintSymbol != null && !hitLocation_.insideTableCell()) {
         //(hitObject == null || hitObject == this)) {
         exitPaintMode();
      }
      if (hitLocation_.isSymbol() && hitLocation_.insideSpecialSymbolList()) {
         enterPaintMode();
         draggedSymbol = null;
      }
      // paint symbol
      if (paintSymbol != null && hitLocation_.insideTableCell()) {
         setCursor(currentCursor);
         // toggle paint: remove symbol if already in cell...
         if (cellContentEqualsPaintSym) {
            removeFromCell_(paintSymbolCursor, hitLocation_.symbolList);
         }
         // ...else add symbol
         else {
            addToCell_(paintSymbol, hitLocation_.symbolList);
            cellContentEqualsPaintSym = true;
            paintSymbolCursor = (Symbol) paintSymbolCursor.clone();
         }
      }
      updateBounds();
   }

   /////////////////////////////////////
   private void removeFromCell_ (Symbol symbol, SymbolList symL) {
      if (symbol == null || symL == null) return;
      TableCell cell = (TableCell) s2c.get(symL);
      if (equivSymbol.toString().equals(symbol.getModel().toString())) {
         cell.clearEquiv();
      } else {
         symL.clearBackup();
         cell.clear();
      }
      updateBounds();
   }

   private void addToCell_ (Symbol symbol, SymbolList symL) {
      if (symbol == null || symL == null) return;
      TableCell cell = (TableCell) s2c.get(symL);
      if (equivSymbol.toString().equals(symbol.getModel().toString())) {
         cell.setEquiv();
      } else {
         cell.set(symbol.getChar());
         model.createEvent(this, TableEvent.CELL_ACTIVATE, cell);
      }
      symL.clearBackup();
      updateBounds();
   }

   private void removeTemporaryInsertedSym_ (Symbol dragSym, MouseEvent e) {
      if (null == dragSym) { return; }
      SymbolList psl = dragSym.getParentSymbolList();
      psl.removeAndRestoreContent(dragSym);
      dragSym.setVisible(true);
      dragSym.setLocation(e.getX(), e.getY()); // move symbol to new position
      if (paintSymbol != null) setCursor(currentCursor);
      droppedIntoSymL = null;
      updateBounds();
   }

   private void temporaryInsertSym_ (Symbol dragSym) {
      if (hitLocation_.insideTableCell()) {
         hitLocation_.symbolList.backupContentAndInsert(dragSym);
         droppedIntoSymL = hitLocation_.symbolList;
         if (dragSym != null) dragSym.setVisible(false);
         if (paintSymbol != null)
               setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
         updateBounds();
      }
   }

   private void enterPaintMode () {
      if (paintSymbol != null) {
         renderer.markSymbol(paintSymbol, null);
      }
      paintSymbol = hitLocation_.symbol;
      paintSymbolCursor = (Symbol) paintSymbol.clone();
      paintSymbolCursor.setVisible(false);
      currentCursor = (Cursor) cursors.get(hitLocation_.symbol);
      setCursor(currentCursor);
      renderer.markSymbol(paintSymbol, renderer.getPaintSymColor());
      if (!equivSymbol.toString().equals(paintSymbol.getModel().toString())) {
         model.createEvent(this, TableEvent.CHAR_SELECT, paintSymbol.getChar());
      } else {
         model.createEvent(this, TableEvent.CHAR_DESELECT_ALL);
      }
   }

   private void exitPaintMode () {
      if (null != paintSymbol) {
         renderer.markSymbol(paintSymbol, null);
         paintSymbol = null;
      }
      paintSymbolCursor = null;
      setCursor(Cursor.getDefaultCursor());
      model.createEvent(this, TableEvent.CHAR_DESELECT_ALL);
   }

   void unmarkAllCells () {
      for (int r = 0; r < nOfRows(); r++) {
         RowUI curr = (RowUI) rows.get(r);
         curr.unmarkAllItems(renderer);
      }
   }

   /** calculates positions of the visible elements. */
   public void updateBounds () {
      int x = origin_X;
      int y = origin_Y;
      charList.calcSpread();
      epsilonList.calcSpread();
      equivList.calcSpread();
      // update widths & heights of cells
      rowHeights = new int[nOfRows()];
      colWidths = new int[nOfRows()];
      for (int r = 0; r < nOfRows(); r++) {
         RowUI curr = (RowUI) rows.get(r);
         curr.calcSpreads();
         for (int c = r; c < nOfCols(); c++) {
            rowHeights[r] = Math.max(rowHeights[r], curr.getHeight(c - r));
            colWidths[c] = Math.max(colWidths[c], curr.getWidth(c - r));
         }
      }
      // arrange elements
      bounds.x = x;
      bounds.y = y;
      int width = x + insets.left;
      int height = insets.top;
      charList.updateBounds(width, y + height, charList.getWidth(), charList
            .getHeight());
      width += charList.getWidth() + RowUI.hgap;
      epsilonList.updateBounds(width, y + height, epsilonList.getWidth(),
            epsilonList.getHeight());
      width += epsilonList.getWidth() + RowUI.hgap;
      equivList.updateBounds(width, y + height, equivList.getWidth(), equivList
            .getHeight());
      width += equivList.getWidth() + RowUI.hgap;
      height += charList.getHeight() + charList_table_vgap;
      int offset = x;
      for (int r = 0; r < nOfRows(); r++) {
         RowUI current = (RowUI) rows.get(r);
         current.updateBounds(offset + insets.left, y + height, rowHeights[r]);
         width = Math.max(width, offset + current.getWidth());
         height += current.getHeight() + rows_vgap;
         offset += colWidths[r] + RowUI.hgap;
      }
      if (colWidths[colWidths.length - 1] > RowLabel.spaceY) {
         lastRowLabel.updateOrigin(offset + insets.left
               - (colWidths[colWidths.length - 1] - RowLabel.spaceX) / 2, y
               + height);
      } else {
         lastRowLabel.updateOrigin(offset + insets.left, y + height);
      }
      height += RowLabel.spaceY;
      bounds.width = width + insets.left + insets.right;
      bounds.height = height - rows_vgap + insets.bottom;
      this.setPreferredSize(new Dimension(bounds.width, bounds.height + 30));
      validate();
   }

   public void paintComponent (Graphics _g) {
      if (null == cursors) {
         // create Cursors for paintmode.
         cursors = new Hashtable();
         for (int i = 0; i < charList.symbols.size(); i++) {
            addSymbolCursor((Symbol) charList.symbols.get(i), cursors);
         }
         addSymbolCursor((Symbol) epsilonList.symbols.get(0), cursors);
         addSymbolCursor((Symbol) equivList.symbols.get(0), cursors);
      }
      renderer.prolog();
      renderer.paintTableView(this, _g);
      Graphics2D g = (Graphics2D) _g;
      if (ANTIALIASED)
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                  RenderingHints.VALUE_ANTIALIAS_ON);
      g.setFont(font);
      // paint table
      charList.paint(renderer, g);
      epsilonList.paint(renderer, g);
      equivList.paint(renderer, g);
      for (int r = 0; r < nOfRows(); r++) {
         RowUI current = (RowUI) rows.get(r);
         current.paint(renderer, g);
      }
      renderer.paintRowLabel(lastRowLabel, _g);
      // paint dragged symbol
      if (null != draggedSymbol && draggedSymbol.visible) { //paintDraggedSym)
         // {
         //g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
         // 0.70f));
         draggedSymbol.paint(renderer, g);
      }
      renderer.epilog();
   }

   /** returns the Object that covers the specified coordinates. */
   private Object objectHitBy (int x, int y) {
      Object obj = null;
      obj = charList.objectHitBy(x, y);
      if (null != obj) return obj;
      obj = epsilonList.objectHitBy(x, y);
      if (null != obj) return obj;
      obj = equivList.objectHitBy(x, y);
      if (null != obj) return obj;
      for (int r = 0; r < nOfRows(); r++) {
         RowUI current = (RowUI) rows.get(r);
         obj = current.objectHitBy(x, y);
         if (null != obj) return obj;
      }
      return this;
   }

   /**
    * Creates Cursor for all Symbols in the alphabet, epsilon and the
    * equivalence-symbol and maps the symbols in the symbolLists
    * (TableView.charList, TableView.epsilonList, TableView.equivList) to these
    * Cursors using the specified Hashtable.
    */
   private void addSymbolCursor (Symbol symbol, Hashtable curs) {
      BufferedImage img = new BufferedImage(32, 32,
            BufferedImage.TYPE_4BYTE_ABGR);
      Graphics2D g = (Graphics2D) img.getGraphics();
      configureGraphics(g);
      WritableRaster raster = img.getAlphaRaster();
      double[] data = new double[32 * 32];
      Arrays.fill(data, 0);
      raster.setPixels(0, 0, 31, 31, data);
      Symbol sym = (Symbol) symbol.clone();
      sym.updateSpread();
      sym.updateOrigin((16 - sym.getWidth() / 2), (16 - sym.getHeight() / 2));
      renderer.markSymbol(sym, renderer.getCursorColor());
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_OFF);
      g.setBackground(Color.white);
      sym.paint(renderer, g);
      curs.put(symbol, Toolkit.getDefaultToolkit().createCustomCursor(img,
            new Point(16, 16), "")); //$NON-NLS-1$
   }
}