/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Frame;

import javax.swing.event.MouseInputListener;

import ch.ethz.exorciser.fsmgui.FSMControllerInterface;
import ch.ethz.exorciser.fsmgui.FSMEditorOperations;
import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMStateInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.fsmgui.State;
import ch.ethz.exorciser.fsmgui.Transition;
import ch.ethz.exorciser.ifa.ViewerDiagramMouseListener;

/**
 * Controller for FA.
 * 
 * @see ch.ethz.exorciser.fsmgui.FSMControllerInterface
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class TwinFAController implements FSMControllerInterface {

   private FSMModelInterface model_;

   private FSMView view_;

   private FSMEditorOperations editorOps_;

   private ViewerDiagramMouseListener mouseListener_;

   private TwinFAPopup popup_;

   /**
    * Create a TwinFAController.
    * 
    * @param model
    *           model of the fa that has to be controlled.
    * @param view
    *           the corresponding view of the fa.
    */
   public TwinFAController (FSMModelInterface model, FSMView view) {
      model_ = model;
      view_ = view;
      editorOps_ = new FSMEditorOperations(model_, view_);
      mouseListener_ = new ViewerDiagramMouseListener(this);
      view_.addMouseListener(mouseListener_);
      view_.addMouseMotionListener(mouseListener_);
   }

   /**
    * Setter for the model of the fa.
    */
   public void setModel (FSMModelInterface model) {
      model_ = model;
      editorOps_.setModel(model_);
      mouseListener_.setEditTarget(this);
   }

   /**
    * Creates a popup-menu.
    */
   public void createDefaultPopup (Frame parent) {
      popup_ = new TwinFAPopup(this, parent);
   }

   /**
    * Creates a popup-menu.
    * 
    * @param ac
    *           ActivityController where activity of InputTracker can be set
    *           from popup-menu.
    */
   public void createDefaultPopup (Frame parent, ActivityController ac) {
      popup_ = new TwinFAPopup(this, parent, ac);
   }

   /**
    * Getter for the popup-menu.
    * 
    * @see ch.ethz.ifa.mdfa.twinfa.popup.TwinFAPopup
    */
   public TwinFAPopup getPopup () {
      return popup_;
   }

   /**
    * Getter for MouseInputListener.
    * 
    * @see ch.ethz.exorciser.ifa.ViewerDiagramMouseListener
    */
   public MouseInputListener getMouseInputListener () {
      return mouseListener_;
   }

   /**
    * Getter for FSMEditorOperations.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMEditorOperations
    */
   public FSMEditorOperations getFSMEditorOperations () {
      return editorOps_;
   }

   /**
    * Getter for the view of the fa.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMView
    */
   public FSMView getView () {
      return view_;
   }

   /**
    * Getter for the fa model.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMModelInterface
    */
   public FSMModelInterface getModel () {
      return model_;
   }

   public void changeFSM (int x, int y) {
      popup_.getPopup(x, y).show(getView(), x, y);
   }

   public FSMStateInterface addState (double x, double y) {
      return null;
   }

   public boolean addTransitionAllowed (FSMStateInterface state) {
      return true;
   }

   public void addTransition (Transition transition, int x, int y) {}

   public void changeState (FSMStateInterface state, int x, int y) {}

   public void selectState (State state) {}

   public void changeTransition (Transition transition, int x, int y) {}

   public void changeTransition (Transition transition, State sOld, State sNew) {}

   public void selectTransition (Transition transition) {}
}