/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import javax.swing.JComponent;

import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.fsmgui.FSMControllerInterface;
import ch.ethz.exorciser.fsmgui.FSMModelInterface;
import ch.ethz.exorciser.fsmgui.FSMView;
import ch.ethz.exorciser.fsmgui.MVCFactory;
import ch.ethz.exorciser.ifa.FAModel;

/**
 * Model view controller factory.
 * 
 * @see ch.ethz.exorciser.fsmgui.MVCFactory
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class TwinFAMVCFactory implements MVCFactory {

   private FSMModelInterface model_;

   private FSMView view_;

   private FSMControllerInterface controller_;

   /**
    * Create a TwinFAMVCFactory that sets default model, view and controller.
    */
   public TwinFAMVCFactory () {
      model_ = new FAModel("new"); //$NON-NLS-1$
      model_.setAlphabet(Alphabet.createABeAlphabet());
      view_ = new FSMView(model_);
      controller_ = new TwinFAController(model_, view_);
   }

   /**
    * Getter for the model of the fa.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMModelInterface
    */
   public FSMModelInterface getModel () {
      return model_;
   }

   /**
    * Getter for the view of the fa.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMView
    */
   public JComponent getView () {
      return view_;
   }

   /**
    * Getter for the controller for the fa.
    * 
    * @see ch.ethz.exorciser.fsmgui.FSMControllerInterface
    */
   public FSMControllerInterface getController () {
      return controller_;
   }

   /**
    * Setter for the model of the fa. Updates view and controller.
    */
   public void setModel (FSMModelInterface model) {
      model_ = model;
      view_.setModel(model_);
      ((TwinFAController) controller_).setModel(model_);
   }
}