/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.minfa;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import ch.ethz.exorciser.Messages;
import ch.ethz.exorciser.fsmgui.DefaultPopup;
import ch.ethz.exorciser.ifa.AccNonAccLayouter;

/**
 * Popup-menu for fas.
 * 
 * @see ch.ethz.exorciser.fsmgui.DefaultPopup
 * @author Alexander Imfeld
 * @author Vincent Tscherter
 * @author Reto Lamprecht
 */
public class TwinFAPopup extends DefaultPopup implements ActionListener {

   private TwinFAController controller_;

   private JCheckBoxMenuItem trackInput_;

   private ActivityController activityController_;

   protected JMenuItem accNaccItem;

   /**
    * Create TwinFAPopup.
    */
   public TwinFAPopup (TwinFAController controller, Frame parent) {
      this(controller, parent, null);
   }

   /**
    * Create TwinFAPopup.
    * 
    * @param activityController
    *           ActivityController where activity of InputTracker can be set.
    */
   public TwinFAPopup (TwinFAController controller, Frame parent,
         ActivityController activityController) {
      super(controller, parent);
      activityController_ = activityController;
      trackInput_ = new JCheckBoxMenuItem(Messages
            .getString("FSM.popup.track_input")); //$NON-NLS-1$
      trackInput_.setActionCommand(Messages.getString("FSM.popup.track_input")); //$NON-NLS-1$
      controller_ = controller;
      accNaccItem = new JMenuItem(Messages.getString("FSM.popup.split")); //$NON-NLS-1$
      accNaccItem.addActionListener(this);
   }

   public void setEditTarget (TwinFAController controller) {
      controller_ = controller;
      this.editorOps = controller_.getFSMEditorOperations();
   }

   /**
    * Getter for the popup-menu.
    */
   public JPopupMenu getPopup (int x, int y) {
      this.lastClickX = x;
      this.lastClickY = y;
      menu.removeAll();
      menu.add(getLayoutMenu());
      menu.add(getStyleMenu());
      menu.addSeparator();
      menu.add(trackInput_);
      trackInput_.addActionListener(this);
      menu.addSeparator();
      menu.add(copyItem);
      return menu;
   }

   protected JMenu getLayoutMenu () {
      layoutMenu.removeAll();
      // "Fit"
      layoutMenu.add(fitItem);
      layoutMenu.addSeparator();
      // "Organic"
      layoutMenu.add(organicItem);
      // "Circle"
      layoutMenu.add(circleItem);
      // "Accept/Non-Accept Split"
      layoutMenu.add(accNaccItem);
      layoutMenu.addSeparator();
      // "Align with grid"
      layoutMenu.add(alignItem);
      if (fsmView.isGridAligned()) {
         alignItem.setState(true);
      }
      return layoutMenu;
   }

   /**
    * Setter for Checkbox state in menu that specifies activity of InputTracker.
    */
   public void setTrackInput (boolean activity) {
      trackInput_.setState(activity);
   }

   /**
    * Handle menu-actions, especially Checkbox state changes for InputTracker.
    */
   public void actionPerformed (ActionEvent e) {
      if (e.getActionCommand().equals(
            Messages.getString("FSM.popup.track_input"))) { //$NON-NLS-1$
         if (activityController_ != null) {
            activityController_.setActivity(trackInput_.getState());
         }
      } else if (Messages.getString("FSM.popup.split").equals( //$NON-NLS-1$
            e.getActionCommand())) { //$NON-NLS-1$
         AccNonAccLayouter.layout(fsmView);
         fsmView.fit();
         fsmView.repaint();
      } else {
         super.actionPerformed(e);
      }
   }
}