/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FA;

/**
 * @author Vincent Tscherter
 */
public class Concatenation extends RegularExpression {

   private static final String OPERATOR = "."; //$NON-NLS-1$

   private List res = new ArrayList();

   private Concatenation () {}

   public static RegularExpression concatenation (RegularExpression r1,
         RegularExpression r2) {
      if (r1 instanceof Epsilon) { return (RegularExpression) r2.clone(); }
      if (r2 instanceof Epsilon) { return (RegularExpression) r1.clone(); }
      Concatenation c = new Concatenation();
      if (r1 instanceof Concatenation) {
         c.res.addAll(((Concatenation) r1).res);
      } else {
         c.res.add(r1);
      } // if .. else
      if (r2 instanceof Concatenation) {
         c.res.addAll(((Concatenation) r2).res);
      } else {
         c.res.add(r2);
      } // if .. else
      try {
         c.fa = (FA) r1.fa.clone();
         c.fa.nDetConcatenation((FA) r2.fa.clone());
         c.fa.makeDetOpt();
         c.fa.detMinimize();
         return c;
      } catch (Exception ex) {
         Debug.showException(ex);
         return null;
      }
   } // concatentation

   public Object clone () {
      Concatenation c = new Concatenation();
      for (Iterator i = this.res.iterator(); i.hasNext();) {
         c.res.add(((RegularExpression) i.next()).clone());
      } // for
      c.fa = (FA) this.fa.clone();
      return c;
   } // clone

   /**
    * @see RegularExpression#getPrefixForm()
    */
   public String getPrefixForm () {
      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < res.size() - 1; i++) {
         buf.append(OPERATOR);
      }
      for (int i = 0; i < res.size(); i++) {
         buf.append(((RegularExpression) res.get(i)).getPrefixForm());
      }
      return buf.toString();
   }

   /**
    * @see RegularExpression#getSuffixForm()
    */
   public String getSuffixForm () {
      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < res.size(); i++) {
         buf.append(((RegularExpression) res.get(i)).getSuffixForm());
      }
      for (int i = 0; i < res.size() - 1; i++) {
         buf.append(OPERATOR);
      }
      return buf.toString();
   }

   public String getInfixForm () {
      StringBuffer buf = new StringBuffer();
      RegularExpression r = (RegularExpression) res.get(0);
      if (r instanceof Union) {
         buf.append("(" + r.getInfixForm() + ")"); //$NON-NLS-1$ //$NON-NLS-2$
      } else {
         buf.append(r.getInfixForm());
      }
      for (int i = 1; i < res.size(); i++) {
         r = (RegularExpression) res.get(i);
         if (r instanceof Union) {
            buf.append("(" + r.getInfixForm() + ")"); //$NON-NLS-1$ //$NON-NLS-2$
         } else {
            buf.append(r.getInfixForm());
         }
      }
      return buf.toString();
   }

   /**
    * @see RegularExpression#getEBNF()
    */
   public String getBackusNaurForm () {
      StringBuffer buf = new StringBuffer();
      RegularExpression r = (RegularExpression) res.get(0);
      if (r instanceof Union) {
         buf.append("(" + r.getBackusNaurForm() + ")"); //$NON-NLS-1$ //$NON-NLS-2$
      } else {
         buf.append(r.getBackusNaurForm());
      }
      for (int i = 1; i < res.size(); i++) {
         r = (RegularExpression) res.get(i);
         if (r instanceof Union) {
            buf.append("(" + r.getBackusNaurForm() + ")"); //$NON-NLS-1$ //$NON-NLS-2$
         } else {
            buf.append(r.getBackusNaurForm());
         }
      }
      return buf.toString();
   }
} // Kleen
