/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.ifa.FAModel;
import ch.ethz.exorciser.ifa.State;

/**
 * @author Vincent Tscherter
 */
public class Epsilon extends RegularExpression {

   public Epsilon () {
      try {
         this.fa = new FAModel();
         this.fa.setAlphabet(Alphabet.createABeAlphabet());
         State one = new State(""); //$NON-NLS-1$
         this.fa.addState(one);
         this.fa.setStartState(one);
         this.fa.setAccepting(one, true);
         this.fa.complete();
      } catch (Exception ex) {
         Debug.showException(ex);
      }
   }

   public Object clone () {
      return new Epsilon();
   }

   /**
    * @see RegularExpression#getPrefixForm()
    */
   public String getPrefixForm () {
      return getInfixForm();
   }

   /**
    * @see RegularExpression#getSuffixForm()
    */
   public String getSuffixForm () {
      return getInfixForm();
   }

   /**
    * @see RegularExpression#getInfixForm()
    */
   public String getInfixForm () {
      return Alphabet.EPSILON.toString();
   }

   /**
    * @see RegularExpression#getEBNF()
    */
   public String getBackusNaurForm () {
      return getInfixForm();
   }
} // Epsilon
