/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FA;

/**
 * @author Vincent Tscherter
 */
public class Intersection extends RegularExpression {

    public static final String OPERATOR = "&"; //$NON-NLS-1$

    private List res = new ArrayList();

    public static RegularExpression intersect(RegularExpression r1,
            RegularExpression r2) {
        try {
            if (r1 instanceof Empty) {
                return new Empty();
            }
            if (r2 instanceof Empty) {
                return new Empty();
            }
            Intersection u = new Intersection();
            if (r1 instanceof Intersection) {
                u.res.addAll(((Intersection) r1).res);
            } else {
                u.res.add(r1);
            }
            if (r2 instanceof Intersection) {
                u.res.addAll(((Intersection) r2).res);
            } else {
                u.res.add(r2);
            }
            u.fa = (FA) r1.fa.clone();
            u.fa.detIntersection((FA) r2.fa.clone());
            u.fa.detMinimize();
            return u;
        } catch (Exception ex) {
            Debug.showException(ex);
            return null;
        }
    } 

    public Object clone() {
        Intersection u = new Intersection();
        for (Iterator i = this.res.iterator(); i.hasNext();) {
            u.res.add(((RegularExpression) i.next()).clone());
        }
        u.fa = (FA) this.fa.clone();
        return u;
    }

    public String getPrefixForm() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < res.size() - 1; i++) {
            buf.append(OPERATOR);
        }
        for (int i = 0; i < res.size(); i++) {
            buf.append(((RegularExpression) res.get(i)).getPrefixForm());
        }
        return buf.toString();
    }

    public String getSuffixForm() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < res.size(); i++) {
            buf.append(((RegularExpression) res.get(i)).getSuffixForm());
        }
        for (int i = 0; i < res.size() - 1; i++) {
            buf.append(OPERATOR);
        }
        return buf.toString();
    }

    public String getInfixForm() {
        StringBuffer buf = new StringBuffer("("); //$NON-NLS-1$
        buf.append(((RegularExpression) res.get(0)).getInfixForm());
        for (int i = 1; i < res.size(); i++) {
            buf.append(OPERATOR
                    + ((RegularExpression) res.get(i)).getInfixForm());
        }
        return buf.append(")").toString(); //$NON-NLS-1$
    }


    public String getBackusNaurForm() {
    	throw new UnsupportedOperationException();
    }

    public static void main(String[] args) {
        
    }
}