/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FA;

/**
 * @author Vincent Tscherter
 */
public class KleeneStar extends RegularExpression {

   public static final String OPERATOR = "*"; //$NON-NLS-1$

   private RegularExpression re;

   private KleeneStar () {}

   public static RegularExpression kleen (RegularExpression re) {
      if (re instanceof Empty) { return new Epsilon(); }
      if (re instanceof Epsilon) { return new Epsilon(); }
      try {
         KleeneStar k = new KleeneStar();
         k.re = re;
         k.fa = (FA) re.fa.clone();
         k.fa.nDetStar();
         k.fa.makeDetOpt();
         k.fa.detMinimize();
         return k;
      } catch (Exception ex) {
         Debug.showException(ex);
         return null;
      }
   }

   public Object clone () {
      KleeneStar k = new KleeneStar();
      k.re = (RegularExpression) this.re.clone();
      k.fa = (FA) this.fa.clone();
      return k;
   }

   /**
    * @see RegularExpression#getPrefixForm()
    */
   public String getPrefixForm () {
      return OPERATOR + re.getPrefixForm();
   }

   /**
    * @see RegularExpression#getSuffixForm()
    */
   public String getSuffixForm () {
      return re.getSuffixForm() + OPERATOR;
   }

   /**
    * @see RegularExpression#getInfixForm()
    */
   public String getInfixForm () {
      if (re instanceof Symbol || re instanceof Empty || re instanceof Epsilon
            || re instanceof KleeneStar) {
         return re.getInfixForm() + OPERATOR;
      }
	return "(" + re.getInfixForm() + ")" + OPERATOR; //$NON-NLS-1$ //$NON-NLS-2$
   }

   /**
    * @see RegularExpression#getEBNF()
    */
   public String getBackusNaurForm () {
      return "{" + re.getBackusNaurForm() + "}"; //$NON-NLS-1$ //$NON-NLS-2$
   }
} // Kleen
