/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FA;

/**
 * @author Vincent Tscherter
 */
public class Minus extends RegularExpression {

	public static final String OPERATOR = "-"; //$NON-NLS-1$

	private RegularExpression r1, r2;

	public static RegularExpression minus(RegularExpression r1,
			RegularExpression r2) {
		try {
			if (r1 instanceof Empty) {
				return new Empty();
			}
			if (r2 instanceof Empty) {
				return (RegularExpression) r1.clone();
			}

			Minus u = new Minus();
			u.r1 = r1;
			u.r2 = r2;
			u.fa = (FA) r1.fa.clone();
			u.fa.detMinus(r2.fa);
			u.fa.detMinimize();
			return u;
		} catch (Exception ex) {
			Debug.showException(ex);
			return null;
		}
	}

	public Object clone() {
		Minus u = new Minus();
		u.r1 = ((RegularExpression) r1.clone());
		u.r2 = ((RegularExpression) r2.clone());
		u.fa = (FA) fa.clone();
		return u;
	}

	public String getPrefixForm() {
		StringBuffer buf = new StringBuffer(OPERATOR);
		buf.append(r1.getPrefixForm());
		buf.append(r2.getPrefixForm());
		return buf.toString();
	}

	public String getSuffixForm() {
		StringBuffer buf = new StringBuffer(r1.getSuffixForm());
		buf.append(r2.getSuffixForm());
		buf.append(OPERATOR);
		return buf.toString();
	}

	public String getInfixForm() {
		String a = r1.getInfixForm();
		String b = r2.getInfixForm();
		return ((a.length() == 1) ? (a) : ("(" + a + ")")) + OPERATOR//$NON-NLS-1$ //$NON-NLS-2$ 
				+ ((b.length() == 1) ? (b) : ("(" + b + ")")); //$NON-NLS-1$ //$NON-NLS-2$ 
	}

	public String getBackusNaurForm() {
		throw new UnsupportedOperationException();
	}

}