/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import ch.ethz.exorciser.Messages;

/**
 * @author Nicolas Lefebvre
 * @author Vincent Tscherter
 */
public class REParserException extends Exception {

   String message;

   public REParserException (String re, int index) {
      if (re.length() <= 0) {
         message = Messages.getString("REParserException.empty"); //$NON-NLS-1$
      } else if (index >= re.length()) {
         message = Messages.getString("REParserException.title").replaceAll( //$NON-NLS-1$
               "\\$EXPRESSION", re); //$NON-NLS-1$
      } else {
         char ch = re.charAt(index);
         if (ch == ')') {
            message = Messages.getString("REParserException.closeRR") //$NON-NLS-1$
                  .replaceAll("\\$EXPRESSION", re).replaceAll( //$NON-NLS-1$
                        "\\$POSITION", index + ""); //$NON-NLS-1$ //$NON-NLS-2$
         } else if (ch == '(') {
            message = Messages.getString("REParserException.closeLR") //$NON-NLS-1$
                  .replaceAll("\\$EXPRESSION", re).replaceAll( //$NON-NLS-1$
                        "\\$POSITION", index + ""); //$NON-NLS-1$ //$NON-NLS-2$
         } else if (ch == '}') {
            message = Messages.getString("REParserException.closeRT") //$NON-NLS-1$
                  .replaceAll("\\$EXPRESSION", re).replaceAll( //$NON-NLS-1$
                        "\\$POSITION", index + ""); //$NON-NLS-1$ //$NON-NLS-2$
         } else if (ch == '{') {
            message = Messages.getString("REParserException.closeLT") //$NON-NLS-1$
                  .replaceAll("\\$EXPRESSION", re).replaceAll( //$NON-NLS-1$
                        "\\$POSITION", index + ""); //$NON-NLS-1$ //$NON-NLS-2$
         } else {
            message = Messages.getString("REParserException.wrong_symbol") //$NON-NLS-1$
                  .replaceAll("\\$EXPRESSION", re).replaceAll( //$NON-NLS-1$
                        "\\$SYMBOL", ch + "").replaceAll("\\$POSITION", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                        index + ""); //$NON-NLS-1$
         }
      }
   }

   public String getMessage () {
      return message;
   }
}