/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.EmptyStackException;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FAModel;
import ch.ethz.exorciser.ifa.IFACompleteEditor;
import ch.ethz.exorciser.rl.re.dictionary.REPrefixDictionary;

/**
 * @author Vincent Tscherter
 */
public class RErpnc extends JPanel {

   protected JList list;

   //	private static final int STACKFRAME = 6;
   protected Stack stack = new Stack();

   private IFACompleteEditor viewer = new IFACompleteEditor(
         IFACompleteEditor.VIEWER);
   static {
      REPrefixDictionary.getInstance().load();
   }

   public RErpnc () {
      this.setLayout(new GridLayout(2, 1));
      JPanel com = new JPanel();
      com.setLayout(new GridLayout(1, 5));
      //
      // alphabeth ....
      //
      JPanel p = new JPanel(new GridLayout(4, 1));
      JButton but = new JButton("a"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            stack.push(Symbol.createASymbol());
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("b"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            stack.push(Symbol.createBSymbol());
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("\u03b5"); // unicode for epsilon I hope //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            stack.push(new Epsilon());
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton(""); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            stack.push(new Empty());
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      com.add(p);
      //
      // regular expressions ....
      //
      p = new JPanel(new GridLayout(5, 1));
      but = new JButton("XY"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.push(Concatenation.concatenation((RegularExpression) stack
                     .pop(), (RegularExpression) stack.pop()));
            } catch (Exception ex) {
               Debug.showException(ex);
            }
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("X|Y"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.push(Union.union((RegularExpression) stack.pop(),
                     (RegularExpression) stack.pop()));
            } catch (Exception ex) {
               Debug.showException(ex);
            }
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("X*"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.push(KleeneStar.kleen((RegularExpression) stack.pop()));
            } catch (Exception ex) {
               Debug.showException(ex);
            }
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("random"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            stack.push(REPrefixDictionary.getInstance().getRandomRE());
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("optimize"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               RegularExpression re = (RegularExpression) stack.peek();
               RegularExpression reopt = REPrefixDictionary.getInstance()
                     .getRE(re.getFA());
               if (reopt != null) {
                  stack.pop();
                  stack.push(reopt);
               }
               list.setListData(stack.toArray());
               updateFAViewer();
            } catch (EmptyStackException ex) {
               Debug.showException(ex);
            }
         }
      });
      p.add(but);
      com.add(p);
      //
      // stack
      //
      list = new JList();
      //list.setListData(stack.toArray());
      com.add(new JScrollPane(list));
      //
      // stackoperations
      //
      p = new JPanel(new GridLayout(5, 1));
      but = new JButton("ENTER"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.push(((RegularExpression) stack.peek()).clone());
               list.setListData(stack.toArray());
               updateFAViewer();
            } catch (Exception ex) {
               Debug.showException(ex);
            } // try .. catch
         }
      });
      p.add(but);
      but = new JButton("CLR X"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.pop();
            } catch (Exception ex) {
               Debug.showException(ex);
            }
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      but = new JButton("CLR ALL"); //$NON-NLS-1$
      but.addActionListener(new ActionListener() {

         public void actionPerformed (ActionEvent e) {
            try {
               stack.clear();
            } catch (Exception ex) {
               Debug.showException(ex);
            }
            list.setListData(stack.toArray());
            updateFAViewer();
         }
      });
      p.add(but);
      p.add(but);
      com.add(p);
      this.add(com);
      this.add(viewer);
   } // constructor

   protected void updateFAViewer () {
      try {
         if (!stack.empty()) {
            viewer.setFA(((RegularExpression) stack.peek()).getFA());
         } else {
            viewer.setFA(new FAModel());
         }
      } catch (Exception ex) {
         Debug.showException(ex);
      } // try .. catch
   } // updateFAViewer

   public static void main (String[] argv) {
      JFrame f = new JFrame();
      // f.setTitle(getString("Frame.title"));
      f.getContentPane().add(new RErpnc());
      f.addWindowListener(new WindowAdapter() {

         public void windowClosing (WindowEvent e) {
            System.exit(0);
         }
      });
      f.pack();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      f.setLocation(screenSize.width / 2 - f.getSize().width / 2,
            screenSize.height / 2 - f.getSize().height / 2);
      f.setVisible(true);
   } // main
} // RErpnc
