/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.rl.re;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.ifa.FA;

/**
 * @author Vincent Tscherter
 */
public class Union extends RegularExpression {

   public static final String OPERATOR = "|"; //$NON-NLS-1$

   private List res = new ArrayList();

   public static RegularExpression union (RegularExpression r1,
         RegularExpression r2) {
      try {
         if (r1 instanceof Empty) { return (RegularExpression) r2.clone(); }
         if (r2 instanceof Empty) { return (RegularExpression) r1.clone(); }
         if (r1 instanceof Epsilon && r2 instanceof KleeneStar) { return (RegularExpression) r2
               .clone(); }
         if (r2 instanceof Epsilon && r1 instanceof KleeneStar) { return (RegularExpression) r1
               .clone(); }
         Union u = new Union();
         if (r1 instanceof Union) {
            u.res.addAll(((Union) r1).res);
         } else {
            u.res.add(r1);
         }
         if (r2 instanceof Union) {
            u.res.addAll(((Union) r2).res);
         } else {
            u.res.add(r2);
         }
         if (u.res.size() > 1) {
            u.fa = (FA) r1.fa.clone();
            u.fa.detUnion((FA) r2.fa.clone());
            u.fa.detMinimize();
            return u;
         }
		return (RegularExpression) u.res.get(0);
      } catch (Exception ex) {
         Debug.showException(ex);
         return null;
      }
   } // union

   public Object clone () {
      Union u = new Union();
      for (Iterator i = this.res.iterator(); i.hasNext();) {
         u.res.add(((RegularExpression) i.next()).clone());
      } // for
      u.fa = (FA) this.fa.clone();
      return u;
   }

   /**
    * @see RegularExpression#getPrefixForm()
    */
   public String getPrefixForm () {
      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < res.size() - 1; i++) {
         buf.append(OPERATOR);
      }
      for (int i = 0; i < res.size(); i++) {
         buf.append(((RegularExpression) res.get(i)).getPrefixForm());
      }
      return buf.toString();
   }

   /**
    * @see RegularExpression#getSuffixForm()
    */
   public String getSuffixForm () {
      StringBuffer buf = new StringBuffer();
      for (int i = 0; i < res.size(); i++) {
         buf.append(((RegularExpression) res.get(i)).getSuffixForm());
      }
      for (int i = 0; i < res.size() - 1; i++) {
         buf.append(OPERATOR);
      }
      return buf.toString();
   }

   /**
    * @see RegularExpression#getInfixForm()
    */
   public String getInfixForm () {
      StringBuffer buf = new StringBuffer();
      buf.append(((RegularExpression) res.get(0)).getInfixForm());
      for (int i = 1; i < res.size(); i++) {
         buf.append(OPERATOR + ((RegularExpression) res.get(i)).getInfixForm());
      }
      return buf.toString();
   }

   /**
    * @see RegularExpression#getEBNF()
    */
   public String getBackusNaurForm () {
      StringBuffer buf = new StringBuffer();
      buf.append(((RegularExpression) res.get(0)).getBackusNaurForm());
      for (int i = 1; i < res.size(); i++) {
         buf.append(OPERATOR
               + ((RegularExpression) res.get(i)).getBackusNaurForm());
      }
      return buf.toString();
   }
} // Kleen
