/*
 * Exorciser - www.educeth.ch/exorciser  Copyright 2000-2004 Swiss Federal
 * Institue of Technology, Zrich Switzerland
 */
package ch.ethz.exorciser.rl.re.dictionary;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import ch.ethz.exorciser.Debug;
import ch.ethz.exorciser.fsmgui.Alphabet;
import ch.ethz.exorciser.ifa.FA;
import ch.ethz.exorciser.ifa.FASerializer;
import ch.ethz.exorciser.rl.re.ABRegExpParser;
import ch.ethz.exorciser.rl.re.REParserException;
import ch.ethz.exorciser.rl.re.RegularExpression;

/**
 * @author Vincent Tscherter
 */
public class REPrefixDictionary implements REDictionary {
	private static final String NEWLINE = "\r\n"; //$NON-NLS-1$

	private static final String SEPARATOR = ";"; //$NON-NLS-1$
	private static REPrefixDictionary singleton = new REPrefixDictionary();

	private static boolean DictionaryFileExists(int number) {
		String name = REPrefixDictionary.class.getPackage().getName().replace('.', '/') + "/prefixre_id" + number + ".dic"; //$NON-NLS-1$ //$NON-NLS-2$
		URL url = ClassLoader.getSystemResource(name);
		return url != null;
	}

	private static File getDictionaryFile(int number) {
		try {
			String dir = REPrefixDictionary.class.getPackage().getName().replace('.', '/');
			dir = ClassLoader.getSystemResource(dir).getFile();
			return new File(new URI("file://" + dir + "/" + "prefixre_id" + number + ".dic")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		} catch (URISyntaxException e) {
			Debug.showException(e);
			return null;
		}
	}

	private static InputStream getDictionaryInputStream(int number) {
		String dir = REPrefixDictionary.class.getPackage().getName().replace('.', '/') + "/prefixre_id" + number + ".dic"; //$NON-NLS-1$ //$NON-NLS-2$
		return ClassLoader.getSystemResourceAsStream(dir);
	}

	public static REPrefixDictionary getInstance() {
		return singleton;
	}

	Object[] dic = new Object[0];
	Object[] dic2 = new Object[200];
	Map id2re = new HashMap();

	private REPrefixDictionary() {}

	private void add(String re, List dn) {
		try {
			String id1 = FASerializer.createDetABSerial(ABRegExpParser.createPrefixFormRE(re).getFA());
			String id2 = FASerializer.createDetABSerial(ABRegExpParser.createPrefixFormRE(sym(re)).getFA());
			if (!id2re.containsKey(id1) && !id2re.containsKey(id2)) {
				id2re.put(id1, re);
				dn.add(re);
				int nofs = FASerializer.NofStates(id1);
				List dn2 = (List) dic2[nofs];
				if (dn2 == null) {
					dn2 = new ArrayList();
					dic2[nofs] = dn2;
				}
				dn2.add(id1);
				System.out.println(re+" added"); //$NON-NLS-1$
			}
		} catch (REParserException e) {}
	}

	private void addNoCheck(String re, String id, List dn) {
		id2re.put(id, re);
		dn.add(re);

		int nofs = FASerializer.NofStates(id);
		List dn2 = (List) dic2[nofs];
		if (dn2 == null) {
			dn2 = new ArrayList();
			dic2[nofs] = dn2;
		}
		dn2.add(id);
	}

	public void expand() {
		if (dic.length == 0) {
			init();
		} else {
			List dn = new ArrayList();
			// star out of d(n-1)
			Object[] d = (Object[]) dic[dic.length - 1];
			for (int i = 0; i < d.length; i++) {
				if (d[i].toString().charAt(0) != '*') {
					add("*" + d[i], dn); //$NON-NLS-1$
					add("*" + sym("" + d[i]), dn); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			// k ...
			for (int k = 0; k < dic.length - k - 1; k++) {
				int l = dic.length - k - 2;
				Object[] dk = (Object[]) dic[k];
				Object[] dl = (Object[]) dic[l];
				for (int i = 0; i < dk.length; i++) {
					for (int j = 0; j < dl.length; j++) {
						add("|" + dk[i] + dl[j], dn); //$NON-NLS-1$
						add("|" + sym("" + dk[i]) + dl[j], dn); //$NON-NLS-1$ //$NON-NLS-2$
						add("." + dk[i] + dl[j], dn); //$NON-NLS-1$
						add("." + sym("" + dk[i]) + dl[j], dn); //$NON-NLS-1$ //$NON-NLS-2$
						add("." + dl[j] + dk[i], dn); //$NON-NLS-1$
						add("." + sym("" + dl[j]) + dk[i], dn); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}
			expand(dn);
		}
	}

	public void expand(int times) {
		for (int i = 0; i < times; i++) {
			expand();
		}
	}

	private void expand(List dn) {
		Object[] dic_ = new Object[dic.length + 1];
		for (int i = 0; i < dic.length; i++) {
			dic_[i] = dic[i];
		}
		dic_[dic.length] = dn.toArray();
		dic = dic_;
	}

	/**
	 * @see ch.ethz.exorciser.rl.re.dictionary.REDictionary#getRandomFA(int)
	 */
	public FA getRandomFA(int nofstates) {
		List d2k = (List) dic2[nofstates];
		if (d2k != null) {
			int i = (int) (Math.random() * d2k.size());
			return FASerializer.createDetABFA("" + d2k.get(i)); //$NON-NLS-1$
		}
		return null;

	}

	public RegularExpression getRandomRE() {
		int i = (int) (Math.random() * id2re.size() - 1);
		int k = 0;
		Object[] d = (Object[]) dic[k];
		while (i >= d.length) {
			i -= d.length;
			k++;
			d = (Object[]) dic[k];
		}
		try {
			return ABRegExpParser.createPrefixFormRE("" + d[i]); //$NON-NLS-1$
		} catch (REParserException e) {
			return null;
		}
	}

	public RegularExpression getRandomRE(int prefixsize) {
		Object[] dk = (Object[]) dic[prefixsize - 1];
		int i = (int) (Math.random() * dk.length);
		try {
			return ABRegExpParser.createPrefixFormRE("" + dk[i]); //$NON-NLS-1$
		} catch (REParserException e) {
			return null;
		}
	}

	public String getRandomString() {
		int i = (int) (Math.random() * id2re.size() - 1);
		int k = 0;
		Object[] d = (Object[]) dic[k];
		while (i >= d.length) {
			i -= d.length;
			k++;
			d = (Object[]) dic[k];
		}
		return "" + d[i]; //$NON-NLS-1$
	}

	public RegularExpression getRE(FA fa) {
		try {
			String id = FASerializer.createDetABSerial(fa);
			if (id2re.containsKey(id)) {
				return ABRegExpParser.createPrefixFormRE("" + id2re.get(id)); //$NON-NLS-1$
			}
			id = FASerializer.createSymetricDetABSerial(fa);
			if (id2re.containsKey(id)) {
				return ABRegExpParser.createPrefixFormRE(sym("" + id2re.get(id))); //$NON-NLS-1$
			}
			return null;
		} catch (REParserException e) {
			return null;
		}
	}

	private void init() {
		List d0 = new ArrayList();
		add("", d0); //$NON-NLS-1$
		add("" + Alphabet.EPSILON, d0); //$NON-NLS-1$
		add("a", d0); //$NON-NLS-1$
		dic = new Object[1];
		dic[0] = d0.toArray();
	}

	public static void main(String[] args) {
//		REPrefixDictionary d = new REPrefixDictionary();
//		d.init();
//		System.out.println("init done");
//		d.expand();
//		System.out.println("expand done");
//		d.save();
		REPrefixDictionary d = getInstance();
		d.load();
		System.out.println("loaded"); //$NON-NLS-1$
		d.expand();
		System.out.println("expanded"); //$NON-NLS-1$
		d.save();
		System.out.println("saved"); //$NON-NLS-1$
	}
	
	public void load() {
		load(Integer.MAX_VALUE);

	}

	public void load(int size) {
		try {
			int i = dic.length + 1;
			InputStream is = getDictionaryInputStream(i);
			while (i <= size && is != null) {
				BufferedReader r = new BufferedReader(new InputStreamReader(is, "UTF-8")); //$NON-NLS-1$
				String line = r.readLine();
				line = r.readLine();
				List di = new ArrayList();
				while (line != null) {
					StringTokenizer t = new StringTokenizer(line);
					String re = t.nextToken(SEPARATOR);
					String id = t.nextToken(SEPARATOR);
					addNoCheck(re, id, di);
					line = r.readLine();
				}
				expand(di);
				i++;
				is = getDictionaryInputStream(i);
			}
		} catch (UnsupportedEncodingException e) {
			Debug.showException(e);
		} catch (FileNotFoundException e) {
			Debug.showException(e);
		} catch (IOException e) {
			Debug.showException(e);
		}
	}

	public void save() {
		try {
			for (int i = 0; i < dic.length; i++) {
				if (!DictionaryFileExists(i + 1)) {
					Writer wi = new OutputStreamWriter(new FileOutputStream(getDictionaryFile(i + 1)), "UTF-8"); //$NON-NLS-1$
					wi.write("// prefix regular expression ; fa id  dic" + (i + 1) + NEWLINE); //$NON-NLS-1$
					Object[] di = (Object[]) dic[i];
					int n = di.length;
					for (int j = 0; j < n; j++) {
						try {
							wi.write(
								di[j]
									+ SEPARATOR
									+ FASerializer.createDetABSerial(ABRegExpParser.createPrefixFormRE("" + di[j]).getFA()) //$NON-NLS-1$
									+ NEWLINE);
						} catch (REParserException e) {}
					}
					wi.close();
				}
			}
		} catch (UnsupportedEncodingException e) {
			Debug.showException(e);
		} catch (FileNotFoundException e) {
			Debug.showException(e);
		} catch (IOException e) {
			Debug.showException(e);
		}
	} //save

	private String sym(String re) {
		String dual = re.replace('a', 't');
		dual = dual.replace('b', 'a');
		dual = dual.replace('t', 'b');
		return dual;
	}

}