/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Graphics;
import java.awt.Rectangle;

/**
 * Abstract edge class used as an interface between a Treebrowser and Tree
 * implementations. Used together with Tree and Node
 * 
 * @author Oliver Biderbost
 */
public abstract class Edge implements Paintable {

   protected boolean _hidden = false;

   protected boolean _highlighted = false;

   protected Node _source = null;

   protected Node _target = null;

   public abstract void paint (Graphics g, Rectangle r);

   public abstract void paintDetails (Graphics g, Rectangle r);

   public abstract void setHide (boolean yes);

   public abstract void setHighlight (boolean yes);

   public Node source () {
      return _source;
   }

   public Node target () {
      return _target;
   }

   public boolean isHidden () {
      return _hidden;
   }

   public boolean isHighlighted () {
      return _highlighted;
   }
}