/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Color;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import ch.ethz.exorciser.Messages;

/**
 * the main class of the treebrowser contains all the stuff needed to create a
 * nice TreeBrowser ;))
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowser extends JPanel {

   private Tree tree;

   private Node currentNode;

   private ArrayList tbc = new ArrayList();

   public TreeBrowser () {
      super();
      init();
   }

   //	public TreeBrowser(boolean isDoubleBuffered) {
   //		super(isDoubleBuffered);
   //		init();
   //	}
   public TreeBrowser (Tree tree) {
      super();
      this.tree = tree;
      init();
   }

   //	public TreeBrowser(Tree tree,boolean isDoubleBuffered) {
   //		super(isDoubleBuffered);
   //		this.tree=tree;
   //		init();
   //	}
   //	private TreeBrowser(LayoutManager layout) {
   //	}
   //	private TreeBrowser(LayoutManager layout,boolean isDoubleBuffered) {
   //	}
   /**
    * Method init. initializes the TreeBrowser and adds all the
    * TreeBrowserComponents
    */
   private void init () {
      //init of parent JPanel
      //setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
      setLayout(new GridLayout(2, 1));
      setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      setBackground(Color.white);
      setNode(tree.root());
      //		setLayout(new BorderLayout());
      //init of the desired treebrowser components
      TreeBrowserTree tbt = new TreeBrowserTree(this);
      tbt.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("TreeBrowser.label.tree"))); //$NON-NLS-1$
      TreeBrowserBrowser tbb = new TreeBrowserBrowser(this);
      JScrollPane tbb_jsp = tbb.getScrollPane();
      //get the scrollpane from the browser component
      tbb_jsp.setBorder(BorderFactory.createTitledBorder(Messages
            .getString("TreeBrowser.label.browser"))); //$NON-NLS-1$
      tbc.add(tbt);
      tbc.add(tbb);
      //the browser and NOT its' scrollpane must be added to the contents
      // list
      add(tbt);
      add(tbb_jsp); //but the scrollpane IS the swing representative
      //		add(tbt, BorderLayout.CENTER);
      //		add(tbb_jsp, BorderLayout.SOUTH);
   }

   /**
    * Method resetTree.
    * 
    * @param tree
    *           sets a new tree
    */
   public void resetTree (Tree t) {
      this.tree = t;
      setNode(t.root());
      updateTB();
   }

   public Tree getTree () {
      return (tree);
   }

   /**
    * Method setNode.
    * 
    * @param node
    *           when the focus changes
    */
   public void setNode (Node node) {
      currentNode = node;
      node.focus();
   }

   public Node getNode () {
      return (currentNode);
   }

   /**
    * Method updateTB. triggers a general update through all
    * TreeBrowserComponents
    */
   public void updateTB () {
      Iterator tbcIter = tbc.iterator();
      while (tbcIter.hasNext())
         ((TreeBrowserComponent) tbcIter.next()).updateTBC();
   }
}