/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JScrollPane;

/**
 * draws and controls the browser controls an ArrayList of
 * TreeBrowserBrowserNode objects if you want a custom design, create a new
 * child of TreeBrowserComponent and fill it with TreeBrowserBrowserNodes
 * according to your desires
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowserBrowser extends TreeBrowserComponent {

   private JScrollPane scrollPane = new JScrollPane(this);

   final static int parent = 0; //index of the parent node

   final static int focus = 1; //index of the focused node

   final static int child = 2; //index of the first child

   private Point2D.Float rect = new Point2D.Float(203, 56);

   // the size of the rectangles
   private Point2D.Float border = new Point2D.Float(10, 2);

   // the size of the border
   private Point2D.Float gapRel = new Point2D.Float((float) 0.2, (float) 0.8);

   // relation between the rectangles and the gaps
   private Point2D.Float smallRectRel = new Point2D.Float((float) 0.5,
         (float) 0.4);

   // relation between the rectangles ant the transition rectangles
   private ArrayList content = new ArrayList();

   //the ArrayList that holds all the components
   private TreeBrowserBrowserContent tbbc;

   //helper variable for the handleMouseMoved method
   private TreeBrowserBrowserContent oldtbbc;

   public TreeBrowserBrowser (TreeBrowser tb) {
      super(tb);
      init();
   }

   //	public TreeBrowserBrowser(TreeBrowser tb,boolean isDoubleBuffered) {
   //		super(tb,isDoubleBuffered);
   //		init();
   //	}
   /**
    * Method init. inits the browser component itself
    */
   private void init () {
      TreeBrowserBrowserListener tbbListener = new TreeBrowserBrowserListener();
      //the listener
      setBackground(Color.white);
      addComponentListener(tbbListener); //add necessary listeners
      addMouseListener(tbbListener);
      addMouseMotionListener(tbbListener);
      scrollPane.setBackground(Color.white);
      scrollPane.setPreferredSize(new Dimension(0, 0));
      initContent();
   }

   //the usual getters and setters
   public JScrollPane getScrollPane () {
      return (scrollPane);
   }

   public void setBorder (float x, float y) {
      border.setLocation(x, y);
   }

   public void setGapRelations (float x, float y) {
      gapRel.setLocation(x, y);
   }

   public void setSmallRectRelations (float x, float y) {
      smallRectRel.setLocation(x, y);
   }

   /**
    * Method initContent. adds all the necessary components to the content
    * ArrayList and initializes them
    */
   private void initContent () {
      content.add(new TreeBrowserBrowserNode());
      content.add(new TreeBrowserBrowserNode((TreeBrowserBrowserNode) content
            .get(parent)));
      ((TreeBrowserBrowserNode) content.get(focus)).setColor(Color.red);
      ((TreeBrowserBrowserNode) content.get(focus)).setStroke(new BasicStroke(
            4.0f));
      updateTBC();
   }

   /**
    * @see nondeterministic.treebrowser.TreeBrowserComponent#updateTBC() the
    *      heart of the browser: creates and initializes all nodes and controls
    *      their dependences is called whenever a event occurs that needs the
    *      browser to be updated
    */
   public void updateTBC () {
      for (int i = 0; i < content.size(); i++)
         ((TreeBrowserBrowserNode) content.get(i)).setActive(false);
      //node recycling
      Node node = tb.getNode(); //get the current node
      if (node != null) { //catch if the focused node is null
         //calculation of the coordinates and sizes
         //main decision: either the whole thing suits in the visible area
         // or not
         //the browser should always be shown in the middle of the visible
         // area
         Rectangle bounds = scrollPane.getViewportBorderBounds();
         Dimension visible = new Dimension(bounds.width - bounds.x,
               bounds.height - bounds.y);
         Dimension dimension = new Dimension(visible);
         int children = node.getChilds().size();
         float xLevel = rect.x * (1 + gapRel.x);
         float yLevel = rect.y * (1 + gapRel.y);
         float xOffset = 0;
         float yOffset = 0;
         float xMiddle;
         if (visible.width < 2 * border.x + xLevel * (children - 1) + rect.x) {
            dimension.width = (int) (2 * border.x + xLevel * (children - 1) + rect.x);
            xMiddle = xMiddle = (float) 0.5 * (children - 1) * xLevel;
         } else {
            xMiddle = (float) 0.5 * (dimension.width - rect.x) - border.x + 1;
            xOffset = (float) 0.5
                  * (dimension.width - xLevel * (children - 1) - rect.x)
                  - border.x;
         }
         if (visible.height < 2 * (border.y + yLevel) + rect.y) dimension.height = (int) (2 * (border.y + yLevel) + rect.y);
         else yOffset = (float) 0.5 * (dimension.height - rect.y) - yLevel
               - border.y;
         TreeBrowserBrowserNode tbbFocus = (TreeBrowserBrowserNode) content
               .get(focus);
         TreeBrowserBrowserNode tbbChild;
         tbbFocus.setNode(node);
         for (int i = 0; i < children; i++) //create all the children
         {
            if (content.size() <= child + i)
                  content.add(new TreeBrowserBrowserNode(
                        (TreeBrowserBrowserNode) content.get(focus)));
            tbbChild = (TreeBrowserBrowserNode) content.get(child + i);
            tbbChild.setNode(((Edge) node.getChilds().get(i)).target());
            tbbChild.setRectangle(new Rectangle2D.Float(border.x + xOffset + i
                  * xLevel, border.y + yOffset + 2 * yLevel, rect.x, rect.y));
            tbbChild.setActive(true);
            tbbChild.setTransitionRect(new Rectangle2D.Float(border.x
                  + (float) 0.5
                  * (xOffset + xMiddle + i * xLevel + rect.x
                        * (1 - smallRectRel.x)), border.y + yOffset
                  + (float) 1.5 * yLevel + (float) 0.5 * rect.y
                  * (1 - smallRectRel.y), rect.x * smallRectRel.x, rect.y
                  * smallRectRel.y));
            tbbChild.setHasTransition(true);
         }
         //create the focused node - the childrens' parent
         tbbFocus.setRectangle(new Rectangle2D.Float(border.x + xMiddle,
               border.y + yOffset + yLevel, rect.x, rect.y));
         tbbFocus.setActive(true);
         tbbFocus.setHasTransition(false);
         //guess that there's no parent of the focused node
         //but if there's one create and init it
         if ((node.getParent() != null) && (node.getParent().source() != null)) {
            node = node.getParent().source();
            TreeBrowserBrowserNode tbbParent = (TreeBrowserBrowserNode) content
                  .get(parent);
            tbbParent.setNode(node);
            tbbParent.setRectangle(new Rectangle2D.Float(border.x + xMiddle,
                  border.y + yOffset, rect.x, rect.y));
            tbbParent.setActive(true);
            tbbFocus.setTransitionRect(new Rectangle2D.Float(border.x + xMiddle
                  + (float) 0.5 * rect.x * (1 - smallRectRel.x), border.y
                  + yOffset + (float) 0.5
                  * (yLevel + rect.y * (1 - smallRectRel.y)), rect.x
                  * smallRectRel.x, rect.y * smallRectRel.y));
            tbbFocus.setHasTransition(true);
         }
         //scrollpane related stuff
         setPreferredSize(dimension);
         scrollPane.revalidate();
         scrollPane.repaint();
         //always scroll to the middle
         if (dimension != visible) {
            Point viewPoint = scrollPane.getViewport().getViewPosition();
            if (dimension.width > visible.width)
                  viewPoint.x = (dimension.width - visible.width) / 2;
            if (dimension.height > visible.height)
                  viewPoint.y = (dimension.height - visible.height) / 2;
            scrollPane.getViewport().setViewPosition(viewPoint);
         }
      }
   }

   /**
    * Method handleMouseClicked.
    * 
    * @param pos
    *           the mouse has been clicked check if a valid node has been
    *           clicked and trigger a general update in this case
    */
   public void handleMouseClicked (Point2D.Float pos) {
      int i = 0;
      while ((i < content.size())
            && ((i == focus) || !((TreeBrowserBrowserContent) content.get(i))
                  .checkRectangle(pos)))
         i++;
      if (i < content.size()) {
         tb.setNode((((TreeBrowserBrowserNode) content.get(i)).getNode()));
         tb.updateTB();
      }
   }

   /**
    * Method handleMouseMoved.
    * 
    * @param pos
    *           if the mouse is moved over a clickable component, draw it blue
    *           to indicate that it is clickable note: this is a very time
    *           critical method and therefore the repaint method is only
    *           triggered when absolutely necessary
    */
   public void handleMouseMoved (Point2D.Float pos) {
      oldtbbc = tbbc;
      tbbc = null;
      int i = 0;
      while ((i < content.size())
            && ((i == focus) || !((TreeBrowserBrowserContent) content.get(i))
                  .checkRectangle(pos)))
         i++;
      if (i < content.size()) {
         tbbc = (TreeBrowserBrowserContent) content.get(i);
         tbbc.setColor(Color.blue);
      }
      if (tbbc != oldtbbc) {
         if (oldtbbc != null) oldtbbc.setColor(Color.black);
         repaint();
      }
   }

   /**
    * @see javax.swing.JComponent#paintComponent(Graphics) paint the whole
    *      browser simply by iterating through its' components
    */
   public void paintComponent (Graphics g) {
      super.paintComponent(g);
      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      Iterator contentIter = content.iterator();
      while (contentIter.hasNext())
         ((TreeBrowserBrowserContent) contentIter.next()).paint(g);
   }
}