/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Stroke;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

/**
 * the parent of all the classes that do anything in the browser
 * 
 * @author Philipp Schlegel
 */
public abstract class TreeBrowserBrowserContent {

   protected boolean isActive = false; //active flag

   protected Rectangle2D.Float rectangle; //the related rectangle

   protected Color color = Color.black; //the current color

   protected Color highlightedColor = Color.green; //the highlighted Color;

   protected Stroke stroke = new BasicStroke(2.0f); //the current stroke

   public abstract void setActive (boolean active);

   public boolean isActive () {
      return (isActive);
   }

   public abstract void setRectangle (Rectangle2D.Float rectangle);

   public void setColor (Color color) {
      this.color = color;
   }

   public void setHighlightedColor (Color color) {
      highlightedColor = color;
   }

   public void setStroke (Stroke stroke) {
      this.stroke = stroke;
   }

   /**
    * Method checkRectangle.
    * 
    * @param point
    * @return boolean needed for the user interaction with the component note:
    *         of course this method returns only true if the component is active
    */
   final public boolean checkRectangle (Point2D.Float point) {
      if (isActive && (point.x > rectangle.x) && (point.y > rectangle.y)
            && (point.x < rectangle.x + rectangle.width)
            && (point.y < rectangle.y + rectangle.height)) {
         return (true);
      }
	return (false);
   }

   public abstract void paint (Graphics g);
}