/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

/**
 * represents the nodes of the browser
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowserBrowserNode extends TreeBrowserBrowserContent {

   private TreeBrowserBrowserNode parent; //the parent

   private Node node; //the node represented through this class

   private boolean hasTransition = false; //if there's a transition or not

   private Rectangle2D.Float transitionRect; //the related rectangles

   private Point2D.Float connectionPoint;

   public TreeBrowserBrowserNode () {}

   /**
    * Method TreeBrowserBrowserNode.
    * 
    * @param parent
    *           a little tree must be created for the transitions
    */
   public TreeBrowserBrowserNode (TreeBrowserBrowserNode parent) {
      this.parent = parent;
   }

   /**
    * @see nondeterministic.treebrowser.TreeBrowserBrowserContent#setActive(boolean)
    *      the usual method to set the component active (visible) with the
    *      necessary constraints
    */
   public void setActive (boolean active) {
      isActive = false;
      if ((active == true) && (node != null) && (rectangle != null))
            isActive = true;
   }

   public void setNode (Node node) {
      this.node = node;
      if (node == null) isActive = false;
   }

   public Node getNode () {
      return (node);
   }

   public void setRectangle (Rectangle2D.Float rectangle) {
      this.rectangle = rectangle;
      if (rectangle != null) {
         connectionPoint = new Point2D.Float(rectangle.x + (float) 0.5
               * rectangle.width, rectangle.y + rectangle.height);
      } else isActive = false;
   }

   public void setHasTransition (boolean hasTransition) {
      this.hasTransition = false;
      if ((hasTransition == true) && (parent != null)
            && (transitionRect != null)) {
         this.hasTransition = true;
      }
   }

   public void setTransitionRect (Rectangle2D.Float transitionRect) {
      this.transitionRect = transitionRect;
      if (transitionRect == null) {
         hasTransition = false;
      }
   }

   /**
    * Method getConnectionPoint.
    * 
    * @return Float get the point where the transition has to be connected
    */
   public Point2D.Float getConnectionPoint () {
      if (rectangle != null) {
         return (connectionPoint);
      }
	return (null);
   }

   public void paint (Graphics g) {
      Graphics2D g2d = (Graphics2D) g;
      if (isActive) {
         Color oldColor = (Color) g2d.getPaint();
         Color transitionColor = oldColor;
         Stroke oldStroke = g2d.getStroke();
         node.paint(g, new Rectangle((int) rectangle.x, (int) rectangle.y,
               (int) rectangle.width, (int) rectangle.height));
         g2d.setPaint(color);
         g2d.setStroke(stroke);
         g2d.draw(rectangle);
         g2d.setStroke(oldStroke);
         g2d.setPaint(oldColor);
         if (hasTransition && parent.isActive() && (node.getParent() != null)) {
            if (node.getParent().isHighlighted())
                  transitionColor = highlightedColor;
            g2d.setPaint(transitionColor);
            g2d.draw(new Line2D.Float(parent.getConnectionPoint(),
                  new Point2D.Float(
                        (float) (rectangle.x + 0.5 * rectangle.width),
                        rectangle.y)));
            g2d.setPaint(Color.white);
            g2d.fill(transitionRect);
            g2d.setPaint(oldColor);
            node.getParent()
                  .paint(
                        g,
                        new Rectangle((int) transitionRect.x,
                              (int) transitionRect.y,
                              (int) transitionRect.width,
                              (int) transitionRect.height));
            g2d.setPaint(transitionColor);
            g2d.draw(transitionRect);
            g2d.setColor(oldColor);
         }
      }
   }
}