/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;

/**
 * does simply paint the detailed view of the focused node
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowserDetails extends TreeBrowserComponent {

   public TreeBrowserDetails (TreeBrowser tb) {
      super(tb);
      init();
   }

   //	public TreeBrowserDetails(TreeBrowser tb,boolean isDoubleBuffered) {
   //		super(tb,isDoubleBuffered);
   //		init();
   //	}
   private void init () {
      setBackground(Color.white);
   }

   /**
    * @see nondeterministic.treebrowser.TreeBrowserComponent#updateTBC()
    */
   public void updateTBC () {
      repaint();
   }

   /**
    * @see javax.swing.JComponent#paintComponent(Graphics) note: the assumption
    *      is that the detailed view is always square
    */
   public void paintComponent (Graphics g) {
      super.paintComponent(g);
      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      Insets insets = getInsets();
      int width = getWidth() - insets.left - insets.right;
      int height = getHeight() - insets.top - insets.bottom;
      int size;
      if (width < height) size = width;
      else size = height;
      tb.getNode().paintDetails(
            g,
            new Rectangle(insets.left + width / 2 - size / 2, insets.top
                  + height / 2 - size / 2, size, size));
   }
}