/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.util.Iterator;

import ch.ethz.exorciser.Messages;

/**
 * draws the Tree in respect of the Node and Tree attributes this turned out to
 * be a very memory and time critical component therefore it is optimized for
 * memory and speed as far this ain't a contradiction although it is optimized
 * there may be algorithms that do an adaption of the tree rather than a
 * complete new calculation each time an update occurs due to the lack of time
 * and the possible complexity of such algorithms and especially because the
 * Java(tm) Graphics Library seems to be the REAL bottleneck I do not intend to
 * implement such algorithms the limit of usability on my PIII@800MHz is at
 * about 10'000 nodes ATTENTION: cycles in the 'tree' will result in infinite
 * loops
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowserTree extends TreeBrowserComponent {

   private int xBorder = 10; //guaranteed gap before the most left node

   private int yBorder = 20;

   private int size = 6; //size of a node

   private int curSize = 10; //size of the focused node

   private Color color = Color.black; //color of the nodes

   private Color curColor = Color.red; //color of the focused node

   private int deltaY = 0;

   private int maxDepth = 50; //absolute maximum tree depth

   private int depth = 0;

   private Tree tree;

   private TreeBrowserTreeNode oldTbtNode = null;

   private TreeBrowserTreeNode tbtNode = null;

   private TreeBrowserTreeNodeStorage content = new TreeBrowserTreeNodeStorage();

   //a sophisticated pool for the nodes
   public TreeBrowserTree (TreeBrowser tb) {
      super(tb);
      init();
   }

   //	public TreeBrowserTree(TreeBrowser tb,boolean isDoubleBuffered) {
   //		super(tb,isDoubleBuffered);
   //		init();
   //	}
   private void init () {
      TreeBrowserTreeListener tbtListener = new TreeBrowserTreeListener();
      setBackground(Color.white);
      addComponentListener(tbtListener);
      addMouseListener(tbtListener);
      tree = tb.getTree();
      //		initTree();
      //		updateTBC();
   }

   /**
    * Method traverseTree.
    * 
    * @param level
    * @param parent
    * @param xPos
    * @param width
    * @param top
    *           traverses the tree node for node in respect to the attributes
    *           and the maxDepth if all children of a leaf are hidden the height
    *           of the drawn tree may be wrong by one level in my opinion this
    *           is negligible and doesn't justify a complicated handlig
    */
   private void traverseTree (int level, TreeBrowserTreeNode parent, int xPos,
         int width, int top) {
      if (level < maxDepth) {
         TreeBrowserTreeNode tbt_Node;
         Node node = parent.getNode();
         Node curNode;
         int nofChildren;
         if ((node.getChilds() != null)
               && ((nofChildren = node.getChilds().size()) != 0)) {
            if (depth < level) depth = level;
            int deltaX = width / nofChildren;
            for (int i = 0; i < nofChildren; i++) {
               if (!(curNode = ((Edge) node.getChilds().get(i)).target())
                     .isHidden()) {
                  tbt_Node = content.getNode();
                  tbt_Node.setParent(parent);
                  tbt_Node.setNode(curNode);
                  tbt_Node.update(xPos + deltaX / 2 + i * deltaX, top, level);
                  tbt_Node.setSize(size);
                  if (curNode == tb.getNode()) {
                     tbt_Node.setSize(curSize);
                     tbt_Node.setVisitedColor(curColor);
                     this.tbtNode = tbt_Node;
                  }
                  tbt_Node.setActive(true);
                  traverseTree(level + 1, tbt_Node, xPos + i * deltaX, deltaX,
                        top);
               }
            }
         }
      }
   }

   /**
    * Method initTree. inits together with traverseTree the whole tree including
    * the positions to save memory only unhidden nodes have been borne in minds
    * thus each time hidden attributes change a reinit of the tree is necessary
    */
   public void initTree () {
      if ((tree != null) && (tree.root() != null)) {
         TreeBrowserTreeNode tbt_Node;
         Insets insets = getInsets();
         int width = getWidth() - insets.left - insets.right - 2 * xBorder;
         depth = 0;
         content.reset();
         this.tbtNode = null;
         tbt_Node = content.getNode();
         tbt_Node.setNode(tree.root());
         tbt_Node.update(insets.left + xBorder + width / 2, insets.top
               + yBorder, 0);
         if (tree.root() == tb.getNode()) {
            tbt_Node.setSize(curSize);
            tbt_Node.setVisitedColor(curColor);
            this.tbtNode = tbt_Node;
         }
         tbt_Node.setActive(true);
         traverseTree(1, tbt_Node, insets.left + xBorder, width, insets.top
               + yBorder);
         content.finish();
         deltaY = (getHeight() - insets.top - insets.bottom - 2 * yBorder)
               / depth;
      }
      repaint();
   }

   /**
    * @see nondeterministic.treebrowser.TreeBrowserComponent#updateTBC()
    */
   public void updateTBC () {
      if (tb.getTree() != tree) tree = tb.getTree();
      oldTbtNode = tbtNode;
      if (oldTbtNode != null) {
         oldTbtNode.setSize(size);
         oldTbtNode.setVisitedColor(color);
      }
      initTree();
   }

   public void handleMouseClicked (int x, int y) {
      TreeBrowserTreeNode node = null;
      Iterator contentIter = content.iterator();
      while ((contentIter.hasNext())
            && (!(node = (TreeBrowserTreeNode) contentIter.next())
                  .checkRectangle(x, y, deltaY))) {}
      if ((node != null) && (node.checkRectangle(x, y, deltaY))) {
         tb.setNode(node.getNode());
         tb.updateTB();
      }
   }

   public void paint (Graphics g) {
      super.paint(g);
      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
      Iterator contentIter = content.iterator();
      while (contentIter.hasNext())
         ((TreeBrowserTreeNode) contentIter.next()).paint(g, deltaY);
      if (tbtNode != null) tbtNode.paint(g, deltaY);
      g2d.drawString(Messages.getString("TreeBrowser.label.number_of_nodes")//$NON-NLS-1$
            + tree.numVisibleNodes(), 10, 30); 
   }
}