/*
 * Exorciser - www.educeth.ch/exorciser  Copyright 2000-2004 Swiss Federal
 * Institue of Technology, Zrich Switzerland
 */
package ch.ethz.exorciser.treebrowser;

import java.awt.Color;
import java.awt.Graphics;

/**
 * represents the nodes of the tree
 * 
 * @author Philipp Schlegel
 */
final public class TreeBrowserTreeNode {

   private boolean isActive;

   private TreeBrowserTreeNode parent;

   private Node node;

   private int x = -1;

   private int top = 0;

   private int level = 0;

   private int size = 4;

   private Color color = Color.gray;

   private Color visitedColor = Color.black;

   private Color highlightedColor = Color.green;

   public void setActive (boolean active) {
      isActive = false;
      if ((active == true) && (node != null) && (x != -1)) isActive = true;
   }

   public boolean isActive () {
      return (isActive);
   }

   public void setParent (TreeBrowserTreeNode parent) {
      this.parent = parent;
   }

   public void setNode (Node node) {
      this.node = node;
      if (node == null) isActive = false;
   }

   public Node getNode () {
      return node;
   }

   public void setVisitedColor (Color color) {
      visitedColor = color;
   }

   public void update (int x_, int top_, int level_) {
      this.x = x_;
      this.top = top_;
      this.level = level_;
      if (x_ == -1) isActive = false;
   }

   public int getX () {
      return (x);
   }

   public int getY (int deltaY) {
      return (top + level * deltaY);
   }

   public void setSize (int size) {
      this.size = size;
   }

   public void setColor (Color color) {
      this.color = color;
   }

   public void setHighlightedColor (Color color) {
      highlightedColor = color;
   }

   /**
    * Method checkRectangle.
    * 
    * @param x
    * @param y
    * @param deltaY
    * @return boolean needed for the user interaction with the component note:
    *         of course this method returns only true if the component is active
    */
   public boolean checkRectangle (int x_, int y_, int delta_y) {
      int _y = top + level * delta_y;
      if (isActive && (x_ > this.x - size / 2) && (y_ > _y - size / 2)
            && (x_ < this.x + size / 2) && (y_ < _y + size / 2)) {
		return (true);
	}
	return (false);
   }

   public void paint (Graphics g, int deltaY) {
      if ((isActive) && (!node.isHidden())) {
         int y = top + level * deltaY;
         Color oldColor = g.getColor();
         if ((parent != null) && (parent.isActive)) {
            if (node.getParent().isHighlighted()) g.setColor(highlightedColor);
            else g.setColor(color);
            g.drawLine(parent.getX(), parent.getY(deltaY), x, y);
         }
         if (node.isHighlighted()) g.setColor(highlightedColor);
         else if (node.visited()) g.setColor(visitedColor);
         else g.setColor(color);
         g.fillRect(x - size / 2, y - size / 2, size, size);
         g.setColor(oldColor);
      }
   }
}