/*
 * Exorciser - Automated generation and grading of exercises in an introductory
 * course on the Theory of Computation,
 * 
 * Copyright (C) 2000-2004 Vincent Tscherter et al., Swiss Federal Institue of
 * Technology, Zrich Switzerland
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package ch.ethz.exorciser.treebrowser;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * a quite sophisticated class to handle TreeBrowserTreeNodes optimized to save
 * time and memory by using an adaption of the common pool concept especially
 * there's no unnecessary pool/storage traversal
 * 
 * @author Philipp Schlegel
 */
public class TreeBrowserTreeNodeStorage {

   private int poolIndex = 0;

   private int index = 0;

   private int oldSize = 0;

   private ArrayList pool = new ArrayList();

   private ArrayList storage = new ArrayList();

   /**
    * Method reset. resets the storage
    */
   public void reset () {
      poolIndex = pool.size() - 1;
      index = 0;
      oldSize = storage.size();
   }

   /**
    * Method getNode.
    * 
    * @return TreeBrowserTreeNode get a new node
    */
   public TreeBrowserTreeNode getNode () {
      if (index < oldSize) return ((TreeBrowserTreeNode) storage.get(index++));
      else if (poolIndex > -1) {
         TreeBrowserTreeNode tmp = (TreeBrowserTreeNode) pool.get(poolIndex);
         pool.remove(poolIndex--);
         storage.add(tmp);
         return (tmp);
      } else {
         TreeBrowserTreeNode tbtn = new TreeBrowserTreeNode();
         storage.add(tbtn);
         return (tbtn);
      }
   }

   /**
    * Method finish. must be called before the use of any iterators on the
    * storage (removes superfluous nodes)
    */
   public void finish () {
      if (index < oldSize) {
         int tmp = oldSize - 1;
         oldSize = index - 1;
         index = tmp;
         while (index > oldSize) {
            ((TreeBrowserTreeNode) storage.get(index)).setNode(null);
            pool.add(storage.get(index));
            storage.remove(index--);
         }
      }
   }

   /**
    * Method iterator.
    * 
    * @return Iterator the iterator
    */
   public Iterator iterator () {
      return (storage.iterator());
   }
}